/* Generated by Cython 0.28.3 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "_GLIBCXX_USE_CXX11_ABI",
                "0"
            ],
            [
                "_FORCE_INLINES",
                "1"
            ]
        ],
        "include_dirs": [
            "/usr/local/cuda/include"
        ],
        "language": "c++",
        "libraries": [
            "cudnn"
        ],
        "library_dirs": [
            "/usr/local/cuda/lib64"
        ],
        "name": "cupy.cudnn",
        "runtime_library_dirs": [
            "/usr/local/cuda/lib64"
        ],
        "sources": [
            "cupy/cudnn.pyx"
        ]
    },
    "module_name": "cupy.cudnn"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_28_3"
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(U other) { return *ptr == other; }
    template<typename U> bool operator !=(U other) { return *ptr != other; }
  private:
    T *ptr;
};

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0; // PyThread_create_key reports success always
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif // TSS (Thread Specific Storage) API
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
  #define PyObject_Unicode             PyObject_Str
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : (Py_INCREF(func), func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__cupy__cudnn
#define __PYX_HAVE_API__cupy__cudnn
/* Early includes */
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <vector>
#include <utility>
#include <map>
#include <stdint.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "cupy/cudnn.pyx",
  "stringsource",
  "cupy/cuda/device.pxd",
  "cupy/cuda/memory.pxd",
  "cupy/cuda/function.pxd",
  "cupy/core/core.pxd",
};

/* "internal.pxd":43
 * cpdef size_t clp2(size_t x)
 * 
 * ctypedef unsigned short _float16             # <<<<<<<<<<<<<<
 * 
 * cpdef uint16_t to_float16(float f)
 */
typedef unsigned short __pyx_t_4cupy_4core_8internal__float16;

/*--- Type declarations ---*/
struct __pyx_obj_4cupy_4cuda_6device_Device;
struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer;
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool;
struct __pyx_obj_4cupy_4cuda_8function_CPointer;
struct __pyx_obj_4cupy_4cuda_8function_Function;
struct __pyx_obj_4cupy_4cuda_8function_Module;
struct __pyx_obj_4cupy_4cuda_8function_LinkState;
struct __pyx_obj_4cupy_4core_4core_ndarray;
struct __pyx_obj_4cupy_4core_4core_broadcast;
struct __pyx_obj_4cupy_4core_4core_Indexer;
struct __pyx_obj_4cupy_5cudnn__Algorithm;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async;
struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator;
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks;

/* "memory.pxd":29
 * 
 *     cpdef copy_from_device(self, MemoryPointer src, Py_ssize_t size)
 *     cpdef copy_from_device_async(self, MemoryPointer src, size_t size,             # <<<<<<<<<<<<<<
 *                                  stream=?)
 *     cpdef copy_from_host(self, mem, size_t size)
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":32
 *                                  stream=?)
 *     cpdef copy_from_host(self, mem, size_t size)
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef copy_from(self, mem, size_t size)
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":34
 *     cpdef copy_from_host_async(self, mem, size_t size, stream=?)
 *     cpdef copy_from(self, mem, size_t size)
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef copy_to_host(self, mem, size_t size)
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":36
 *     cpdef copy_from_async(self, mem, size_t size, stream=?)
 *     cpdef copy_to_host(self, mem, size_t size)
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef memset(self, int value, size_t size)
 *     cpdef memset_async(self, int value, size_t size, stream=?)
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":38
 *     cpdef copy_to_host_async(self, mem, size_t size, stream=?)
 *     cpdef memset(self, int value, size_t size)
 *     cpdef memset_async(self, int value, size_t size, stream=?)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async {
  int __pyx_n;
  PyObject *stream;
};

/* "memory.pxd":44
 * 
 * 
 * cpdef set_allocator(allocator=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_set_allocator {
  int __pyx_n;
  PyObject *allocator;
};

/* "memory.pxd":53
 * 
 *     cpdef MemoryPointer malloc(self, Py_ssize_t size)
 *     cpdef free_all_blocks(self, stream=?)             # <<<<<<<<<<<<<<
 *     cpdef free_all_free(self)
 *     cpdef n_free_blocks(self)
 */
struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks {
  int __pyx_n;
  PyObject *stream;
};
struct __pyx_opt_args_4cupy_4cuda_8function_8Function_linear_launch;

/* "cupy/cuda/function.pxd":11
 *         public size_t ptr
 * 
 *     cpdef linear_launch(self, size_t size, args, size_t shared_mem=*,             # <<<<<<<<<<<<<<
 *                         size_t block_max_size=*, stream=*)
 * 
 */
struct __pyx_opt_args_4cupy_4cuda_8function_8Function_linear_launch {
  int __pyx_n;
  size_t shared_mem;
  size_t block_max_size;
  PyObject *stream;
};
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_tofile;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_astype;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_copy;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_view;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_squeeze;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_take;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_repeat;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_choose;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_sort;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argsort;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_partition;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argpartition;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_diagonal;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_max;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argmax;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_min;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argmin;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_clip;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_round;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_trace;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_sum;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_cumsum;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_mean;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_var;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_std;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_prod;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_cumprod;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_all;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_any;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_get;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_set;
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_reduced_view;
struct __pyx_opt_args_4cupy_4core_4core_ascontiguousarray;
struct __pyx_opt_args_4cupy_4core_4core_compile_with_cache;

/* "core.pxd":22
 * 
 *     cpdef tolist(self)
 *     cpdef tofile(self, fid, sep=*, format=*)             # <<<<<<<<<<<<<<
 *     cpdef dump(self, file)
 *     cpdef dumps(self)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_tofile {
  int __pyx_n;
  PyObject *sep;
  PyObject *format;
};

/* "core.pxd":25
 *     cpdef dump(self, file)
 *     cpdef dumps(self)
 *     cpdef ndarray astype(self, dtype, order=*, casting=*, subok=*, copy=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray copy(self, order=*)
 *     cpdef ndarray view(self, dtype=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_astype {
  int __pyx_n;
  PyObject *order;
  PyObject *casting;
  PyObject *subok;
  PyObject *copy;
};

/* "core.pxd":26
 *     cpdef dumps(self)
 *     cpdef ndarray astype(self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef ndarray copy(self, order=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray view(self, dtype=*)
 *     cpdef fill(self, value)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_copy {
  int __pyx_n;
  PyObject *order;
};

/* "core.pxd":27
 *     cpdef ndarray astype(self, dtype, order=*, casting=*, subok=*, copy=*)
 *     cpdef ndarray copy(self, order=*)
 *     cpdef ndarray view(self, dtype=*)             # <<<<<<<<<<<<<<
 *     cpdef fill(self, value)
 *     cpdef ndarray _reshape(self, vector.vector[Py_ssize_t] shape)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_view {
  int __pyx_n;
  PyObject *dtype;
};

/* "core.pxd":34
 *     cpdef ndarray flatten(self)
 *     cpdef ndarray ravel(self)
 *     cpdef ndarray squeeze(self, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray take(self, indices, axis=*, out=*)
 *     cpdef repeat(self, repeats, axis=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_squeeze {
  int __pyx_n;
  PyObject *axis;
};

/* "core.pxd":35
 *     cpdef ndarray ravel(self)
 *     cpdef ndarray squeeze(self, axis=*)
 *     cpdef ndarray take(self, indices, axis=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef repeat(self, repeats, axis=*)
 *     cpdef choose(self, choices, out=*, mode=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_take {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
};

/* "core.pxd":36
 *     cpdef ndarray squeeze(self, axis=*)
 *     cpdef ndarray take(self, indices, axis=*, out=*)
 *     cpdef repeat(self, repeats, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef choose(self, choices, out=*, mode=*)
 *     cpdef sort(self, int axis=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_repeat {
  int __pyx_n;
  PyObject *axis;
};

/* "core.pxd":37
 *     cpdef ndarray take(self, indices, axis=*, out=*)
 *     cpdef repeat(self, repeats, axis=*)
 *     cpdef choose(self, choices, out=*, mode=*)             # <<<<<<<<<<<<<<
 *     cpdef sort(self, int axis=*)
 *     cpdef ndarray argsort(self, axis=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_choose {
  int __pyx_n;
  PyObject *out;
  PyObject *mode;
};

/* "core.pxd":38
 *     cpdef repeat(self, repeats, axis=*)
 *     cpdef choose(self, choices, out=*, mode=*)
 *     cpdef sort(self, int axis=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray argsort(self, axis=*)
 *     cpdef partition(self, kth, int axis=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_sort {
  int __pyx_n;
  int axis;
};

/* "core.pxd":39
 *     cpdef choose(self, choices, out=*, mode=*)
 *     cpdef sort(self, int axis=*)
 *     cpdef ndarray argsort(self, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef partition(self, kth, int axis=*)
 *     cpdef ndarray argpartition(self, kth, axis=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argsort {
  int __pyx_n;
  PyObject *axis;
};

/* "core.pxd":40
 *     cpdef sort(self, int axis=*)
 *     cpdef ndarray argsort(self, axis=*)
 *     cpdef partition(self, kth, int axis=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray argpartition(self, kth, axis=*)
 *     cpdef tuple nonzero(self)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_partition {
  int __pyx_n;
  int axis;
};

/* "core.pxd":41
 *     cpdef ndarray argsort(self, axis=*)
 *     cpdef partition(self, kth, int axis=*)
 *     cpdef ndarray argpartition(self, kth, axis=*)             # <<<<<<<<<<<<<<
 *     cpdef tuple nonzero(self)
 *     cpdef ndarray diagonal(self, offset=*, axis1=*, axis2=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argpartition {
  int __pyx_n;
  PyObject *axis;
};

/* "core.pxd":43
 *     cpdef ndarray argpartition(self, kth, axis=*)
 *     cpdef tuple nonzero(self)
 *     cpdef ndarray diagonal(self, offset=*, axis1=*, axis2=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray max(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef ndarray argmax(self, axis=*, out=*, dtype=*,
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_diagonal {
  int __pyx_n;
  PyObject *offset;
  PyObject *axis1;
  PyObject *axis2;
};

/* "core.pxd":44
 *     cpdef tuple nonzero(self)
 *     cpdef ndarray diagonal(self, offset=*, axis1=*, axis2=*)
 *     cpdef ndarray max(self, axis=*, out=*, dtype=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray argmax(self, axis=*, out=*, dtype=*,
 *                          keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_max {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *dtype;
  PyObject *keepdims;
};

/* "core.pxd":45
 *     cpdef ndarray diagonal(self, offset=*, axis1=*, axis2=*)
 *     cpdef ndarray max(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef ndarray argmax(self, axis=*, out=*, dtype=*,             # <<<<<<<<<<<<<<
 *                          keepdims=*)
 *     cpdef ndarray min(self, axis=*, out=*, dtype=*, keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argmax {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *dtype;
  PyObject *keepdims;
};

/* "core.pxd":47
 *     cpdef ndarray argmax(self, axis=*, out=*, dtype=*,
 *                          keepdims=*)
 *     cpdef ndarray min(self, axis=*, out=*, dtype=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray argmin(self, axis=*, out=*, dtype=*,
 *                          keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_min {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *dtype;
  PyObject *keepdims;
};

/* "core.pxd":48
 *                          keepdims=*)
 *     cpdef ndarray min(self, axis=*, out=*, dtype=*, keepdims=*)
 *     cpdef ndarray argmin(self, axis=*, out=*, dtype=*,             # <<<<<<<<<<<<<<
 *                          keepdims=*)
 *     cpdef ndarray clip(self, a_min=*, a_max=*, out=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argmin {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *dtype;
  PyObject *keepdims;
};

/* "core.pxd":50
 *     cpdef ndarray argmin(self, axis=*, out=*, dtype=*,
 *                          keepdims=*)
 *     cpdef ndarray clip(self, a_min=*, a_max=*, out=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray round(self, decimals=*, out=*)
 * 
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_clip {
  int __pyx_n;
  PyObject *a_min;
  PyObject *a_max;
  PyObject *out;
};

/* "core.pxd":51
 *                          keepdims=*)
 *     cpdef ndarray clip(self, a_min=*, a_max=*, out=*)
 *     cpdef ndarray round(self, decimals=*, out=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef ndarray trace(self, offset=*, axis1=*, axis2=*, dtype=*,
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_round {
  int __pyx_n;
  PyObject *decimals;
  PyObject *out;
};

/* "core.pxd":53
 *     cpdef ndarray round(self, decimals=*, out=*)
 * 
 *     cpdef ndarray trace(self, offset=*, axis1=*, axis2=*, dtype=*,             # <<<<<<<<<<<<<<
 *                         out=*)
 *     cpdef ndarray sum(self, axis=*, dtype=*, out=*, keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_trace {
  int __pyx_n;
  PyObject *offset;
  PyObject *axis1;
  PyObject *axis2;
  PyObject *dtype;
  PyObject *out;
};

/* "core.pxd":55
 *     cpdef ndarray trace(self, offset=*, axis1=*, axis2=*, dtype=*,
 *                         out=*)
 *     cpdef ndarray sum(self, axis=*, dtype=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray cumsum(self, axis=*, dtype=*, out=*)
 * 
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_sum {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *keepdims;
};

/* "core.pxd":56
 *                         out=*)
 *     cpdef ndarray sum(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef ndarray cumsum(self, axis=*, dtype=*, out=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef ndarray mean(self, axis=*, dtype=*, out=*, keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_cumsum {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
};

/* "core.pxd":58
 *     cpdef ndarray cumsum(self, axis=*, dtype=*, out=*)
 * 
 *     cpdef ndarray mean(self, axis=*, dtype=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray var(self, axis=*, dtype=*, out=*, ddof=*,
 *                       keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_mean {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *keepdims;
};

/* "core.pxd":59
 * 
 *     cpdef ndarray mean(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef ndarray var(self, axis=*, dtype=*, out=*, ddof=*,             # <<<<<<<<<<<<<<
 *                       keepdims=*)
 *     cpdef ndarray std(self, axis=*, dtype=*, out=*, ddof=*,
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_var {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *ddof;
  PyObject *keepdims;
};

/* "core.pxd":61
 *     cpdef ndarray var(self, axis=*, dtype=*, out=*, ddof=*,
 *                       keepdims=*)
 *     cpdef ndarray std(self, axis=*, dtype=*, out=*, ddof=*,             # <<<<<<<<<<<<<<
 *                       keepdims=*)
 *     cpdef ndarray prod(self, axis=*, dtype=*, out=*, keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_std {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *ddof;
  PyObject *keepdims;
};

/* "core.pxd":63
 *     cpdef ndarray std(self, axis=*, dtype=*, out=*, ddof=*,
 *                       keepdims=*)
 *     cpdef ndarray prod(self, axis=*, dtype=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray cumprod(a, axis=*, dtype=*, out=*)
 * 
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_prod {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
  PyObject *keepdims;
};

/* "core.pxd":64
 *                       keepdims=*)
 *     cpdef ndarray prod(self, axis=*, dtype=*, out=*, keepdims=*)
 *     cpdef ndarray cumprod(a, axis=*, dtype=*, out=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef ndarray all(self, axis=*, out=*, keepdims=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_cumprod {
  int __pyx_n;
  PyObject *axis;
  PyObject *dtype;
  PyObject *out;
};

/* "core.pxd":66
 *     cpdef ndarray cumprod(a, axis=*, dtype=*, out=*)
 * 
 *     cpdef ndarray all(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray any(self, axis=*, out=*, keepdims=*)
 *     cpdef ndarray conj(self)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_all {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "core.pxd":67
 * 
 *     cpdef ndarray all(self, axis=*, out=*, keepdims=*)
 *     cpdef ndarray any(self, axis=*, out=*, keepdims=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray conj(self)
 *     cpdef get(self, stream=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_any {
  int __pyx_n;
  PyObject *axis;
  PyObject *out;
  PyObject *keepdims;
};

/* "core.pxd":69
 *     cpdef ndarray any(self, axis=*, out=*, keepdims=*)
 *     cpdef ndarray conj(self)
 *     cpdef get(self, stream=*)             # <<<<<<<<<<<<<<
 *     cpdef set(self, arr, stream=*)
 *     cpdef ndarray reduced_view(self, dtype=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_get {
  int __pyx_n;
  PyObject *stream;
};

/* "core.pxd":70
 *     cpdef ndarray conj(self)
 *     cpdef get(self, stream=*)
 *     cpdef set(self, arr, stream=*)             # <<<<<<<<<<<<<<
 *     cpdef ndarray reduced_view(self, dtype=*)
 *     cpdef _update_c_contiguity(self)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_set {
  int __pyx_n;
  PyObject *stream;
};

/* "core.pxd":71
 *     cpdef get(self, stream=*)
 *     cpdef set(self, arr, stream=*)
 *     cpdef ndarray reduced_view(self, dtype=*)             # <<<<<<<<<<<<<<
 *     cpdef _update_c_contiguity(self)
 *     cpdef _update_f_contiguity(self)
 */
struct __pyx_opt_args_4cupy_4core_4core_7ndarray_reduced_view {
  int __pyx_n;
  PyObject *dtype;
};

/* "core.pxd":102
 * 
 * 
 * cpdef ndarray ascontiguousarray(ndarray a, dtype=*)             # <<<<<<<<<<<<<<
 * cpdef Module compile_with_cache(str source, tuple options=*, arch=*,
 *                                 cachd_dir=*, prepend_cupy_headers=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_ascontiguousarray {
  int __pyx_n;
  PyObject *dtype;
};

/* "core.pxd":103
 * 
 * cpdef ndarray ascontiguousarray(ndarray a, dtype=*)
 * cpdef Module compile_with_cache(str source, tuple options=*, arch=*,             # <<<<<<<<<<<<<<
 *                                 cachd_dir=*, prepend_cupy_headers=*)
 */
struct __pyx_opt_args_4cupy_4core_4core_compile_with_cache {
  int __pyx_n;
  PyObject *options;
  PyObject *arch;
  PyObject *cachd_dir;
  PyObject *prepend_cupy_headers;
};

/* "cudnn.pxd":6
 * ###############################################################################
 * 
 * cpdef enum:             # <<<<<<<<<<<<<<
 *     CUDNN_DATA_FLOAT = 0
 *     CUDNN_DATA_DOUBLE = 1
 */
enum  {
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_FLOAT = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_DOUBLE = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_HALF = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_PROPAGATE_NAN = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NON_DETERMINISTIC = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DETERMINISTIC = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NHWC = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_OP_TENSOR_ADD = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_OP_TENSOR_MUL = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_OP_TENSOR_MIN = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_OP_TENSOR_MAX = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_OP_TENSOR_SQRT = 4,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_OP_TENSOR_NOT = 5,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_ADD = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_MUL = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_MIN = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_MAX = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_AMAX = 4,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_AVG = 5,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_NORM1 = 6,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_NORM2 = 7,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_MUL_NO_ZEROS = 8,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_NO_INDICES = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_FLATTENED_INDICES = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_32BIT_INDICES = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_64BIT_INDICES = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_16BIT_INDICES = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_8BIT_INDICES = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ADD_IMAGE = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ADD_SAME_HW = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ADD_FEATURE_MAP = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ADD_SAME_CHW = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ADD_SAME_C = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ADD_FULL_TENSOR = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CROSS_CORRELATION = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_NO_WORKSPACE = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_PREFER_FASTEST = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_SPECIFY_WORKSPACE_LIMIT = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_IMPLICIT_GEMM = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_IMPLICIT_PRECOMP_GEMM = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_GEMM = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_DIRECT = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_FFT = 4,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_FFT_TILING = 5,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_WINOGRAD = 6,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_ALGO_WINOGRAD_NONFUSED = 7,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_NO_WORKSPACE = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_PREFER_FASTEST = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_SPECIFY_WORKSPACE_LIMIT = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_0 = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_1 = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_FFT = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_3 = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_WINOGRAD = 4,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_WINOGRAD_NONFUSED = 5,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_NO_WORKSPACE = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_PREFER_FASTEST = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_SPECIFY_WORKSPACE_LIMIT = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_0 = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_1 = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_FFT = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_FFT_TILING = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_WINOGRAD = 4,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_WINOGRAD_NONFUSED = 5,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SOFTMAX_FAST = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SOFTMAX_ACCURATE = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SOFTMAX_LOG = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SOFTMAX_MODE_INSTANCE = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SOFTMAX_MODE_CHANNEL = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_POOLING_MAX = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_POOLING_AVERAGE_COUNT_INCLUDE_PADDING = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_POOLING_AVERAGE_COUNT_EXCLUDE_PADDING = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ACTIVATION_SIGMOID = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ACTIVATION_RELU = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ACTIVATION_TANH = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ACTIVATION_CLIPPED_RELU = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ACTIVATION_ELU = 4,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_LRN_CROSS_CHANNEL_DIM1 = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DIVNORM_PRECOMPUTED_MEANS = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_BATCHNORM_PER_ACTIVATION = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_BATCHNORM_SPATIAL = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_BATCHNORM_SPATIAL_PERSISTENT = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_RELU = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_TANH = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_LSTM = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_GRU = 3,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_UNIDIRECTIONAL = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_BIDIRECTIONAL = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_ALGO_STANDARD = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_ALGO_PERSIST_STATIC = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_ALGO_PERSIST_DYNAMIC = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_DATA_LAYOUT_SEQ_MAJOR_UNPACKED = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_DATA_LAYOUT_SEQ_MAJOR_PACKED = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_DATA_LAYOUT_BATCH_MAJOR_UNPACKED = 2,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_PADDED_IO_DISABLED = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_PADDED_IO_ENABLED = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_LINEAR_INPUT = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SKIP_INPUT = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_SAMPLER_BILINEAR = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_STATUS_SUCCESS = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_STATUS_RUNTIME_PREREQUISITE_MISSING = 11,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_STATUS_RUNTIME_IN_PROGRESS = 12,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_STATUS_RUNTIME_FP_OVERFLOW = 13,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ERRQUERY_RAWCODE = 0,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ERRQUERY_NONBLOCKING = 1,
  __pyx_e_4cupy_4cuda_5cudnn_CUDNN_ERRQUERY_BLOCKING = 2
};
struct __pyx_opt_args_4cupy_4core_8internal_prod;
struct __pyx_opt_args_4cupy_4core_8internal_prod_ssize_t;

/* "internal.pxd":5
 * 
 * 
 * cpdef Py_ssize_t prod(args, Py_ssize_t init=*) except? -1             # <<<<<<<<<<<<<<
 * 
 * cpdef Py_ssize_t prod_ssize_t(
 */
struct __pyx_opt_args_4cupy_4core_8internal_prod {
  int __pyx_n;
  Py_ssize_t init;
};

/* "internal.pxd":7
 * cpdef Py_ssize_t prod(args, Py_ssize_t init=*) except? -1
 * 
 * cpdef Py_ssize_t prod_ssize_t(             # <<<<<<<<<<<<<<
 *     vector.vector[Py_ssize_t]& arr, Py_ssize_t init=*)
 * 
 */
struct __pyx_opt_args_4cupy_4core_8internal_prod_ssize_t {
  int __pyx_n;
  Py_ssize_t init;
};
struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor;
struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor;
struct __pyx_opt_args_4cupy_5cudnn__get_algorithm;

/* "cupy/cudnn.pyx":115
 * 
 * cpdef _create_tensor_nd_descriptor(
 *         size_t desc, core.ndarray arr, int data_type=-1):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_shape, c_strides
 *     cdef Py_ssize_t itemsize, s
 */
struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor {
  int __pyx_n;
  int data_type;
};

/* "cupy/cudnn.pyx":157
 * 
 * cpdef _create_filter_descriptor(
 *         size_t desc, core.ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_shape
 *     cdef Py_ssize_t s, ndim = arr._shape.size()
 */
struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor {
  int __pyx_n;
  int format;
};

/* "cupy/cudnn.pyx":554
 * 
 * 
 * cdef _Algorithm _get_algorithm(int algo, size_t memory, int mathType=0):             # <<<<<<<<<<<<<<
 *     cdef _Algorithm ret = _Algorithm.__new__(_Algorithm)
 *     ret.algo = algo
 */
struct __pyx_opt_args_4cupy_5cudnn__get_algorithm {
  int __pyx_n;
  int mathType;
};

/* "device.pxd":7
 * cpdef str get_compute_capability()
 * 
 * cdef class Device:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int id
 */
struct __pyx_obj_4cupy_4cuda_6device_Device {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6device_Device *__pyx_vtab;
  int id;
  PyObject *_device_stack;
};


/* "memory.pxd":10
 * 
 * @cython.no_gc
 * cdef class BaseMemory:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory {
  PyObject_HEAD
  size_t ptr;
  Py_ssize_t size;
  int device_id;
};


/* "memory.pxd":19
 * 
 * @cython.final
 * cdef class MemoryPointer:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *__pyx_vtab;
  size_t ptr;
  int device_id;
  struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *mem;
};


/* "memory.pxd":47
 * 
 * 
 * cdef class MemoryPool:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *__pyx_vtab;
  PyObject *_pools;
};


/* "cupy/cuda/function.pxd":1
 * cdef class CPointer:             # <<<<<<<<<<<<<<
 *     cdef void* ptr
 * 
 */
struct __pyx_obj_4cupy_4cuda_8function_CPointer {
  PyObject_HEAD
  void *ptr;
};


/* "cupy/cuda/function.pxd":5
 * 
 * 
 * cdef class Function:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_4cupy_4cuda_8function_Function {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_8function_Function *__pyx_vtab;
  struct __pyx_obj_4cupy_4cuda_8function_Module *module;
  size_t ptr;
};


/* "cupy/cuda/function.pxd":15
 * 
 * 
 * cdef class Module:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_4cupy_4cuda_8function_Module {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_8function_Module *__pyx_vtab;
  size_t ptr;
};


/* "cupy/cuda/function.pxd":26
 * 
 * 
 * cdef class LinkState:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_4cupy_4cuda_8function_LinkState {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState *__pyx_vtab;
  size_t ptr;
};


/* "core.pxd":7
 * from cupy.cuda.function cimport Module
 * 
 * cdef class ndarray:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object __weakref__
 */
struct __pyx_obj_4cupy_4core_4core_ndarray {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4core_4core_ndarray *__pyx_vtab;
  PyObject *__weakref__;
  Py_ssize_t size;
  std::vector<Py_ssize_t>  _shape;
  std::vector<Py_ssize_t>  _strides;
  int _c_contiguous;
  int _f_contiguous;
  PyObject *dtype;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *data;
  struct __pyx_obj_4cupy_4core_4core_ndarray *base;
};


/* "core.pxd":86
 * 
 * 
 * cdef class broadcast:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly tuple values
 */
struct __pyx_obj_4cupy_4core_4core_broadcast {
  PyObject_HEAD
  PyObject *values;
  PyObject *shape;
  Py_ssize_t size;
  Py_ssize_t nd;
};


/* "core.pxd":94
 * 
 * 
 * cdef class Indexer:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly Py_ssize_t size
 */
struct __pyx_obj_4cupy_4core_4core_Indexer {
  PyObject_HEAD
  struct __pyx_vtabstruct_4cupy_4core_4core_Indexer *__pyx_vtab;
  Py_ssize_t size;
  PyObject *shape;
};


/* "cupy/cudnn.pyx":547
 * 
 * 
 * cdef class _Algorithm:             # <<<<<<<<<<<<<<
 *     cdef:
 *         int algo
 */
struct __pyx_obj_4cupy_5cudnn__Algorithm {
  PyObject_HEAD
  int algo;
  int mathType;
  size_t memory;
};



/* "device.pxd":7
 * cpdef str get_compute_capability()
 * 
 * cdef class Device:             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int id
 */

struct __pyx_vtabstruct_4cupy_4cuda_6device_Device {
  PyObject *(*use)(struct __pyx_obj_4cupy_4cuda_6device_Device *, int __pyx_skip_dispatch);
  PyObject *(*synchronize)(struct __pyx_obj_4cupy_4cuda_6device_Device *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6device_Device *__pyx_vtabptr_4cupy_4cuda_6device_Device;


/* "memory.pxd":19
 * 
 * @cython.final
 * cdef class MemoryPointer:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer {
  PyObject *(*_init)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory *, Py_ssize_t);
  PyObject *(*copy_from_device)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, Py_ssize_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_device_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_device_async *__pyx_optional_args);
  PyObject *(*copy_from_host)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_host_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_host_async *__pyx_optional_args);
  PyObject *(*copy_from)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_from_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_from_async *__pyx_optional_args);
  PyObject *(*copy_to_host)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch);
  PyObject *(*copy_to_host_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, PyObject *, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_copy_to_host_async *__pyx_optional_args);
  PyObject *(*memset)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch);
  PyObject *(*memset_async)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *, int, size_t, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_13MemoryPointer_memset_async *__pyx_optional_args);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer;


/* "memory.pxd":47
 * 
 * 
 * cdef class MemoryPool:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool {
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*malloc)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, Py_ssize_t, int __pyx_skip_dispatch);
  PyObject *(*free_all_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_6memory_10MemoryPool_free_all_blocks *__pyx_optional_args);
  PyObject *(*free_all_free)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*n_free_blocks)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*used_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*free_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
  PyObject *(*total_bytes)(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool *__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool;


/* "cupy/cuda/function.pxd":5
 * 
 * 
 * cdef class Function:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_4cupy_4cuda_8function_Function {
  PyObject *(*linear_launch)(struct __pyx_obj_4cupy_4cuda_8function_Function *, size_t, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4cuda_8function_8Function_linear_launch *__pyx_optional_args);
};
static struct __pyx_vtabstruct_4cupy_4cuda_8function_Function *__pyx_vtabptr_4cupy_4cuda_8function_Function;


/* "cupy/cuda/function.pxd":15
 * 
 * 
 * cdef class Module:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_4cupy_4cuda_8function_Module {
  PyObject *(*load_file)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*load)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_global_var)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_function)(struct __pyx_obj_4cupy_4cuda_8function_Module *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_8function_Module *__pyx_vtabptr_4cupy_4cuda_8function_Module;


/* "cupy/cuda/function.pxd":26
 * 
 * 
 * cdef class LinkState:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState {
  PyObject *(*add_ptr_data)(struct __pyx_obj_4cupy_4cuda_8function_LinkState *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*complete)(struct __pyx_obj_4cupy_4cuda_8function_LinkState *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState *__pyx_vtabptr_4cupy_4cuda_8function_LinkState;


/* "core.pxd":7
 * from cupy.cuda.function cimport Module
 * 
 * cdef class ndarray:             # <<<<<<<<<<<<<<
 *     cdef:
 *         object __weakref__
 */

struct __pyx_vtabstruct_4cupy_4core_4core_ndarray {
  PyObject *(*tolist)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  PyObject *(*tofile)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_tofile *__pyx_optional_args);
  PyObject *(*dump)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*dumps)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*astype)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_astype *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*copy)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_copy *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*view)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_view *__pyx_optional_args);
  PyObject *(*fill)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*_reshape)(struct __pyx_obj_4cupy_4core_4core_ndarray *, std::vector<Py_ssize_t> , int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*_transpose)(struct __pyx_obj_4cupy_4core_4core_ndarray *, std::vector<Py_ssize_t> , int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*swapaxes)(struct __pyx_obj_4cupy_4core_4core_ndarray *, Py_ssize_t, Py_ssize_t, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*flatten)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*ravel)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*squeeze)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_squeeze *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*take)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_take *__pyx_optional_args);
  PyObject *(*repeat)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_repeat *__pyx_optional_args);
  PyObject *(*choose)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_choose *__pyx_optional_args);
  PyObject *(*sort)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_sort *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*argsort)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argsort *__pyx_optional_args);
  PyObject *(*partition)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_partition *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*argpartition)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argpartition *__pyx_optional_args);
  PyObject *(*nonzero)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*diagonal)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_diagonal *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*max)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_max *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*argmax)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argmax *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*min)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_min *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*argmin)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_argmin *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*clip)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_clip *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*round)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_round *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*trace)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_trace *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*sum)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_sum *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*cumsum)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_cumsum *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*mean)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_mean *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*var)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_var *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*std)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_std *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*prod)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_prod *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*cumprod)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_cumprod *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*all)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_all *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*any)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_any *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*conj)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_get *__pyx_optional_args);
  PyObject *(*set)(struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_set *__pyx_optional_args);
  struct __pyx_obj_4cupy_4core_4core_ndarray *(*reduced_view)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_7ndarray_reduced_view *__pyx_optional_args);
  PyObject *(*_update_c_contiguity)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  PyObject *(*_update_f_contiguity)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  PyObject *(*_update_contiguity)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
  PyObject *(*_set_shape_and_strides)(struct __pyx_obj_4cupy_4core_4core_ndarray *, std::vector<Py_ssize_t>  &, std::vector<Py_ssize_t>  &, int, int, int __pyx_skip_dispatch);
  PyObject *(*_set_shape_and_contiguous_strides)(struct __pyx_obj_4cupy_4core_4core_ndarray *, std::vector<Py_ssize_t>  &, Py_ssize_t, int, int __pyx_skip_dispatch);
  struct __pyx_obj_4cupy_4cuda_8function_CPointer *(*get_pointer)(struct __pyx_obj_4cupy_4core_4core_ndarray *);
  PyObject *(*toDlpack)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_4cupy_4core_4core_ndarray *__pyx_vtabptr_4cupy_4core_4core_ndarray;


/* "core.pxd":94
 * 
 * 
 * cdef class Indexer:             # <<<<<<<<<<<<<<
 *     cdef:
 *         readonly Py_ssize_t size
 */

struct __pyx_vtabstruct_4cupy_4core_4core_Indexer {
  struct __pyx_obj_4cupy_4cuda_8function_CPointer *(*get_pointer)(struct __pyx_obj_4cupy_4core_4core_Indexer *);
};
static struct __pyx_vtabstruct_4cupy_4core_4core_Indexer *__pyx_vtabptr_4cupy_4core_4core_Indexer;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* None.proto */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t, Py_ssize_t);

/* UnaryNegOverflows.proto */
#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod1.proto */
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* CallUnboundCMethod2.proto */
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* RaiseKeywordRequired.proto */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CppExceptionConversion.proto */
#ifndef __Pyx_CppExn2PyErr
#include <new>
#include <typeinfo>
#include <stdexcept>
#include <ios>
static void __Pyx_CppExn2PyErr() {
  try {
    if (PyErr_Occurred())
      ; // let the latest Python exn pass through and ignore the current one
    else
      throw;
  } catch (const std::bad_alloc& exn) {
    PyErr_SetString(PyExc_MemoryError, exn.what());
  } catch (const std::bad_cast& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::bad_typeid& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::domain_error& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::invalid_argument& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::ios_base::failure& exn) {
    PyErr_SetString(PyExc_IOError, exn.what());
  } catch (const std::out_of_range& exn) {
    PyErr_SetString(PyExc_IndexError, exn.what());
  } catch (const std::overflow_error& exn) {
    PyErr_SetString(PyExc_OverflowError, exn.what());
  } catch (const std::range_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::underflow_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::exception& exn) {
    PyErr_SetString(PyExc_RuntimeError, exn.what());
  }
  catch (...)
  {
    PyErr_SetString(PyExc_RuntimeError, "Unknown exception");
  }
}
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* None.proto */
#include <new>

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_As_unsigned_PY_LONG_LONG(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libcpp' */

/* Module declarations from 'libcpp.vector' */

/* Module declarations from 'cupy.core' */

/* Module declarations from 'cupy.cuda' */

/* Module declarations from 'cython' */

/* Module declarations from 'libcpp.utility' */

/* Module declarations from 'libcpp.map' */

/* Module declarations from 'cupy.cuda.device' */
static PyTypeObject *__pyx_ptype_4cupy_4cuda_6device_Device = 0;
static int (*__pyx_f_4cupy_4cuda_6device_get_device_id)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_6device_get_compute_capability)(int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'cupy.cuda.memory' */
static PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_BaseMemory = 0;
static PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer = 0;
static PyTypeObject *__pyx_ptype_4cupy_4cuda_6memory_MemoryPool = 0;
static struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(*__pyx_f_4cupy_4cuda_6memory_alloc)(PyObject *, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'cupy.cuda.function' */
static PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_CPointer = 0;
static PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_Function = 0;
static PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_Module = 0;
static PyTypeObject *__pyx_ptype_4cupy_4cuda_8function_LinkState = 0;

/* Module declarations from 'cupy.core.core' */
static PyTypeObject *__pyx_ptype_4cupy_4core_4core_ndarray = 0;
static PyTypeObject *__pyx_ptype_4cupy_4core_4core_broadcast = 0;
static PyTypeObject *__pyx_ptype_4cupy_4core_4core_Indexer = 0;
static struct __pyx_obj_4cupy_4core_4core_ndarray *(*__pyx_f_4cupy_4core_4core_ascontiguousarray)(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_ascontiguousarray *__pyx_optional_args); /*proto*/

/* Module declarations from 'cupy.cuda.cudnn' */
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_getVersion)(int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_create)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_destroy)(size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setTensor4dDescriptor)(size_t, int, int, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setTensorNdDescriptor)(size_t, int, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor)(size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_addTensor_v3)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createOpTensorDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setOpTensorDescriptor)(size_t, int, int, int, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createReduceTensorDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setReduceTensorDescriptor)(size_t, int, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setFilter4dDescriptor_v4)(size_t, int, int, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setFilterNdDescriptor_v4)(size_t, int, int, int, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_getFilterNdDescriptor)(size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor)(size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createConvolutionDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setConvolutionMathType)(size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setConvolutionGroupCount)(size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setConvolution2dDescriptor_v4)(size_t, int, int, int, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setConvolution2dDescriptor_v5)(size_t, int, int, int, int, int, int, int, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setConvolutionNdDescriptor_v3)(size_t, int, size_t, size_t, size_t, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor)(size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_findConvolutionForwardAlgorithmEx)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_findConvolutionForwardAlgorithmEx_v7)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardAlgorithm_v6)(size_t, size_t, size_t, size_t, size_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardAlgorithm_v7)(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static Py_ssize_t (*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardWorkspaceSize)(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_convolutionForward)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardFilterAlgorithmEx)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardFilterAlgorithmEx_v7)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterAlgorithm_v6)(size_t, size_t, size_t, size_t, size_t, int, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterAlgorithm_v7)(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static Py_ssize_t (*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterWorkspaceSize)(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_convolutionBackwardFilter_v3)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardDataAlgorithmEx)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardDataAlgorithmEx_v7)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static int (*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataAlgorithm_v6)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataAlgorithm_v7)(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static Py_ssize_t (*__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataWorkspaceSize)(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_convolutionBackwardData_v3)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createPoolingDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setPooling2dDescriptor_v4)(size_t, int, int, int, int, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setPoolingNdDescriptor_v4)(size_t, int, int, int, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createActivationDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setActivationDescriptor)(size_t, int, int, double, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_destroyActivationDescriptor)(size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_activationForward_v4)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_activationBackward_v4)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createDropoutDescriptor)(int __pyx_skip_dispatch); /*proto*/
static Py_ssize_t (*__pyx_f_4cupy_4cuda_5cudnn_dropoutGetStatesSize)(size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setDropoutDescriptor)(size_t, size_t, float, size_t, size_t, unsigned PY_LONG_LONG, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_getDropoutReserveSpaceSize)(size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_dropoutForward)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_dropoutBackward)(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createRNNDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setRNNDescriptor_v5)(size_t, int, int, size_t, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setRNNDescriptor_v6)(size_t, size_t, int, int, size_t, int, int, int, int, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_getRNNLinLayerMatrixParams)(size_t, size_t, int, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_getRNNLinLayerBiasParams)(size_t, size_t, int, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch); /*proto*/
static size_t (*__pyx_f_4cupy_4cuda_5cudnn_createSpatialTransformerDescriptor)(int __pyx_skip_dispatch); /*proto*/
static PyObject *(*__pyx_f_4cupy_4cuda_5cudnn_setSpatialTransformerDescriptor)(size_t, size_t, int, int, size_t, int __pyx_skip_dispatch); /*proto*/

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'cupy.core.internal' */
static Py_ssize_t (*__pyx_f_4cupy_4core_8internal_prod)(PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_8internal_prod *__pyx_optional_args); /*proto*/

/* Module declarations from 'cupy.cudnn' */
static PyTypeObject *__pyx_ptype_4cupy_5cudnn__Algorithm = 0;
static int __pyx_v_4cupy_5cudnn__cudnn_version;
static PyObject *__pyx_v_4cupy_5cudnn__thread_local = 0;
static std::vector<size_t>  __pyx_v_4cupy_5cudnn__handles;
static size_t __pyx_v_4cupy_5cudnn__max_workspace_size;
static PyObject *__pyx_v_4cupy_5cudnn__algorithm_fwd_cache = 0;
static PyObject *__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache = 0;
static PyObject *__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache = 0;
static size_t __pyx_f_4cupy_5cudnn_get_handle(int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__get_nd_tensor_cache(int __pyx_skip_dispatch); /*proto*/
static size_t __pyx_f_4cupy_5cudnn_get_max_workspace_size(int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn_set_max_workspace_size(PyObject *, int __pyx_skip_dispatch); /*proto*/
static int __pyx_f_4cupy_5cudnn_get_data_type(PyObject *, int __pyx_skip_dispatch); /*proto*/
static int __pyx_f_4cupy_5cudnn__get_byte_size(int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(size_t, struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_tensor_descriptor(size_t, struct __pyx_obj_4cupy_4core_4core_ndarray *, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(size_t, struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_filter_descriptor(size_t, struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_convolution_descriptor(size_t, PyObject *, PyObject *, PyObject *, int, PyObject *, int, int, int __pyx_skip_dispatch); /*proto*/
static int __pyx_f_4cupy_5cudnn_is_tensor_core_available(PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm(int, size_t, struct __pyx_opt_args_4cupy_5cudnn__get_algorithm *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__warn_algorithm_fwd(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__find_algorithm_fwd(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, size_t, size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm_fwd(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, size_t, size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__warn_algorithm_bwd_filter(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__find_algorithm_bwd_filter(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, size_t, size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm_bwd_filter(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, size_t, size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__warn_algorithm_bwd_data(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__find_algorithm_bwd_data(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, size_t, size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm_bwd_data(struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, struct __pyx_obj_4cupy_4core_4core_ndarray *, PyObject *, size_t, size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch); /*proto*/
static int __pyx_f_4cupy_5cudnn__should_use_tensor_core(PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn___pyx_unpickle__Algorithm__set_state(struct __pyx_obj_4cupy_5cudnn__Algorithm *, PyObject *); /*proto*/
static std::vector<int>  __pyx_convert_vector_from_py_int(PyObject *); /*proto*/
static PyObject *__pyx_convert_vector_to_py_Py_ssize_t(const std::vector<Py_ssize_t>  &); /*proto*/
#define __Pyx_MODULE_NAME "cupy.cudnn"
extern int __pyx_module_is_main_cupy__cudnn;
int __pyx_module_is_main_cupy__cudnn = 0;

/* Implementation of 'cupy.cudnn' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_DeprecationWarning;
static PyObject *__pyx_builtin_RuntimeWarning;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_range;
static const char __pyx_k_W[] = "W";
static const char __pyx_k_b[] = "b";
static const char __pyx_k_d[] = "d";
static const char __pyx_k_e[] = "e";
static const char __pyx_k_i[] = "i";
static const char __pyx_k_w[] = "w";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_dW[] = "dW";
static const char __pyx_k_dx[] = "dx";
static const char __pyx_k_dy[] = "dy";
static const char __pyx_k_gW[] = "gW";
static const char __pyx_k_gx[] = "gx";
static const char __pyx_k_gy[] = "gy";
static const char __pyx_k_id[] = "id";
static const char __pyx_k__42[] = "_";
static const char __pyx_k_arr[] = "arr";
static const char __pyx_k_del[] = "__del__";
static const char __pyx_k_dim[] = "dim";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_mat[] = "mat";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_one[] = "one";
static const char __pyx_k_pad[] = "pad";
static const char __pyx_k_ptr[] = "ptr";
static const char __pyx_k_algo[] = "algo";
static const char __pyx_k_auto[] = "auto";
static const char __pyx_k_beta[] = "beta";
static const char __pyx_k_bias[] = "bias";
static const char __pyx_k_char[] = "char";
static const char __pyx_k_coef[] = "coef";
static const char __pyx_k_desc[] = "desc";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mode[] = "mode";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_ndim[] = "ndim";
static const char __pyx_k_perf[] = "perf";
static const char __pyx_k_seed[] = "seed";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_warn[] = "warn";
static const char __pyx_k_zero[] = "zero";
static const char __pyx_k_alpha[] = "alpha";
static const char __pyx_k_c_pad[] = "c_pad";
static const char __pyx_k_cache[] = "cache";
static const char __pyx_k_cudnn[] = "cudnn";
static const char __pyx_k_dim_A[] = "dim_A";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_flags[] = "flags";
static const char __pyx_k_ksize[] = "ksize";
static const char __pyx_k_layer[] = "layer";
static const char __pyx_k_local[] = "local";
static const char __pyx_k_never[] = "never";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_always[] = "always";
static const char __pyx_k_atexit[] = "atexit";
static const char __pyx_k_b_desc[] = "b_desc";
static const char __pyx_k_desc_2[] = "_desc";
static const char __pyx_k_dev_id[] = "dev_id";
static const char __pyx_k_device[] = "device";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_groups[] = "groups";
static const char __pyx_k_handle[] = "handle";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_memory[] = "memory";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_offset[] = "offset";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_states[] = "states";
static const char __pyx_k_stride[] = "stride";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_w_desc[] = "w_desc";
static const char __pyx_k_x_desc[] = "x_desc";
static const char __pyx_k_y_desc[] = "y_desc";
static const char __pyx_k_b_shape[] = "b_shape";
static const char __pyx_k_c_ksize[] = "c_ksize";
static const char __pyx_k_destroy[] = "destroy";
static const char __pyx_k_dropout[] = "dropout";
static const char __pyx_k_dy_desc[] = "dy_desc";
static const char __pyx_k_forward[] = "forward";
static const char __pyx_k_gy_desc[] = "gy_desc";
static const char __pyx_k_indices[] = "indices";
static const char __pyx_k_nb_dims[] = "nb_dims";
static const char __pyx_k_op_type[] = "op_type";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_act_desc[] = "act_desc";
static const char __pyx_k_backward[] = "backward";
static const char __pyx_k_biasData[] = "biasData";
static const char __pyx_k_biasDesc[] = "biasDesc";
static const char __pyx_k_c_stride[] = "c_stride";
static const char __pyx_k_dilation[] = "dilation";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_handle_2[] = "_handle";
static const char __pyx_k_itemsize[] = "itemsize";
static const char __pyx_k_mat_desc[] = "mat_desc";
static const char __pyx_k_mathType[] = "mathType";
static const char __pyx_k_py_cudnn[] = "py_cudnn";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_register[] = "register";
static const char __pyx_k_rnn_desc[] = "rnn_desc";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_states_2[] = "_states";
static const char __pyx_k_warnings[] = "warnings";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_auto_tune[] = "auto_tune";
static const char __pyx_k_bias_desc[] = "bias_desc";
static const char __pyx_k_byte_size[] = "byte_size";
static const char __pyx_k_conv_desc[] = "conv_desc";
static const char __pyx_k_cupy_cuda[] = "cupy.cuda";
static const char __pyx_k_data_type[] = "data_type";
static const char __pyx_k_destroyer[] = "destroyer";
static const char __pyx_k_direction[] = "direction";
static const char __pyx_k_enumerate[] = "enumerate";
static const char __pyx_k_float_one[] = "float_one";
static const char __pyx_k_math_type[] = "math_type";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_threading[] = "threading";
static const char __pyx_k_workspace[] = "workspace";
static const char __pyx_k_Descriptor[] = "Descriptor";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_add_tensor[] = "add_tensor";
static const char __pyx_k_conv_param[] = "conv_param";
static const char __pyx_k_cupy_cudnn[] = "cupy.cudnn";
static const char __pyx_k_descriptor[] = "descriptor";
static const char __pyx_k_double_one[] = "double_one";
static const char __pyx_k_float_zero[] = "float_zero";
static const char __pyx_k_input_mode[] = "input_mode";
static const char __pyx_k_num_layers[] = "num_layers";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_state_size[] = "state_size";
static const char __pyx_k_w_data_ptr[] = "w_data_ptr";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_double_zero[] = "double_zero";
static const char __pyx_k_filter_desc[] = "filter_desc";
static const char __pyx_k_hidden_size[] = "hidden_size";
static const char __pyx_k_reduce_type[] = "reduce_type";
static const char __pyx_k_srcDestData[] = "srcDestData";
static const char __pyx_k_srcDestDesc[] = "srcDestDesc";
static const char __pyx_k_tensor_core[] = "tensor_core";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_c_contiguous[] = "c_contiguous";
static const char __pyx_k_dropout_desc[] = "dropout_desc";
static const char __pyx_k_lin_layer_id[] = "lin_layer_id";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_reserve_size[] = "reserve_size";
static const char __pyx_k_sampler_type[] = "sampler_type";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_DropoutStates[] = "DropoutStates";
static const char __pyx_k_deterministic[] = "deterministic";
static const char __pyx_k_dropout_ratio[] = "dropout_ratio";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_reserve_space[] = "reserve_space";
static const char __pyx_k_reset_handles[] = "reset_handles";
static const char __pyx_k_RuntimeWarning[] = "RuntimeWarning";
static const char __pyx_k_cupy_cudnn_pyx[] = "cupy/cudnn.pyx";
static const char __pyx_k_workspace_size[] = "workspace_size";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_use_tensor_core[] = "use_tensor_core";
static const char __pyx_k_Descriptor___del[] = "Descriptor.__del__";
static const char __pyx_k_tensor_core_mode[] = "tensor_core_mode";
static const char __pyx_k_Descriptor___init[] = "Descriptor.__init__";
static const char __pyx_k_DeprecationWarning[] = "DeprecationWarning";
static const char __pyx_k_activation_forward[] = "activation_forward";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_max_workspace_size[] = "max_workspace_size";
static const char __pyx_k_activation_backward[] = "activation_backward";
static const char __pyx_k_convolution_forward[] = "convolution_forward";
static const char __pyx_k_state_size_in_bytes[] = "state_size_in_bytes";
static const char __pyx_k_DropoutStates___init[] = "DropoutStates.__init__";
static const char __pyx_k_destroyRNNDescriptor[] = "destroyRNNDescriptor";
static const char __pyx_k_DropoutStates_forward[] = "DropoutStates.forward";
static const char __pyx_k_create_dropout_states[] = "create_dropout_states";
static const char __pyx_k_create_rnn_descriptor[] = "create_rnn_descriptor";
static const char __pyx_k_cudnn_nd_tensor_cache[] = "cudnn_nd_tensor_cache";
static const char __pyx_k_DropoutStates_backward[] = "DropoutStates.backward";
static const char __pyx_k_set_dropout_descriptor[] = "set_dropout_descriptor";
static const char __pyx_k_Invalid_cuDNN_data_type[] = "Invalid cuDNN data type: {}";
static const char __pyx_k_destroyFilterDescriptor[] = "destroyFilterDescriptor";
static const char __pyx_k_destroyTensorDescriptor[] = "destroyTensorDescriptor";
static const char __pyx_k_pyx_unpickle__Algorithm[] = "__pyx_unpickle__Algorithm";
static const char __pyx_k_create_filter_descriptor[] = "create_filter_descriptor";
static const char __pyx_k_create_tensor_descriptor[] = "create_tensor_descriptor";
static const char __pyx_k_destroyDropoutDescriptor[] = "destroyDropoutDescriptor";
static const char __pyx_k_destroyPoolingDescriptor[] = "destroyPoolingDescriptor";
static const char __pyx_k_convolution_backward_data[] = "convolution_backward_data";
static const char __pyx_k_create_dropout_descriptor[] = "create_dropout_descriptor";
static const char __pyx_k_create_pooling_descriptor[] = "create_pooling_descriptor";
static const char __pyx_k_destroyOpTensorDescriptor[] = "destroyOpTensorDescriptor";
static const char __pyx_k_convolution_backward_filter[] = "convolution_backward_filter";
static const char __pyx_k_create_op_tensor_descriptor[] = "create_op_tensor_descriptor";
static const char __pyx_k_create_tensor_nd_descriptor[] = "create_tensor_nd_descriptor";
static const char __pyx_k_destroyConvolutionDescriptor[] = "destroyConvolutionDescriptor";
static const char __pyx_k_create_convolution_descriptor[] = "create_convolution_descriptor";
static const char __pyx_k_destroyReduceTensorDescriptor[] = "destroyReduceTensorDescriptor";
static const char __pyx_k_get_rnn_lin_layer_bias_params[] = "get_rnn_lin_layer_bias_params";
static const char __pyx_k_Dtype_is_not_supported_in_cuDNN[] = "Dtype {} is not supported in cuDNN";
static const char __pyx_k_No_conv_bwd_data_algo_available[] = "No conv bwd data algo available with workspace size less equal {}";
static const char __pyx_k_No_conv_fwd_algo_available_with[] = "No conv fwd algo available with workspace size less equal {}";
static const char __pyx_k_Tensor_Core_mode_is_set_but_the[] = "Tensor Core mode is set but the selected convolution forward algorithm is not a Tensor Core enabled algorithm. This might be due to lack of workspace memory. x.shape:{}, W.shape:{}, y.shape:{}, pad:{}, stride:{}";
static const char __pyx_k_The_best_algo_of_conv_fwd_might[] = "The best algo of conv fwd might not be selected due to lack of workspace size ({})";
static const char __pyx_k_create_reduce_tensor_descriptor[] = "create_reduce_tensor_descriptor";
static const char __pyx_k_dilation_must_be_one_when_cuDNN[] = "dilation must be one when cuDNN < 6.0";
static const char __pyx_k_get_rnn_lin_layer_matrix_params[] = "get_rnn_lin_layer_matrix_params";
static const char __pyx_k_groups_must_be_one_when_cuDNN_7[] = "groups must be one when cuDNN < 7.0";
static const char __pyx_k_ksize_stride_and_pad_must_be_of[] = "ksize, stride, and pad must be of same length";
static const char __pyx_k_tensor_code_mode_must_be_either[] = "tensor_code_mode must be either of \"always\", \"auto\", or \"never\".";
static const char __pyx_k_Incompatible_checksums_s_vs_0x47[] = "Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))";
static const char __pyx_k_No_conv_bwd_filter_algo_availabl[] = "No conv bwd filter algo available with workspace size less equal {}";
static const char __pyx_k_The_best_algo_of_conv_bwd_data_m[] = "The best algo of conv bwd data might not not selected due to lack of workspace size ({})";
static const char __pyx_k_The_best_algo_of_conv_bwd_filter[] = "The best algo of conv bwd filter might not not selected due to lack of workspace size ({})";
static const char __pyx_k_create_dropout_states_is_depreca[] = "create_dropout_states is deprecated.Please use DropoutStates class instead.";
static const char __pyx_k_create_spatial_transformer_descr[] = "create_spatial_transformer_descriptor";
static const char __pyx_k_create_uninitialized_tensor_desc[] = "create_uninitialized_tensor_descriptor";
static const char __pyx_k_cupy_cudnn_supports_c_contiguous[] = "cupy.cudnn supports c-contiguous arrays only";
static const char __pyx_k_destroySpatialTransformerDescrip[] = "destroySpatialTransformerDescriptor";
static const char __pyx_k_pad_and_stride_must_be_of_same_l[] = "pad and stride must be of same length";
static const char __pyx_k_Tensor_Core_mode_is_set_but_the_2[] = "Tensor Core mode is set but the selected convolution backward filter algorithm is not a Tensor Core enabled algorithm. This might be due to lack of workspace memory. x.shape:{}, dy.shape:{}, dW.shape:{}, pad:{}, stride:{}";
static const char __pyx_k_Tensor_Core_mode_is_set_but_the_3[] = "Tensor Core mode is set but the selected convolution backward filter algorithm is not a Tensor Core enabled algorithm. This might be due to lack of workspace memory. W.shape:{}, x.shape:{}, y.shape:{}, pad:{}, stride:{}";
static PyObject *__pyx_n_s_DeprecationWarning;
static PyObject *__pyx_n_s_Descriptor;
static PyObject *__pyx_n_s_Descriptor___del;
static PyObject *__pyx_n_s_Descriptor___init;
static PyObject *__pyx_n_s_DropoutStates;
static PyObject *__pyx_n_s_DropoutStates___init;
static PyObject *__pyx_n_s_DropoutStates_backward;
static PyObject *__pyx_n_s_DropoutStates_forward;
static PyObject *__pyx_kp_s_Dtype_is_not_supported_in_cuDNN;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x47;
static PyObject *__pyx_kp_s_Invalid_cuDNN_data_type;
static PyObject *__pyx_kp_s_No_conv_bwd_data_algo_available;
static PyObject *__pyx_kp_s_No_conv_bwd_filter_algo_availabl;
static PyObject *__pyx_kp_s_No_conv_fwd_algo_available_with;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_RuntimeWarning;
static PyObject *__pyx_kp_s_Tensor_Core_mode_is_set_but_the;
static PyObject *__pyx_kp_s_Tensor_Core_mode_is_set_but_the_2;
static PyObject *__pyx_kp_s_Tensor_Core_mode_is_set_but_the_3;
static PyObject *__pyx_kp_s_The_best_algo_of_conv_bwd_data_m;
static PyObject *__pyx_kp_s_The_best_algo_of_conv_bwd_filter;
static PyObject *__pyx_kp_s_The_best_algo_of_conv_fwd_might;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_W;
static PyObject *__pyx_n_s__42;
static PyObject *__pyx_n_s_act_desc;
static PyObject *__pyx_n_s_activation_backward;
static PyObject *__pyx_n_s_activation_forward;
static PyObject *__pyx_n_s_add_tensor;
static PyObject *__pyx_n_s_algo;
static PyObject *__pyx_n_s_alpha;
static PyObject *__pyx_n_s_always;
static PyObject *__pyx_n_s_arr;
static PyObject *__pyx_n_s_atexit;
static PyObject *__pyx_n_s_auto;
static PyObject *__pyx_n_s_auto_tune;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_b_desc;
static PyObject *__pyx_n_s_b_shape;
static PyObject *__pyx_n_s_backward;
static PyObject *__pyx_n_s_beta;
static PyObject *__pyx_n_s_bias;
static PyObject *__pyx_n_s_biasData;
static PyObject *__pyx_n_s_biasDesc;
static PyObject *__pyx_n_s_bias_desc;
static PyObject *__pyx_n_s_byte_size;
static PyObject *__pyx_n_s_c_contiguous;
static PyObject *__pyx_n_s_c_ksize;
static PyObject *__pyx_n_s_c_pad;
static PyObject *__pyx_n_s_c_stride;
static PyObject *__pyx_n_s_cache;
static PyObject *__pyx_n_s_char;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_coef;
static PyObject *__pyx_n_s_conv_desc;
static PyObject *__pyx_n_s_conv_param;
static PyObject *__pyx_n_s_convolution_backward_data;
static PyObject *__pyx_n_s_convolution_backward_filter;
static PyObject *__pyx_n_s_convolution_forward;
static PyObject *__pyx_n_s_create_convolution_descriptor;
static PyObject *__pyx_n_s_create_dropout_descriptor;
static PyObject *__pyx_n_s_create_dropout_states;
static PyObject *__pyx_kp_s_create_dropout_states_is_depreca;
static PyObject *__pyx_n_s_create_filter_descriptor;
static PyObject *__pyx_n_s_create_op_tensor_descriptor;
static PyObject *__pyx_n_s_create_pooling_descriptor;
static PyObject *__pyx_n_s_create_reduce_tensor_descriptor;
static PyObject *__pyx_n_s_create_rnn_descriptor;
static PyObject *__pyx_n_s_create_spatial_transformer_descr;
static PyObject *__pyx_n_s_create_tensor_descriptor;
static PyObject *__pyx_n_s_create_tensor_nd_descriptor;
static PyObject *__pyx_n_s_create_uninitialized_tensor_desc;
static PyObject *__pyx_n_s_cudnn;
static PyObject *__pyx_n_s_cudnn_nd_tensor_cache;
static PyObject *__pyx_n_s_cupy_cuda;
static PyObject *__pyx_n_s_cupy_cudnn;
static PyObject *__pyx_kp_s_cupy_cudnn_pyx;
static PyObject *__pyx_kp_s_cupy_cudnn_supports_c_contiguous;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_n_s_dW;
static PyObject *__pyx_n_s_data_type;
static PyObject *__pyx_n_s_del;
static PyObject *__pyx_n_s_desc;
static PyObject *__pyx_n_s_desc_2;
static PyObject *__pyx_n_s_descriptor;
static PyObject *__pyx_n_s_destroy;
static PyObject *__pyx_n_s_destroyConvolutionDescriptor;
static PyObject *__pyx_n_s_destroyDropoutDescriptor;
static PyObject *__pyx_n_s_destroyFilterDescriptor;
static PyObject *__pyx_n_s_destroyOpTensorDescriptor;
static PyObject *__pyx_n_s_destroyPoolingDescriptor;
static PyObject *__pyx_n_s_destroyRNNDescriptor;
static PyObject *__pyx_n_s_destroyReduceTensorDescriptor;
static PyObject *__pyx_n_s_destroySpatialTransformerDescrip;
static PyObject *__pyx_n_s_destroyTensorDescriptor;
static PyObject *__pyx_n_s_destroyer;
static PyObject *__pyx_n_s_deterministic;
static PyObject *__pyx_n_s_dev_id;
static PyObject *__pyx_n_s_device;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_dilation;
static PyObject *__pyx_kp_s_dilation_must_be_one_when_cuDNN;
static PyObject *__pyx_n_s_dim;
static PyObject *__pyx_n_s_dim_A;
static PyObject *__pyx_n_s_direction;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_double_one;
static PyObject *__pyx_n_s_double_zero;
static PyObject *__pyx_n_s_dropout;
static PyObject *__pyx_n_s_dropout_desc;
static PyObject *__pyx_n_s_dropout_ratio;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_dx;
static PyObject *__pyx_n_s_dy;
static PyObject *__pyx_n_s_dy_desc;
static PyObject *__pyx_n_s_e;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_filter_desc;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_float_one;
static PyObject *__pyx_n_s_float_zero;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_forward;
static PyObject *__pyx_n_s_gW;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_get_rnn_lin_layer_bias_params;
static PyObject *__pyx_n_s_get_rnn_lin_layer_matrix_params;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_groups;
static PyObject *__pyx_kp_s_groups_must_be_one_when_cuDNN_7;
static PyObject *__pyx_n_s_gx;
static PyObject *__pyx_n_s_gy;
static PyObject *__pyx_n_s_gy_desc;
static PyObject *__pyx_n_s_handle;
static PyObject *__pyx_n_s_handle_2;
static PyObject *__pyx_n_s_hidden_size;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_indices;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_input_mode;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_ksize;
static PyObject *__pyx_kp_s_ksize_stride_and_pad_must_be_of;
static PyObject *__pyx_n_s_layer;
static PyObject *__pyx_n_s_lin_layer_id;
static PyObject *__pyx_n_s_local;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mat;
static PyObject *__pyx_n_s_mat_desc;
static PyObject *__pyx_n_s_mathType;
static PyObject *__pyx_n_s_math_type;
static PyObject *__pyx_n_s_max_workspace_size;
static PyObject *__pyx_n_s_memory;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_nb_dims;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_never;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_num_layers;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_n_s_offset;
static PyObject *__pyx_n_s_one;
static PyObject *__pyx_n_s_op_type;
static PyObject *__pyx_n_s_pad;
static PyObject *__pyx_kp_s_pad_and_stride_must_be_of_same_l;
static PyObject *__pyx_n_s_perf;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_ptr;
static PyObject *__pyx_n_s_py_cudnn;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle__Algorithm;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_reduce_type;
static PyObject *__pyx_n_s_register;
static PyObject *__pyx_n_s_reserve_size;
static PyObject *__pyx_n_s_reserve_space;
static PyObject *__pyx_n_s_reset_handles;
static PyObject *__pyx_n_s_rnn_desc;
static PyObject *__pyx_n_s_sampler_type;
static PyObject *__pyx_n_s_seed;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_set_dropout_descriptor;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_srcDestData;
static PyObject *__pyx_n_s_srcDestDesc;
static PyObject *__pyx_n_s_state_size;
static PyObject *__pyx_n_s_state_size_in_bytes;
static PyObject *__pyx_n_s_states;
static PyObject *__pyx_n_s_states_2;
static PyObject *__pyx_n_s_stride;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_kp_s_tensor_code_mode_must_be_either;
static PyObject *__pyx_n_s_tensor_core;
static PyObject *__pyx_n_s_tensor_core_mode;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_threading;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_tensor_core;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_w;
static PyObject *__pyx_n_s_w_data_ptr;
static PyObject *__pyx_n_s_w_desc;
static PyObject *__pyx_n_s_warn;
static PyObject *__pyx_n_s_warnings;
static PyObject *__pyx_n_s_workspace;
static PyObject *__pyx_n_s_workspace_size;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_x_desc;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_y_desc;
static PyObject *__pyx_n_s_zero;
static PyObject *__pyx_pf_4cupy_5cudnn_get_handle(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_2reset_handles(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_4_get_nd_tensor_cache(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_6get_max_workspace_size(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_8set_max_workspace_size(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_10Descriptor___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_descriptor, PyObject *__pyx_v_destroyer); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_10Descriptor_2__del__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_10get_data_type(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_12_get_byte_size(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_data_type); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_14_create_tensor_nd_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_data_type); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_16_create_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_format); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_18_create_tensor_descriptor_as4darray(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_20_create_filter_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_format); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_22_create_convolution_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, PyObject *__pyx_v_dtype, int __pyx_v_mode, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_24create_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr, PyObject *__pyx_v_format); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_26create_uninitialized_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_28create_tensor_nd_descriptor(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_30create_filter_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr, PyObject *__pyx_v_format); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_32create_convolution_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dtype, PyObject *__pyx_v_mode, PyObject *__pyx_v_dilation, PyObject *__pyx_v_use_tensor_core, PyObject *__pyx_v_groups); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_34create_pooling_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_ksize, PyObject *__pyx_v_stride, PyObject *__pyx_v_pad, int __pyx_v_mode); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_36activation_forward(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, int __pyx_v_mode, double __pyx_v_coef); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_38activation_backward(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gy, int __pyx_v_mode, float __pyx_v_coef); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_40create_dropout_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_dropout, PyObject *__pyx_v_states, PyObject *__pyx_v_state_size_in_bytes, PyObject *__pyx_v_seed); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_42set_dropout_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_desc, PyObject *__pyx_v_handle, PyObject *__pyx_v_dropout); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_44create_rnn_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_hidden_size, PyObject *__pyx_v_num_layers, PyObject *__pyx_v_dropout_desc, PyObject *__pyx_v_input_mode, PyObject *__pyx_v_direction, PyObject *__pyx_v_mode, PyObject *__pyx_v_data_type, PyObject *__pyx_v_algo); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_46get_rnn_lin_layer_matrix_params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_rnn_desc, PyObject *__pyx_v_layer, PyObject *__pyx_v_x_desc, PyObject *__pyx_v_w_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_w, PyObject *__pyx_v_lin_layer_id); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_48get_rnn_lin_layer_bias_params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_rnn_desc, PyObject *__pyx_v_layer, PyObject *__pyx_v_x_desc, PyObject *__pyx_v_w_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_w, PyObject *__pyx_v_lin_layer_id); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_50create_dropout_states(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_52create_spatial_transformer_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sampler_type, PyObject *__pyx_v_dtype, PyObject *__pyx_v_nb_dims, PyObject *__pyx_v_dim_A); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_54add_tensor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_alpha, PyObject *__pyx_v_biasDesc, PyObject *__pyx_v_biasData, PyObject *__pyx_v_beta, PyObject *__pyx_v_srcDestDesc, PyObject *__pyx_v_srcDestData); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_56create_op_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_op_type, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_58create_reduce_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_reduce_type, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_60is_tensor_core_available(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_13DropoutStates___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_seed); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_13DropoutStates_2forward(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_handle, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, PyObject *__pyx_v_dropout_ratio); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_13DropoutStates_4backward(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_handle, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, PyObject *__pyx_v_dropout_ratio, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_reserve_space); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_10_Algorithm___reduce_cython__(struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_10_Algorithm_2__setstate_cython__(struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_62_warn_algorithm_fwd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_64_find_algorithm_fwd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_66_get_algorithm_fwd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_68_warn_algorithm_bwd_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_70_find_algorithm_bwd_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_dy_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_72_get_algorithm_bwd_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_gy_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_74_warn_algorithm_bwd_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_76_find_algorithm_bwd_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_filter_desc, size_t __pyx_v_x_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_78_get_algorithm_bwd_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_filter_desc, size_t __pyx_v_x_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_80_should_use_tensor_core(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tensor_core_mode, PyObject *__pyx_v_dtype); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_82convolution_forward(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_b, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, int __pyx_v_auto_tune, PyObject *__pyx_v_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_84convolution_backward_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gW, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, int __pyx_v_deterministic, int __pyx_v_auto_tune, PyObject *__pyx_v_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_86convolution_backward_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_b, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, int __pyx_v_deterministic, int __pyx_v_auto_tune, PyObject *__pyx_v_tensor_core); /* proto */
static PyObject *__pyx_pf_4cupy_5cudnn_88__pyx_unpickle__Algorithm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_4cupy_5cudnn__Algorithm(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get = {0, &__pyx_n_s_get, 0, 0, 0};
static PyObject *__pyx_float_0_;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_70;
static PyObject *__pyx_int_74537800;
static int __pyx_k__2;
static PyObject *__pyx_k__7;
static PyObject *__pyx_k__9;
static PyObject *__pyx_k__10;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_codeobj__15;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
static PyObject *__pyx_codeobj__22;
static PyObject *__pyx_codeobj__23;
static PyObject *__pyx_codeobj__25;
static PyObject *__pyx_codeobj__27;
static PyObject *__pyx_codeobj__29;
static PyObject *__pyx_codeobj__31;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__41;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
static PyObject *__pyx_codeobj__48;
static PyObject *__pyx_codeobj__50;
static PyObject *__pyx_codeobj__52;
static PyObject *__pyx_codeobj__54;
static PyObject *__pyx_codeobj__56;
static PyObject *__pyx_codeobj__59;
static PyObject *__pyx_codeobj__61;
static PyObject *__pyx_codeobj__63;
static PyObject *__pyx_codeobj__65;
static PyObject *__pyx_codeobj__67;
static PyObject *__pyx_codeobj__69;
static PyObject *__pyx_codeobj__71;
/* Late includes */

/* "cupy/cudnn.pyx":24
 * 
 * 
 * cpdef size_t get_handle() except? 0:             # <<<<<<<<<<<<<<
 *     cdef int dev
 *     dev = device.get_device_id()
 */

static PyObject *__pyx_pw_4cupy_5cudnn_1get_handle(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static size_t __pyx_f_4cupy_5cudnn_get_handle(CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_dev;
  size_t __pyx_v_ret;
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  size_t __pyx_t_3;
  __Pyx_RefNannySetupContext("get_handle", 0);

  /* "cupy/cudnn.pyx":26
 * cpdef size_t get_handle() except? 0:
 *     cdef int dev
 *     dev = device.get_device_id()             # <<<<<<<<<<<<<<
 *     if _handles.size() <= dev:
 *         _handles.resize(dev + 1, 0)
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_6device_get_device_id(0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 26, __pyx_L1_error)
  __pyx_v_dev = __pyx_t_1;

  /* "cupy/cudnn.pyx":27
 *     cdef int dev
 *     dev = device.get_device_id()
 *     if _handles.size() <= dev:             # <<<<<<<<<<<<<<
 *         _handles.resize(dev + 1, 0)
 *     ret = _handles[dev]
 */
  __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__handles.size() <= __pyx_v_dev) != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":28
 *     dev = device.get_device_id()
 *     if _handles.size() <= dev:
 *         _handles.resize(dev + 1, 0)             # <<<<<<<<<<<<<<
 *     ret = _handles[dev]
 *     if ret != 0:
 */
    try {
      __pyx_v_4cupy_5cudnn__handles.resize((__pyx_v_dev + 1), 0);
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 28, __pyx_L1_error)
    }

    /* "cupy/cudnn.pyx":27
 *     cdef int dev
 *     dev = device.get_device_id()
 *     if _handles.size() <= dev:             # <<<<<<<<<<<<<<
 *         _handles.resize(dev + 1, 0)
 *     ret = _handles[dev]
 */
  }

  /* "cupy/cudnn.pyx":29
 *     if _handles.size() <= dev:
 *         _handles.resize(dev + 1, 0)
 *     ret = _handles[dev]             # <<<<<<<<<<<<<<
 *     if ret != 0:
 *         return ret
 */
  __pyx_v_ret = (__pyx_v_4cupy_5cudnn__handles[__pyx_v_dev]);

  /* "cupy/cudnn.pyx":30
 *         _handles.resize(dev + 1, 0)
 *     ret = _handles[dev]
 *     if ret != 0:             # <<<<<<<<<<<<<<
 *         return ret
 *     ret = cudnn.create()
 */
  __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":31
 *     ret = _handles[dev]
 *     if ret != 0:
 *         return ret             # <<<<<<<<<<<<<<
 *     ret = cudnn.create()
 *     _handles[dev] = ret
 */
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":30
 *         _handles.resize(dev + 1, 0)
 *     ret = _handles[dev]
 *     if ret != 0:             # <<<<<<<<<<<<<<
 *         return ret
 *     ret = cudnn.create()
 */
  }

  /* "cupy/cudnn.pyx":32
 *     if ret != 0:
 *         return ret
 *     ret = cudnn.create()             # <<<<<<<<<<<<<<
 *     _handles[dev] = ret
 *     return ret
 */
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_create(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 32, __pyx_L1_error)
  __pyx_v_ret = __pyx_t_3;

  /* "cupy/cudnn.pyx":33
 *         return ret
 *     ret = cudnn.create()
 *     _handles[dev] = ret             # <<<<<<<<<<<<<<
 *     return ret
 * 
 */
  (__pyx_v_4cupy_5cudnn__handles[__pyx_v_dev]) = __pyx_v_ret;

  /* "cupy/cudnn.pyx":34
 *     ret = cudnn.create()
 *     _handles[dev] = ret
 *     return ret             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":24
 * 
 * 
 * cpdef size_t get_handle() except? 0:             # <<<<<<<<<<<<<<
 *     cdef int dev
 *     dev = device.get_device_id()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cudnn.get_handle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_1get_handle(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4cupy_5cudnn_get_handle[] = "get_handle() -> size_t";
static PyObject *__pyx_pw_4cupy_5cudnn_1get_handle(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_handle (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_get_handle(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_get_handle(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_handle", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_1 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 24, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn.get_handle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":38
 * 
 * @atexit.register
 * def reset_handles():             # <<<<<<<<<<<<<<
 *     for handle in _handles:
 *         if handle:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_3reset_handles(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4cupy_5cudnn_2reset_handles[] = "reset_handles()";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_3reset_handles = {"reset_handles", (PyCFunction)__pyx_pw_4cupy_5cudnn_3reset_handles, METH_NOARGS, __pyx_doc_4cupy_5cudnn_2reset_handles};
static PyObject *__pyx_pw_4cupy_5cudnn_3reset_handles(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reset_handles (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_2reset_handles(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_2reset_handles(CYTHON_UNUSED PyObject *__pyx_self) {
  size_t __pyx_v_handle;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  std::vector<size_t> ::iterator __pyx_t_1;
  size_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("reset_handles", 0);

  /* "cupy/cudnn.pyx":39
 * @atexit.register
 * def reset_handles():
 *     for handle in _handles:             # <<<<<<<<<<<<<<
 *         if handle:
 *             cudnn.destroy(handle)
 */
  __pyx_t_1 = __pyx_v_4cupy_5cudnn__handles.begin();
  for (;;) {
    if (!(__pyx_t_1 != __pyx_v_4cupy_5cudnn__handles.end())) break;
    __pyx_t_2 = *__pyx_t_1;
    ++__pyx_t_1;
    __pyx_v_handle = __pyx_t_2;

    /* "cupy/cudnn.pyx":40
 * def reset_handles():
 *     for handle in _handles:
 *         if handle:             # <<<<<<<<<<<<<<
 *             cudnn.destroy(handle)
 *     _handles.clear()
 */
    __pyx_t_3 = (__pyx_v_handle != 0);
    if (__pyx_t_3) {

      /* "cupy/cudnn.pyx":41
 *     for handle in _handles:
 *         if handle:
 *             cudnn.destroy(handle)             # <<<<<<<<<<<<<<
 *     _handles.clear()
 * 
 */
      __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_destroy(__pyx_v_handle, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 41, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":40
 * def reset_handles():
 *     for handle in _handles:
 *         if handle:             # <<<<<<<<<<<<<<
 *             cudnn.destroy(handle)
 *     _handles.clear()
 */
    }

    /* "cupy/cudnn.pyx":39
 * @atexit.register
 * def reset_handles():
 *     for handle in _handles:             # <<<<<<<<<<<<<<
 *         if handle:
 *             cudnn.destroy(handle)
 */
  }

  /* "cupy/cudnn.pyx":42
 *         if handle:
 *             cudnn.destroy(handle)
 *     _handles.clear()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_4cupy_5cudnn__handles.clear();

  /* "cupy/cudnn.pyx":38
 * 
 * @atexit.register
 * def reset_handles():             # <<<<<<<<<<<<<<
 *     for handle in _handles:
 *         if handle:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cudnn.reset_handles", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":45
 * 
 * 
 * cpdef dict _get_nd_tensor_cache():             # <<<<<<<<<<<<<<
 *     if not hasattr(_thread_local, 'cudnn_nd_tensor_cache'):
 *         _thread_local.cudnn_nd_tensor_cache = {}
 */

static PyObject *__pyx_pw_4cupy_5cudnn_5_get_nd_tensor_cache(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__get_nd_tensor_cache(CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_get_nd_tensor_cache", 0);

  /* "cupy/cudnn.pyx":46
 * 
 * cpdef dict _get_nd_tensor_cache():
 *     if not hasattr(_thread_local, 'cudnn_nd_tensor_cache'):             # <<<<<<<<<<<<<<
 *         _thread_local.cudnn_nd_tensor_cache = {}
 *     return _thread_local.cudnn_nd_tensor_cache
 */
  __pyx_t_1 = __pyx_v_4cupy_5cudnn__thread_local;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_HasAttr(__pyx_t_1, __pyx_n_s_cudnn_nd_tensor_cache); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_3) {

    /* "cupy/cudnn.pyx":47
 * cpdef dict _get_nd_tensor_cache():
 *     if not hasattr(_thread_local, 'cudnn_nd_tensor_cache'):
 *         _thread_local.cudnn_nd_tensor_cache = {}             # <<<<<<<<<<<<<<
 *     return _thread_local.cudnn_nd_tensor_cache
 * 
 */
    __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_4cupy_5cudnn__thread_local, __pyx_n_s_cudnn_nd_tensor_cache, __pyx_t_1) < 0) __PYX_ERR(0, 47, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":46
 * 
 * cpdef dict _get_nd_tensor_cache():
 *     if not hasattr(_thread_local, 'cudnn_nd_tensor_cache'):             # <<<<<<<<<<<<<<
 *         _thread_local.cudnn_nd_tensor_cache = {}
 *     return _thread_local.cudnn_nd_tensor_cache
 */
  }

  /* "cupy/cudnn.pyx":48
 *     if not hasattr(_thread_local, 'cudnn_nd_tensor_cache'):
 *         _thread_local.cudnn_nd_tensor_cache = {}
 *     return _thread_local.cudnn_nd_tensor_cache             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_4cupy_5cudnn__thread_local, __pyx_n_s_cudnn_nd_tensor_cache); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":45
 * 
 * 
 * cpdef dict _get_nd_tensor_cache():             # <<<<<<<<<<<<<<
 *     if not hasattr(_thread_local, 'cudnn_nd_tensor_cache'):
 *         _thread_local.cudnn_nd_tensor_cache = {}
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._get_nd_tensor_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_5_get_nd_tensor_cache(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4cupy_5cudnn_4_get_nd_tensor_cache[] = "_get_nd_tensor_cache() -> dict";
static PyObject *__pyx_pw_4cupy_5cudnn_5_get_nd_tensor_cache(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_nd_tensor_cache (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_4_get_nd_tensor_cache(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_4_get_nd_tensor_cache(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_get_nd_tensor_cache", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__get_nd_tensor_cache(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._get_nd_tensor_cache", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":54
 * 
 * 
 * cpdef size_t get_max_workspace_size():             # <<<<<<<<<<<<<<
 *     """Gets the workspace size for cuDNN.
 * 
 */

static PyObject *__pyx_pw_4cupy_5cudnn_7get_max_workspace_size(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static size_t __pyx_f_4cupy_5cudnn_get_max_workspace_size(CYTHON_UNUSED int __pyx_skip_dispatch) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_max_workspace_size", 0);

  /* "cupy/cudnn.pyx":63
 * 
 *     """
 *     return _max_workspace_size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_4cupy_5cudnn__max_workspace_size;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":54
 * 
 * 
 * cpdef size_t get_max_workspace_size():             # <<<<<<<<<<<<<<
 *     """Gets the workspace size for cuDNN.
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_7get_max_workspace_size(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4cupy_5cudnn_6get_max_workspace_size[] = "get_max_workspace_size() -> size_t\nGets the workspace size for cuDNN.\n\n    Check \"cuDNN Library User Guide\" for detail.\n\n    Returns:\n        int: The workspace size for cuDNN.\n\n    ";
static PyObject *__pyx_pw_4cupy_5cudnn_7get_max_workspace_size(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_max_workspace_size (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_6get_max_workspace_size(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_6get_max_workspace_size(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_max_workspace_size", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_f_4cupy_5cudnn_get_max_workspace_size(0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn.get_max_workspace_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":66
 * 
 * 
 * cpdef set_max_workspace_size(size):             # <<<<<<<<<<<<<<
 *     """Sets the workspace size for cuDNN.
 * 
 */

static PyObject *__pyx_pw_4cupy_5cudnn_9set_max_workspace_size(PyObject *__pyx_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn_set_max_workspace_size(PyObject *__pyx_v_size, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  __Pyx_RefNannySetupContext("set_max_workspace_size", 0);

  /* "cupy/cudnn.pyx":76
 *     """
 *     global _max_workspace_size
 *     _max_workspace_size = size             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_size); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 76, __pyx_L1_error)
  __pyx_v_4cupy_5cudnn__max_workspace_size = __pyx_t_1;

  /* "cupy/cudnn.pyx":66
 * 
 * 
 * cpdef set_max_workspace_size(size):             # <<<<<<<<<<<<<<
 *     """Sets the workspace size for cuDNN.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cudnn.set_max_workspace_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_9set_max_workspace_size(PyObject *__pyx_self, PyObject *__pyx_v_size); /*proto*/
static char __pyx_doc_4cupy_5cudnn_8set_max_workspace_size[] = "set_max_workspace_size(size)\nSets the workspace size for cuDNN.\n\n    Check \"cuDNN Library User Guide\" for detail.\n\n    Args:\n        size: The workspace size for cuDNN.\n\n    ";
static PyObject *__pyx_pw_4cupy_5cudnn_9set_max_workspace_size(PyObject *__pyx_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_max_workspace_size (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_8set_max_workspace_size(__pyx_self, ((PyObject *)__pyx_v_size));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_8set_max_workspace_size(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("set_max_workspace_size", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn_set_max_workspace_size(__pyx_v_size, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn.set_max_workspace_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":81
 * class Descriptor(object):
 * 
 *     def __init__(self, descriptor, destroyer):             # <<<<<<<<<<<<<<
 *         self.value = descriptor
 *         self.destroy = destroyer
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_10Descriptor_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_10Descriptor___init__[] = "Descriptor.__init__(self, descriptor, destroyer)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_10Descriptor_1__init__ = {"__init__", (PyCFunction)__pyx_pw_4cupy_5cudnn_10Descriptor_1__init__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_10Descriptor___init__};
static PyObject *__pyx_pw_4cupy_5cudnn_10Descriptor_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_descriptor = 0;
  PyObject *__pyx_v_destroyer = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_descriptor,&__pyx_n_s_destroyer,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_descriptor)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 81, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_destroyer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 81, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 81, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_descriptor = values[1];
    __pyx_v_destroyer = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 81, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.Descriptor.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_10Descriptor___init__(__pyx_self, __pyx_v_self, __pyx_v_descriptor, __pyx_v_destroyer);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_10Descriptor___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_descriptor, PyObject *__pyx_v_destroyer) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cudnn.pyx":82
 * 
 *     def __init__(self, descriptor, destroyer):
 *         self.value = descriptor             # <<<<<<<<<<<<<<
 *         self.destroy = destroyer
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_value, __pyx_v_descriptor) < 0) __PYX_ERR(0, 82, __pyx_L1_error)

  /* "cupy/cudnn.pyx":83
 *     def __init__(self, descriptor, destroyer):
 *         self.value = descriptor
 *         self.destroy = destroyer             # <<<<<<<<<<<<<<
 * 
 *     def __del__(self):
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_destroy, __pyx_v_destroyer) < 0) __PYX_ERR(0, 83, __pyx_L1_error)

  /* "cupy/cudnn.pyx":81
 * class Descriptor(object):
 * 
 *     def __init__(self, descriptor, destroyer):             # <<<<<<<<<<<<<<
 *         self.value = descriptor
 *         self.destroy = destroyer
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("cupy.cudnn.Descriptor.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":85
 *         self.destroy = destroyer
 * 
 *     def __del__(self):             # <<<<<<<<<<<<<<
 *         if self.value:
 *             self.destroy(self.value)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_10Descriptor_3__del__(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static char __pyx_doc_4cupy_5cudnn_10Descriptor_2__del__[] = "Descriptor.__del__(self)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_10Descriptor_3__del__ = {"__del__", (PyCFunction)__pyx_pw_4cupy_5cudnn_10Descriptor_3__del__, METH_O, __pyx_doc_4cupy_5cudnn_10Descriptor_2__del__};
static PyObject *__pyx_pw_4cupy_5cudnn_10Descriptor_3__del__(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_10Descriptor_2__del__(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_10Descriptor_2__del__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__del__", 0);

  /* "cupy/cudnn.pyx":86
 * 
 *     def __del__(self):
 *         if self.value:             # <<<<<<<<<<<<<<
 *             self.destroy(self.value)
 *             self.value = None
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":87
 *     def __del__(self):
 *         if self.value:
 *             self.destroy(self.value)             # <<<<<<<<<<<<<<
 *             self.value = None
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_destroy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 87, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":88
 *         if self.value:
 *             self.destroy(self.value)
 *             self.value = None             # <<<<<<<<<<<<<<
 * 
 * 
 */
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_value, Py_None) < 0) __PYX_ERR(0, 88, __pyx_L1_error)

    /* "cupy/cudnn.pyx":86
 * 
 *     def __del__(self):
 *         if self.value:             # <<<<<<<<<<<<<<
 *             self.destroy(self.value)
 *             self.value = None
 */
  }

  /* "cupy/cudnn.pyx":85
 *         self.destroy = destroyer
 * 
 *     def __del__(self):             # <<<<<<<<<<<<<<
 *         if self.value:
 *             self.destroy(self.value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn.Descriptor.__del__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":91
 * 
 * 
 * cpdef int get_data_type(dtype) except? -1:             # <<<<<<<<<<<<<<
 *     cdef char t = ord(dtype.char)
 *     if t == 'f':
 */

static PyObject *__pyx_pw_4cupy_5cudnn_11get_data_type(PyObject *__pyx_self, PyObject *__pyx_v_dtype); /*proto*/
static int __pyx_f_4cupy_5cudnn_get_data_type(PyObject *__pyx_v_dtype, CYTHON_UNUSED int __pyx_skip_dispatch) {
  char __pyx_v_t;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("get_data_type", 0);

  /* "cupy/cudnn.pyx":92
 * 
 * cpdef int get_data_type(dtype) except? -1:
 *     cdef char t = ord(dtype.char)             # <<<<<<<<<<<<<<
 *     if t == 'f':
 *         return cudnn.CUDNN_DATA_FLOAT
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_char); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Ord(__pyx_t_1); if (unlikely(__pyx_t_2 == ((long)(long)(Py_UCS4)-1))) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_t = __pyx_t_2;

  /* "cupy/cudnn.pyx":93
 * cpdef int get_data_type(dtype) except? -1:
 *     cdef char t = ord(dtype.char)
 *     if t == 'f':             # <<<<<<<<<<<<<<
 *         return cudnn.CUDNN_DATA_FLOAT
 *     elif t == 'd':
 */
  switch (__pyx_v_t) {
    case 'f':

    /* "cupy/cudnn.pyx":94
 *     cdef char t = ord(dtype.char)
 *     if t == 'f':
 *         return cudnn.CUDNN_DATA_FLOAT             # <<<<<<<<<<<<<<
 *     elif t == 'd':
 *         return cudnn.CUDNN_DATA_DOUBLE
 */
    __pyx_r = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_FLOAT;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":93
 * cpdef int get_data_type(dtype) except? -1:
 *     cdef char t = ord(dtype.char)
 *     if t == 'f':             # <<<<<<<<<<<<<<
 *         return cudnn.CUDNN_DATA_FLOAT
 *     elif t == 'd':
 */
    break;

    /* "cupy/cudnn.pyx":95
 *     if t == 'f':
 *         return cudnn.CUDNN_DATA_FLOAT
 *     elif t == 'd':             # <<<<<<<<<<<<<<
 *         return cudnn.CUDNN_DATA_DOUBLE
 *     elif t == 'e':
 */
    case 'd':

    /* "cupy/cudnn.pyx":96
 *         return cudnn.CUDNN_DATA_FLOAT
 *     elif t == 'd':
 *         return cudnn.CUDNN_DATA_DOUBLE             # <<<<<<<<<<<<<<
 *     elif t == 'e':
 *         return cudnn.CUDNN_DATA_HALF
 */
    __pyx_r = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_DOUBLE;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":95
 *     if t == 'f':
 *         return cudnn.CUDNN_DATA_FLOAT
 *     elif t == 'd':             # <<<<<<<<<<<<<<
 *         return cudnn.CUDNN_DATA_DOUBLE
 *     elif t == 'e':
 */
    break;

    /* "cupy/cudnn.pyx":97
 *     elif t == 'd':
 *         return cudnn.CUDNN_DATA_DOUBLE
 *     elif t == 'e':             # <<<<<<<<<<<<<<
 *         return cudnn.CUDNN_DATA_HALF
 *     else:
 */
    case 'e':

    /* "cupy/cudnn.pyx":98
 *         return cudnn.CUDNN_DATA_DOUBLE
 *     elif t == 'e':
 *         return cudnn.CUDNN_DATA_HALF             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError('Dtype {} is not supported in cuDNN'.format(dtype))
 */
    __pyx_r = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_HALF;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":97
 *     elif t == 'd':
 *         return cudnn.CUDNN_DATA_DOUBLE
 *     elif t == 'e':             # <<<<<<<<<<<<<<
 *         return cudnn.CUDNN_DATA_HALF
 *     else:
 */
    break;
    default:

    /* "cupy/cudnn.pyx":100
 *         return cudnn.CUDNN_DATA_HALF
 *     else:
 *         raise TypeError('Dtype {} is not supported in cuDNN'.format(dtype))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Dtype_is_not_supported_in_cuDNN, __pyx_n_s_format); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_dtype); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_dtype};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_v_dtype};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 100, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(__pyx_v_dtype);
        __Pyx_GIVEREF(__pyx_v_dtype);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_v_dtype);
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 100, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 100, __pyx_L1_error)
    break;
  }

  /* "cupy/cudnn.pyx":91
 * 
 * 
 * cpdef int get_data_type(dtype) except? -1:             # <<<<<<<<<<<<<<
 *     cdef char t = ord(dtype.char)
 *     if t == 'f':
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cudnn.get_data_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_11get_data_type(PyObject *__pyx_self, PyObject *__pyx_v_dtype); /*proto*/
static char __pyx_doc_4cupy_5cudnn_10get_data_type[] = "get_data_type(dtype) -> int";
static PyObject *__pyx_pw_4cupy_5cudnn_11get_data_type(PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_data_type (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_10get_data_type(__pyx_self, ((PyObject *)__pyx_v_dtype));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_10get_data_type(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_data_type", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_v_dtype, 0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 91, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn.get_data_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":103
 * 
 * 
 * cpdef int _get_byte_size(int data_type) except -1:             # <<<<<<<<<<<<<<
 *     if data_type == cudnn.CUDNN_DATA_HALF:
 *         return 2
 */

static PyObject *__pyx_pw_4cupy_5cudnn_13_get_byte_size(PyObject *__pyx_self, PyObject *__pyx_arg_data_type); /*proto*/
static int __pyx_f_4cupy_5cudnn__get_byte_size(int __pyx_v_data_type, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("_get_byte_size", 0);

  /* "cupy/cudnn.pyx":104
 * 
 * cpdef int _get_byte_size(int data_type) except -1:
 *     if data_type == cudnn.CUDNN_DATA_HALF:             # <<<<<<<<<<<<<<
 *         return 2
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:
 */
  switch (__pyx_v_data_type) {
    case __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_HALF:

    /* "cupy/cudnn.pyx":105
 * cpdef int _get_byte_size(int data_type) except -1:
 *     if data_type == cudnn.CUDNN_DATA_HALF:
 *         return 2             # <<<<<<<<<<<<<<
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:
 *         return 4
 */
    __pyx_r = 2;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":104
 * 
 * cpdef int _get_byte_size(int data_type) except -1:
 *     if data_type == cudnn.CUDNN_DATA_HALF:             # <<<<<<<<<<<<<<
 *         return 2
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:
 */
    break;

    /* "cupy/cudnn.pyx":106
 *     if data_type == cudnn.CUDNN_DATA_HALF:
 *         return 2
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:             # <<<<<<<<<<<<<<
 *         return 4
 *     elif data_type == cudnn.CUDNN_DATA_DOUBLE:
 */
    case __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_FLOAT:

    /* "cupy/cudnn.pyx":107
 *         return 2
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:
 *         return 4             # <<<<<<<<<<<<<<
 *     elif data_type == cudnn.CUDNN_DATA_DOUBLE:
 *         return 8
 */
    __pyx_r = 4;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":106
 *     if data_type == cudnn.CUDNN_DATA_HALF:
 *         return 2
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:             # <<<<<<<<<<<<<<
 *         return 4
 *     elif data_type == cudnn.CUDNN_DATA_DOUBLE:
 */
    break;

    /* "cupy/cudnn.pyx":108
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:
 *         return 4
 *     elif data_type == cudnn.CUDNN_DATA_DOUBLE:             # <<<<<<<<<<<<<<
 *         return 8
 *     else:
 */
    case __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_DOUBLE:

    /* "cupy/cudnn.pyx":109
 *         return 4
 *     elif data_type == cudnn.CUDNN_DATA_DOUBLE:
 *         return 8             # <<<<<<<<<<<<<<
 *     else:
 *         raise TypeError('Invalid cuDNN data type: {}'.format(data_type))
 */
    __pyx_r = 8;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":108
 *     elif data_type == cudnn.CUDNN_DATA_FLOAT:
 *         return 4
 *     elif data_type == cudnn.CUDNN_DATA_DOUBLE:             # <<<<<<<<<<<<<<
 *         return 8
 *     else:
 */
    break;
    default:

    /* "cupy/cudnn.pyx":111
 *         return 8
 *     else:
 *         raise TypeError('Invalid cuDNN data type: {}'.format(data_type))             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Invalid_cuDNN_data_type, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_data_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 111, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 111, __pyx_L1_error)
    break;
  }

  /* "cupy/cudnn.pyx":103
 * 
 * 
 * cpdef int _get_byte_size(int data_type) except -1:             # <<<<<<<<<<<<<<
 *     if data_type == cudnn.CUDNN_DATA_HALF:
 *         return 2
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cudnn._get_byte_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_13_get_byte_size(PyObject *__pyx_self, PyObject *__pyx_arg_data_type); /*proto*/
static char __pyx_doc_4cupy_5cudnn_12_get_byte_size[] = "_get_byte_size(int data_type) -> int";
static PyObject *__pyx_pw_4cupy_5cudnn_13_get_byte_size(PyObject *__pyx_self, PyObject *__pyx_arg_data_type) {
  int __pyx_v_data_type;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_byte_size (wrapper)", 0);
  assert(__pyx_arg_data_type); {
    __pyx_v_data_type = __Pyx_PyInt_As_int(__pyx_arg_data_type); if (unlikely((__pyx_v_data_type == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 103, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._get_byte_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_12_get_byte_size(__pyx_self, ((int)__pyx_v_data_type));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_12_get_byte_size(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_data_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_get_byte_size", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__get_byte_size(__pyx_v_data_type, 0); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(0, 103, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn._get_byte_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":114
 * 
 * 
 * cpdef _create_tensor_nd_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, core.ndarray arr, int data_type=-1):
 *     cdef vector.vector[int] c_shape, c_strides
 */

static PyObject *__pyx_pw_4cupy_5cudnn_15_create_tensor_nd_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor *__pyx_optional_args) {
  int __pyx_v_data_type = ((int)-1);
  std::vector<int>  __pyx_v_c_shape;
  std::vector<int>  __pyx_v_c_strides;
  Py_ssize_t __pyx_v_itemsize;
  Py_ssize_t __pyx_v_s;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  std::vector<Py_ssize_t> ::iterator __pyx_t_5;
  std::vector<Py_ssize_t>  *__pyx_t_6;
  __Pyx_RefNannySetupContext("_create_tensor_nd_descriptor", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_data_type = __pyx_optional_args->data_type;
    }
  }

  /* "cupy/cudnn.pyx":118
 *     cdef vector.vector[int] c_shape, c_strides
 *     cdef Py_ssize_t itemsize, s
 *     if data_type == -1:  # `-1` is used instead of `None`             # <<<<<<<<<<<<<<
 *         data_type = get_data_type(arr.dtype)
 *     itemsize = arr.itemsize
 */
  __pyx_t_1 = ((__pyx_v_data_type == -1L) != 0);
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":119
 *     cdef Py_ssize_t itemsize, s
 *     if data_type == -1:  # `-1` is used instead of `None`
 *         data_type = get_data_type(arr.dtype)             # <<<<<<<<<<<<<<
 *     itemsize = arr.itemsize
 *     for s in arr._strides:
 */
    __pyx_t_2 = __pyx_v_arr->dtype;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_t_2, 0); if (unlikely(__pyx_t_3 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_data_type = __pyx_t_3;

    /* "cupy/cudnn.pyx":118
 *     cdef vector.vector[int] c_shape, c_strides
 *     cdef Py_ssize_t itemsize, s
 *     if data_type == -1:  # `-1` is used instead of `None`             # <<<<<<<<<<<<<<
 *         data_type = get_data_type(arr.dtype)
 *     itemsize = arr.itemsize
 */
  }

  /* "cupy/cudnn.pyx":120
 *     if data_type == -1:  # `-1` is used instead of `None`
 *         data_type = get_data_type(arr.dtype)
 *     itemsize = arr.itemsize             # <<<<<<<<<<<<<<
 *     for s in arr._strides:
 *         c_strides.push_back(s // itemsize)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_arr), __pyx_n_s_itemsize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyIndex_AsSsize_t(__pyx_t_2); if (unlikely((__pyx_t_4 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_itemsize = __pyx_t_4;

  /* "cupy/cudnn.pyx":121
 *         data_type = get_data_type(arr.dtype)
 *     itemsize = arr.itemsize
 *     for s in arr._strides:             # <<<<<<<<<<<<<<
 *         c_strides.push_back(s // itemsize)
 *     for s in arr._shape:
 */
  __pyx_t_6 = &__pyx_v_arr->_strides;
  __pyx_t_5 = __pyx_t_6->begin();
  for (;;) {
    if (!(__pyx_t_5 != __pyx_t_6->end())) break;
    __pyx_t_4 = *__pyx_t_5;
    ++__pyx_t_5;
    __pyx_v_s = __pyx_t_4;

    /* "cupy/cudnn.pyx":122
 *     itemsize = arr.itemsize
 *     for s in arr._strides:
 *         c_strides.push_back(s // itemsize)             # <<<<<<<<<<<<<<
 *     for s in arr._shape:
 *         c_shape.push_back(s)
 */
    if (unlikely(__pyx_v_itemsize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      __PYX_ERR(0, 122, __pyx_L1_error)
    }
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_s))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      __PYX_ERR(0, 122, __pyx_L1_error)
    }
    try {
      __pyx_v_c_strides.push_back(__Pyx_div_Py_ssize_t(__pyx_v_s, __pyx_v_itemsize));
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 122, __pyx_L1_error)
    }

    /* "cupy/cudnn.pyx":121
 *         data_type = get_data_type(arr.dtype)
 *     itemsize = arr.itemsize
 *     for s in arr._strides:             # <<<<<<<<<<<<<<
 *         c_strides.push_back(s // itemsize)
 *     for s in arr._shape:
 */
  }

  /* "cupy/cudnn.pyx":123
 *     for s in arr._strides:
 *         c_strides.push_back(s // itemsize)
 *     for s in arr._shape:             # <<<<<<<<<<<<<<
 *         c_shape.push_back(s)
 *     cudnn.setTensorNdDescriptor(
 */
  __pyx_t_6 = &__pyx_v_arr->_shape;
  __pyx_t_5 = __pyx_t_6->begin();
  for (;;) {
    if (!(__pyx_t_5 != __pyx_t_6->end())) break;
    __pyx_t_4 = *__pyx_t_5;
    ++__pyx_t_5;
    __pyx_v_s = __pyx_t_4;

    /* "cupy/cudnn.pyx":124
 *         c_strides.push_back(s // itemsize)
 *     for s in arr._shape:
 *         c_shape.push_back(s)             # <<<<<<<<<<<<<<
 *     cudnn.setTensorNdDescriptor(
 *         desc, data_type, arr._shape.size(), <size_t>&c_shape[0],
 */
    try {
      __pyx_v_c_shape.push_back(__pyx_v_s);
    } catch(...) {
      __Pyx_CppExn2PyErr();
      __PYX_ERR(0, 124, __pyx_L1_error)
    }

    /* "cupy/cudnn.pyx":123
 *     for s in arr._strides:
 *         c_strides.push_back(s // itemsize)
 *     for s in arr._shape:             # <<<<<<<<<<<<<<
 *         c_shape.push_back(s)
 *     cudnn.setTensorNdDescriptor(
 */
  }

  /* "cupy/cudnn.pyx":125
 *     for s in arr._shape:
 *         c_shape.push_back(s)
 *     cudnn.setTensorNdDescriptor(             # <<<<<<<<<<<<<<
 *         desc, data_type, arr._shape.size(), <size_t>&c_shape[0],
 *         <size_t>&c_strides[0])
 */
  __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_setTensorNdDescriptor(__pyx_v_desc, __pyx_v_data_type, __pyx_v_arr->_shape.size(), ((size_t)(&(__pyx_v_c_shape[0]))), ((size_t)(&(__pyx_v_c_strides[0]))), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":114
 * 
 * 
 * cpdef _create_tensor_nd_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, core.ndarray arr, int data_type=-1):
 *     cdef vector.vector[int] c_shape, c_strides
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_nd_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_15_create_tensor_nd_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_14_create_tensor_nd_descriptor[] = "_create_tensor_nd_descriptor(size_t desc, ndarray arr, int data_type=-1)";
static PyObject *__pyx_pw_4cupy_5cudnn_15_create_tensor_nd_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_desc;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr = 0;
  int __pyx_v_data_type;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_tensor_nd_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,&__pyx_n_s_arr,&__pyx_n_s_data_type,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_desc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_tensor_nd_descriptor", 0, 2, 3, 1); __PYX_ERR(0, 114, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data_type);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_create_tensor_nd_descriptor") < 0)) __PYX_ERR(0, 114, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_desc = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 115, __pyx_L3_error)
    __pyx_v_arr = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    if (values[2]) {
      __pyx_v_data_type = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_data_type == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 115, __pyx_L3_error)
    } else {
      __pyx_v_data_type = ((int)-1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_tensor_nd_descriptor", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 114, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_nd_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_arr), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "arr", 0))) __PYX_ERR(0, 115, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_14_create_tensor_nd_descriptor(__pyx_self, __pyx_v_desc, __pyx_v_arr, __pyx_v_data_type);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_14_create_tensor_nd_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_data_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor __pyx_t_2;
  __Pyx_RefNannySetupContext("_create_tensor_nd_descriptor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.data_type = __pyx_v_data_type;
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_desc, __pyx_v_arr, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_nd_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":130
 * 
 * 
 * cpdef _create_tensor_descriptor(size_t desc, core.ndarray arr,             # <<<<<<<<<<<<<<
 *                                 int format):
 *     if not arr._c_contiguous:
 */

static PyObject *__pyx_pw_4cupy_5cudnn_17_create_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_tensor_descriptor(size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_format, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_data_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_create_tensor_descriptor", 0);

  /* "cupy/cudnn.pyx":132
 * cpdef _create_tensor_descriptor(size_t desc, core.ndarray arr,
 *                                 int format):
 *     if not arr._c_contiguous:             # <<<<<<<<<<<<<<
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     if arr._shape.size() == 4:
 */
  __pyx_t_1 = ((!(__pyx_v_arr->_c_contiguous != 0)) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "cupy/cudnn.pyx":133
 *                                 int format):
 *     if not arr._c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')             # <<<<<<<<<<<<<<
 *     if arr._shape.size() == 4:
 *         data_type = get_data_type(arr.dtype)
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 133, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 133, __pyx_L1_error)

    /* "cupy/cudnn.pyx":132
 * cpdef _create_tensor_descriptor(size_t desc, core.ndarray arr,
 *                                 int format):
 *     if not arr._c_contiguous:             # <<<<<<<<<<<<<<
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     if arr._shape.size() == 4:
 */
  }

  /* "cupy/cudnn.pyx":134
 *     if not arr._c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     if arr._shape.size() == 4:             # <<<<<<<<<<<<<<
 *         data_type = get_data_type(arr.dtype)
 *         cudnn.setTensor4dDescriptor(desc, format, data_type,
 */
  __pyx_t_1 = ((__pyx_v_arr->_shape.size() == 4) != 0);
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":135
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     if arr._shape.size() == 4:
 *         data_type = get_data_type(arr.dtype)             # <<<<<<<<<<<<<<
 *         cudnn.setTensor4dDescriptor(desc, format, data_type,
 *                                     arr._shape[0], arr._shape[1],
 */
    __pyx_t_2 = __pyx_v_arr->dtype;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_t_2, 0); if (unlikely(__pyx_t_3 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 135, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_data_type = __pyx_t_3;

    /* "cupy/cudnn.pyx":136
 *     if arr._shape.size() == 4:
 *         data_type = get_data_type(arr.dtype)
 *         cudnn.setTensor4dDescriptor(desc, format, data_type,             # <<<<<<<<<<<<<<
 *                                     arr._shape[0], arr._shape[1],
 *                                     arr._shape[2], arr._shape[3])
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_setTensor4dDescriptor(__pyx_v_desc, __pyx_v_format, __pyx_v_data_type, (__pyx_v_arr->_shape[0]), (__pyx_v_arr->_shape[1]), (__pyx_v_arr->_shape[2]), (__pyx_v_arr->_shape[3]), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":134
 *     if not arr._c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     if arr._shape.size() == 4:             # <<<<<<<<<<<<<<
 *         data_type = get_data_type(arr.dtype)
 *         cudnn.setTensor4dDescriptor(desc, format, data_type,
 */
    goto __pyx_L4;
  }

  /* "cupy/cudnn.pyx":140
 *                                     arr._shape[2], arr._shape[3])
 *     else:
 *         _create_tensor_nd_descriptor(desc, arr)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_desc, __pyx_v_arr, 0, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L4:;

  /* "cupy/cudnn.pyx":130
 * 
 * 
 * cpdef _create_tensor_descriptor(size_t desc, core.ndarray arr,             # <<<<<<<<<<<<<<
 *                                 int format):
 *     if not arr._c_contiguous:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_17_create_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_16_create_tensor_descriptor[] = "_create_tensor_descriptor(size_t desc, ndarray arr, int format)";
static PyObject *__pyx_pw_4cupy_5cudnn_17_create_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_desc;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr = 0;
  int __pyx_v_format;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_tensor_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,&__pyx_n_s_arr,&__pyx_n_s_format,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_desc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_tensor_descriptor", 1, 3, 3, 1); __PYX_ERR(0, 130, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_tensor_descriptor", 1, 3, 3, 2); __PYX_ERR(0, 130, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_create_tensor_descriptor") < 0)) __PYX_ERR(0, 130, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_desc = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L3_error)
    __pyx_v_arr = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_format = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_format == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 131, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_tensor_descriptor", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 130, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_arr), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "arr", 0))) __PYX_ERR(0, 130, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_16_create_tensor_descriptor(__pyx_self, __pyx_v_desc, __pyx_v_arr, __pyx_v_format);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_16_create_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_format) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_create_tensor_descriptor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor(__pyx_v_desc, __pyx_v_arr, __pyx_v_format, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":143
 * 
 * 
 * cpdef _create_tensor_descriptor_as4darray(size_t desc,             # <<<<<<<<<<<<<<
 *                                           core.ndarray arr):
 *     cdef Py_ssize_t dim1, dim2
 */

static PyObject *__pyx_pw_4cupy_5cudnn_19_create_tensor_descriptor_as4darray(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, CYTHON_UNUSED int __pyx_skip_dispatch) {
  Py_ssize_t __pyx_v_dim1;
  Py_ssize_t __pyx_v_dim2;
  int __pyx_v_data_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("_create_tensor_descriptor_as4darray", 0);

  /* "cupy/cudnn.pyx":146
 *                                           core.ndarray arr):
 *     cdef Py_ssize_t dim1, dim2
 *     assert arr._c_contiguous             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(arr.dtype)
 *     dim1 = 1
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!(__pyx_v_arr->_c_contiguous != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 146, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":147
 *     cdef Py_ssize_t dim1, dim2
 *     assert arr._c_contiguous
 *     data_type = get_data_type(arr.dtype)             # <<<<<<<<<<<<<<
 *     dim1 = 1
 *     if arr._shape.size() > 0:
 */
  __pyx_t_1 = __pyx_v_arr->dtype;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_t_1, 0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 147, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_data_type = __pyx_t_2;

  /* "cupy/cudnn.pyx":148
 *     assert arr._c_contiguous
 *     data_type = get_data_type(arr.dtype)
 *     dim1 = 1             # <<<<<<<<<<<<<<
 *     if arr._shape.size() > 0:
 *         dim1 = arr._shape[0]
 */
  __pyx_v_dim1 = 1;

  /* "cupy/cudnn.pyx":149
 *     data_type = get_data_type(arr.dtype)
 *     dim1 = 1
 *     if arr._shape.size() > 0:             # <<<<<<<<<<<<<<
 *         dim1 = arr._shape[0]
 *     dim2 = arr.size // dim1
 */
  __pyx_t_3 = ((__pyx_v_arr->_shape.size() > 0) != 0);
  if (__pyx_t_3) {

    /* "cupy/cudnn.pyx":150
 *     dim1 = 1
 *     if arr._shape.size() > 0:
 *         dim1 = arr._shape[0]             # <<<<<<<<<<<<<<
 *     dim2 = arr.size // dim1
 *     cudnn.setTensor4dDescriptor(desc, cudnn.CUDNN_TENSOR_NCHW, data_type,
 */
    __pyx_v_dim1 = (__pyx_v_arr->_shape[0]);

    /* "cupy/cudnn.pyx":149
 *     data_type = get_data_type(arr.dtype)
 *     dim1 = 1
 *     if arr._shape.size() > 0:             # <<<<<<<<<<<<<<
 *         dim1 = arr._shape[0]
 *     dim2 = arr.size // dim1
 */
  }

  /* "cupy/cudnn.pyx":151
 *     if arr._shape.size() > 0:
 *         dim1 = arr._shape[0]
 *     dim2 = arr.size // dim1             # <<<<<<<<<<<<<<
 *     cudnn.setTensor4dDescriptor(desc, cudnn.CUDNN_TENSOR_NCHW, data_type,
 *                                 dim1, dim2, 1, 1)
 */
  if (unlikely(__pyx_v_dim1 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
    __PYX_ERR(0, 151, __pyx_L1_error)
  }
  else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_dim1 == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_arr->size))) {
    PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
    __PYX_ERR(0, 151, __pyx_L1_error)
  }
  __pyx_v_dim2 = __Pyx_div_Py_ssize_t(__pyx_v_arr->size, __pyx_v_dim1);

  /* "cupy/cudnn.pyx":152
 *         dim1 = arr._shape[0]
 *     dim2 = arr.size // dim1
 *     cudnn.setTensor4dDescriptor(desc, cudnn.CUDNN_TENSOR_NCHW, data_type,             # <<<<<<<<<<<<<<
 *                                 dim1, dim2, 1, 1)
 * 
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setTensor4dDescriptor(__pyx_v_desc, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW, __pyx_v_data_type, __pyx_v_dim1, __pyx_v_dim2, 1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":143
 * 
 * 
 * cpdef _create_tensor_descriptor_as4darray(size_t desc,             # <<<<<<<<<<<<<<
 *                                           core.ndarray arr):
 *     cdef Py_ssize_t dim1, dim2
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_descriptor_as4darray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_19_create_tensor_descriptor_as4darray(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_18_create_tensor_descriptor_as4darray[] = "_create_tensor_descriptor_as4darray(size_t desc, ndarray arr)";
static PyObject *__pyx_pw_4cupy_5cudnn_19_create_tensor_descriptor_as4darray(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_desc;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_tensor_descriptor_as4darray (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,&__pyx_n_s_arr,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_desc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_tensor_descriptor_as4darray", 1, 2, 2, 1); __PYX_ERR(0, 143, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_create_tensor_descriptor_as4darray") < 0)) __PYX_ERR(0, 143, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_desc = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 143, __pyx_L3_error)
    __pyx_v_arr = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_tensor_descriptor_as4darray", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 143, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_descriptor_as4darray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_arr), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "arr", 0))) __PYX_ERR(0, 144, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_18_create_tensor_descriptor_as4darray(__pyx_self, __pyx_v_desc, __pyx_v_arr);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_18_create_tensor_descriptor_as4darray(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_create_tensor_descriptor_as4darray", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(__pyx_v_desc, __pyx_v_arr, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_tensor_descriptor_as4darray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":156
 * 
 * 
 * cpdef _create_filter_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, core.ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW):
 *     cdef vector.vector[int] c_shape
 */

static PyObject *__pyx_pw_4cupy_5cudnn_21_create_filter_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_filter_descriptor(size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, CYTHON_UNUSED int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor *__pyx_optional_args) {
  int __pyx_v_format = __pyx_k__2;
  std::vector<int>  __pyx_v_c_shape;
  Py_ssize_t __pyx_v_s;
  Py_ssize_t __pyx_v_ndim;
  int __pyx_v_data_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  std::vector<Py_ssize_t> ::iterator __pyx_t_4;
  std::vector<Py_ssize_t>  *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __Pyx_RefNannySetupContext("_create_filter_descriptor", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_format = __pyx_optional_args->format;
    }
  }

  /* "cupy/cudnn.pyx":159
 *         size_t desc, core.ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW):
 *     cdef vector.vector[int] c_shape
 *     cdef Py_ssize_t s, ndim = arr._shape.size()             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(arr.dtype)
 *     if ndim == 4:
 */
  __pyx_v_ndim = __pyx_v_arr->_shape.size();

  /* "cupy/cudnn.pyx":160
 *     cdef vector.vector[int] c_shape
 *     cdef Py_ssize_t s, ndim = arr._shape.size()
 *     data_type = get_data_type(arr.dtype)             # <<<<<<<<<<<<<<
 *     if ndim == 4:
 *         cudnn.setFilter4dDescriptor_v4(
 */
  __pyx_t_1 = __pyx_v_arr->dtype;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_t_1, 0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_data_type = __pyx_t_2;

  /* "cupy/cudnn.pyx":161
 *     cdef Py_ssize_t s, ndim = arr._shape.size()
 *     data_type = get_data_type(arr.dtype)
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         cudnn.setFilter4dDescriptor_v4(
 *             desc, data_type, format,
 */
  __pyx_t_3 = ((__pyx_v_ndim == 4) != 0);
  if (__pyx_t_3) {

    /* "cupy/cudnn.pyx":162
 *     data_type = get_data_type(arr.dtype)
 *     if ndim == 4:
 *         cudnn.setFilter4dDescriptor_v4(             # <<<<<<<<<<<<<<
 *             desc, data_type, format,
 *             arr._shape[0], arr._shape[1], arr._shape[2], arr._shape[3])
 */
    __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setFilter4dDescriptor_v4(__pyx_v_desc, __pyx_v_data_type, __pyx_v_format, (__pyx_v_arr->_shape[0]), (__pyx_v_arr->_shape[1]), (__pyx_v_arr->_shape[2]), (__pyx_v_arr->_shape[3]), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":161
 *     cdef Py_ssize_t s, ndim = arr._shape.size()
 *     data_type = get_data_type(arr.dtype)
 *     if ndim == 4:             # <<<<<<<<<<<<<<
 *         cudnn.setFilter4dDescriptor_v4(
 *             desc, data_type, format,
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":166
 *             arr._shape[0], arr._shape[1], arr._shape[2], arr._shape[3])
 *     else:
 *         for s in arr._shape:             # <<<<<<<<<<<<<<
 *             c_shape.push_back(s)
 *         cudnn.setFilterNdDescriptor_v4(
 */
  /*else*/ {
    __pyx_t_5 = &__pyx_v_arr->_shape;
    __pyx_t_4 = __pyx_t_5->begin();
    for (;;) {
      if (!(__pyx_t_4 != __pyx_t_5->end())) break;
      __pyx_t_6 = *__pyx_t_4;
      ++__pyx_t_4;
      __pyx_v_s = __pyx_t_6;

      /* "cupy/cudnn.pyx":167
 *     else:
 *         for s in arr._shape:
 *             c_shape.push_back(s)             # <<<<<<<<<<<<<<
 *         cudnn.setFilterNdDescriptor_v4(
 *             desc, data_type, format, ndim, <size_t>&c_shape[0])
 */
      try {
        __pyx_v_c_shape.push_back(__pyx_v_s);
      } catch(...) {
        __Pyx_CppExn2PyErr();
        __PYX_ERR(0, 167, __pyx_L1_error)
      }

      /* "cupy/cudnn.pyx":166
 *             arr._shape[0], arr._shape[1], arr._shape[2], arr._shape[3])
 *     else:
 *         for s in arr._shape:             # <<<<<<<<<<<<<<
 *             c_shape.push_back(s)
 *         cudnn.setFilterNdDescriptor_v4(
 */
    }

    /* "cupy/cudnn.pyx":168
 *         for s in arr._shape:
 *             c_shape.push_back(s)
 *         cudnn.setFilterNdDescriptor_v4(             # <<<<<<<<<<<<<<
 *             desc, data_type, format, ndim, <size_t>&c_shape[0])
 * 
 */
    __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setFilterNdDescriptor_v4(__pyx_v_desc, __pyx_v_data_type, __pyx_v_format, __pyx_v_ndim, ((size_t)(&(__pyx_v_c_shape[0]))), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":156
 * 
 * 
 * cpdef _create_filter_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, core.ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW):
 *     cdef vector.vector[int] c_shape
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_filter_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_21_create_filter_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_20_create_filter_descriptor[] = "_create_filter_descriptor(size_t desc, ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW)";
static PyObject *__pyx_pw_4cupy_5cudnn_21_create_filter_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_desc;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr = 0;
  int __pyx_v_format;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_filter_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,&__pyx_n_s_arr,&__pyx_n_s_format,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_desc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_filter_descriptor", 0, 2, 3, 1); __PYX_ERR(0, 156, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_create_filter_descriptor") < 0)) __PYX_ERR(0, 156, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_desc = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 157, __pyx_L3_error)
    __pyx_v_arr = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    if (values[2]) {
      __pyx_v_format = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_format == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 157, __pyx_L3_error)
    } else {
      __pyx_v_format = __pyx_k__2;
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_filter_descriptor", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 156, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._create_filter_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_arr), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "arr", 0))) __PYX_ERR(0, 157, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_20_create_filter_descriptor(__pyx_self, __pyx_v_desc, __pyx_v_arr, __pyx_v_format);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_20_create_filter_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr, int __pyx_v_format) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor __pyx_t_2;
  __Pyx_RefNannySetupContext("_create_filter_descriptor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 1;
  __pyx_t_2.format = __pyx_v_format;
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_filter_descriptor(__pyx_v_desc, __pyx_v_arr, 0, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_filter_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":172
 * 
 * 
 * cpdef _create_convolution_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, tuple pad, tuple stride, tuple dilation, int groups,
 *         object dtype, int mode, bint use_tensor_core):
 */

static PyObject *__pyx_pw_4cupy_5cudnn_23_create_convolution_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__create_convolution_descriptor(size_t __pyx_v_desc, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, PyObject *__pyx_v_dtype, int __pyx_v_mode, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_v_d0;
  int __pyx_v_d1;
  int __pyx_v_p0;
  int __pyx_v_p1;
  int __pyx_v_s0;
  int __pyx_v_s1;
  std::vector<int>  __pyx_v_c_pad;
  std::vector<int>  __pyx_v_c_stride;
  std::vector<int>  __pyx_v_c_dilation;
  Py_ssize_t __pyx_v_ndim;
  int __pyx_v_compute_type;
  int __pyx_v_i;
  int __pyx_v_math_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  std::vector<int>  __pyx_t_5;
  int __pyx_t_6;
  std::vector<int> ::iterator __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  __Pyx_RefNannySetupContext("_create_convolution_descriptor", 0);

  /* "cupy/cudnn.pyx":177
 *     cdef int d0, d1, p0, p1, s0, s1
 *     cdef vector.vector[int] c_pad, c_stride, c_dilation
 *     ndim = len(pad)             # <<<<<<<<<<<<<<
 *     if ndim != len(stride):
 *         raise ValueError('pad and stride must be of same length')
 */
  if (unlikely(__pyx_v_pad == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 177, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_pad); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 177, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "cupy/cudnn.pyx":178
 *     cdef vector.vector[int] c_pad, c_stride, c_dilation
 *     ndim = len(pad)
 *     if ndim != len(stride):             # <<<<<<<<<<<<<<
 *         raise ValueError('pad and stride must be of same length')
 * 
 */
  if (unlikely(__pyx_v_stride == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 178, __pyx_L1_error)
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_stride); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 178, __pyx_L1_error)
  __pyx_t_2 = ((__pyx_v_ndim != __pyx_t_1) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "cupy/cudnn.pyx":179
 *     ndim = len(pad)
 *     if ndim != len(stride):
 *         raise ValueError('pad and stride must be of same length')             # <<<<<<<<<<<<<<
 * 
 *     compute_type = get_data_type(dtype)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 179, __pyx_L1_error)

    /* "cupy/cudnn.pyx":178
 *     cdef vector.vector[int] c_pad, c_stride, c_dilation
 *     ndim = len(pad)
 *     if ndim != len(stride):             # <<<<<<<<<<<<<<
 *         raise ValueError('pad and stride must be of same length')
 * 
 */
  }

  /* "cupy/cudnn.pyx":181
 *         raise ValueError('pad and stride must be of same length')
 * 
 *     compute_type = get_data_type(dtype)             # <<<<<<<<<<<<<<
 *     # TODO(takagi) Temporarily use computing precision of FP32 for
 *     #     storing precision of FP16.
 */
  __pyx_t_4 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_v_dtype, 0); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_v_compute_type = __pyx_t_4;

  /* "cupy/cudnn.pyx":184
 *     # TODO(takagi) Temporarily use computing precision of FP32 for
 *     #     storing precision of FP16.
 *     if compute_type == cudnn.CUDNN_DATA_HALF:             # <<<<<<<<<<<<<<
 *         compute_type = cudnn.CUDNN_DATA_FLOAT
 * 
 */
  __pyx_t_2 = ((__pyx_v_compute_type == __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_HALF) != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":185
 *     #     storing precision of FP16.
 *     if compute_type == cudnn.CUDNN_DATA_HALF:
 *         compute_type = cudnn.CUDNN_DATA_FLOAT             # <<<<<<<<<<<<<<
 * 
 *     if ndim != 2:
 */
    __pyx_v_compute_type = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DATA_FLOAT;

    /* "cupy/cudnn.pyx":184
 *     # TODO(takagi) Temporarily use computing precision of FP32 for
 *     #     storing precision of FP16.
 *     if compute_type == cudnn.CUDNN_DATA_HALF:             # <<<<<<<<<<<<<<
 *         compute_type = cudnn.CUDNN_DATA_FLOAT
 * 
 */
  }

  /* "cupy/cudnn.pyx":187
 *         compute_type = cudnn.CUDNN_DATA_FLOAT
 * 
 *     if ndim != 2:             # <<<<<<<<<<<<<<
 *         c_pad = pad
 *         c_stride = stride
 */
  __pyx_t_2 = ((__pyx_v_ndim != 2) != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":188
 * 
 *     if ndim != 2:
 *         c_pad = pad             # <<<<<<<<<<<<<<
 *         c_stride = stride
 *         if dilation is None:
 */
    __pyx_t_5 = __pyx_convert_vector_from_py_int(__pyx_v_pad); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 188, __pyx_L1_error)
    __pyx_v_c_pad = __pyx_t_5;

    /* "cupy/cudnn.pyx":189
 *     if ndim != 2:
 *         c_pad = pad
 *         c_stride = stride             # <<<<<<<<<<<<<<
 *         if dilation is None:
 *             c_dilation.assign(ndim, 1)
 */
    __pyx_t_5 = __pyx_convert_vector_from_py_int(__pyx_v_stride); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 189, __pyx_L1_error)
    __pyx_v_c_stride = __pyx_t_5;

    /* "cupy/cudnn.pyx":190
 *         c_pad = pad
 *         c_stride = stride
 *         if dilation is None:             # <<<<<<<<<<<<<<
 *             c_dilation.assign(ndim, 1)
 *         else:
 */
    __pyx_t_2 = (__pyx_v_dilation == ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_2 != 0);
    if (__pyx_t_6) {

      /* "cupy/cudnn.pyx":191
 *         c_stride = stride
 *         if dilation is None:
 *             c_dilation.assign(ndim, 1)             # <<<<<<<<<<<<<<
 *         else:
 *             c_dilation = dilation
 */
      __pyx_v_c_dilation.assign(__pyx_v_ndim, 1);

      /* "cupy/cudnn.pyx":190
 *         c_pad = pad
 *         c_stride = stride
 *         if dilation is None:             # <<<<<<<<<<<<<<
 *             c_dilation.assign(ndim, 1)
 *         else:
 */
      goto __pyx_L6;
    }

    /* "cupy/cudnn.pyx":193
 *             c_dilation.assign(ndim, 1)
 *         else:
 *             c_dilation = dilation             # <<<<<<<<<<<<<<
 *             if _cudnn_version < 6000:
 *                 for i in c_dilation:
 */
    /*else*/ {
      __pyx_t_5 = __pyx_convert_vector_from_py_int(__pyx_v_dilation); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 193, __pyx_L1_error)
      __pyx_v_c_dilation = __pyx_t_5;

      /* "cupy/cudnn.pyx":194
 *         else:
 *             c_dilation = dilation
 *             if _cudnn_version < 6000:             # <<<<<<<<<<<<<<
 *                 for i in c_dilation:
 *                     if i != 1:
 */
      __pyx_t_6 = ((__pyx_v_4cupy_5cudnn__cudnn_version < 0x1770) != 0);
      if (__pyx_t_6) {

        /* "cupy/cudnn.pyx":195
 *             c_dilation = dilation
 *             if _cudnn_version < 6000:
 *                 for i in c_dilation:             # <<<<<<<<<<<<<<
 *                     if i != 1:
 *                         raise ValueError(
 */
        __pyx_t_7 = __pyx_v_c_dilation.begin();
        for (;;) {
          if (!(__pyx_t_7 != __pyx_v_c_dilation.end())) break;
          __pyx_t_4 = *__pyx_t_7;
          ++__pyx_t_7;
          __pyx_v_i = __pyx_t_4;

          /* "cupy/cudnn.pyx":196
 *             if _cudnn_version < 6000:
 *                 for i in c_dilation:
 *                     if i != 1:             # <<<<<<<<<<<<<<
 *                         raise ValueError(
 *                             'dilation must be one when cuDNN < 6.0')
 */
          __pyx_t_6 = ((__pyx_v_i != 1) != 0);
          if (unlikely(__pyx_t_6)) {

            /* "cupy/cudnn.pyx":197
 *                 for i in c_dilation:
 *                     if i != 1:
 *                         raise ValueError(             # <<<<<<<<<<<<<<
 *                             'dilation must be one when cuDNN < 6.0')
 *         cudnn.setConvolutionNdDescriptor_v3(
 */
            __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 197, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_Raise(__pyx_t_3, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __PYX_ERR(0, 197, __pyx_L1_error)

            /* "cupy/cudnn.pyx":196
 *             if _cudnn_version < 6000:
 *                 for i in c_dilation:
 *                     if i != 1:             # <<<<<<<<<<<<<<
 *                         raise ValueError(
 *                             'dilation must be one when cuDNN < 6.0')
 */
          }

          /* "cupy/cudnn.pyx":195
 *             c_dilation = dilation
 *             if _cudnn_version < 6000:
 *                 for i in c_dilation:             # <<<<<<<<<<<<<<
 *                     if i != 1:
 *                         raise ValueError(
 */
        }

        /* "cupy/cudnn.pyx":194
 *         else:
 *             c_dilation = dilation
 *             if _cudnn_version < 6000:             # <<<<<<<<<<<<<<
 *                 for i in c_dilation:
 *                     if i != 1:
 */
      }
    }
    __pyx_L6:;

    /* "cupy/cudnn.pyx":199
 *                         raise ValueError(
 *                             'dilation must be one when cuDNN < 6.0')
 *         cudnn.setConvolutionNdDescriptor_v3(             # <<<<<<<<<<<<<<
 *             desc, ndim, <size_t>&c_pad[0], <size_t>&c_stride[0],
 *             <size_t>&c_dilation[0], mode, compute_type)
 */
    __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_setConvolutionNdDescriptor_v3(__pyx_v_desc, __pyx_v_ndim, ((size_t)(&(__pyx_v_c_pad[0]))), ((size_t)(&(__pyx_v_c_stride[0]))), ((size_t)(&(__pyx_v_c_dilation[0]))), __pyx_v_mode, __pyx_v_compute_type, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 199, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":187
 *         compute_type = cudnn.CUDNN_DATA_FLOAT
 * 
 *     if ndim != 2:             # <<<<<<<<<<<<<<
 *         c_pad = pad
 *         c_stride = stride
 */
    goto __pyx_L5;
  }

  /* "cupy/cudnn.pyx":203
 *             <size_t>&c_dilation[0], mode, compute_type)
 *     else:
 *         if dilation is None:             # <<<<<<<<<<<<<<
 *             d0 = d1 = 1
 *         else:
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_dilation == ((PyObject*)Py_None));
    __pyx_t_2 = (__pyx_t_6 != 0);
    if (__pyx_t_2) {

      /* "cupy/cudnn.pyx":204
 *     else:
 *         if dilation is None:
 *             d0 = d1 = 1             # <<<<<<<<<<<<<<
 *         else:
 *             d0, d1 = dilation
 */
      __pyx_v_d0 = 1;
      __pyx_v_d1 = 1;

      /* "cupy/cudnn.pyx":203
 *             <size_t>&c_dilation[0], mode, compute_type)
 *     else:
 *         if dilation is None:             # <<<<<<<<<<<<<<
 *             d0 = d1 = 1
 *         else:
 */
      goto __pyx_L11;
    }

    /* "cupy/cudnn.pyx":206
 *             d0 = d1 = 1
 *         else:
 *             d0, d1 = dilation             # <<<<<<<<<<<<<<
 *             if _cudnn_version < 6000 and (d0 != 1 or d1 != 1):
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')
 */
    /*else*/ {
      if (likely(__pyx_v_dilation != Py_None)) {
        PyObject* sequence = __pyx_v_dilation;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 206, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_8);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 206, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 206, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        #endif
      } else {
        __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 206, __pyx_L1_error)
      }
      __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_8); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 206, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_v_d0 = __pyx_t_4;
      __pyx_v_d1 = __pyx_t_9;

      /* "cupy/cudnn.pyx":207
 *         else:
 *             d0, d1 = dilation
 *             if _cudnn_version < 6000 and (d0 != 1 or d1 != 1):             # <<<<<<<<<<<<<<
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')
 *         p0, p1 = pad
 */
      __pyx_t_6 = ((__pyx_v_4cupy_5cudnn__cudnn_version < 0x1770) != 0);
      if (__pyx_t_6) {
      } else {
        __pyx_t_2 = __pyx_t_6;
        goto __pyx_L13_bool_binop_done;
      }
      __pyx_t_6 = ((__pyx_v_d0 != 1) != 0);
      if (!__pyx_t_6) {
      } else {
        __pyx_t_2 = __pyx_t_6;
        goto __pyx_L13_bool_binop_done;
      }
      __pyx_t_6 = ((__pyx_v_d1 != 1) != 0);
      __pyx_t_2 = __pyx_t_6;
      __pyx_L13_bool_binop_done:;
      if (unlikely(__pyx_t_2)) {

        /* "cupy/cudnn.pyx":208
 *             d0, d1 = dilation
 *             if _cudnn_version < 6000 and (d0 != 1 or d1 != 1):
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')             # <<<<<<<<<<<<<<
 *         p0, p1 = pad
 *         s0, s1 = stride
 */
        __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 208, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_Raise(__pyx_t_8, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __PYX_ERR(0, 208, __pyx_L1_error)

        /* "cupy/cudnn.pyx":207
 *         else:
 *             d0, d1 = dilation
 *             if _cudnn_version < 6000 and (d0 != 1 or d1 != 1):             # <<<<<<<<<<<<<<
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')
 *         p0, p1 = pad
 */
      }
    }
    __pyx_L11:;

    /* "cupy/cudnn.pyx":209
 *             if _cudnn_version < 6000 and (d0 != 1 or d1 != 1):
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')
 *         p0, p1 = pad             # <<<<<<<<<<<<<<
 *         s0, s1 = stride
 *         if _cudnn_version >= 5000:
 */
    if (likely(__pyx_v_pad != Py_None)) {
      PyObject* sequence = __pyx_v_pad;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 209, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_3);
      #else
      __pyx_t_8 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 209, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 209, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 209, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_8); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_p0 = __pyx_t_9;
    __pyx_v_p1 = __pyx_t_4;

    /* "cupy/cudnn.pyx":210
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')
 *         p0, p1 = pad
 *         s0, s1 = stride             # <<<<<<<<<<<<<<
 *         if _cudnn_version >= 5000:
 *             cudnn.setConvolution2dDescriptor_v5(
 */
    if (likely(__pyx_v_stride != Py_None)) {
      PyObject* sequence = __pyx_v_stride;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 210, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_8);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_8 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 210, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 210, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 210, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_8); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 210, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_v_s0 = __pyx_t_4;
    __pyx_v_s1 = __pyx_t_9;

    /* "cupy/cudnn.pyx":211
 *         p0, p1 = pad
 *         s0, s1 = stride
 *         if _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolution2dDescriptor_v5(
 *                 desc, p0, p1, s0, s1, d0, d1, mode, compute_type)
 */
    __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1388) != 0);
    if (__pyx_t_2) {

      /* "cupy/cudnn.pyx":212
 *         s0, s1 = stride
 *         if _cudnn_version >= 5000:
 *             cudnn.setConvolution2dDescriptor_v5(             # <<<<<<<<<<<<<<
 *                 desc, p0, p1, s0, s1, d0, d1, mode, compute_type)
 *         else:
 */
      __pyx_t_8 = __pyx_f_4cupy_4cuda_5cudnn_setConvolution2dDescriptor_v5(__pyx_v_desc, __pyx_v_p0, __pyx_v_p1, __pyx_v_s0, __pyx_v_s1, __pyx_v_d0, __pyx_v_d1, __pyx_v_mode, __pyx_v_compute_type, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 212, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cudnn.pyx":211
 *         p0, p1 = pad
 *         s0, s1 = stride
 *         if _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolution2dDescriptor_v5(
 *                 desc, p0, p1, s0, s1, d0, d1, mode, compute_type)
 */
      goto __pyx_L16;
    }

    /* "cupy/cudnn.pyx":215
 *                 desc, p0, p1, s0, s1, d0, d1, mode, compute_type)
 *         else:
 *             cudnn.setConvolution2dDescriptor_v4(             # <<<<<<<<<<<<<<
 *                 desc, p0, p1, s0, s1, 1, 1, mode)
 *     if _cudnn_version >= 7000:
 */
    /*else*/ {

      /* "cupy/cudnn.pyx":216
 *         else:
 *             cudnn.setConvolution2dDescriptor_v4(
 *                 desc, p0, p1, s0, s1, 1, 1, mode)             # <<<<<<<<<<<<<<
 *     if _cudnn_version >= 7000:
 *         if use_tensor_core:
 */
      __pyx_t_8 = __pyx_f_4cupy_4cuda_5cudnn_setConvolution2dDescriptor_v4(__pyx_v_desc, __pyx_v_p0, __pyx_v_p1, __pyx_v_s0, __pyx_v_s1, 1, 1, __pyx_v_mode, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 215, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    }
    __pyx_L16:;
  }
  __pyx_L5:;

  /* "cupy/cudnn.pyx":217
 *             cudnn.setConvolution2dDescriptor_v4(
 *                 desc, p0, p1, s0, s1, 1, 1, mode)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         if use_tensor_core:
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 */
  __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":218
 *                 desc, p0, p1, s0, s1, 1, 1, mode)
 *     if _cudnn_version >= 7000:
 *         if use_tensor_core:             # <<<<<<<<<<<<<<
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 *             cudnn.setConvolutionMathType(desc, math_type)
 */
    __pyx_t_2 = (__pyx_v_use_tensor_core != 0);
    if (__pyx_t_2) {

      /* "cupy/cudnn.pyx":219
 *     if _cudnn_version >= 7000:
 *         if use_tensor_core:
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(desc, math_type)
 *         if groups > 1:
 */
      __pyx_v_math_type = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH;

      /* "cupy/cudnn.pyx":220
 *         if use_tensor_core:
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 *             cudnn.setConvolutionMathType(desc, math_type)             # <<<<<<<<<<<<<<
 *         if groups > 1:
 *             cudnn.setConvolutionGroupCount(desc, groups)
 */
      __pyx_t_8 = __pyx_f_4cupy_4cuda_5cudnn_setConvolutionMathType(__pyx_v_desc, __pyx_v_math_type, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 220, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cudnn.pyx":218
 *                 desc, p0, p1, s0, s1, 1, 1, mode)
 *     if _cudnn_version >= 7000:
 *         if use_tensor_core:             # <<<<<<<<<<<<<<
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 *             cudnn.setConvolutionMathType(desc, math_type)
 */
    }

    /* "cupy/cudnn.pyx":221
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 *             cudnn.setConvolutionMathType(desc, math_type)
 *         if groups > 1:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionGroupCount(desc, groups)
 *     elif groups > 1:
 */
    __pyx_t_2 = ((__pyx_v_groups > 1) != 0);
    if (__pyx_t_2) {

      /* "cupy/cudnn.pyx":222
 *             cudnn.setConvolutionMathType(desc, math_type)
 *         if groups > 1:
 *             cudnn.setConvolutionGroupCount(desc, groups)             # <<<<<<<<<<<<<<
 *     elif groups > 1:
 *         raise ValueError('groups must be one when cuDNN < 7.0')
 */
      __pyx_t_8 = __pyx_f_4cupy_4cuda_5cudnn_setConvolutionGroupCount(__pyx_v_desc, __pyx_v_groups, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 222, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cudnn.pyx":221
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 *             cudnn.setConvolutionMathType(desc, math_type)
 *         if groups > 1:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionGroupCount(desc, groups)
 *     elif groups > 1:
 */
    }

    /* "cupy/cudnn.pyx":217
 *             cudnn.setConvolution2dDescriptor_v4(
 *                 desc, p0, p1, s0, s1, 1, 1, mode)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         if use_tensor_core:
 *             math_type = cudnn.CUDNN_TENSOR_OP_MATH
 */
    goto __pyx_L17;
  }

  /* "cupy/cudnn.pyx":223
 *         if groups > 1:
 *             cudnn.setConvolutionGroupCount(desc, groups)
 *     elif groups > 1:             # <<<<<<<<<<<<<<
 *         raise ValueError('groups must be one when cuDNN < 7.0')
 * 
 */
  __pyx_t_2 = ((__pyx_v_groups > 1) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "cupy/cudnn.pyx":224
 *             cudnn.setConvolutionGroupCount(desc, groups)
 *     elif groups > 1:
 *         raise ValueError('groups must be one when cuDNN < 7.0')             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __PYX_ERR(0, 224, __pyx_L1_error)

    /* "cupy/cudnn.pyx":223
 *         if groups > 1:
 *             cudnn.setConvolutionGroupCount(desc, groups)
 *     elif groups > 1:             # <<<<<<<<<<<<<<
 *         raise ValueError('groups must be one when cuDNN < 7.0')
 * 
 */
  }
  __pyx_L17:;

  /* "cupy/cudnn.pyx":172
 * 
 * 
 * cpdef _create_convolution_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, tuple pad, tuple stride, tuple dilation, int groups,
 *         object dtype, int mode, bint use_tensor_core):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn._create_convolution_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_23_create_convolution_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_22_create_convolution_descriptor[] = "_create_convolution_descriptor(size_t desc, tuple pad, tuple stride, tuple dilation, int groups, dtype, int mode, bool use_tensor_core)";
static PyObject *__pyx_pw_4cupy_5cudnn_23_create_convolution_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_desc;
  PyObject *__pyx_v_pad = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_dilation = 0;
  int __pyx_v_groups;
  PyObject *__pyx_v_dtype = 0;
  int __pyx_v_mode;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_create_convolution_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,&__pyx_n_s_pad,&__pyx_n_s_stride,&__pyx_n_s_dilation,&__pyx_n_s_groups,&__pyx_n_s_dtype,&__pyx_n_s_mode,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_desc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pad)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 1); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 2); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dilation)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 3); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 4); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 5); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 6); __PYX_ERR(0, 172, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, 7); __PYX_ERR(0, 172, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_create_convolution_descriptor") < 0)) __PYX_ERR(0, 172, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
    }
    __pyx_v_desc = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
    __pyx_v_pad = ((PyObject*)values[1]);
    __pyx_v_stride = ((PyObject*)values[2]);
    __pyx_v_dilation = ((PyObject*)values[3]);
    __pyx_v_groups = __Pyx_PyInt_As_int(values[4]); if (unlikely((__pyx_v_groups == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 173, __pyx_L3_error)
    __pyx_v_dtype = values[5];
    __pyx_v_mode = __Pyx_PyInt_As_int(values[6]); if (unlikely((__pyx_v_mode == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 174, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[7]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 174, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_create_convolution_descriptor", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 172, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._create_convolution_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_pad), (&PyTuple_Type), 1, "pad", 1))) __PYX_ERR(0, 173, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stride), (&PyTuple_Type), 1, "stride", 1))) __PYX_ERR(0, 173, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dilation), (&PyTuple_Type), 1, "dilation", 1))) __PYX_ERR(0, 173, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_22_create_convolution_descriptor(__pyx_self, __pyx_v_desc, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_v_dtype, __pyx_v_mode, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_22_create_convolution_descriptor(CYTHON_UNUSED PyObject *__pyx_self, size_t __pyx_v_desc, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, PyObject *__pyx_v_dtype, int __pyx_v_mode, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_create_convolution_descriptor", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_convolution_descriptor(__pyx_v_desc, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_v_dtype, __pyx_v_mode, __pyx_v_use_tensor_core, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._create_convolution_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":227
 * 
 * 
 * def create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_25create_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_24create_tensor_descriptor[] = "create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_25create_tensor_descriptor = {"create_tensor_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_25create_tensor_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_24create_tensor_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_25create_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_arr = 0;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_tensor_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_arr,&__pyx_n_s_format,0};
    PyObject* values[2] = {0,0};
    values[1] = __pyx_k__7;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_tensor_descriptor") < 0)) __PYX_ERR(0, 227, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_arr = values[0];
    __pyx_v_format = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_tensor_descriptor", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 227, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_24create_tensor_descriptor(__pyx_self, __pyx_v_arr, __pyx_v_format);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_24create_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr, PyObject *__pyx_v_format) {
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("create_tensor_descriptor", 0);

  /* "cupy/cudnn.pyx":228
 * 
 * def create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):
 *     desc = Descriptor(cudnn.createTensorDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyTensorDescriptor)
 *     _create_tensor_descriptor(desc.value, arr, format)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 228, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":229
 * def create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)             # <<<<<<<<<<<<<<
 *     _create_tensor_descriptor(desc.value, arr, format)
 *     return desc
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyTensorDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":230
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)
 *     _create_tensor_descriptor(desc.value, arr, format)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_v_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_arr, __pyx_ptype_4cupy_4core_4core_ndarray))))) __PYX_ERR(0, 230, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_format); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 230, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor(__pyx_t_3, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_v_arr), __pyx_t_7, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":231
 *                       py_cudnn.destroyTensorDescriptor)
 *     _create_tensor_descriptor(desc.value, arr, format)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":227
 * 
 * 
 * def create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":234
 * 
 * 
 * def create_uninitialized_tensor_descriptor():             # <<<<<<<<<<<<<<
 *     """Create uninitialized tensor descriptor.
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_27create_uninitialized_tensor_descriptor(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4cupy_5cudnn_26create_uninitialized_tensor_descriptor[] = "create_uninitialized_tensor_descriptor()\nCreate uninitialized tensor descriptor.\n\n    Create a cudnnCreateTensorDescriptor_t that is not yet initialized.\n    This is used by the batch normalization functions.\n    ";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_27create_uninitialized_tensor_descriptor = {"create_uninitialized_tensor_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_27create_uninitialized_tensor_descriptor, METH_NOARGS, __pyx_doc_4cupy_5cudnn_26create_uninitialized_tensor_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_27create_uninitialized_tensor_descriptor(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_uninitialized_tensor_descriptor (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_26create_uninitialized_tensor_descriptor(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_26create_uninitialized_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("create_uninitialized_tensor_descriptor", 0);

  /* "cupy/cudnn.pyx":240
 *     This is used by the batch normalization functions.
 *     """
 *     return Descriptor(cudnn.createTensorDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyTensorDescriptor)
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 240, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":241
 *     """
 *     return Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyTensorDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 241, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":234
 * 
 * 
 * def create_uninitialized_tensor_descriptor():             # <<<<<<<<<<<<<<
 *     """Create uninitialized tensor descriptor.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_uninitialized_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":244
 * 
 * 
 * def create_tensor_nd_descriptor(core.ndarray arr):             # <<<<<<<<<<<<<<
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_29create_tensor_nd_descriptor(PyObject *__pyx_self, PyObject *__pyx_v_arr); /*proto*/
static char __pyx_doc_4cupy_5cudnn_28create_tensor_nd_descriptor[] = "create_tensor_nd_descriptor(ndarray arr)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_29create_tensor_nd_descriptor = {"create_tensor_nd_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_29create_tensor_nd_descriptor, METH_O, __pyx_doc_4cupy_5cudnn_28create_tensor_nd_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_29create_tensor_nd_descriptor(PyObject *__pyx_self, PyObject *__pyx_v_arr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_tensor_nd_descriptor (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_arr), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "arr", 0))) __PYX_ERR(0, 244, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_28create_tensor_nd_descriptor(__pyx_self, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_v_arr));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_28create_tensor_nd_descriptor(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_arr) {
  PyObject *__pyx_v_cache = 0;
  int __pyx_v_data_type;
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor __pyx_t_11;
  __Pyx_RefNannySetupContext("create_tensor_nd_descriptor", 0);

  /* "cupy/cudnn.pyx":246
 * def create_tensor_nd_descriptor(core.ndarray arr):
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:             # <<<<<<<<<<<<<<
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     data_type = get_data_type(arr.dtype)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_arr), __pyx_n_s_flags); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_c_contiguous); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = ((!__pyx_t_3) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "cupy/cudnn.pyx":247
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(arr.dtype)
 *     key = (data_type, tuple(arr._shape))
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 247, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 247, __pyx_L1_error)

    /* "cupy/cudnn.pyx":246
 * def create_tensor_nd_descriptor(core.ndarray arr):
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:             # <<<<<<<<<<<<<<
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     data_type = get_data_type(arr.dtype)
 */
  }

  /* "cupy/cudnn.pyx":248
 *     if not arr.flags.c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     data_type = get_data_type(arr.dtype)             # <<<<<<<<<<<<<<
 *     key = (data_type, tuple(arr._shape))
 *     cache = _get_nd_tensor_cache()
 */
  __pyx_t_2 = __pyx_v_arr->dtype;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_5 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_t_2, 0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_data_type = __pyx_t_5;

  /* "cupy/cudnn.pyx":249
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')
 *     data_type = get_data_type(arr.dtype)
 *     key = (data_type, tuple(arr._shape))             # <<<<<<<<<<<<<<
 *     cache = _get_nd_tensor_cache()
 *     if key in cache:
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_data_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __pyx_convert_vector_to_py_Py_ssize_t(__pyx_v_arr->_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
  __pyx_t_2 = 0;
  __pyx_t_6 = 0;
  __pyx_v_key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":250
 *     data_type = get_data_type(arr.dtype)
 *     key = (data_type, tuple(arr._shape))
 *     cache = _get_nd_tensor_cache()             # <<<<<<<<<<<<<<
 *     if key in cache:
 *         return cache[key]
 */
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__get_nd_tensor_cache(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_cache = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":251
 *     key = (data_type, tuple(arr._shape))
 *     cache = _get_nd_tensor_cache()
 *     if key in cache:             # <<<<<<<<<<<<<<
 *         return cache[key]
 * 
 */
  if (unlikely(__pyx_v_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 251, __pyx_L1_error)
  }
  __pyx_t_4 = (__Pyx_PyDict_ContainsTF(__pyx_v_key, __pyx_v_cache, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 251, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_4 != 0);
  if (__pyx_t_3) {

    /* "cupy/cudnn.pyx":252
 *     cache = _get_nd_tensor_cache()
 *     if key in cache:
 *         return cache[key]             # <<<<<<<<<<<<<<
 * 
 *     # numpy's stride is defined in bytes, but cudnn's stride is defined in
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_cache == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 252, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_cache, __pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 252, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":251
 *     key = (data_type, tuple(arr._shape))
 *     cache = _get_nd_tensor_cache()
 *     if key in cache:             # <<<<<<<<<<<<<<
 *         return cache[key]
 * 
 */
  }

  /* "cupy/cudnn.pyx":256
 *     # numpy's stride is defined in bytes, but cudnn's stride is defined in
 *     # size of element
 *     desc = Descriptor(cudnn.createTensorDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyTensorDescriptor)
 *     _create_tensor_nd_descriptor(desc.value, arr, data_type)
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 256, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_FromSize_t(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 256, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "cupy/cudnn.pyx":257
 *     # size of element
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)             # <<<<<<<<<<<<<<
 *     _create_tensor_nd_descriptor(desc.value, arr, data_type)
 *     cache[key] = desc
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_destroyTensorDescriptor); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_2, __pyx_t_9};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_2, __pyx_t_9};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_10 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__pyx_t_8) {
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_5, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_5, __pyx_t_9);
    __pyx_t_2 = 0;
    __pyx_t_9 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":258
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)
 *     _create_tensor_nd_descriptor(desc.value, arr, data_type)             # <<<<<<<<<<<<<<
 *     cache[key] = desc
 *     return desc
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_7 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_11.__pyx_n = 1;
  __pyx_t_11.data_type = __pyx_v_data_type;
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_t_7, __pyx_v_arr, 0, &__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 258, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":259
 *                       py_cudnn.destroyTensorDescriptor)
 *     _create_tensor_nd_descriptor(desc.value, arr, data_type)
 *     cache[key] = desc             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  if (unlikely(__pyx_v_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 259, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_v_cache, __pyx_v_key, __pyx_v_desc) < 0)) __PYX_ERR(0, 259, __pyx_L1_error)

  /* "cupy/cudnn.pyx":260
 *     _create_tensor_nd_descriptor(desc.value, arr, data_type)
 *     cache[key] = desc
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":244
 * 
 * 
 * def create_tensor_nd_descriptor(core.ndarray arr):             # <<<<<<<<<<<<<<
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("cupy.cudnn.create_tensor_nd_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cache);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":263
 * 
 * 
 * def create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createFilterDescriptor(),
 *                       py_cudnn.destroyFilterDescriptor)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_31create_filter_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_30create_filter_descriptor[] = "create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_31create_filter_descriptor = {"create_filter_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_31create_filter_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_30create_filter_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_31create_filter_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_arr = 0;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_filter_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_arr,&__pyx_n_s_format,0};
    PyObject* values[2] = {0,0};
    values[1] = __pyx_k__9;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_arr)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_format);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_filter_descriptor") < 0)) __PYX_ERR(0, 263, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_arr = values[0];
    __pyx_v_format = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_filter_descriptor", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 263, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_filter_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_30create_filter_descriptor(__pyx_self, __pyx_v_arr, __pyx_v_format);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_30create_filter_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr, PyObject *__pyx_v_format) {
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor __pyx_t_9;
  __Pyx_RefNannySetupContext("create_filter_descriptor", 0);

  /* "cupy/cudnn.pyx":264
 * 
 * def create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):
 *     desc = Descriptor(cudnn.createFilterDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyFilterDescriptor)
 *     _create_filter_descriptor(desc.value, arr, format)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 264, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":265
 * def create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):
 *     desc = Descriptor(cudnn.createFilterDescriptor(),
 *                       py_cudnn.destroyFilterDescriptor)             # <<<<<<<<<<<<<<
 *     _create_filter_descriptor(desc.value, arr, format)
 *     return desc
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyFilterDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 265, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":266
 *     desc = Descriptor(cudnn.createFilterDescriptor(),
 *                       py_cudnn.destroyFilterDescriptor)
 *     _create_filter_descriptor(desc.value, arr, format)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_v_arr) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_arr, __pyx_ptype_4cupy_4core_4core_ndarray))))) __PYX_ERR(0, 266, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_format); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 266, __pyx_L1_error)
  __pyx_t_9.__pyx_n = 1;
  __pyx_t_9.format = __pyx_t_7;
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_filter_descriptor(__pyx_t_3, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_v_arr), 0, &__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":267
 *                       py_cudnn.destroyFilterDescriptor)
 *     _create_filter_descriptor(desc.value, arr, format)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":263
 * 
 * 
 * def create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createFilterDescriptor(),
 *                       py_cudnn.destroyFilterDescriptor)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_filter_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":270
 * 
 * 
 * def create_convolution_descriptor(pad, stride, dtype,             # <<<<<<<<<<<<<<
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_33create_convolution_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_32create_convolution_descriptor[] = "create_convolution_descriptor(pad, stride, dtype, mode=cudnn.CUDNN_CROSS_CORRELATION, dilation=None, use_tensor_core=False, groups=1)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_33create_convolution_descriptor = {"create_convolution_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_33create_convolution_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_32create_convolution_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_33create_convolution_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_pad = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_dilation = 0;
  PyObject *__pyx_v_use_tensor_core = 0;
  PyObject *__pyx_v_groups = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_convolution_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pad,&__pyx_n_s_stride,&__pyx_n_s_dtype,&__pyx_n_s_mode,&__pyx_n_s_dilation,&__pyx_n_s_use_tensor_core,&__pyx_n_s_groups,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    values[3] = __pyx_k__10;

    /* "cupy/cudnn.pyx":272
 * def create_convolution_descriptor(pad, stride, dtype,
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,             # <<<<<<<<<<<<<<
 *                                   use_tensor_core=False,
 *                                   groups=1):
 */
    values[4] = ((PyObject *)Py_None);

    /* "cupy/cudnn.pyx":273
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,
 *                                   use_tensor_core=False,             # <<<<<<<<<<<<<<
 *                                   groups=1):
 *     desc = Descriptor(cudnn.createConvolutionDescriptor(),
 */
    values[5] = ((PyObject *)Py_False);
    values[6] = ((PyObject *)__pyx_int_1);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pad)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_convolution_descriptor", 0, 3, 7, 1); __PYX_ERR(0, 270, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_convolution_descriptor", 0, 3, 7, 2); __PYX_ERR(0, 270, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode);
          if (value) { values[3] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dilation);
          if (value) { values[4] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core);
          if (value) { values[5] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups);
          if (value) { values[6] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_convolution_descriptor") < 0)) __PYX_ERR(0, 270, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_pad = values[0];
    __pyx_v_stride = values[1];
    __pyx_v_dtype = values[2];
    __pyx_v_mode = values[3];
    __pyx_v_dilation = values[4];
    __pyx_v_use_tensor_core = values[5];
    __pyx_v_groups = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_convolution_descriptor", 0, 3, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 270, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_convolution_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_32create_convolution_descriptor(__pyx_self, __pyx_v_pad, __pyx_v_stride, __pyx_v_dtype, __pyx_v_mode, __pyx_v_dilation, __pyx_v_use_tensor_core, __pyx_v_groups);

  /* "cupy/cudnn.pyx":270
 * 
 * 
 * def create_convolution_descriptor(pad, stride, dtype,             # <<<<<<<<<<<<<<
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_32create_convolution_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dtype, PyObject *__pyx_v_mode, PyObject *__pyx_v_dilation, PyObject *__pyx_v_use_tensor_core, PyObject *__pyx_v_groups) {
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("create_convolution_descriptor", 0);

  /* "cupy/cudnn.pyx":275
 *                                   use_tensor_core=False,
 *                                   groups=1):
 *     desc = Descriptor(cudnn.createConvolutionDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyConvolutionDescriptor)
 *     _create_convolution_descriptor(
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createConvolutionDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 275, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":276
 *                                   groups=1):
 *     desc = Descriptor(cudnn.createConvolutionDescriptor(),
 *                       py_cudnn.destroyConvolutionDescriptor)             # <<<<<<<<<<<<<<
 *     _create_convolution_descriptor(
 *         desc.value, pad, stride, dilation, groups,
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyConvolutionDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":278
 *                       py_cudnn.destroyConvolutionDescriptor)
 *     _create_convolution_descriptor(
 *         desc.value, pad, stride, dilation, groups,             # <<<<<<<<<<<<<<
 *         dtype, mode, use_tensor_core)
 *     return desc
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyTuple_CheckExact(__pyx_v_pad))||((__pyx_v_pad) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_pad)->tp_name), 0))) __PYX_ERR(0, 278, __pyx_L1_error)
  if (!(likely(PyTuple_CheckExact(__pyx_v_stride))||((__pyx_v_stride) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_stride)->tp_name), 0))) __PYX_ERR(0, 278, __pyx_L1_error)
  if (!(likely(PyTuple_CheckExact(__pyx_v_dilation))||((__pyx_v_dilation) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v_dilation)->tp_name), 0))) __PYX_ERR(0, 278, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_groups); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 278, __pyx_L1_error)

  /* "cupy/cudnn.pyx":279
 *     _create_convolution_descriptor(
 *         desc.value, pad, stride, dilation, groups,
 *         dtype, mode, use_tensor_core)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_v_mode); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 279, __pyx_L1_error)
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_use_tensor_core); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 279, __pyx_L1_error)

  /* "cupy/cudnn.pyx":277
 *     desc = Descriptor(cudnn.createConvolutionDescriptor(),
 *                       py_cudnn.destroyConvolutionDescriptor)
 *     _create_convolution_descriptor(             # <<<<<<<<<<<<<<
 *         desc.value, pad, stride, dilation, groups,
 *         dtype, mode, use_tensor_core)
 */
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_convolution_descriptor(__pyx_t_3, ((PyObject*)__pyx_v_pad), ((PyObject*)__pyx_v_stride), ((PyObject*)__pyx_v_dilation), __pyx_t_7, __pyx_v_dtype, __pyx_t_9, __pyx_t_10, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":280
 *         desc.value, pad, stride, dilation, groups,
 *         dtype, mode, use_tensor_core)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":270
 * 
 * 
 * def create_convolution_descriptor(pad, stride, dtype,             # <<<<<<<<<<<<<<
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_convolution_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":283
 * 
 * 
 * def create_pooling_descriptor(ksize, stride, pad, int mode):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_35create_pooling_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_34create_pooling_descriptor[] = "create_pooling_descriptor(ksize, stride, pad, int mode)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_35create_pooling_descriptor = {"create_pooling_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_35create_pooling_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_34create_pooling_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_35create_pooling_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_ksize = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_pad = 0;
  int __pyx_v_mode;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_pooling_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_ksize,&__pyx_n_s_stride,&__pyx_n_s_pad,&__pyx_n_s_mode,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_ksize)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_pooling_descriptor", 1, 4, 4, 1); __PYX_ERR(0, 283, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pad)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_pooling_descriptor", 1, 4, 4, 2); __PYX_ERR(0, 283, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_pooling_descriptor", 1, 4, 4, 3); __PYX_ERR(0, 283, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_pooling_descriptor") < 0)) __PYX_ERR(0, 283, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_ksize = values[0];
    __pyx_v_stride = values[1];
    __pyx_v_pad = values[2];
    __pyx_v_mode = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_mode == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 283, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_pooling_descriptor", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 283, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_pooling_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_34create_pooling_descriptor(__pyx_self, __pyx_v_ksize, __pyx_v_stride, __pyx_v_pad, __pyx_v_mode);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_34create_pooling_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_ksize, PyObject *__pyx_v_stride, PyObject *__pyx_v_pad, int __pyx_v_mode) {
  std::vector<int>  __pyx_v_c_ksize;
  std::vector<int>  __pyx_v_c_pad;
  std::vector<int>  __pyx_v_c_stride;
  int __pyx_v_ndim;
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  std::vector<int>  __pyx_t_17;
  __Pyx_RefNannySetupContext("create_pooling_descriptor", 0);

  /* "cupy/cudnn.pyx":285
 * def create_pooling_descriptor(ksize, stride, pad, int mode):
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)             # <<<<<<<<<<<<<<
 *     if ndim != len(stride) or ndim != len(pad):
 *         raise ValueError('ksize, stride, and pad must be of same length')
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_ksize); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 285, __pyx_L1_error)
  __pyx_v_ndim = __pyx_t_1;

  /* "cupy/cudnn.pyx":286
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)
 *     if ndim != len(stride) or ndim != len(pad):             # <<<<<<<<<<<<<<
 *         raise ValueError('ksize, stride, and pad must be of same length')
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_stride); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 286, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_v_ndim != __pyx_t_1) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_1 = PyObject_Length(__pyx_v_pad); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 286, __pyx_L1_error)
  __pyx_t_3 = ((__pyx_v_ndim != __pyx_t_1) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_2)) {

    /* "cupy/cudnn.pyx":287
 *     cdef int ndim = len(ksize)
 *     if ndim != len(stride) or ndim != len(pad):
 *         raise ValueError('ksize, stride, and pad must be of same length')             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 *                       py_cudnn.destroyPoolingDescriptor)
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 287, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 287, __pyx_L1_error)

    /* "cupy/cudnn.pyx":286
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)
 *     if ndim != len(stride) or ndim != len(pad):             # <<<<<<<<<<<<<<
 *         raise ValueError('ksize, stride, and pad must be of same length')
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 */
  }

  /* "cupy/cudnn.pyx":288
 *     if ndim != len(stride) or ndim != len(pad):
 *         raise ValueError('ksize, stride, and pad must be of same length')
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyPoolingDescriptor)
 *     if ndim == 2:
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __pyx_f_4cupy_4cuda_5cudnn_createPoolingDescriptor(0); if (unlikely(__pyx_t_6 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 288, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_FromSize_t(__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "cupy/cudnn.pyx":289
 *         raise ValueError('ksize, stride, and pad must be of same length')
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 *                       py_cudnn.destroyPoolingDescriptor)             # <<<<<<<<<<<<<<
 *     if ndim == 2:
 *         cudnn.setPooling2dDescriptor_v4(
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_destroyPoolingDescriptor); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = NULL;
  __pyx_t_10 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_10 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_7, __pyx_t_9};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 288, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_7, __pyx_t_9};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 288, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_11 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 288, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__pyx_t_8) {
      __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_8); __pyx_t_8 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_10, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_10, __pyx_t_9);
    __pyx_t_7 = 0;
    __pyx_t_9 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 288, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_desc = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":290
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 *                       py_cudnn.destroyPoolingDescriptor)
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         cudnn.setPooling2dDescriptor_v4(
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ksize[0],
 */
  __pyx_t_2 = ((__pyx_v_ndim == 2) != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":292
 *     if ndim == 2:
 *         cudnn.setPooling2dDescriptor_v4(
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ksize[0],             # <<<<<<<<<<<<<<
 *             ksize[1], pad[0], pad[1], stride[0], stride[1])
 *     else:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_ksize, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_10 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 292, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":293
 *         cudnn.setPooling2dDescriptor_v4(
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ksize[0],
 *             ksize[1], pad[0], pad[1], stride[0], stride[1])             # <<<<<<<<<<<<<<
 *     else:
 *         c_ksize = ksize
 */
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_ksize, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_pad, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_pad, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_stride, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_15 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_15 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_stride, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_16 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_16 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":291
 *                       py_cudnn.destroyPoolingDescriptor)
 *     if ndim == 2:
 *         cudnn.setPooling2dDescriptor_v4(             # <<<<<<<<<<<<<<
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ksize[0],
 *             ksize[1], pad[0], pad[1], stride[0], stride[1])
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_setPooling2dDescriptor_v4(__pyx_t_6, __pyx_v_mode, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN, __pyx_t_10, __pyx_t_12, __pyx_t_13, __pyx_t_14, __pyx_t_15, __pyx_t_16, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 291, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":290
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 *                       py_cudnn.destroyPoolingDescriptor)
 *     if ndim == 2:             # <<<<<<<<<<<<<<
 *         cudnn.setPooling2dDescriptor_v4(
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ksize[0],
 */
    goto __pyx_L6;
  }

  /* "cupy/cudnn.pyx":295
 *             ksize[1], pad[0], pad[1], stride[0], stride[1])
 *     else:
 *         c_ksize = ksize             # <<<<<<<<<<<<<<
 *         c_pad = pad
 *         c_stride = stride
 */
  /*else*/ {
    __pyx_t_17 = __pyx_convert_vector_from_py_int(__pyx_v_ksize); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 295, __pyx_L1_error)
    __pyx_v_c_ksize = __pyx_t_17;

    /* "cupy/cudnn.pyx":296
 *     else:
 *         c_ksize = ksize
 *         c_pad = pad             # <<<<<<<<<<<<<<
 *         c_stride = stride
 *         cudnn.setPoolingNdDescriptor_v4(
 */
    __pyx_t_17 = __pyx_convert_vector_from_py_int(__pyx_v_pad); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 296, __pyx_L1_error)
    __pyx_v_c_pad = __pyx_t_17;

    /* "cupy/cudnn.pyx":297
 *         c_ksize = ksize
 *         c_pad = pad
 *         c_stride = stride             # <<<<<<<<<<<<<<
 *         cudnn.setPoolingNdDescriptor_v4(
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ndim,
 */
    __pyx_t_17 = __pyx_convert_vector_from_py_int(__pyx_v_stride); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 297, __pyx_L1_error)
    __pyx_v_c_stride = __pyx_t_17;

    /* "cupy/cudnn.pyx":299
 *         c_stride = stride
 *         cudnn.setPoolingNdDescriptor_v4(
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ndim,             # <<<<<<<<<<<<<<
 *             <size_t>&c_ksize[0], <size_t>&c_pad[0], <size_t>&c_stride[0])
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 299, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 299, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":298
 *         c_pad = pad
 *         c_stride = stride
 *         cudnn.setPoolingNdDescriptor_v4(             # <<<<<<<<<<<<<<
 *             desc.value, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, ndim,
 *             <size_t>&c_ksize[0], <size_t>&c_pad[0], <size_t>&c_stride[0])
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_setPoolingNdDescriptor_v4(__pyx_t_6, __pyx_v_mode, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN, __pyx_v_ndim, ((size_t)(&(__pyx_v_c_ksize[0]))), ((size_t)(&(__pyx_v_c_pad[0]))), ((size_t)(&(__pyx_v_c_stride[0]))), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 298, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __pyx_L6:;

  /* "cupy/cudnn.pyx":302
 *             <size_t>&c_ksize[0], <size_t>&c_pad[0], <size_t>&c_stride[0])
 * 
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":283
 * 
 * 
 * def create_pooling_descriptor(ksize, stride, pad, int mode):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cudnn.create_pooling_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":305
 * 
 * 
 * def activation_forward(core.ndarray x, int mode, double coef=0.0):             # <<<<<<<<<<<<<<
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_37activation_forward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_36activation_forward[] = "activation_forward(ndarray x, int mode, double coef=0.0)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_37activation_forward = {"activation_forward", (PyCFunction)__pyx_pw_4cupy_5cudnn_37activation_forward, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_36activation_forward};
static PyObject *__pyx_pw_4cupy_5cudnn_37activation_forward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  int __pyx_v_mode;
  double __pyx_v_coef;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("activation_forward (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_mode,&__pyx_n_s_coef,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("activation_forward", 0, 2, 3, 1); __PYX_ERR(0, 305, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_coef);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "activation_forward") < 0)) __PYX_ERR(0, 305, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_mode = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_mode == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L3_error)
    if (values[2]) {
      __pyx_v_coef = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_coef == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L3_error)
    } else {
      __pyx_v_coef = ((double)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("activation_forward", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 305, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.activation_forward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 305, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_36activation_forward(__pyx_self, __pyx_v_x, __pyx_v_mode, __pyx_v_coef);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_36activation_forward(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, int __pyx_v_mode, double __pyx_v_coef) {
  float __pyx_v_float_zero;
  float __pyx_v_float_one;
  double __pyx_v_double_zero;
  double __pyx_v_double_one;
  size_t __pyx_v_zero;
  size_t __pyx_v_one;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_desc;
  size_t __pyx_v_act_desc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  char const *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("activation_forward", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_x);

  /* "cupy/cudnn.pyx":306
 * 
 * def activation_forward(core.ndarray x, int mode, double coef=0.0):
 *     cdef float float_zero = 0, float_one = 1             # <<<<<<<<<<<<<<
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 */
  __pyx_v_float_zero = 0.0;
  __pyx_v_float_one = 1.0;

  /* "cupy/cudnn.pyx":307
 * def activation_forward(core.ndarray x, int mode, double coef=0.0):
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1             # <<<<<<<<<<<<<<
 *     cdef size_t zero, one
 *     cdef core.ndarray y
 */
  __pyx_v_double_zero = 0.0;
  __pyx_v_double_one = 1.0;

  /* "cupy/cudnn.pyx":310
 *     cdef size_t zero, one
 *     cdef core.ndarray y
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_x->dtype, __pyx_n_s_d, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 310, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":311
 *     cdef core.ndarray y
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&double_one
 *     else:
 */
    __pyx_v_zero = ((size_t)(&__pyx_v_double_zero));

    /* "cupy/cudnn.pyx":312
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one             # <<<<<<<<<<<<<<
 *     else:
 *         zero = <size_t>&float_zero
 */
    __pyx_v_one = ((size_t)(&__pyx_v_double_one));

    /* "cupy/cudnn.pyx":310
 *     cdef size_t zero, one
 *     cdef core.ndarray y
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":314
 *         one = <size_t>&double_one
 *     else:
 *         zero = <size_t>&float_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&float_one
 * 
 */
  /*else*/ {
    __pyx_v_zero = ((size_t)(&__pyx_v_float_zero));

    /* "cupy/cudnn.pyx":315
 *     else:
 *         zero = <size_t>&float_zero
 *         one = <size_t>&float_one             # <<<<<<<<<<<<<<
 * 
 *     x = core.ascontiguousarray(x)
 */
    __pyx_v_one = ((size_t)(&__pyx_v_float_one));
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":317
 *         one = <size_t>&float_one
 * 
 *     x = core.ascontiguousarray(x)             # <<<<<<<<<<<<<<
 *     y = core.ndarray(x._shape, x.dtype)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_x, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_x, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":318
 * 
 *     x = core.ascontiguousarray(x)
 *     y = core.ndarray(x._shape, x.dtype)             # <<<<<<<<<<<<<<
 * 
 *     handle = get_handle()
 */
  __pyx_t_2 = __pyx_convert_vector_to_py_Py_ssize_t(__pyx_v_x->_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_v_x->dtype);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_x->dtype);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4cupy_4core_4core_ndarray), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 318, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":320
 *     y = core.ndarray(x._shape, x.dtype)
 * 
 *     handle = get_handle()             # <<<<<<<<<<<<<<
 *     desc = cudnn.createTensorDescriptor()
 *     act_desc = cudnn.createActivationDescriptor()
 */
  __pyx_t_4 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_4 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 320, __pyx_L1_error)
  __pyx_v_handle = __pyx_t_4;

  /* "cupy/cudnn.pyx":321
 * 
 *     handle = get_handle()
 *     desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     act_desc = cudnn.createActivationDescriptor()
 *     try:
 */
  __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_4 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 321, __pyx_L1_error)
  __pyx_v_desc = __pyx_t_4;

  /* "cupy/cudnn.pyx":322
 *     handle = get_handle()
 *     desc = cudnn.createTensorDescriptor()
 *     act_desc = cudnn.createActivationDescriptor()             # <<<<<<<<<<<<<<
 *     try:
 *         _create_tensor_descriptor_as4darray(desc, x)
 */
  __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_createActivationDescriptor(0); if (unlikely(__pyx_t_4 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 322, __pyx_L1_error)
  __pyx_v_act_desc = __pyx_t_4;

  /* "cupy/cudnn.pyx":323
 *     desc = cudnn.createTensorDescriptor()
 *     act_desc = cudnn.createActivationDescriptor()
 *     try:             # <<<<<<<<<<<<<<
 *         _create_tensor_descriptor_as4darray(desc, x)
 *         cudnn.setActivationDescriptor(
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":324
 *     act_desc = cudnn.createActivationDescriptor()
 *     try:
 *         _create_tensor_descriptor_as4darray(desc, x)             # <<<<<<<<<<<<<<
 *         cudnn.setActivationDescriptor(
 *             act_desc, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, coef)
 */
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(__pyx_v_desc, __pyx_v_x, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 324, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":325
 *     try:
 *         _create_tensor_descriptor_as4darray(desc, x)
 *         cudnn.setActivationDescriptor(             # <<<<<<<<<<<<<<
 *             act_desc, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, coef)
 *         cudnn.activationForward_v4(
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_setActivationDescriptor(__pyx_v_act_desc, __pyx_v_mode, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN, __pyx_v_coef, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":327
 *         cudnn.setActivationDescriptor(
 *             act_desc, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, coef)
 *         cudnn.activationForward_v4(             # <<<<<<<<<<<<<<
 *             handle, act_desc, one, desc, x.data.ptr,
 *             zero, desc, y.data.ptr)
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_activationForward_v4(__pyx_v_handle, __pyx_v_act_desc, __pyx_v_one, __pyx_v_desc, __pyx_v_x->data->ptr, __pyx_v_zero, __pyx_v_desc, __pyx_v_y->data->ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 327, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "cupy/cudnn.pyx":331
 *             zero, desc, y.data.ptr)
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(desc)
 *     return y
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyActivationDescriptor(__pyx_v_act_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cudnn.pyx":332
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)
 *         cudnn.destroyTensorDescriptor(desc)             # <<<<<<<<<<<<<<
 *     return y
 * 
 */
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L6;
    }
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10) < 0)) __Pyx_ErrFetch(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __pyx_t_5 = __pyx_lineno; __pyx_t_6 = __pyx_clineno; __pyx_t_7 = __pyx_filename;
      {

        /* "cupy/cudnn.pyx":331
 *             zero, desc, y.data.ptr)
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(desc)
 *     return y
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyActivationDescriptor(__pyx_v_act_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "cupy/cudnn.pyx":332
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)
 *         cudnn.destroyTensorDescriptor(desc)             # <<<<<<<<<<<<<<
 *     return y
 * 
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      }
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ErrRestore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_6; __pyx_filename = __pyx_t_7;
      goto __pyx_L1_error;
      __pyx_L8_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      }
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L6:;
  }

  /* "cupy/cudnn.pyx":333
 *         cudnn.destroyActivationDescriptor(act_desc)
 *         cudnn.destroyTensorDescriptor(desc)
 *     return y             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_y));
  __pyx_r = ((PyObject *)__pyx_v_y);
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":305
 * 
 * 
 * def activation_forward(core.ndarray x, int mode, double coef=0.0):             # <<<<<<<<<<<<<<
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cudnn.activation_forward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_y);
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":336
 * 
 * 
 * def activation_backward(core.ndarray x, core.ndarray y, core.ndarray gy,             # <<<<<<<<<<<<<<
 *                         int mode, float coef=0.0):
 *     cdef float float_zero = 0, float_one = 1
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_39activation_backward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_38activation_backward[] = "activation_backward(ndarray x, ndarray y, ndarray gy, int mode, float coef=0.0)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_39activation_backward = {"activation_backward", (PyCFunction)__pyx_pw_4cupy_5cudnn_39activation_backward, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_38activation_backward};
static PyObject *__pyx_pw_4cupy_5cudnn_39activation_backward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gy = 0;
  int __pyx_v_mode;
  float __pyx_v_coef;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("activation_backward (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_gy,&__pyx_n_s_mode,&__pyx_n_s_coef,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("activation_backward", 0, 4, 5, 1); __PYX_ERR(0, 336, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_gy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("activation_backward", 0, 4, 5, 2); __PYX_ERR(0, 336, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("activation_backward", 0, 4, 5, 3); __PYX_ERR(0, 336, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_coef);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "activation_backward") < 0)) __PYX_ERR(0, 336, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_gy = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_mode = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_mode == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 337, __pyx_L3_error)
    if (values[4]) {
      __pyx_v_coef = __pyx_PyFloat_AsFloat(values[4]); if (unlikely((__pyx_v_coef == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 337, __pyx_L3_error)
    } else {
      __pyx_v_coef = ((float)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("activation_backward", 0, 4, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 336, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.activation_backward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 336, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 336, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_gy), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "gy", 0))) __PYX_ERR(0, 336, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_38activation_backward(__pyx_self, __pyx_v_x, __pyx_v_y, __pyx_v_gy, __pyx_v_mode, __pyx_v_coef);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_38activation_backward(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gy, int __pyx_v_mode, float __pyx_v_coef) {
  float __pyx_v_float_zero;
  float __pyx_v_float_one;
  double __pyx_v_double_zero;
  double __pyx_v_double_one;
  size_t __pyx_v_zero;
  size_t __pyx_v_one;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gx = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_desc;
  size_t __pyx_v_act_desc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  char const *__pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_RefNannySetupContext("activation_backward", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_x);
  __Pyx_INCREF((PyObject *)__pyx_v_y);
  __Pyx_INCREF((PyObject *)__pyx_v_gy);

  /* "cupy/cudnn.pyx":338
 * def activation_backward(core.ndarray x, core.ndarray y, core.ndarray gy,
 *                         int mode, float coef=0.0):
 *     cdef float float_zero = 0, float_one = 1             # <<<<<<<<<<<<<<
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 */
  __pyx_v_float_zero = 0.0;
  __pyx_v_float_one = 1.0;

  /* "cupy/cudnn.pyx":339
 *                         int mode, float coef=0.0):
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1             # <<<<<<<<<<<<<<
 *     cdef size_t zero, one
 *     cdef core.ndarray gx
 */
  __pyx_v_double_zero = 0.0;
  __pyx_v_double_one = 1.0;

  /* "cupy/cudnn.pyx":342
 *     cdef size_t zero, one
 *     cdef core.ndarray gx
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_x->dtype, __pyx_n_s_d, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 342, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":343
 *     cdef core.ndarray gx
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&double_one
 *     else:
 */
    __pyx_v_zero = ((size_t)(&__pyx_v_double_zero));

    /* "cupy/cudnn.pyx":344
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one             # <<<<<<<<<<<<<<
 *     else:
 *         zero = <size_t>&float_zero
 */
    __pyx_v_one = ((size_t)(&__pyx_v_double_one));

    /* "cupy/cudnn.pyx":342
 *     cdef size_t zero, one
 *     cdef core.ndarray gx
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":346
 *         one = <size_t>&double_one
 *     else:
 *         zero = <size_t>&float_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&float_one
 * 
 */
  /*else*/ {
    __pyx_v_zero = ((size_t)(&__pyx_v_float_zero));

    /* "cupy/cudnn.pyx":347
 *     else:
 *         zero = <size_t>&float_zero
 *         one = <size_t>&float_one             # <<<<<<<<<<<<<<
 * 
 *     gx = core.ndarray(x._shape, x.dtype)
 */
    __pyx_v_one = ((size_t)(&__pyx_v_float_one));
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":349
 *         one = <size_t>&float_one
 * 
 *     gx = core.ndarray(x._shape, x.dtype)             # <<<<<<<<<<<<<<
 *     x = core.ascontiguousarray(x)
 *     y = core.ascontiguousarray(y)
 */
  __pyx_t_2 = __pyx_convert_vector_to_py_Py_ssize_t(__pyx_v_x->_shape); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_v_x->dtype);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_v_x->dtype);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4cupy_4core_4core_ndarray), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_gx = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":350
 * 
 *     gx = core.ndarray(x._shape, x.dtype)
 *     x = core.ascontiguousarray(x)             # <<<<<<<<<<<<<<
 *     y = core.ascontiguousarray(y)
 *     gy = core.ascontiguousarray(gy)
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_x, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_x, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":351
 *     gx = core.ndarray(x._shape, x.dtype)
 *     x = core.ascontiguousarray(x)
 *     y = core.ascontiguousarray(y)             # <<<<<<<<<<<<<<
 *     gy = core.ascontiguousarray(gy)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_y, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_y, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":352
 *     x = core.ascontiguousarray(x)
 *     y = core.ascontiguousarray(y)
 *     gy = core.ascontiguousarray(gy)             # <<<<<<<<<<<<<<
 * 
 *     handle = get_handle()
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_gy, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_gy, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":354
 *     gy = core.ascontiguousarray(gy)
 * 
 *     handle = get_handle()             # <<<<<<<<<<<<<<
 *     desc = cudnn.createTensorDescriptor()
 *     act_desc = cudnn.createActivationDescriptor()
 */
  __pyx_t_4 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_4 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 354, __pyx_L1_error)
  __pyx_v_handle = __pyx_t_4;

  /* "cupy/cudnn.pyx":355
 * 
 *     handle = get_handle()
 *     desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     act_desc = cudnn.createActivationDescriptor()
 *     try:
 */
  __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_4 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 355, __pyx_L1_error)
  __pyx_v_desc = __pyx_t_4;

  /* "cupy/cudnn.pyx":356
 *     handle = get_handle()
 *     desc = cudnn.createTensorDescriptor()
 *     act_desc = cudnn.createActivationDescriptor()             # <<<<<<<<<<<<<<
 *     try:
 *         _create_tensor_descriptor_as4darray(desc, y)
 */
  __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_createActivationDescriptor(0); if (unlikely(__pyx_t_4 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 356, __pyx_L1_error)
  __pyx_v_act_desc = __pyx_t_4;

  /* "cupy/cudnn.pyx":357
 *     desc = cudnn.createTensorDescriptor()
 *     act_desc = cudnn.createActivationDescriptor()
 *     try:             # <<<<<<<<<<<<<<
 *         _create_tensor_descriptor_as4darray(desc, y)
 *         cudnn.setActivationDescriptor(
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":358
 *     act_desc = cudnn.createActivationDescriptor()
 *     try:
 *         _create_tensor_descriptor_as4darray(desc, y)             # <<<<<<<<<<<<<<
 *         cudnn.setActivationDescriptor(
 *             act_desc, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, coef)
 */
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(__pyx_v_desc, __pyx_v_y, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":359
 *     try:
 *         _create_tensor_descriptor_as4darray(desc, y)
 *         cudnn.setActivationDescriptor(             # <<<<<<<<<<<<<<
 *             act_desc, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, coef)
 *         cudnn.activationBackward_v4(
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_setActivationDescriptor(__pyx_v_act_desc, __pyx_v_mode, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN, __pyx_v_coef, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 359, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":361
 *         cudnn.setActivationDescriptor(
 *             act_desc, mode, cudnn.CUDNN_NOT_PROPAGATE_NAN, coef)
 *         cudnn.activationBackward_v4(             # <<<<<<<<<<<<<<
 *             handle, act_desc, one, desc, y.data.ptr,
 *             desc, gy.data.ptr, desc, x.data.ptr,
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_activationBackward_v4(__pyx_v_handle, __pyx_v_act_desc, __pyx_v_one, __pyx_v_desc, __pyx_v_y->data->ptr, __pyx_v_desc, __pyx_v_gy->data->ptr, __pyx_v_desc, __pyx_v_x->data->ptr, __pyx_v_zero, __pyx_v_desc, __pyx_v_gx->data->ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 361, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "cupy/cudnn.pyx":366
 *             zero, desc, gx.data.ptr)
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(desc)
 *     return gx
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyActivationDescriptor(__pyx_v_act_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 366, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cudnn.pyx":367
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)
 *         cudnn.destroyTensorDescriptor(desc)             # <<<<<<<<<<<<<<
 *     return gx
 * 
 */
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 367, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L6;
    }
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10) < 0)) __Pyx_ErrFetch(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __pyx_t_5 = __pyx_lineno; __pyx_t_6 = __pyx_clineno; __pyx_t_7 = __pyx_filename;
      {

        /* "cupy/cudnn.pyx":366
 *             zero, desc, gx.data.ptr)
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(desc)
 *     return gx
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyActivationDescriptor(__pyx_v_act_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 366, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "cupy/cudnn.pyx":367
 *     finally:
 *         cudnn.destroyActivationDescriptor(act_desc)
 *         cudnn.destroyTensorDescriptor(desc)             # <<<<<<<<<<<<<<
 *     return gx
 * 
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 367, __pyx_L8_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      }
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_ErrRestore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_6; __pyx_filename = __pyx_t_7;
      goto __pyx_L1_error;
      __pyx_L8_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_XGIVEREF(__pyx_t_12);
        __Pyx_XGIVEREF(__pyx_t_13);
        __Pyx_ExceptionReset(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      }
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L6:;
  }

  /* "cupy/cudnn.pyx":368
 *         cudnn.destroyActivationDescriptor(act_desc)
 *         cudnn.destroyTensorDescriptor(desc)
 *     return gx             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_gx));
  __pyx_r = ((PyObject *)__pyx_v_gx);
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":336
 * 
 * 
 * def activation_backward(core.ndarray x, core.ndarray y, core.ndarray gy,             # <<<<<<<<<<<<<<
 *                         int mode, float coef=0.0):
 *     cdef float float_zero = 0, float_one = 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cudnn.activation_backward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_gx);
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XDECREF((PyObject *)__pyx_v_y);
  __Pyx_XDECREF((PyObject *)__pyx_v_gy);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":371
 * 
 * 
 * def create_dropout_descriptor(             # <<<<<<<<<<<<<<
 *         handle, dropout, states, state_size_in_bytes, seed):
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_41create_dropout_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_40create_dropout_descriptor[] = "create_dropout_descriptor(handle, dropout, states, state_size_in_bytes, seed)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_41create_dropout_descriptor = {"create_dropout_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_41create_dropout_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_40create_dropout_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_41create_dropout_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_dropout = 0;
  PyObject *__pyx_v_states = 0;
  PyObject *__pyx_v_state_size_in_bytes = 0;
  PyObject *__pyx_v_seed = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_dropout_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handle,&__pyx_n_s_dropout,&__pyx_n_s_states,&__pyx_n_s_state_size_in_bytes,&__pyx_n_s_seed,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dropout)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_dropout_descriptor", 1, 5, 5, 1); __PYX_ERR(0, 371, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_states)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_dropout_descriptor", 1, 5, 5, 2); __PYX_ERR(0, 371, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_state_size_in_bytes)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_dropout_descriptor", 1, 5, 5, 3); __PYX_ERR(0, 371, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_seed)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_dropout_descriptor", 1, 5, 5, 4); __PYX_ERR(0, 371, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_dropout_descriptor") < 0)) __PYX_ERR(0, 371, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_handle = values[0];
    __pyx_v_dropout = values[1];
    __pyx_v_states = values[2];
    __pyx_v_state_size_in_bytes = values[3];
    __pyx_v_seed = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_dropout_descriptor", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 371, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_dropout_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_40create_dropout_descriptor(__pyx_self, __pyx_v_handle, __pyx_v_dropout, __pyx_v_states, __pyx_v_state_size_in_bytes, __pyx_v_seed);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_40create_dropout_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_dropout, PyObject *__pyx_v_states, PyObject *__pyx_v_state_size_in_bytes, PyObject *__pyx_v_seed) {
  PyObject *__pyx_v_desc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  float __pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  unsigned PY_LONG_LONG __pyx_t_13;
  __Pyx_RefNannySetupContext("create_dropout_descriptor", 0);

  /* "cupy/cudnn.pyx":373
 * def create_dropout_descriptor(
 *         handle, dropout, states, state_size_in_bytes, seed):
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyDropoutDescriptor)
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout,
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createDropoutDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 373, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":374
 *         handle, dropout, states, state_size_in_bytes, seed):
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 *                       py_cudnn.destroyDropoutDescriptor)             # <<<<<<<<<<<<<<
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout,
 *                                states, state_size_in_bytes, seed)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyDropoutDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 373, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":375
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 *                       py_cudnn.destroyDropoutDescriptor)
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout,             # <<<<<<<<<<<<<<
 *                                states, state_size_in_bytes, seed)
 *     return desc
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_9 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)
  __pyx_t_10 = __pyx_PyFloat_AsFloat(__pyx_v_dropout); if (unlikely((__pyx_t_10 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L1_error)

  /* "cupy/cudnn.pyx":376
 *                       py_cudnn.destroyDropoutDescriptor)
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout,
 *                                states, state_size_in_bytes, seed)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_11 = __Pyx_PyInt_As_size_t(__pyx_v_states); if (unlikely((__pyx_t_11 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)
  __pyx_t_12 = __Pyx_PyInt_As_size_t(__pyx_v_state_size_in_bytes); if (unlikely((__pyx_t_12 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)
  __pyx_t_13 = __Pyx_PyInt_As_unsigned_PY_LONG_LONG(__pyx_v_seed); if (unlikely((__pyx_t_13 == (unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)

  /* "cupy/cudnn.pyx":375
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 *                       py_cudnn.destroyDropoutDescriptor)
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout,             # <<<<<<<<<<<<<<
 *                                states, state_size_in_bytes, seed)
 *     return desc
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setDropoutDescriptor(__pyx_t_3, __pyx_t_9, __pyx_t_10, __pyx_t_11, __pyx_t_12, __pyx_t_13, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 375, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":377
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout,
 *                                states, state_size_in_bytes, seed)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":371
 * 
 * 
 * def create_dropout_descriptor(             # <<<<<<<<<<<<<<
 *         handle, dropout, states, state_size_in_bytes, seed):
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_dropout_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":380
 * 
 * 
 * def set_dropout_descriptor(desc, handle, dropout):             # <<<<<<<<<<<<<<
 *     # When the fourth argument is NULL, random state is not updated.
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout, 0, 0, 0)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_43set_dropout_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_42set_dropout_descriptor[] = "set_dropout_descriptor(desc, handle, dropout)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_43set_dropout_descriptor = {"set_dropout_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_43set_dropout_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_42set_dropout_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_43set_dropout_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_desc = 0;
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_dropout = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_dropout_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_desc,&__pyx_n_s_handle,&__pyx_n_s_dropout,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_desc)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_dropout_descriptor", 1, 3, 3, 1); __PYX_ERR(0, 380, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dropout)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_dropout_descriptor", 1, 3, 3, 2); __PYX_ERR(0, 380, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_dropout_descriptor") < 0)) __PYX_ERR(0, 380, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_desc = values[0];
    __pyx_v_handle = values[1];
    __pyx_v_dropout = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_dropout_descriptor", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 380, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.set_dropout_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_42set_dropout_descriptor(__pyx_self, __pyx_v_desc, __pyx_v_handle, __pyx_v_dropout);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_42set_dropout_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_desc, PyObject *__pyx_v_handle, PyObject *__pyx_v_dropout) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  float __pyx_t_4;
  __Pyx_RefNannySetupContext("set_dropout_descriptor", 0);

  /* "cupy/cudnn.pyx":382
 * def set_dropout_descriptor(desc, handle, dropout):
 *     # When the fourth argument is NULL, random state is not updated.
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout, 0, 0, 0)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L1_error)
  __pyx_t_4 = __pyx_PyFloat_AsFloat(__pyx_v_dropout); if (unlikely((__pyx_t_4 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setDropoutDescriptor(__pyx_t_2, __pyx_t_3, __pyx_t_4, 0, 0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":380
 * 
 * 
 * def set_dropout_descriptor(desc, handle, dropout):             # <<<<<<<<<<<<<<
 *     # When the fourth argument is NULL, random state is not updated.
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout, 0, 0, 0)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn.set_dropout_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":385
 * 
 * 
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,             # <<<<<<<<<<<<<<
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_45create_rnn_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_44create_rnn_descriptor[] = "create_rnn_descriptor(hidden_size, num_layers, dropout_desc, input_mode, direction, mode, data_type, algo=None)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_45create_rnn_descriptor = {"create_rnn_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_45create_rnn_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_44create_rnn_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_45create_rnn_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_hidden_size = 0;
  PyObject *__pyx_v_num_layers = 0;
  PyObject *__pyx_v_dropout_desc = 0;
  PyObject *__pyx_v_input_mode = 0;
  PyObject *__pyx_v_direction = 0;
  PyObject *__pyx_v_mode = 0;
  PyObject *__pyx_v_data_type = 0;
  PyObject *__pyx_v_algo = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_rnn_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_hidden_size,&__pyx_n_s_num_layers,&__pyx_n_s_dropout_desc,&__pyx_n_s_input_mode,&__pyx_n_s_direction,&__pyx_n_s_mode,&__pyx_n_s_data_type,&__pyx_n_s_algo,0};
    PyObject* values[8] = {0,0,0,0,0,0,0,0};

    /* "cupy/cudnn.pyx":386
 * 
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,
 *                           input_mode, direction, mode, data_type, algo=None):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 *                       py_cudnn.destroyRNNDescriptor)
 */
    values[7] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_hidden_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_num_layers)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, 1); __PYX_ERR(0, 385, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dropout_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, 2); __PYX_ERR(0, 385, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_input_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, 3); __PYX_ERR(0, 385, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_direction)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, 4); __PYX_ERR(0, 385, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mode)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, 5); __PYX_ERR(0, 385, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_data_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, 6); __PYX_ERR(0, 385, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_algo);
          if (value) { values[7] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_rnn_descriptor") < 0)) __PYX_ERR(0, 385, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_hidden_size = values[0];
    __pyx_v_num_layers = values[1];
    __pyx_v_dropout_desc = values[2];
    __pyx_v_input_mode = values[3];
    __pyx_v_direction = values[4];
    __pyx_v_mode = values[5];
    __pyx_v_data_type = values[6];
    __pyx_v_algo = values[7];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_rnn_descriptor", 0, 7, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 385, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_rnn_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_44create_rnn_descriptor(__pyx_self, __pyx_v_hidden_size, __pyx_v_num_layers, __pyx_v_dropout_desc, __pyx_v_input_mode, __pyx_v_direction, __pyx_v_mode, __pyx_v_data_type, __pyx_v_algo);

  /* "cupy/cudnn.pyx":385
 * 
 * 
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,             # <<<<<<<<<<<<<<
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_44create_rnn_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_hidden_size, PyObject *__pyx_v_num_layers, PyObject *__pyx_v_dropout_desc, PyObject *__pyx_v_input_mode, PyObject *__pyx_v_direction, PyObject *__pyx_v_mode, PyObject *__pyx_v_data_type, PyObject *__pyx_v_algo) {
  PyObject *__pyx_v_desc = NULL;
  size_t __pyx_v__handle;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  size_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  __Pyx_RefNannySetupContext("create_rnn_descriptor", 0);
  __Pyx_INCREF(__pyx_v_algo);

  /* "cupy/cudnn.pyx":387
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyRNNDescriptor)
 *     if _cudnn_version >= 6000:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createRNNDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 387, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 387, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":388
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 *                       py_cudnn.destroyRNNDescriptor)             # <<<<<<<<<<<<<<
 *     if _cudnn_version >= 6000:
 *         _handle = get_handle()
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyRNNDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":389
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 *                       py_cudnn.destroyRNNDescriptor)
 *     if _cudnn_version >= 6000:             # <<<<<<<<<<<<<<
 *         _handle = get_handle()
 *         if algo is None:
 */
  __pyx_t_9 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1770) != 0);
  if (__pyx_t_9) {

    /* "cupy/cudnn.pyx":390
 *                       py_cudnn.destroyRNNDescriptor)
 *     if _cudnn_version >= 6000:
 *         _handle = get_handle()             # <<<<<<<<<<<<<<
 *         if algo is None:
 *             algo = cudnn.CUDNN_RNN_ALGO_STANDARD
 */
    __pyx_t_3 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 390, __pyx_L1_error)
    __pyx_v__handle = __pyx_t_3;

    /* "cupy/cudnn.pyx":391
 *     if _cudnn_version >= 6000:
 *         _handle = get_handle()
 *         if algo is None:             # <<<<<<<<<<<<<<
 *             algo = cudnn.CUDNN_RNN_ALGO_STANDARD
 *         cudnn.setRNNDescriptor_v6(
 */
    __pyx_t_9 = (__pyx_v_algo == Py_None);
    __pyx_t_10 = (__pyx_t_9 != 0);
    if (__pyx_t_10) {

      /* "cupy/cudnn.pyx":392
 *         _handle = get_handle()
 *         if algo is None:
 *             algo = cudnn.CUDNN_RNN_ALGO_STANDARD             # <<<<<<<<<<<<<<
 *         cudnn.setRNNDescriptor_v6(
 *             _handle, desc.value, hidden_size, num_layers, dropout_desc.value,
 */
      __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_RNN_ALGO_STANDARD); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_algo, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":391
 *     if _cudnn_version >= 6000:
 *         _handle = get_handle()
 *         if algo is None:             # <<<<<<<<<<<<<<
 *             algo = cudnn.CUDNN_RNN_ALGO_STANDARD
 *         cudnn.setRNNDescriptor_v6(
 */
    }

    /* "cupy/cudnn.pyx":394
 *             algo = cudnn.CUDNN_RNN_ALGO_STANDARD
 *         cudnn.setRNNDescriptor_v6(
 *             _handle, desc.value, hidden_size, num_layers, dropout_desc.value,             # <<<<<<<<<<<<<<
 *             input_mode, direction, mode, algo, data_type)
 *     else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_hidden_size); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 394, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_v_num_layers); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 394, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dropout_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_12 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_12 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":395
 *         cudnn.setRNNDescriptor_v6(
 *             _handle, desc.value, hidden_size, num_layers, dropout_desc.value,
 *             input_mode, direction, mode, algo, data_type)             # <<<<<<<<<<<<<<
 *     else:
 *         cudnn.setRNNDescriptor_v5(
 */
    __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_v_input_mode); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 395, __pyx_L1_error)
    __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_v_direction); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 395, __pyx_L1_error)
    __pyx_t_15 = __Pyx_PyInt_As_int(__pyx_v_mode); if (unlikely((__pyx_t_15 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 395, __pyx_L1_error)
    __pyx_t_16 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_16 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 395, __pyx_L1_error)
    __pyx_t_17 = __Pyx_PyInt_As_int(__pyx_v_data_type); if (unlikely((__pyx_t_17 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 395, __pyx_L1_error)

    /* "cupy/cudnn.pyx":393
 *         if algo is None:
 *             algo = cudnn.CUDNN_RNN_ALGO_STANDARD
 *         cudnn.setRNNDescriptor_v6(             # <<<<<<<<<<<<<<
 *             _handle, desc.value, hidden_size, num_layers, dropout_desc.value,
 *             input_mode, direction, mode, algo, data_type)
 */
    __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setRNNDescriptor_v6(__pyx_v__handle, __pyx_t_3, __pyx_t_7, __pyx_t_11, __pyx_t_12, __pyx_t_13, __pyx_t_14, __pyx_t_15, __pyx_t_16, __pyx_t_17, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":389
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 *                       py_cudnn.destroyRNNDescriptor)
 *     if _cudnn_version >= 6000:             # <<<<<<<<<<<<<<
 *         _handle = get_handle()
 *         if algo is None:
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":397
 *             input_mode, direction, mode, algo, data_type)
 *     else:
 *         cudnn.setRNNDescriptor_v5(             # <<<<<<<<<<<<<<
 *             desc.value, hidden_size, num_layers, dropout_desc.value,
 *             input_mode, direction, mode, data_type)
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":398
 *     else:
 *         cudnn.setRNNDescriptor_v5(
 *             desc.value, hidden_size, num_layers, dropout_desc.value,             # <<<<<<<<<<<<<<
 *             input_mode, direction, mode, data_type)
 *     return desc
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_12 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_12 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 398, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_17 = __Pyx_PyInt_As_int(__pyx_v_hidden_size); if (unlikely((__pyx_t_17 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 398, __pyx_L1_error)
    __pyx_t_16 = __Pyx_PyInt_As_int(__pyx_v_num_layers); if (unlikely((__pyx_t_16 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 398, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dropout_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 398, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":399
 *         cudnn.setRNNDescriptor_v5(
 *             desc.value, hidden_size, num_layers, dropout_desc.value,
 *             input_mode, direction, mode, data_type)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
    __pyx_t_15 = __Pyx_PyInt_As_int(__pyx_v_input_mode); if (unlikely((__pyx_t_15 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 399, __pyx_L1_error)
    __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_v_direction); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 399, __pyx_L1_error)
    __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_v_mode); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 399, __pyx_L1_error)
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_v_data_type); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 399, __pyx_L1_error)

    /* "cupy/cudnn.pyx":397
 *             input_mode, direction, mode, algo, data_type)
 *     else:
 *         cudnn.setRNNDescriptor_v5(             # <<<<<<<<<<<<<<
 *             desc.value, hidden_size, num_layers, dropout_desc.value,
 *             input_mode, direction, mode, data_type)
 */
    __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setRNNDescriptor_v5(__pyx_t_12, __pyx_t_17, __pyx_t_16, __pyx_t_3, __pyx_t_15, __pyx_t_14, __pyx_t_13, __pyx_t_11, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":400
 *             desc.value, hidden_size, num_layers, dropout_desc.value,
 *             input_mode, direction, mode, data_type)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":385
 * 
 * 
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,             # <<<<<<<<<<<<<<
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_rnn_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XDECREF(__pyx_v_algo);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":403
 * 
 * 
 * def get_rnn_lin_layer_matrix_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_47get_rnn_lin_layer_matrix_params(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_46get_rnn_lin_layer_matrix_params[] = "get_rnn_lin_layer_matrix_params(handle, rnn_desc, layer, x_desc, w_desc, ndarray w, lin_layer_id)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_47get_rnn_lin_layer_matrix_params = {"get_rnn_lin_layer_matrix_params", (PyCFunction)__pyx_pw_4cupy_5cudnn_47get_rnn_lin_layer_matrix_params, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_46get_rnn_lin_layer_matrix_params};
static PyObject *__pyx_pw_4cupy_5cudnn_47get_rnn_lin_layer_matrix_params(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_rnn_desc = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_x_desc = 0;
  PyObject *__pyx_v_w_desc = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_w = 0;
  PyObject *__pyx_v_lin_layer_id = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_rnn_lin_layer_matrix_params (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handle,&__pyx_n_s_rnn_desc,&__pyx_n_s_layer,&__pyx_n_s_x_desc,&__pyx_n_s_w_desc,&__pyx_n_s_w,&__pyx_n_s_lin_layer_id,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rnn_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, 1); __PYX_ERR(0, 403, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_layer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, 2); __PYX_ERR(0, 403, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, 3); __PYX_ERR(0, 403, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_w_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, 4); __PYX_ERR(0, 403, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, 5); __PYX_ERR(0, 403, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_lin_layer_id)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, 6); __PYX_ERR(0, 403, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_rnn_lin_layer_matrix_params") < 0)) __PYX_ERR(0, 403, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
    }
    __pyx_v_handle = values[0];
    __pyx_v_rnn_desc = values[1];
    __pyx_v_layer = values[2];
    __pyx_v_x_desc = values[3];
    __pyx_v_w_desc = values[4];
    __pyx_v_w = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[5]);
    __pyx_v_lin_layer_id = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_matrix_params", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 403, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.get_rnn_lin_layer_matrix_params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_w), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "w", 0))) __PYX_ERR(0, 404, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_46get_rnn_lin_layer_matrix_params(__pyx_self, __pyx_v_handle, __pyx_v_rnn_desc, __pyx_v_layer, __pyx_v_x_desc, __pyx_v_w_desc, __pyx_v_w, __pyx_v_lin_layer_id);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_46get_rnn_lin_layer_matrix_params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_rnn_desc, PyObject *__pyx_v_layer, PyObject *__pyx_v_x_desc, PyObject *__pyx_v_w_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_w, PyObject *__pyx_v_lin_layer_id) {
  size_t __pyx_v_ptr;
  CYTHON_UNUSED size_t __pyx_v_w_data_ptr;
  size_t __pyx_v_mat_desc;
  PyObject *__pyx_v_data_type = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_dim = NULL;
  PyObject *__pyx_v_byte_size = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_v_mat = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  char const *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  Py_ssize_t __pyx_t_21;
  __Pyx_RefNannySetupContext("get_rnn_lin_layer_matrix_params", 0);

  /* "cupy/cudnn.pyx":405
 * def get_rnn_lin_layer_matrix_params(
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0             # <<<<<<<<<<<<<<
 *     w_data_ptr = w.data.ptr
 *     mat_desc = cudnn.createFilterDescriptor()
 */
  __pyx_v_ptr = 0;

  /* "cupy/cudnn.pyx":406
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 *     w_data_ptr = w.data.ptr             # <<<<<<<<<<<<<<
 *     mat_desc = cudnn.createFilterDescriptor()
 *     try:
 */
  __pyx_t_1 = __pyx_v_w->data->ptr;
  __pyx_v_w_data_ptr = __pyx_t_1;

  /* "cupy/cudnn.pyx":407
 *     cdef size_t ptr = 0
 *     w_data_ptr = w.data.ptr
 *     mat_desc = cudnn.createFilterDescriptor()             # <<<<<<<<<<<<<<
 *     try:
 *         cudnn.getRNNLinLayerMatrixParams(
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor(0); if (unlikely(__pyx_t_1 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 407, __pyx_L1_error)
  __pyx_v_mat_desc = __pyx_t_1;

  /* "cupy/cudnn.pyx":408
 *     w_data_ptr = w.data.ptr
 *     mat_desc = cudnn.createFilterDescriptor()
 *     try:             # <<<<<<<<<<<<<<
 *         cudnn.getRNNLinLayerMatrixParams(
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":410
 *     try:
 *         cudnn.getRNNLinLayerMatrixParams(
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,             # <<<<<<<<<<<<<<
 *             w.data.ptr, lin_layer_id, mat_desc, <size_t>&ptr)
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(mat_desc, 3)
 */
    __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L4_error)
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_rnn_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 410, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_layer); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L4_error)
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_x_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 410, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_w_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 410, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":411
 *         cudnn.getRNNLinLayerMatrixParams(
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 *             w.data.ptr, lin_layer_id, mat_desc, <size_t>&ptr)             # <<<<<<<<<<<<<<
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(mat_desc, 3)
 *     finally:
 */
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_lin_layer_id); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 411, __pyx_L4_error)

    /* "cupy/cudnn.pyx":409
 *     mat_desc = cudnn.createFilterDescriptor()
 *     try:
 *         cudnn.getRNNLinLayerMatrixParams(             # <<<<<<<<<<<<<<
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 *             w.data.ptr, lin_layer_id, mat_desc, <size_t>&ptr)
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_getRNNLinLayerMatrixParams(__pyx_t_1, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_v_w->data->ptr, __pyx_t_7, __pyx_v_mat_desc, ((size_t)(&__pyx_v_ptr)), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 409, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":412
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 *             w.data.ptr, lin_layer_id, mat_desc, <size_t>&ptr)
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(mat_desc, 3)             # <<<<<<<<<<<<<<
 *     finally:
 *         cudnn.destroyFilterDescriptor(mat_desc)
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_getFilterNdDescriptor(__pyx_v_mat_desc, 3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 412, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 412, __pyx_L4_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_8,&__pyx_t_9,&__pyx_t_10,&__pyx_t_11};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 412, __pyx_L4_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_8,&__pyx_t_9,&__pyx_t_10,&__pyx_t_11};
      __pyx_t_12 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 412, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_13(__pyx_t_12); if (unlikely(!item)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 4) < 0) __PYX_ERR(0, 412, __pyx_L4_error)
      __pyx_t_13 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_13 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 412, __pyx_L4_error)
      __pyx_L7_unpacking_done:;
    }
    __pyx_v_data_type = __pyx_t_8;
    __pyx_t_8 = 0;
    __pyx_v__ = __pyx_t_9;
    __pyx_t_9 = 0;
    __Pyx_DECREF_SET(__pyx_v__, __pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_v_dim = __pyx_t_11;
    __pyx_t_11 = 0;
  }

  /* "cupy/cudnn.pyx":414
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(mat_desc, 3)
 *     finally:
 *         cudnn.destroyFilterDescriptor(mat_desc)             # <<<<<<<<<<<<<<
 *     byte_size = _get_byte_size(data_type)
 *     offset = (ptr - w.data.ptr) // byte_size
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_mat_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __pyx_t_7 = __pyx_lineno; __pyx_t_4 = __pyx_clineno; __pyx_t_14 = __pyx_filename;
      {
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_mat_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 414, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_ErrRestore(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      __pyx_lineno = __pyx_t_7; __pyx_clineno = __pyx_t_4; __pyx_filename = __pyx_t_14;
      goto __pyx_L1_error;
      __pyx_L9_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      }
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "cupy/cudnn.pyx":415
 *     finally:
 *         cudnn.destroyFilterDescriptor(mat_desc)
 *     byte_size = _get_byte_size(data_type)             # <<<<<<<<<<<<<<
 *     offset = (ptr - w.data.ptr) // byte_size
 *     size = internal.prod(dim)
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_data_type); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 415, __pyx_L1_error)
  __pyx_t_7 = __pyx_f_4cupy_5cudnn__get_byte_size(__pyx_t_4, 0); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 415, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 415, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_byte_size = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":416
 *         cudnn.destroyFilterDescriptor(mat_desc)
 *     byte_size = _get_byte_size(data_type)
 *     offset = (ptr - w.data.ptr) // byte_size             # <<<<<<<<<<<<<<
 *     size = internal.prod(dim)
 *     mat = w[offset: offset + size]
 */
  __pyx_t_2 = __Pyx_PyInt_FromSize_t((__pyx_v_ptr - __pyx_v_w->data->ptr)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_11 = PyNumber_FloorDivide(__pyx_t_2, __pyx_v_byte_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 416, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_offset = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "cupy/cudnn.pyx":417
 *     byte_size = _get_byte_size(data_type)
 *     offset = (ptr - w.data.ptr) // byte_size
 *     size = internal.prod(dim)             # <<<<<<<<<<<<<<
 *     mat = w[offset: offset + size]
 *     return mat
 */
  __pyx_t_21 = __pyx_f_4cupy_4core_8internal_prod(__pyx_v_dim, 0, NULL); if (unlikely(__pyx_t_21 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 417, __pyx_L1_error)
  __pyx_t_11 = PyInt_FromSsize_t(__pyx_t_21); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 417, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v_size = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "cupy/cudnn.pyx":418
 *     offset = (ptr - w.data.ptr) // byte_size
 *     size = internal.prod(dim)
 *     mat = w[offset: offset + size]             # <<<<<<<<<<<<<<
 *     return mat
 * 
 */
  __pyx_t_11 = PyNumber_Add(__pyx_v_offset, __pyx_v_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_w), 0, 0, &__pyx_v_offset, &__pyx_t_11, NULL, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_mat = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":419
 *     size = internal.prod(dim)
 *     mat = w[offset: offset + size]
 *     return mat             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_mat);
  __pyx_r = __pyx_v_mat;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":403
 * 
 * 
 * def get_rnn_lin_layer_matrix_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupy.cudnn.get_rnn_lin_layer_matrix_params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data_type);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_dim);
  __Pyx_XDECREF(__pyx_v_byte_size);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XDECREF(__pyx_v_mat);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":422
 * 
 * 
 * def get_rnn_lin_layer_bias_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_49get_rnn_lin_layer_bias_params(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_48get_rnn_lin_layer_bias_params[] = "get_rnn_lin_layer_bias_params(handle, rnn_desc, layer, x_desc, w_desc, ndarray w, lin_layer_id)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_49get_rnn_lin_layer_bias_params = {"get_rnn_lin_layer_bias_params", (PyCFunction)__pyx_pw_4cupy_5cudnn_49get_rnn_lin_layer_bias_params, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_48get_rnn_lin_layer_bias_params};
static PyObject *__pyx_pw_4cupy_5cudnn_49get_rnn_lin_layer_bias_params(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_rnn_desc = 0;
  PyObject *__pyx_v_layer = 0;
  PyObject *__pyx_v_x_desc = 0;
  PyObject *__pyx_v_w_desc = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_w = 0;
  PyObject *__pyx_v_lin_layer_id = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_rnn_lin_layer_bias_params (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handle,&__pyx_n_s_rnn_desc,&__pyx_n_s_layer,&__pyx_n_s_x_desc,&__pyx_n_s_w_desc,&__pyx_n_s_w,&__pyx_n_s_lin_layer_id,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_rnn_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, 1); __PYX_ERR(0, 422, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_layer)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, 2); __PYX_ERR(0, 422, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, 3); __PYX_ERR(0, 422, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_w_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, 4); __PYX_ERR(0, 422, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_w)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, 5); __PYX_ERR(0, 422, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_lin_layer_id)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, 6); __PYX_ERR(0, 422, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_rnn_lin_layer_bias_params") < 0)) __PYX_ERR(0, 422, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
    }
    __pyx_v_handle = values[0];
    __pyx_v_rnn_desc = values[1];
    __pyx_v_layer = values[2];
    __pyx_v_x_desc = values[3];
    __pyx_v_w_desc = values[4];
    __pyx_v_w = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[5]);
    __pyx_v_lin_layer_id = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_rnn_lin_layer_bias_params", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 422, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.get_rnn_lin_layer_bias_params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_w), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "w", 0))) __PYX_ERR(0, 423, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_48get_rnn_lin_layer_bias_params(__pyx_self, __pyx_v_handle, __pyx_v_rnn_desc, __pyx_v_layer, __pyx_v_x_desc, __pyx_v_w_desc, __pyx_v_w, __pyx_v_lin_layer_id);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_48get_rnn_lin_layer_bias_params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_rnn_desc, PyObject *__pyx_v_layer, PyObject *__pyx_v_x_desc, PyObject *__pyx_v_w_desc, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_w, PyObject *__pyx_v_lin_layer_id) {
  size_t __pyx_v_ptr;
  size_t __pyx_v_bias_desc;
  PyObject *__pyx_v_data_type = NULL;
  CYTHON_UNUSED PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_dim = NULL;
  PyObject *__pyx_v_byte_size = NULL;
  PyObject *__pyx_v_offset = NULL;
  PyObject *__pyx_v_size = NULL;
  PyObject *__pyx_v_bias = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *(*__pyx_t_13)(PyObject *);
  char const *__pyx_t_14;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  Py_ssize_t __pyx_t_21;
  __Pyx_RefNannySetupContext("get_rnn_lin_layer_bias_params", 0);

  /* "cupy/cudnn.pyx":424
 * def get_rnn_lin_layer_bias_params(
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0             # <<<<<<<<<<<<<<
 *     bias_desc = cudnn.createFilterDescriptor()
 *     try:
 */
  __pyx_v_ptr = 0;

  /* "cupy/cudnn.pyx":425
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 *     bias_desc = cudnn.createFilterDescriptor()             # <<<<<<<<<<<<<<
 *     try:
 *         cudnn.getRNNLinLayerBiasParams(
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor(0); if (unlikely(__pyx_t_1 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 425, __pyx_L1_error)
  __pyx_v_bias_desc = __pyx_t_1;

  /* "cupy/cudnn.pyx":426
 *     cdef size_t ptr = 0
 *     bias_desc = cudnn.createFilterDescriptor()
 *     try:             # <<<<<<<<<<<<<<
 *         cudnn.getRNNLinLayerBiasParams(
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":428
 *     try:
 *         cudnn.getRNNLinLayerBiasParams(
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,             # <<<<<<<<<<<<<<
 *             w.data.ptr, lin_layer_id, bias_desc, <size_t>&ptr)
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(bias_desc, 3)
 */
    __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 428, __pyx_L4_error)
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_rnn_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 428, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_layer); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 428, __pyx_L4_error)
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_x_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 428, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_w_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 428, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 428, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":429
 *         cudnn.getRNNLinLayerBiasParams(
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 *             w.data.ptr, lin_layer_id, bias_desc, <size_t>&ptr)             # <<<<<<<<<<<<<<
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(bias_desc, 3)
 *     finally:
 */
    __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_lin_layer_id); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 429, __pyx_L4_error)

    /* "cupy/cudnn.pyx":427
 *     bias_desc = cudnn.createFilterDescriptor()
 *     try:
 *         cudnn.getRNNLinLayerBiasParams(             # <<<<<<<<<<<<<<
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 *             w.data.ptr, lin_layer_id, bias_desc, <size_t>&ptr)
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_getRNNLinLayerBiasParams(__pyx_t_1, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_v_w->data->ptr, __pyx_t_7, __pyx_v_bias_desc, ((size_t)(&__pyx_v_ptr)), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":430
 *             handle, rnn_desc.value, layer, x_desc.value, w_desc.value,
 *             w.data.ptr, lin_layer_id, bias_desc, <size_t>&ptr)
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(bias_desc, 3)             # <<<<<<<<<<<<<<
 *     finally:
 *         cudnn.destroyFilterDescriptor(bias_desc)
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_getFilterNdDescriptor(__pyx_v_bias_desc, 3, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 430, __pyx_L4_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_8 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_8,&__pyx_t_9,&__pyx_t_10,&__pyx_t_11};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 430, __pyx_L4_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_8,&__pyx_t_9,&__pyx_t_10,&__pyx_t_11};
      __pyx_t_12 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 430, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_13 = Py_TYPE(__pyx_t_12)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_13(__pyx_t_12); if (unlikely(!item)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_13(__pyx_t_12), 4) < 0) __PYX_ERR(0, 430, __pyx_L4_error)
      __pyx_t_13 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_13 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 430, __pyx_L4_error)
      __pyx_L7_unpacking_done:;
    }
    __pyx_v_data_type = __pyx_t_8;
    __pyx_t_8 = 0;
    __pyx_v__ = __pyx_t_9;
    __pyx_t_9 = 0;
    __Pyx_DECREF_SET(__pyx_v__, __pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_v_dim = __pyx_t_11;
    __pyx_t_11 = 0;
  }

  /* "cupy/cudnn.pyx":432
 *         data_type, _, _, dim = cudnn.getFilterNdDescriptor(bias_desc, 3)
 *     finally:
 *         cudnn.destroyFilterDescriptor(bias_desc)             # <<<<<<<<<<<<<<
 *     byte_size = _get_byte_size(data_type)
 *     offset = (ptr - w.data.ptr) // byte_size
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_bias_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17) < 0)) __Pyx_ErrFetch(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __pyx_t_7 = __pyx_lineno; __pyx_t_4 = __pyx_clineno; __pyx_t_14 = __pyx_filename;
      {
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_bias_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      }
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_17);
      __Pyx_ErrRestore(__pyx_t_15, __pyx_t_16, __pyx_t_17);
      __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      __pyx_lineno = __pyx_t_7; __pyx_clineno = __pyx_t_4; __pyx_filename = __pyx_t_14;
      goto __pyx_L1_error;
      __pyx_L9_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_XGIVEREF(__pyx_t_19);
        __Pyx_XGIVEREF(__pyx_t_20);
        __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      }
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "cupy/cudnn.pyx":433
 *     finally:
 *         cudnn.destroyFilterDescriptor(bias_desc)
 *     byte_size = _get_byte_size(data_type)             # <<<<<<<<<<<<<<
 *     offset = (ptr - w.data.ptr) // byte_size
 *     size = internal.prod(dim)
 */
  __pyx_t_4 = __Pyx_PyInt_As_int(__pyx_v_data_type); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 433, __pyx_L1_error)
  __pyx_t_7 = __pyx_f_4cupy_5cudnn__get_byte_size(__pyx_t_4, 0); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 433, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_byte_size = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":434
 *         cudnn.destroyFilterDescriptor(bias_desc)
 *     byte_size = _get_byte_size(data_type)
 *     offset = (ptr - w.data.ptr) // byte_size             # <<<<<<<<<<<<<<
 *     size = internal.prod(dim)
 *     bias = w[offset: offset + size]
 */
  __pyx_t_2 = __Pyx_PyInt_FromSize_t((__pyx_v_ptr - __pyx_v_w->data->ptr)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_11 = PyNumber_FloorDivide(__pyx_t_2, __pyx_v_byte_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 434, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_offset = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "cupy/cudnn.pyx":435
 *     byte_size = _get_byte_size(data_type)
 *     offset = (ptr - w.data.ptr) // byte_size
 *     size = internal.prod(dim)             # <<<<<<<<<<<<<<
 *     bias = w[offset: offset + size]
 *     return bias
 */
  __pyx_t_21 = __pyx_f_4cupy_4core_8internal_prod(__pyx_v_dim, 0, NULL); if (unlikely(__pyx_t_21 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 435, __pyx_L1_error)
  __pyx_t_11 = PyInt_FromSsize_t(__pyx_t_21); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_v_size = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "cupy/cudnn.pyx":436
 *     offset = (ptr - w.data.ptr) // byte_size
 *     size = internal.prod(dim)
 *     bias = w[offset: offset + size]             # <<<<<<<<<<<<<<
 *     return bias
 * 
 */
  __pyx_t_11 = PyNumber_Add(__pyx_v_offset, __pyx_v_size); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_w), 0, 0, &__pyx_v_offset, &__pyx_t_11, NULL, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 436, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_bias = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":437
 *     size = internal.prod(dim)
 *     bias = w[offset: offset + size]
 *     return bias             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_bias);
  __pyx_r = __pyx_v_bias;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":422
 * 
 * 
 * def get_rnn_lin_layer_bias_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupy.cudnn.get_rnn_lin_layer_bias_params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_data_type);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_dim);
  __Pyx_XDECREF(__pyx_v_byte_size);
  __Pyx_XDECREF(__pyx_v_offset);
  __Pyx_XDECREF(__pyx_v_size);
  __Pyx_XDECREF(__pyx_v_bias);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":440
 * 
 * 
 * def create_dropout_states(handle):             # <<<<<<<<<<<<<<
 *     warnings.warn('create_dropout_states is deprecated.'
 *                   'Please use DropoutStates class instead.',
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_51create_dropout_states(PyObject *__pyx_self, PyObject *__pyx_v_handle); /*proto*/
static char __pyx_doc_4cupy_5cudnn_50create_dropout_states[] = "create_dropout_states(handle)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_51create_dropout_states = {"create_dropout_states", (PyCFunction)__pyx_pw_4cupy_5cudnn_51create_dropout_states, METH_O, __pyx_doc_4cupy_5cudnn_50create_dropout_states};
static PyObject *__pyx_pw_4cupy_5cudnn_51create_dropout_states(PyObject *__pyx_self, PyObject *__pyx_v_handle) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_dropout_states (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_50create_dropout_states(__pyx_self, ((PyObject *)__pyx_v_handle));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_50create_dropout_states(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle) {
  Py_ssize_t __pyx_v_state_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __Pyx_RefNannySetupContext("create_dropout_states", 0);

  /* "cupy/cudnn.pyx":441
 * 
 * def create_dropout_states(handle):
 *     warnings.warn('create_dropout_states is deprecated.'             # <<<<<<<<<<<<<<
 *                   'Please use DropoutStates class instead.',
 *                   DeprecationWarning)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_warn); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":443
 *     warnings.warn('create_dropout_states is deprecated.'
 *                   'Please use DropoutStates class instead.',
 *                   DeprecationWarning)             # <<<<<<<<<<<<<<
 *     state_size = cudnn.dropoutGetStatesSize(handle)
 *     return core.ndarray((state_size,), 'b')
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":444
 *                   'Please use DropoutStates class instead.',
 *                   DeprecationWarning)
 *     state_size = cudnn.dropoutGetStatesSize(handle)             # <<<<<<<<<<<<<<
 *     return core.ndarray((state_size,), 'b')
 * 
 */
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 444, __pyx_L1_error)
  __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_dropoutGetStatesSize(__pyx_t_3, 0); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 444, __pyx_L1_error)
  __pyx_v_state_size = __pyx_t_4;

  /* "cupy/cudnn.pyx":445
 *                   DeprecationWarning)
 *     state_size = cudnn.dropoutGetStatesSize(handle)
 *     return core.ndarray((state_size,), 'b')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_state_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_b);
  __Pyx_GIVEREF(__pyx_n_s_b);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_b);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4cupy_4core_4core_ndarray), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":440
 * 
 * 
 * def create_dropout_states(handle):             # <<<<<<<<<<<<<<
 *     warnings.warn('create_dropout_states is deprecated.'
 *                   'Please use DropoutStates class instead.',
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn.create_dropout_states", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":448
 * 
 * 
 * def create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),
 *                       py_cudnn.destroySpatialTransformerDescriptor)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_53create_spatial_transformer_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_52create_spatial_transformer_descriptor[] = "create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_53create_spatial_transformer_descriptor = {"create_spatial_transformer_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_53create_spatial_transformer_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_52create_spatial_transformer_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_53create_spatial_transformer_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sampler_type = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_nb_dims = 0;
  PyObject *__pyx_v_dim_A = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_spatial_transformer_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sampler_type,&__pyx_n_s_dtype,&__pyx_n_s_nb_dims,&__pyx_n_s_dim_A,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sampler_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_spatial_transformer_descriptor", 1, 4, 4, 1); __PYX_ERR(0, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_nb_dims)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_spatial_transformer_descriptor", 1, 4, 4, 2); __PYX_ERR(0, 448, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dim_A)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_spatial_transformer_descriptor", 1, 4, 4, 3); __PYX_ERR(0, 448, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_spatial_transformer_descriptor") < 0)) __PYX_ERR(0, 448, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_sampler_type = values[0];
    __pyx_v_dtype = values[1];
    __pyx_v_nb_dims = values[2];
    __pyx_v_dim_A = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_spatial_transformer_descriptor", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 448, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_spatial_transformer_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_52create_spatial_transformer_descriptor(__pyx_self, __pyx_v_sampler_type, __pyx_v_dtype, __pyx_v_nb_dims, __pyx_v_dim_A);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_52create_spatial_transformer_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_sampler_type, PyObject *__pyx_v_dtype, PyObject *__pyx_v_nb_dims, PyObject *__pyx_v_dim_A) {
  PyObject *__pyx_v_desc = NULL;
  int __pyx_v_data_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  __Pyx_RefNannySetupContext("create_spatial_transformer_descriptor", 0);

  /* "cupy/cudnn.pyx":449
 * 
 * def create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A):
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroySpatialTransformerDescriptor)
 *     data_type = get_data_type(dtype)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createSpatialTransformerDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":450
 * def create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A):
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),
 *                       py_cudnn.destroySpatialTransformerDescriptor)             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(dtype)
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroySpatialTransformerDescrip); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 449, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":451
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),
 *                       py_cudnn.destroySpatialTransformerDescriptor)
 *     data_type = get_data_type(dtype)             # <<<<<<<<<<<<<<
 * 
 *     cudnn.setSpatialTransformerDescriptor(
 */
  __pyx_t_7 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_v_dtype, 0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 451, __pyx_L1_error)
  __pyx_v_data_type = __pyx_t_7;

  /* "cupy/cudnn.pyx":454
 * 
 *     cudnn.setSpatialTransformerDescriptor(
 *         desc.value, sampler_type, data_type, nb_dims, dim_A)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = __Pyx_PyInt_As_size_t(__pyx_v_sampler_type); if (unlikely((__pyx_t_9 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_nb_dims); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)
  __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_v_dim_A); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)

  /* "cupy/cudnn.pyx":453
 *     data_type = get_data_type(dtype)
 * 
 *     cudnn.setSpatialTransformerDescriptor(             # <<<<<<<<<<<<<<
 *         desc.value, sampler_type, data_type, nb_dims, dim_A)
 *     return desc
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setSpatialTransformerDescriptor(__pyx_t_3, __pyx_t_9, __pyx_v_data_type, __pyx_t_7, __pyx_t_10, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 453, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":455
 *     cudnn.setSpatialTransformerDescriptor(
 *         desc.value, sampler_type, data_type, nb_dims, dim_A)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":448
 * 
 * 
 * def create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),
 *                       py_cudnn.destroySpatialTransformerDescriptor)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_spatial_transformer_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":458
 * 
 * 
 * def add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc,             # <<<<<<<<<<<<<<
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_55add_tensor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_54add_tensor[] = "add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc, srcDestData)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_55add_tensor = {"add_tensor", (PyCFunction)__pyx_pw_4cupy_5cudnn_55add_tensor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_54add_tensor};
static PyObject *__pyx_pw_4cupy_5cudnn_55add_tensor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_alpha = 0;
  PyObject *__pyx_v_biasDesc = 0;
  PyObject *__pyx_v_biasData = 0;
  PyObject *__pyx_v_beta = 0;
  PyObject *__pyx_v_srcDestDesc = 0;
  PyObject *__pyx_v_srcDestData = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_tensor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handle,&__pyx_n_s_alpha,&__pyx_n_s_biasDesc,&__pyx_n_s_biasData,&__pyx_n_s_beta,&__pyx_n_s_srcDestDesc,&__pyx_n_s_srcDestData,0};
    PyObject* values[7] = {0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, 1); __PYX_ERR(0, 458, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_biasDesc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, 2); __PYX_ERR(0, 458, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_biasData)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, 3); __PYX_ERR(0, 458, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_beta)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, 4); __PYX_ERR(0, 458, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_srcDestDesc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, 5); __PYX_ERR(0, 458, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_srcDestData)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, 6); __PYX_ERR(0, 458, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_tensor") < 0)) __PYX_ERR(0, 458, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
    }
    __pyx_v_handle = values[0];
    __pyx_v_alpha = values[1];
    __pyx_v_biasDesc = values[2];
    __pyx_v_biasData = values[3];
    __pyx_v_beta = values[4];
    __pyx_v_srcDestDesc = values[5];
    __pyx_v_srcDestData = values[6];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_tensor", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 458, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.add_tensor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_54add_tensor(__pyx_self, __pyx_v_handle, __pyx_v_alpha, __pyx_v_biasDesc, __pyx_v_biasData, __pyx_v_beta, __pyx_v_srcDestDesc, __pyx_v_srcDestData);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_54add_tensor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_alpha, PyObject *__pyx_v_biasDesc, PyObject *__pyx_v_biasData, PyObject *__pyx_v_beta, PyObject *__pyx_v_srcDestDesc, PyObject *__pyx_v_srcDestData) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("add_tensor", 0);

  /* "cupy/cudnn.pyx":460
 * def add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc,
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,             # <<<<<<<<<<<<<<
 *                        biasData, beta, srcDestDesc, srcDestData)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyInt_As_size_t(__pyx_v_alpha); if (unlikely((__pyx_t_2 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_v_biasDesc); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)

  /* "cupy/cudnn.pyx":461
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,
 *                        biasData, beta, srcDestDesc, srcDestData)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = __Pyx_PyInt_As_size_t(__pyx_v_biasData); if (unlikely((__pyx_t_4 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 461, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyInt_As_size_t(__pyx_v_beta); if (unlikely((__pyx_t_5 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 461, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyInt_As_size_t(__pyx_v_srcDestDesc); if (unlikely((__pyx_t_6 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 461, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyInt_As_size_t(__pyx_v_srcDestData); if (unlikely((__pyx_t_7 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 461, __pyx_L1_error)

  /* "cupy/cudnn.pyx":460
 * def add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc,
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,             # <<<<<<<<<<<<<<
 *                        biasData, beta, srcDestDesc, srcDestData)
 * 
 */
  __pyx_t_8 = __pyx_f_4cupy_4cuda_5cudnn_addTensor_v3(__pyx_t_1, __pyx_t_2, __pyx_t_3, __pyx_t_4, __pyx_t_5, __pyx_t_6, __pyx_t_7, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "cupy/cudnn.pyx":458
 * 
 * 
 * def add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc,             # <<<<<<<<<<<<<<
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.add_tensor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":464
 * 
 * 
 * def create_op_tensor_descriptor(op_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),
 *                       py_cudnn.destroyOpTensorDescriptor)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_57create_op_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_56create_op_tensor_descriptor[] = "create_op_tensor_descriptor(op_type, dtype)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_57create_op_tensor_descriptor = {"create_op_tensor_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_57create_op_tensor_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_56create_op_tensor_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_57create_op_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_op_type = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_op_tensor_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_op_type,&__pyx_n_s_dtype,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_op_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_op_tensor_descriptor", 1, 2, 2, 1); __PYX_ERR(0, 464, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_op_tensor_descriptor") < 0)) __PYX_ERR(0, 464, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_op_type = values[0];
    __pyx_v_dtype = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_op_tensor_descriptor", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 464, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_op_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_56create_op_tensor_descriptor(__pyx_self, __pyx_v_op_type, __pyx_v_dtype);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_56create_op_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_op_type, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_v_desc = NULL;
  int __pyx_v_data_type;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("create_op_tensor_descriptor", 0);

  /* "cupy/cudnn.pyx":465
 * 
 * def create_op_tensor_descriptor(op_type, dtype):
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyOpTensorDescriptor)
 *     data_type = get_data_type(dtype)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createOpTensorDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 465, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":466
 * def create_op_tensor_descriptor(op_type, dtype):
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),
 *                       py_cudnn.destroyOpTensorDescriptor)             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(dtype)
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyOpTensorDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":467
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),
 *                       py_cudnn.destroyOpTensorDescriptor)
 *     data_type = get_data_type(dtype)             # <<<<<<<<<<<<<<
 * 
 *     cudnn.setOpTensorDescriptor(desc.value, op_type, data_type,
 */
  __pyx_t_7 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_v_dtype, 0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 467, __pyx_L1_error)
  __pyx_v_data_type = __pyx_t_7;

  /* "cupy/cudnn.pyx":469
 *     data_type = get_data_type(dtype)
 * 
 *     cudnn.setOpTensorDescriptor(desc.value, op_type, data_type,             # <<<<<<<<<<<<<<
 *                                 cudnn.CUDNN_NOT_PROPAGATE_NAN)
 *     return desc
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_op_type); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L1_error)

  /* "cupy/cudnn.pyx":470
 * 
 *     cudnn.setOpTensorDescriptor(desc.value, op_type, data_type,
 *                                 cudnn.CUDNN_NOT_PROPAGATE_NAN)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setOpTensorDescriptor(__pyx_t_3, __pyx_t_7, __pyx_v_data_type, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":471
 *     cudnn.setOpTensorDescriptor(desc.value, op_type, data_type,
 *                                 cudnn.CUDNN_NOT_PROPAGATE_NAN)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":464
 * 
 * 
 * def create_op_tensor_descriptor(op_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),
 *                       py_cudnn.destroyOpTensorDescriptor)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_op_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":474
 * 
 * 
 * def create_reduce_tensor_descriptor(reduce_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),
 *                       py_cudnn.destroyReduceTensorDescriptor)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_59create_reduce_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_58create_reduce_tensor_descriptor[] = "create_reduce_tensor_descriptor(reduce_type, dtype)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_59create_reduce_tensor_descriptor = {"create_reduce_tensor_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_59create_reduce_tensor_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_58create_reduce_tensor_descriptor};
static PyObject *__pyx_pw_4cupy_5cudnn_59create_reduce_tensor_descriptor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_reduce_type = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_reduce_tensor_descriptor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_reduce_type,&__pyx_n_s_dtype,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reduce_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_reduce_tensor_descriptor", 1, 2, 2, 1); __PYX_ERR(0, 474, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_reduce_tensor_descriptor") < 0)) __PYX_ERR(0, 474, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_reduce_type = values[0];
    __pyx_v_dtype = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_reduce_tensor_descriptor", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 474, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.create_reduce_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_58create_reduce_tensor_descriptor(__pyx_self, __pyx_v_reduce_type, __pyx_v_dtype);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_58create_reduce_tensor_descriptor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_reduce_type, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_v_desc = NULL;
  int __pyx_v_data_type;
  int __pyx_v_indices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("create_reduce_tensor_descriptor", 0);

  /* "cupy/cudnn.pyx":475
 * 
 * def create_reduce_tensor_descriptor(reduce_type, dtype):
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),             # <<<<<<<<<<<<<<
 *                       py_cudnn.destroyReduceTensorDescriptor)
 *     data_type = get_data_type(dtype)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_Descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_createReduceTensorDescriptor(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 475, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":476
 * def create_reduce_tensor_descriptor(reduce_type, dtype):
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),
 *                       py_cudnn.destroyReduceTensorDescriptor)             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(dtype)
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_py_cudnn); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_destroyReduceTensorDescriptor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_4, __pyx_t_6};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_6);
    __pyx_t_4 = 0;
    __pyx_t_6 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_desc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":477
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),
 *                       py_cudnn.destroyReduceTensorDescriptor)
 *     data_type = get_data_type(dtype)             # <<<<<<<<<<<<<<
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,
 *                        cudnn.CUDNN_REDUCE_TENSOR_MAX):
 */
  __pyx_t_7 = __pyx_f_4cupy_5cudnn_get_data_type(__pyx_v_dtype, 0); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 477, __pyx_L1_error)
  __pyx_v_data_type = __pyx_t_7;

  /* "cupy/cudnn.pyx":478
 *                       py_cudnn.destroyReduceTensorDescriptor)
 *     data_type = get_data_type(dtype)
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,             # <<<<<<<<<<<<<<
 *                        cudnn.CUDNN_REDUCE_TENSOR_MAX):
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_FLATTENED_INDICES
 */
  __Pyx_INCREF(__pyx_v_reduce_type);
  __pyx_t_1 = __pyx_v_reduce_type;
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_MIN); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L4_bool_binop_done;
  }

  /* "cupy/cudnn.pyx":479
 *     data_type = get_data_type(dtype)
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,
 *                        cudnn.CUDNN_REDUCE_TENSOR_MAX):             # <<<<<<<<<<<<<<
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_FLATTENED_INDICES
 *     else:
 */
  __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_MAX); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_t_8, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "cupy/cudnn.pyx":478
 *                       py_cudnn.destroyReduceTensorDescriptor)
 *     data_type = get_data_type(dtype)
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,             # <<<<<<<<<<<<<<
 *                        cudnn.CUDNN_REDUCE_TENSOR_MAX):
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_FLATTENED_INDICES
 */
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_9 = __pyx_t_10;
  __pyx_L4_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "cupy/cudnn.pyx":480
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,
 *                        cudnn.CUDNN_REDUCE_TENSOR_MAX):
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_FLATTENED_INDICES             # <<<<<<<<<<<<<<
 *     else:
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_NO_INDICES
 */
    __pyx_v_indices = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_FLATTENED_INDICES;

    /* "cupy/cudnn.pyx":478
 *                       py_cudnn.destroyReduceTensorDescriptor)
 *     data_type = get_data_type(dtype)
 *     if reduce_type in (cudnn.CUDNN_REDUCE_TENSOR_MIN,             # <<<<<<<<<<<<<<
 *                        cudnn.CUDNN_REDUCE_TENSOR_MAX):
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_FLATTENED_INDICES
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":482
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_FLATTENED_INDICES
 *     else:
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_NO_INDICES             # <<<<<<<<<<<<<<
 * 
 *     cudnn.setReduceTensorDescriptor(desc.value, reduce_type, data_type,
 */
  /*else*/ {
    __pyx_v_indices = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_REDUCE_TENSOR_NO_INDICES;
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":484
 *         indices = cudnn.CUDNN_REDUCE_TENSOR_NO_INDICES
 * 
 *     cudnn.setReduceTensorDescriptor(desc.value, reduce_type, data_type,             # <<<<<<<<<<<<<<
 *                                     cudnn.CUDNN_NOT_PROPAGATE_NAN,
 *                                     indices,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_desc, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_int(__pyx_v_reduce_type); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 484, __pyx_L1_error)

  /* "cupy/cudnn.pyx":487
 *                                     cudnn.CUDNN_NOT_PROPAGATE_NAN,
 *                                     indices,
 *                                     cudnn.CUDNN_32BIT_INDICES)             # <<<<<<<<<<<<<<
 *     return desc
 * 
 */
  __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_setReduceTensorDescriptor(__pyx_t_3, __pyx_t_7, __pyx_v_data_type, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_NOT_PROPAGATE_NAN, __pyx_v_indices, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_32BIT_INDICES, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":488
 *                                     indices,
 *                                     cudnn.CUDNN_32BIT_INDICES)
 *     return desc             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_desc);
  __pyx_r = __pyx_v_desc;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":474
 * 
 * 
 * def create_reduce_tensor_descriptor(reduce_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),
 *                       py_cudnn.destroyReduceTensorDescriptor)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("cupy.cudnn.create_reduce_tensor_descriptor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_desc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":491
 * 
 * 
 * cpdef bint is_tensor_core_available(dtype) except *:             # <<<<<<<<<<<<<<
 *     return (_cudnn_version >= 7000 and
 *             (<str>dtype.char) == 'e' and
 */

static PyObject *__pyx_pw_4cupy_5cudnn_61is_tensor_core_available(PyObject *__pyx_self, PyObject *__pyx_v_dtype); /*proto*/
static int __pyx_f_4cupy_5cudnn_is_tensor_core_available(PyObject *__pyx_v_dtype, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("is_tensor_core_available", 0);

  /* "cupy/cudnn.pyx":492
 * 
 * cpdef bint is_tensor_core_available(dtype) except *:
 *     return (_cudnn_version >= 7000 and             # <<<<<<<<<<<<<<
 *             (<str>dtype.char) == 'e' and
 *             int(device.get_compute_capability()) == 70)
 */
  __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }

  /* "cupy/cudnn.pyx":493
 * cpdef bint is_tensor_core_available(dtype) except *:
 *     return (_cudnn_version >= 7000 and
 *             (<str>dtype.char) == 'e' and             # <<<<<<<<<<<<<<
 *             int(device.get_compute_capability()) == 70)
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_char); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_t_3, __pyx_n_s_e, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = (__pyx_t_2 != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_1 = __pyx_t_4;
    goto __pyx_L3_bool_binop_done;
  }

  /* "cupy/cudnn.pyx":494
 *     return (_cudnn_version >= 7000 and
 *             (<str>dtype.char) == 'e' and
 *             int(device.get_compute_capability()) == 70)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = __pyx_f_4cupy_4cuda_6device_get_compute_capability(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyNumber_Int(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_t_5, __pyx_int_70, 70, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_1 = __pyx_t_4;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":491
 * 
 * 
 * cpdef bint is_tensor_core_available(dtype) except *:             # <<<<<<<<<<<<<<
 *     return (_cudnn_version >= 7000 and
 *             (<str>dtype.char) == 'e' and
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("cupy.cudnn.is_tensor_core_available", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_61is_tensor_core_available(PyObject *__pyx_self, PyObject *__pyx_v_dtype); /*proto*/
static char __pyx_doc_4cupy_5cudnn_60is_tensor_core_available[] = "is_tensor_core_available(dtype) -> bool";
static PyObject *__pyx_pw_4cupy_5cudnn_61is_tensor_core_available(PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_tensor_core_available (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_60is_tensor_core_available(__pyx_self, ((PyObject *)__pyx_v_dtype));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_60is_tensor_core_available(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("is_tensor_core_available", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn_is_tensor_core_available(__pyx_v_dtype, 0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 491, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn.is_tensor_core_available", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":499
 * class DropoutStates(object):
 * 
 *     def __init__(self, handle, seed):             # <<<<<<<<<<<<<<
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_13DropoutStates_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_13DropoutStates___init__[] = "DropoutStates.__init__(self, handle, seed)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_13DropoutStates_1__init__ = {"__init__", (PyCFunction)__pyx_pw_4cupy_5cudnn_13DropoutStates_1__init__, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_13DropoutStates___init__};
static PyObject *__pyx_pw_4cupy_5cudnn_13DropoutStates_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_handle = 0;
  PyObject *__pyx_v_seed = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_handle,&__pyx_n_s_seed,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 1); __PYX_ERR(0, 499, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_seed)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, 2); __PYX_ERR(0, 499, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 499, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = values[0];
    __pyx_v_handle = values[1];
    __pyx_v_seed = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 499, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.DropoutStates.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_13DropoutStates___init__(__pyx_self, __pyx_v_self, __pyx_v_handle, __pyx_v_seed);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_13DropoutStates___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_handle, PyObject *__pyx_v_seed) {
  Py_ssize_t __pyx_v_state_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "cupy/cudnn.pyx":500
 * 
 *     def __init__(self, handle, seed):
 *         state_size = cudnn.dropoutGetStatesSize(handle)             # <<<<<<<<<<<<<<
 *         self._states = memory.alloc(state_size)
 *         self._desc = create_dropout_descriptor(
 */
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 500, __pyx_L1_error)
  __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_dropoutGetStatesSize(__pyx_t_1, 0); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 500, __pyx_L1_error)
  __pyx_v_state_size = __pyx_t_2;

  /* "cupy/cudnn.pyx":501
 *     def __init__(self, handle, seed):
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)             # <<<<<<<<<<<<<<
 *         self._desc = create_dropout_descriptor(
 *             handle, 0., self._states.ptr,
 */
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_state_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_3, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_states_2, __pyx_t_4) < 0) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":502
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)
 *         self._desc = create_dropout_descriptor(             # <<<<<<<<<<<<<<
 *             handle, 0., self._states.ptr,
 *             state_size, seed)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_create_dropout_descriptor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "cupy/cudnn.pyx":503
 *         self._states = memory.alloc(state_size)
 *         self._desc = create_dropout_descriptor(
 *             handle, 0., self._states.ptr,             # <<<<<<<<<<<<<<
 *             state_size, seed)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_states_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_ptr); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 503, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "cupy/cudnn.pyx":504
 *         self._desc = create_dropout_descriptor(
 *             handle, 0., self._states.ptr,
 *             state_size, seed)             # <<<<<<<<<<<<<<
 * 
 *     def forward(self, handle, core.ndarray x, dropout_ratio):
 */
  __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_state_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[6] = {__pyx_t_7, __pyx_v_handle, __pyx_float_0_, __pyx_t_6, __pyx_t_5, __pyx_v_seed};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 5+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[6] = {__pyx_t_7, __pyx_v_handle, __pyx_float_0_, __pyx_t_6, __pyx_t_5, __pyx_v_seed};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 5+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(5+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_handle);
    __Pyx_GIVEREF(__pyx_v_handle);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_handle);
    __Pyx_INCREF(__pyx_float_0_);
    __Pyx_GIVEREF(__pyx_float_0_);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_float_0_);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_9, 3+__pyx_t_8, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_seed);
    __Pyx_GIVEREF(__pyx_v_seed);
    PyTuple_SET_ITEM(__pyx_t_9, 4+__pyx_t_8, __pyx_v_seed);
    __pyx_t_6 = 0;
    __pyx_t_5 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "cupy/cudnn.pyx":502
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)
 *         self._desc = create_dropout_descriptor(             # <<<<<<<<<<<<<<
 *             handle, 0., self._states.ptr,
 *             state_size, seed)
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_desc_2, __pyx_t_4) < 0) __PYX_ERR(0, 502, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":499
 * class DropoutStates(object):
 * 
 *     def __init__(self, handle, seed):             # <<<<<<<<<<<<<<
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("cupy.cudnn.DropoutStates.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":506
 *             state_size, seed)
 * 
 *     def forward(self, handle, core.ndarray x, dropout_ratio):             # <<<<<<<<<<<<<<
 *         cdef core.ndarray y, reserve_space
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_13DropoutStates_3forward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_13DropoutStates_2forward[] = "DropoutStates.forward(self, handle, ndarray x, dropout_ratio)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_13DropoutStates_3forward = {"forward", (PyCFunction)__pyx_pw_4cupy_5cudnn_13DropoutStates_3forward, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_13DropoutStates_2forward};
static PyObject *__pyx_pw_4cupy_5cudnn_13DropoutStates_3forward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_handle = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  PyObject *__pyx_v_dropout_ratio = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("forward (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_handle,&__pyx_n_s_x,&__pyx_n_s_dropout_ratio,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("forward", 1, 4, 4, 1); __PYX_ERR(0, 506, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("forward", 1, 4, 4, 2); __PYX_ERR(0, 506, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dropout_ratio)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("forward", 1, 4, 4, 3); __PYX_ERR(0, 506, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "forward") < 0)) __PYX_ERR(0, 506, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_self = values[0];
    __pyx_v_handle = values[1];
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_dropout_ratio = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("forward", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 506, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.DropoutStates.forward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 506, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_13DropoutStates_2forward(__pyx_self, __pyx_v_self, __pyx_v_handle, __pyx_v_x, __pyx_v_dropout_ratio);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_13DropoutStates_2forward(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_handle, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, PyObject *__pyx_v_dropout_ratio) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_reserve_space = 0;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_reserve_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_t_9;
  char const *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("forward", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_x);

  /* "cupy/cudnn.pyx":508
 *     def forward(self, handle, core.ndarray x, dropout_ratio):
 *         cdef core.ndarray y, reserve_space
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)             # <<<<<<<<<<<<<<
 * 
 *         x = core.ascontiguousarray(x)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_set_dropout_descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_desc_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 508, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, __pyx_v_handle, __pyx_v_dropout_ratio};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, __pyx_v_handle, __pyx_v_dropout_ratio};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_handle);
    __Pyx_GIVEREF(__pyx_v_handle);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_handle);
    __Pyx_INCREF(__pyx_v_dropout_ratio);
    __Pyx_GIVEREF(__pyx_v_dropout_ratio);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_dropout_ratio);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":510
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)
 * 
 *         x = core.ascontiguousarray(x)             # <<<<<<<<<<<<<<
 *         y = core.ndarray(x._shape, x.dtype)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_x, 0, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_x, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":511
 * 
 *         x = core.ascontiguousarray(x)
 *         y = core.ndarray(x._shape, x.dtype)             # <<<<<<<<<<<<<<
 * 
 *         x_desc = cudnn.createTensorDescriptor()
 */
  __pyx_t_1 = __pyx_convert_vector_to_py_Py_ssize_t(__pyx_v_x->_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_v_x->dtype);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_x->dtype);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4cupy_4core_4core_ndarray), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":513
 *         y = core.ndarray(x._shape, x.dtype)
 * 
 *         x_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *         try:
 *             _create_tensor_descriptor_as4darray(x_desc, x)
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 513, __pyx_L1_error)
  __pyx_v_x_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":514
 * 
 *         x_desc = cudnn.createTensorDescriptor()
 *         try:             # <<<<<<<<<<<<<<
 *             _create_tensor_descriptor_as4darray(x_desc, x)
 *             reserve_size = cudnn.getDropoutReserveSpaceSize(x_desc)
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":515
 *         x_desc = cudnn.createTensorDescriptor()
 *         try:
 *             _create_tensor_descriptor_as4darray(x_desc, x)             # <<<<<<<<<<<<<<
 *             reserve_size = cudnn.getDropoutReserveSpaceSize(x_desc)
 *             reserve_space = core.ndarray((reserve_size,), 'b')
 */
    __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(__pyx_v_x_desc, __pyx_v_x, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":516
 *         try:
 *             _create_tensor_descriptor_as4darray(x_desc, x)
 *             reserve_size = cudnn.getDropoutReserveSpaceSize(x_desc)             # <<<<<<<<<<<<<<
 *             reserve_space = core.ndarray((reserve_size,), 'b')
 * 
 */
    __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_getDropoutReserveSpaceSize(__pyx_v_x_desc, 0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 516, __pyx_L4_error)
    __pyx_v_reserve_size = __pyx_t_7;

    /* "cupy/cudnn.pyx":517
 *             _create_tensor_descriptor_as4darray(x_desc, x)
 *             reserve_size = cudnn.getDropoutReserveSpaceSize(x_desc)
 *             reserve_space = core.ndarray((reserve_size,), 'b')             # <<<<<<<<<<<<<<
 * 
 *             cudnn.dropoutForward(handle, self._desc.value,
 */
    __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_reserve_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 517, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 517, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_b);
    __Pyx_GIVEREF(__pyx_n_s_b);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_b);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4cupy_4core_4core_ndarray), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_reserve_space = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":519
 *             reserve_space = core.ndarray((reserve_size,), 'b')
 * 
 *             cudnn.dropoutForward(handle, self._desc.value,             # <<<<<<<<<<<<<<
 *                                  x_desc, x.data.ptr, x_desc, y.data.ptr,
 *                                  reserve_space.data.ptr, reserve_size)
 */
    __pyx_t_7 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_7 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 519, __pyx_L4_error)
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_desc_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 519, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_8 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 519, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":521
 *             cudnn.dropoutForward(handle, self._desc.value,
 *                                  x_desc, x.data.ptr, x_desc, y.data.ptr,
 *                                  reserve_space.data.ptr, reserve_size)             # <<<<<<<<<<<<<<
 *         finally:
 *             cudnn.destroyTensorDescriptor(x_desc)
 */
    __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_dropoutForward(__pyx_t_7, __pyx_t_8, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_x_desc, __pyx_v_y->data->ptr, __pyx_v_reserve_space->data->ptr, __pyx_v_reserve_size, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 519, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "cupy/cudnn.pyx":523
 *                                  reserve_space.data.ptr, reserve_size)
 *         finally:
 *             cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         return reserve_space, y
 * 
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __pyx_t_5 = __pyx_lineno; __pyx_t_9 = __pyx_clineno; __pyx_t_10 = __pyx_filename;
      {
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_14);
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      }
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ErrRestore(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_9; __pyx_filename = __pyx_t_10;
      goto __pyx_L1_error;
      __pyx_L7_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_14);
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      }
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "cupy/cudnn.pyx":524
 *         finally:
 *             cudnn.destroyTensorDescriptor(x_desc)
 *         return reserve_space, y             # <<<<<<<<<<<<<<
 * 
 *     def backward(self, handle, core.ndarray dy, dropout_ratio,
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_reserve_space));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_reserve_space));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_reserve_space));
  __Pyx_INCREF(((PyObject *)__pyx_v_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_y));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_y));
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":506
 *             state_size, seed)
 * 
 *     def forward(self, handle, core.ndarray x, dropout_ratio):             # <<<<<<<<<<<<<<
 *         cdef core.ndarray y, reserve_space
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn.DropoutStates.forward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_y);
  __Pyx_XDECREF((PyObject *)__pyx_v_reserve_space);
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":526
 *         return reserve_space, y
 * 
 *     def backward(self, handle, core.ndarray dy, dropout_ratio,             # <<<<<<<<<<<<<<
 *                  core.ndarray reserve_space):
 *         cdef core.ndarray dx
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_13DropoutStates_5backward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_13DropoutStates_4backward[] = "DropoutStates.backward(self, handle, ndarray dy, dropout_ratio, ndarray reserve_space)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_13DropoutStates_5backward = {"backward", (PyCFunction)__pyx_pw_4cupy_5cudnn_13DropoutStates_5backward, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_13DropoutStates_4backward};
static PyObject *__pyx_pw_4cupy_5cudnn_13DropoutStates_5backward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_handle = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy = 0;
  PyObject *__pyx_v_dropout_ratio = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_reserve_space = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("backward (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_handle,&__pyx_n_s_dy,&__pyx_n_s_dropout_ratio,&__pyx_n_s_reserve_space,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("backward", 1, 5, 5, 1); __PYX_ERR(0, 526, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("backward", 1, 5, 5, 2); __PYX_ERR(0, 526, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dropout_ratio)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("backward", 1, 5, 5, 3); __PYX_ERR(0, 526, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_reserve_space)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("backward", 1, 5, 5, 4); __PYX_ERR(0, 526, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "backward") < 0)) __PYX_ERR(0, 526, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_self = values[0];
    __pyx_v_handle = values[1];
    __pyx_v_dy = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_dropout_ratio = values[3];
    __pyx_v_reserve_space = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[4]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("backward", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 526, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.DropoutStates.backward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dy), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dy", 0))) __PYX_ERR(0, 526, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_reserve_space), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "reserve_space", 0))) __PYX_ERR(0, 527, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_13DropoutStates_4backward(__pyx_self, __pyx_v_self, __pyx_v_handle, __pyx_v_dy, __pyx_v_dropout_ratio, __pyx_v_reserve_space);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_13DropoutStates_4backward(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_handle, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, PyObject *__pyx_v_dropout_ratio, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_reserve_space) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dx = 0;
  size_t __pyx_v_dy_desc;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_t_9;
  char const *__pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_RefNannySetupContext("backward", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_dy);

  /* "cupy/cudnn.pyx":529
 *                  core.ndarray reserve_space):
 *         cdef core.ndarray dx
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)             # <<<<<<<<<<<<<<
 * 
 *         dy = core.ascontiguousarray(dy)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_set_dropout_descriptor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_desc_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, __pyx_v_handle, __pyx_v_dropout_ratio};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, __pyx_v_handle, __pyx_v_dropout_ratio};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_handle);
    __Pyx_GIVEREF(__pyx_v_handle);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_handle);
    __Pyx_INCREF(__pyx_v_dropout_ratio);
    __Pyx_GIVEREF(__pyx_v_dropout_ratio);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_dropout_ratio);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":531
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)
 * 
 *         dy = core.ascontiguousarray(dy)             # <<<<<<<<<<<<<<
 *         dx = core.ndarray(dy._shape, dy.dtype)
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_dy, 0, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_dy, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":532
 * 
 *         dy = core.ascontiguousarray(dy)
 *         dx = core.ndarray(dy._shape, dy.dtype)             # <<<<<<<<<<<<<<
 * 
 *         dy_desc = cudnn.createTensorDescriptor()
 */
  __pyx_t_1 = __pyx_convert_vector_to_py_Py_ssize_t(__pyx_v_dy->_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_dy->dtype);
  __Pyx_GIVEREF(__pyx_v_dy->dtype);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_dy->dtype);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_4cupy_4core_4core_ndarray), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dx = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":534
 *         dx = core.ndarray(dy._shape, dy.dtype)
 * 
 *         dy_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *         try:
 *             _create_tensor_descriptor_as4darray(dy_desc, dy)
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 534, __pyx_L1_error)
  __pyx_v_dy_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":535
 * 
 *         dy_desc = cudnn.createTensorDescriptor()
 *         try:             # <<<<<<<<<<<<<<
 *             _create_tensor_descriptor_as4darray(dy_desc, dy)
 *             cudnn.dropoutBackward(handle, self._desc.value,
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":536
 *         dy_desc = cudnn.createTensorDescriptor()
 *         try:
 *             _create_tensor_descriptor_as4darray(dy_desc, dy)             # <<<<<<<<<<<<<<
 *             cudnn.dropoutBackward(handle, self._desc.value,
 *                                   dy_desc, dy.data.ptr,
 */
    __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_descriptor_as4darray(__pyx_v_dy_desc, __pyx_v_dy, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 536, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "cupy/cudnn.pyx":537
 *         try:
 *             _create_tensor_descriptor_as4darray(dy_desc, dy)
 *             cudnn.dropoutBackward(handle, self._desc.value,             # <<<<<<<<<<<<<<
 *                                   dy_desc, dy.data.ptr,
 *                                   dy_desc, dx.data.ptr,
 */
    __pyx_t_7 = __Pyx_PyInt_As_size_t(__pyx_v_handle); if (unlikely((__pyx_t_7 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 537, __pyx_L4_error)
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_desc_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 537, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_8 = __Pyx_PyInt_As_size_t(__pyx_t_2); if (unlikely((__pyx_t_8 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 537, __pyx_L4_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":541
 *                                   dy_desc, dx.data.ptr,
 *                                   reserve_space.data.ptr,
 *                                   reserve_space.size)             # <<<<<<<<<<<<<<
 *         finally:
 *             cudnn.destroyTensorDescriptor(dy_desc)
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_dropoutBackward(__pyx_t_7, __pyx_t_8, __pyx_v_dy_desc, __pyx_v_dy->data->ptr, __pyx_v_dy_desc, __pyx_v_dx->data->ptr, __pyx_v_reserve_space->data->ptr, __pyx_v_reserve_space->size, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "cupy/cudnn.pyx":543
 *                                   reserve_space.size)
 *         finally:
 *             cudnn.destroyTensorDescriptor(dy_desc)             # <<<<<<<<<<<<<<
 *         return dx
 * 
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_dy_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L5;
    }
    __pyx_L4_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_11, &__pyx_t_12, &__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __pyx_t_5 = __pyx_lineno; __pyx_t_9 = __pyx_clineno; __pyx_t_10 = __pyx_filename;
      {
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_dy_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_14);
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      }
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_ErrRestore(__pyx_t_11, __pyx_t_12, __pyx_t_13);
      __pyx_t_11 = 0; __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      __pyx_lineno = __pyx_t_5; __pyx_clineno = __pyx_t_9; __pyx_filename = __pyx_t_10;
      goto __pyx_L1_error;
      __pyx_L7_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_14);
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
      }
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L5:;
  }

  /* "cupy/cudnn.pyx":544
 *         finally:
 *             cudnn.destroyTensorDescriptor(dy_desc)
 *         return dx             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_dx));
  __pyx_r = ((PyObject *)__pyx_v_dx);
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":526
 *         return reserve_space, y
 * 
 *     def backward(self, handle, core.ndarray dy, dropout_ratio,             # <<<<<<<<<<<<<<
 *                  core.ndarray reserve_space):
 *         cdef core.ndarray dx
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn.DropoutStates.backward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dx);
  __Pyx_XDECREF((PyObject *)__pyx_v_dy);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.algo, self.mathType, self.memory)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_10_Algorithm_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_4cupy_5cudnn_10_Algorithm___reduce_cython__[] = "_Algorithm.__reduce_cython__(self)";
static PyObject *__pyx_pw_4cupy_5cudnn_10_Algorithm_1__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_10_Algorithm___reduce_cython__(((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_10_Algorithm___reduce_cython__(struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self.algo, self.mathType, self.memory)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->algo); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->mathType); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->memory); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self.algo, self.mathType, self.memory)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":5
 *     state = (self.algo, self.mathType, self.memory)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = False
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self.algo, self.mathType, self.memory)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = False             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, None), state
 */
  /*else*/ {
    __pyx_v_use_setstate = 0;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":11
 *         use_setstate = False
 *     if use_setstate:
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle__Algorithm); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_74537800);
    __Pyx_GIVEREF(__pyx_int_74537800);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_74537800);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = False
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, None), state
 *     else:
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__Algorithm__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle__Algorithm); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_74537800);
    __Pyx_GIVEREF(__pyx_int_74537800);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_74537800);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self.algo, self.mathType, self.memory)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cudnn._Algorithm.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__Algorithm__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_10_Algorithm_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_4cupy_5cudnn_10_Algorithm_2__setstate_cython__[] = "_Algorithm.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_4cupy_5cudnn_10_Algorithm_3__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_4cupy_5cudnn_10_Algorithm_2__setstate_cython__(((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_10_Algorithm_2__setstate_cython__(struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__Algorithm__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_4cupy_5cudnn___pyx_unpickle__Algorithm__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle__Algorithm, (type(self), 0x4715b48, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__Algorithm__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._Algorithm.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":554
 * 
 * 
 * cdef _Algorithm _get_algorithm(int algo, size_t memory, int mathType=0):             # <<<<<<<<<<<<<<
 *     cdef _Algorithm ret = _Algorithm.__new__(_Algorithm)
 *     ret.algo = algo
 */

static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm(int __pyx_v_algo, size_t __pyx_v_memory, struct __pyx_opt_args_4cupy_5cudnn__get_algorithm *__pyx_optional_args) {
  int __pyx_v_mathType = ((int)0);
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_ret = 0;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_get_algorithm", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_mathType = __pyx_optional_args->mathType;
    }
  }

  /* "cupy/cudnn.pyx":555
 * 
 * cdef _Algorithm _get_algorithm(int algo, size_t memory, int mathType=0):
 *     cdef _Algorithm ret = _Algorithm.__new__(_Algorithm)             # <<<<<<<<<<<<<<
 *     ret.algo = algo
 *     ret.mathType = mathType
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_4cupy_5cudnn__Algorithm(((PyTypeObject *)__pyx_ptype_4cupy_5cudnn__Algorithm), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_ret = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":556
 * cdef _Algorithm _get_algorithm(int algo, size_t memory, int mathType=0):
 *     cdef _Algorithm ret = _Algorithm.__new__(_Algorithm)
 *     ret.algo = algo             # <<<<<<<<<<<<<<
 *     ret.mathType = mathType
 *     ret.memory = memory
 */
  __pyx_v_ret->algo = __pyx_v_algo;

  /* "cupy/cudnn.pyx":557
 *     cdef _Algorithm ret = _Algorithm.__new__(_Algorithm)
 *     ret.algo = algo
 *     ret.mathType = mathType             # <<<<<<<<<<<<<<
 *     ret.memory = memory
 *     return ret
 */
  __pyx_v_ret->mathType = __pyx_v_mathType;

  /* "cupy/cudnn.pyx":558
 *     ret.algo = algo
 *     ret.mathType = mathType
 *     ret.memory = memory             # <<<<<<<<<<<<<<
 *     return ret
 * 
 */
  __pyx_v_ret->memory = __pyx_v_memory;

  /* "cupy/cudnn.pyx":559
 *     ret.mathType = mathType
 *     ret.memory = memory
 *     return ret             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_ret));
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":554
 * 
 * 
 * cdef _Algorithm _get_algorithm(int algo, size_t memory, int mathType=0):             # <<<<<<<<<<<<<<
 *     cdef _Algorithm ret = _Algorithm.__new__(_Algorithm)
 *     ret.algo = algo
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ret);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":567
 * 
 * 
 * cpdef _warn_algorithm_fwd(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param):
 *     warnings.warn(
 */

static PyObject *__pyx_pw_4cupy_5cudnn_63_warn_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__warn_algorithm_fwd(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("_warn_algorithm_fwd", 0);

  /* "cupy/cudnn.pyx":569
 * cpdef _warn_algorithm_fwd(
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param):
 *     warnings.warn(             # <<<<<<<<<<<<<<
 *         'Tensor Core mode is set but the selected convolution forward '
 *         'algorithm is not a Tensor Core enabled algorithm. '
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_warn); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":574
 *         'This might be due to lack of workspace memory. '
 *         'x.shape:{}, W.shape:{}, y.shape:{}, pad:{}, stride:{}'
 *         .format(x.shape, W.shape, y.shape, conv_param[0], conv_param[1]),             # <<<<<<<<<<<<<<
 *         RuntimeWarning)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Tensor_Core_mode_is_set_but_the, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x), __pyx_n_s_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_W), __pyx_n_s_shape); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y), __pyx_n_s_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (unlikely(__pyx_v_conv_param == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 574, __pyx_L1_error)
  }
  __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v_conv_param, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (unlikely(__pyx_v_conv_param == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 574, __pyx_L1_error)
  }
  __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v_conv_param, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 574, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[6] = {__pyx_t_10, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 5+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[6] = {__pyx_t_10, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 5+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_12 = PyTuple_New(5+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_11, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_12, 3+__pyx_t_11, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_12, 4+__pyx_t_11, __pyx_t_9);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":575
 *         'x.shape:{}, W.shape:{}, y.shape:{}, pad:{}, stride:{}'
 *         .format(x.shape, W.shape, y.shape, conv_param[0], conv_param[1]),
 *         RuntimeWarning)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_builtin_RuntimeWarning};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_builtin_RuntimeWarning};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  {
    __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_2);
    __Pyx_INCREF(__pyx_builtin_RuntimeWarning);
    __Pyx_GIVEREF(__pyx_builtin_RuntimeWarning);
    PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_builtin_RuntimeWarning);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":567
 * 
 * 
 * cpdef _warn_algorithm_fwd(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param):
 *     warnings.warn(
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_63_warn_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_62_warn_algorithm_fwd[] = "_warn_algorithm_fwd(ndarray x, ndarray W, ndarray y, tuple conv_param)";
static PyObject *__pyx_pw_4cupy_5cudnn_63_warn_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_conv_param = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_warn_algorithm_fwd (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_W,&__pyx_n_s_y,&__pyx_n_s_conv_param,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_fwd", 1, 4, 4, 1); __PYX_ERR(0, 567, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_fwd", 1, 4, 4, 2); __PYX_ERR(0, 567, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_fwd", 1, 4, 4, 3); __PYX_ERR(0, 567, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_warn_algorithm_fwd") < 0)) __PYX_ERR(0, 567, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_warn_algorithm_fwd", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 567, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 568, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 568, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 568, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 568, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_62_warn_algorithm_fwd(__pyx_self, __pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_62_warn_algorithm_fwd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_warn_algorithm_fwd", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__warn_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 567, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":578
 * 
 * 
 * cpdef _Algorithm _find_algorithm_fwd(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t filter_desc, size_t conv_desc,
 */

static PyObject *__pyx_pw_4cupy_5cudnn_65_find_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__find_algorithm_fwd(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_algo = 0;
  PyObject *__pyx_v_key = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_v_perf = NULL;
  PyObject *__pyx_v_perf_old = NULL;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_t_11;
  struct __pyx_opt_args_4cupy_5cudnn__get_algorithm __pyx_t_12;
  __Pyx_RefNannySetupContext("_find_algorithm_fwd", 0);

  /* "cupy/cudnn.pyx":583
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, x.shape, W.shape, y.shape, conv_param,             # <<<<<<<<<<<<<<
 *            max_workspace_size)
 *     algo = _algorithm_fwd_cache.get(key, None)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x), __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_W), __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y), __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":584
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, x.shape, W.shape, y.shape, conv_param,
 *            max_workspace_size)             # <<<<<<<<<<<<<<
 *     algo = _algorithm_fwd_cache.get(key, None)
 *     if algo is not None:
 */
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "cupy/cudnn.pyx":583
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, x.shape, W.shape, y.shape, conv_param,             # <<<<<<<<<<<<<<
 *            max_workspace_size)
 *     algo = _algorithm_fwd_cache.get(key, None)
 */
  __pyx_t_6 = PyTuple_New(6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 583, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_conv_param);
  __Pyx_GIVEREF(__pyx_v_conv_param);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_conv_param);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_key = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "cupy/cudnn.pyx":585
 *     key = (x.data.device.id, x.shape, W.shape, y.shape, conv_param,
 *            max_workspace_size)
 *     algo = _algorithm_fwd_cache.get(key, None)             # <<<<<<<<<<<<<<
 *     if algo is not None:
 *         return algo
 */
  if (unlikely(__pyx_v_4cupy_5cudnn__algorithm_fwd_cache == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 585, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyDict_GetItemDefault(__pyx_v_4cupy_5cudnn__algorithm_fwd_cache, __pyx_v_key, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 585, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_4cupy_5cudnn__Algorithm))))) __PYX_ERR(0, 585, __pyx_L1_error)
  __pyx_v_algo = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "cupy/cudnn.pyx":586
 *            max_workspace_size)
 *     algo = _algorithm_fwd_cache.get(key, None)
 *     if algo is not None:             # <<<<<<<<<<<<<<
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 */
  __pyx_t_7 = (((PyObject *)__pyx_v_algo) != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "cupy/cudnn.pyx":587
 *     algo = _algorithm_fwd_cache.get(key, None)
 *     if algo is not None:
 *         return algo             # <<<<<<<<<<<<<<
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_algo));
    __pyx_r = __pyx_v_algo;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":586
 *            max_workspace_size)
 *     algo = _algorithm_fwd_cache.get(key, None)
 *     if algo is not None:             # <<<<<<<<<<<<<<
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 */
  }

  /* "cupy/cudnn.pyx":588
 *     if algo is not None:
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)             # <<<<<<<<<<<<<<
 *     if _cudnn_version >= 7000:
 *         perf = cudnn.findConvolutionForwardAlgorithmEx_v7(
 */
  __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_6, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 588, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "cupy/cudnn.pyx":589
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         perf = cudnn.findConvolutionForwardAlgorithmEx_v7(
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 */
  __pyx_t_8 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  if (__pyx_t_8) {

    /* "cupy/cudnn.pyx":590
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:
 *         perf = cudnn.findConvolutionForwardAlgorithmEx_v7(             # <<<<<<<<<<<<<<
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 */
    __pyx_t_5 = __pyx_f_4cupy_4cuda_5cudnn_findConvolutionForwardAlgorithmEx_v7(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_filter_desc, __pyx_v_W->data->ptr, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_y->data->ptr, 1, __pyx_v_workspace->ptr, __pyx_v_max_workspace_size, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "cupy/cudnn.pyx":592
 *         perf = cudnn.findConvolutionForwardAlgorithmEx_v7(
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]             # <<<<<<<<<<<<<<
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_perf = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "cupy/cudnn.pyx":593
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 */
    __pyx_t_7 = (__pyx_v_use_tensor_core != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_8) {

      /* "cupy/cudnn.pyx":594
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_fwd(x, W, y, conv_param)             # <<<<<<<<<<<<<<
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *     else:
 */
      __pyx_t_4 = __pyx_f_4cupy_5cudnn__warn_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":593
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 */
    }

    /* "cupy/cudnn.pyx":595
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)             # <<<<<<<<<<<<<<
 *     else:
 *         perf_old = cudnn.findConvolutionForwardAlgorithmEx(
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_algo); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.mathType = __pyx_t_11;
    __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_9, __pyx_t_10, &__pyx_t_12)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_algo, ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":589
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         perf = cudnn.findConvolutionForwardAlgorithmEx_v7(
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 */
    goto __pyx_L4;
  }

  /* "cupy/cudnn.pyx":597
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *     else:
 *         perf_old = cudnn.findConvolutionForwardAlgorithmEx(             # <<<<<<<<<<<<<<
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":599
 *         perf_old = cudnn.findConvolutionForwardAlgorithmEx(
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]             # <<<<<<<<<<<<<<
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_findConvolutionForwardAlgorithmEx(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_filter_desc, __pyx_v_W->data->ptr, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_y->data->ptr, 1, __pyx_v_workspace->ptr, __pyx_v_max_workspace_size, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 597, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_perf_old = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "cupy/cudnn.pyx":601
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)             # <<<<<<<<<<<<<<
 *     _algorithm_fwd_cache[key] = algo
 *     return algo
 */
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_perf_old, __pyx_n_s_algo); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_perf_old, __pyx_n_s_memory); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_5); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 601, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "cupy/cudnn.pyx":600
 *             handle, x_desc, x.data.ptr, filter_desc, W.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(             # <<<<<<<<<<<<<<
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_fwd_cache[key] = algo
 */
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.mathType = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH;
    __pyx_t_5 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_11, __pyx_t_10, &__pyx_t_12)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_algo, ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_5));
    __pyx_t_5 = 0;
  }
  __pyx_L4:;

  /* "cupy/cudnn.pyx":602
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_fwd_cache[key] = algo             # <<<<<<<<<<<<<<
 *     return algo
 * 
 */
  if (unlikely(__pyx_v_4cupy_5cudnn__algorithm_fwd_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 602, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_v_4cupy_5cudnn__algorithm_fwd_cache, __pyx_v_key, ((PyObject *)__pyx_v_algo)) < 0)) __PYX_ERR(0, 602, __pyx_L1_error)

  /* "cupy/cudnn.pyx":603
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_fwd_cache[key] = algo
 *     return algo             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_algo));
  __pyx_r = __pyx_v_algo;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":578
 * 
 * 
 * cpdef _Algorithm _find_algorithm_fwd(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t filter_desc, size_t conv_desc,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_algo);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF(__pyx_v_perf);
  __Pyx_XDECREF(__pyx_v_perf_old);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_65_find_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_64_find_algorithm_fwd[] = "_find_algorithm_fwd(ndarray x, ndarray W, ndarray y, tuple conv_param, size_t handle, size_t x_desc, size_t filter_desc, size_t conv_desc, size_t y_desc, size_t max_workspace_size, bool use_tensor_core) -> _Algorithm";
static PyObject *__pyx_pw_4cupy_5cudnn_65_find_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_y_desc;
  size_t __pyx_v_max_workspace_size;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_find_algorithm_fwd (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_W,&__pyx_n_s_y,&__pyx_n_s_conv_param,&__pyx_n_s_handle,&__pyx_n_s_x_desc,&__pyx_n_s_filter_desc,&__pyx_n_s_conv_desc,&__pyx_n_s_y_desc,&__pyx_n_s_max_workspace_size,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 1); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 2); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 3); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 4); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 5); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filter_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 6); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 7); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 8); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_workspace_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 9); __PYX_ERR(0, 578, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, 10); __PYX_ERR(0, 578, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_find_algorithm_fwd") < 0)) __PYX_ERR(0, 578, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 11) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
    __pyx_v_handle = __Pyx_PyInt_As_size_t(values[4]); if (unlikely((__pyx_v_handle == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L3_error)
    __pyx_v_x_desc = __Pyx_PyInt_As_size_t(values[5]); if (unlikely((__pyx_v_x_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L3_error)
    __pyx_v_filter_desc = __Pyx_PyInt_As_size_t(values[6]); if (unlikely((__pyx_v_filter_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L3_error)
    __pyx_v_conv_desc = __Pyx_PyInt_As_size_t(values[7]); if (unlikely((__pyx_v_conv_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L3_error)
    __pyx_v_y_desc = __Pyx_PyInt_As_size_t(values[8]); if (unlikely((__pyx_v_y_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 581, __pyx_L3_error)
    __pyx_v_max_workspace_size = __Pyx_PyInt_As_size_t(values[9]); if (unlikely((__pyx_v_max_workspace_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 581, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 581, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_find_algorithm_fwd", 1, 11, 11, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 578, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 579, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 579, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 579, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 579, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_64_find_algorithm_fwd(__pyx_self, __pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_64_find_algorithm_fwd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_find_algorithm_fwd", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5cudnn__find_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 578, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":606
 * 
 * 
 * cpdef _Algorithm _get_algorithm_fwd(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t filter_desc, size_t conv_desc,
 */

static PyObject *__pyx_pw_4cupy_5cudnn_67_get_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm_fwd(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_ret = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_perf = NULL;
  PyObject *__pyx_v_algo = NULL;
  PyObject *__pyx_v_workspace_size = NULL;
  PyObject *__pyx_v_math_type = NULL;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  size_t __pyx_t_13;
  int __pyx_t_14;
  struct __pyx_opt_args_4cupy_5cudnn__get_algorithm __pyx_t_15;
  __Pyx_RefNannySetupContext("_get_algorithm_fwd", 0);

  /* "cupy/cudnn.pyx":611
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         ret = cudnn.getConvolutionForwardAlgorithm_v7(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 */
  __pyx_t_2 = (__pyx_v_use_tensor_core != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":612
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:
 *         ret = cudnn.getConvolutionForwardAlgorithm_v7(             # <<<<<<<<<<<<<<
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):
 */
    __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardAlgorithm_v7(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, 10, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 612, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 612, __pyx_L1_error)
    __pyx_v_ret = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":614
 *         ret = cudnn.getConvolutionForwardAlgorithm_v7(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    __pyx_t_4 = 0;
    __pyx_t_3 = __pyx_v_ret; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    for (;;) {
      if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 614, __pyx_L1_error)
      #else
      __pyx_t_6 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 614, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_perf, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_v_i = __pyx_t_4;
      __pyx_t_4 = (__pyx_t_4 + 1);

      /* "cupy/cudnn.pyx":615
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:             # <<<<<<<<<<<<<<
 *                 break
 *         else:
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 615, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 615, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(__pyx_t_6, __pyx_t_7, Py_LE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 615, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 615, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (__pyx_t_1) {

        /* "cupy/cudnn.pyx":616
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:
 *                 break             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError('No conv fwd algo available with workspace size'
 */
        goto __pyx_L7_break;

        /* "cupy/cudnn.pyx":615
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:             # <<<<<<<<<<<<<<
 *                 break
 *         else:
 */
      }

      /* "cupy/cudnn.pyx":614
 *         ret = cudnn.getConvolutionForwardAlgorithm_v7(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    }
    /*else*/ {

      /* "cupy/cudnn.pyx":619
 *         else:
 *             raise RuntimeError('No conv fwd algo available with workspace size'
 *                                ' less equal {}'.format(max_workspace_size))             # <<<<<<<<<<<<<<
 *         if i != 0:
 *             warnings.warn(
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_No_conv_fwd_algo_available_with, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_9) {
        __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 619, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_8);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 619, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 619, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 619, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 619, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "cupy/cudnn.pyx":618
 *                 break
 *         else:
 *             raise RuntimeError('No conv fwd algo available with workspace size'             # <<<<<<<<<<<<<<
 *                                ' less equal {}'.format(max_workspace_size))
 *         if i != 0:
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_RuntimeError, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 618, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 618, __pyx_L1_error)
    }

    /* "cupy/cudnn.pyx":614
 *         ret = cudnn.getConvolutionForwardAlgorithm_v7(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    __pyx_L7_break:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":620
 *             raise RuntimeError('No conv fwd algo available with workspace size'
 *                                ' less equal {}'.format(max_workspace_size))
 *         if i != 0:             # <<<<<<<<<<<<<<
 *             warnings.warn(
 *                 'The best algo of conv fwd might not be selected due to '
 */
    __pyx_t_1 = ((__pyx_v_i != 0) != 0);
    if (__pyx_t_1) {

      /* "cupy/cudnn.pyx":621
 *                                ' less equal {}'.format(max_workspace_size))
 *         if i != 0:
 *             warnings.warn(             # <<<<<<<<<<<<<<
 *                 'The best algo of conv fwd might not be selected due to '
 *                 'lack of workspace size ({})'.format(max_workspace_size))
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 621, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_warn); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 621, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "cupy/cudnn.pyx":623
 *             warnings.warn(
 *                 'The best algo of conv fwd might not be selected due to '
 *                 'lack of workspace size ({})'.format(max_workspace_size))             # <<<<<<<<<<<<<<
 *         algo = perf.algo
 *         workspace_size = perf.memory
 */
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_The_best_algo_of_conv_fwd_might, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 623, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 623, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      if (!__pyx_t_9) {
        __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 623, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 623, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 623, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 623, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 623, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (!__pyx_t_10) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 621, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 621, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 621, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 621, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_11, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 621, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "cupy/cudnn.pyx":620
 *             raise RuntimeError('No conv fwd algo available with workspace size'
 *                                ' less equal {}'.format(max_workspace_size))
 *         if i != 0:             # <<<<<<<<<<<<<<
 *             warnings.warn(
 *                 'The best algo of conv fwd might not be selected due to '
 */
    }

    /* "cupy/cudnn.pyx":624
 *                 'The best algo of conv fwd might not be selected due to '
 *                 'lack of workspace size ({})'.format(max_workspace_size))
 *         algo = perf.algo             # <<<<<<<<<<<<<<
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_algo); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 624, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_algo = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":625
 *                 'lack of workspace size ({})'.format(max_workspace_size))
 *         algo = perf.algo
 *         workspace_size = perf.memory             # <<<<<<<<<<<<<<
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 625, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_workspace_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":626
 *         algo = perf.algo
 *         workspace_size = perf.memory
 *         math_type = perf.mathType             # <<<<<<<<<<<<<<
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 626, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_math_type = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":627
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *     else:
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = PyObject_RichCompare(__pyx_v_math_type, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 627, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_1) {

      /* "cupy/cudnn.pyx":628
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_fwd(x, W, y, conv_param)             # <<<<<<<<<<<<<<
 *     else:
 *         algo = cudnn.getConvolutionForwardAlgorithm_v6(
 */
      __pyx_t_8 = __pyx_f_4cupy_5cudnn__warn_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 628, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cudnn.pyx":627
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *     else:
 */
    }

    /* "cupy/cudnn.pyx":611
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         ret = cudnn.getConvolutionForwardAlgorithm_v7(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, 10)
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":630
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *     else:
 *         algo = cudnn.getConvolutionForwardAlgorithm_v6(             # <<<<<<<<<<<<<<
 *             handle, x_desc, filter_desc, conv_desc, y_desc,
 *             cudnn.CUDNN_CONVOLUTION_FWD_SPECIFY_WORKSPACE_LIMIT,
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":633
 *             handle, x_desc, filter_desc, conv_desc, y_desc,
 *             cudnn.CUDNN_CONVOLUTION_FWD_SPECIFY_WORKSPACE_LIMIT,
 *             max_workspace_size)             # <<<<<<<<<<<<<<
 *         workspace_size = cudnn.getConvolutionForwardWorkspaceSize(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, algo)
 */
    __pyx_t_12 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardAlgorithm_v6(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_FWD_SPECIFY_WORKSPACE_LIMIT, __pyx_v_max_workspace_size, 0); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 630, __pyx_L1_error)

    /* "cupy/cudnn.pyx":630
 *             _warn_algorithm_fwd(x, W, y, conv_param)
 *     else:
 *         algo = cudnn.getConvolutionForwardAlgorithm_v6(             # <<<<<<<<<<<<<<
 *             handle, x_desc, filter_desc, conv_desc, y_desc,
 *             cudnn.CUDNN_CONVOLUTION_FWD_SPECIFY_WORKSPACE_LIMIT,
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 630, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_algo = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cupy/cudnn.pyx":635
 *             max_workspace_size)
 *         workspace_size = cudnn.getConvolutionForwardWorkspaceSize(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, algo)             # <<<<<<<<<<<<<<
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 *     return _get_algorithm(algo, workspace_size, math_type)
 */
    __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 635, __pyx_L1_error)

    /* "cupy/cudnn.pyx":634
 *             cudnn.CUDNN_CONVOLUTION_FWD_SPECIFY_WORKSPACE_LIMIT,
 *             max_workspace_size)
 *         workspace_size = cudnn.getConvolutionForwardWorkspaceSize(             # <<<<<<<<<<<<<<
 *             handle, x_desc, filter_desc, conv_desc, y_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardWorkspaceSize(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_t_12, 0); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 634, __pyx_L1_error)
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 634, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_workspace_size = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cupy/cudnn.pyx":636
 *         workspace_size = cudnn.getConvolutionForwardWorkspaceSize(
 *             handle, x_desc, filter_desc, conv_desc, y_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH             # <<<<<<<<<<<<<<
 *     return _get_algorithm(algo, workspace_size, math_type)
 * 
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 636, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_math_type = __pyx_t_8;
    __pyx_t_8 = 0;
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":637
 *             handle, x_desc, filter_desc, conv_desc, y_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 *     return _get_algorithm(algo, workspace_size, math_type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 637, __pyx_L1_error)
  __pyx_t_13 = __Pyx_PyInt_As_size_t(__pyx_v_workspace_size); if (unlikely((__pyx_t_13 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 637, __pyx_L1_error)
  __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_v_math_type); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 637, __pyx_L1_error)
  __pyx_t_15.__pyx_n = 1;
  __pyx_t_15.mathType = __pyx_t_14;
  __pyx_t_8 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_12, __pyx_t_13, &__pyx_t_15)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_8);
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":606
 * 
 * 
 * cpdef _Algorithm _get_algorithm_fwd(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t filter_desc, size_t conv_desc,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_perf);
  __Pyx_XDECREF(__pyx_v_algo);
  __Pyx_XDECREF(__pyx_v_workspace_size);
  __Pyx_XDECREF(__pyx_v_math_type);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_67_get_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_66_get_algorithm_fwd[] = "_get_algorithm_fwd(ndarray x, ndarray W, ndarray y, tuple conv_param, size_t handle, size_t x_desc, size_t filter_desc, size_t conv_desc, size_t y_desc, size_t max_workspace_size, bool use_tensor_core) -> _Algorithm";
static PyObject *__pyx_pw_4cupy_5cudnn_67_get_algorithm_fwd(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_y_desc;
  size_t __pyx_v_max_workspace_size;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_algorithm_fwd (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_W,&__pyx_n_s_y,&__pyx_n_s_conv_param,&__pyx_n_s_handle,&__pyx_n_s_x_desc,&__pyx_n_s_filter_desc,&__pyx_n_s_conv_desc,&__pyx_n_s_y_desc,&__pyx_n_s_max_workspace_size,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 1); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 2); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 3); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 4); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 5); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filter_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 6); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 7); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 8); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_workspace_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 9); __PYX_ERR(0, 606, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, 10); __PYX_ERR(0, 606, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_algorithm_fwd") < 0)) __PYX_ERR(0, 606, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 11) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
    __pyx_v_handle = __Pyx_PyInt_As_size_t(values[4]); if (unlikely((__pyx_v_handle == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 608, __pyx_L3_error)
    __pyx_v_x_desc = __Pyx_PyInt_As_size_t(values[5]); if (unlikely((__pyx_v_x_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 608, __pyx_L3_error)
    __pyx_v_filter_desc = __Pyx_PyInt_As_size_t(values[6]); if (unlikely((__pyx_v_filter_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 608, __pyx_L3_error)
    __pyx_v_conv_desc = __Pyx_PyInt_As_size_t(values[7]); if (unlikely((__pyx_v_conv_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 608, __pyx_L3_error)
    __pyx_v_y_desc = __Pyx_PyInt_As_size_t(values[8]); if (unlikely((__pyx_v_y_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 609, __pyx_L3_error)
    __pyx_v_max_workspace_size = __Pyx_PyInt_As_size_t(values[9]); if (unlikely((__pyx_v_max_workspace_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 609, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 609, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_algorithm_fwd", 1, 11, 11, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 606, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 607, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 607, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 607, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 607, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_66_get_algorithm_fwd(__pyx_self, __pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_66_get_algorithm_fwd(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_get_algorithm_fwd", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_fwd", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":640
 * 
 * 
 * cpdef _warn_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param):
 *     warnings.warn(
 */

static PyObject *__pyx_pw_4cupy_5cudnn_69_warn_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__warn_algorithm_bwd_filter(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("_warn_algorithm_bwd_filter", 0);

  /* "cupy/cudnn.pyx":642
 * cpdef _warn_algorithm_bwd_filter(
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param):
 *     warnings.warn(             # <<<<<<<<<<<<<<
 *         'Tensor Core mode is set but the selected convolution backward '
 *         'filter algorithm is not a Tensor Core enabled algorithm. '
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_warn); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":647
 *         'This might be due to lack of workspace memory. '
 *         'x.shape:{}, dy.shape:{}, dW.shape:{}, pad:{}, stride:{}'
 *         .format(x.shape, dy.shape, dW.shape, conv_param[0], conv_param[1]),             # <<<<<<<<<<<<<<
 *         RuntimeWarning)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Tensor_Core_mode_is_set_but_the_2, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x), __pyx_n_s_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dy), __pyx_n_s_shape); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dW), __pyx_n_s_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (unlikely(__pyx_v_conv_param == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 647, __pyx_L1_error)
  }
  __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v_conv_param, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (unlikely(__pyx_v_conv_param == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 647, __pyx_L1_error)
  }
  __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v_conv_param, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[6] = {__pyx_t_10, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 5+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[6] = {__pyx_t_10, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 5+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_12 = PyTuple_New(5+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_11, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_12, 3+__pyx_t_11, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_12, 4+__pyx_t_11, __pyx_t_9);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 647, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":648
 *         'x.shape:{}, dy.shape:{}, dW.shape:{}, pad:{}, stride:{}'
 *         .format(x.shape, dy.shape, dW.shape, conv_param[0], conv_param[1]),
 *         RuntimeWarning)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_builtin_RuntimeWarning};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 642, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_builtin_RuntimeWarning};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 642, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  {
    __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 642, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_2);
    __Pyx_INCREF(__pyx_builtin_RuntimeWarning);
    __Pyx_GIVEREF(__pyx_builtin_RuntimeWarning);
    PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_builtin_RuntimeWarning);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 642, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":640
 * 
 * 
 * cpdef _warn_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param):
 *     warnings.warn(
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_69_warn_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_68_warn_algorithm_bwd_filter[] = "_warn_algorithm_bwd_filter(ndarray x, ndarray dy, ndarray dW, tuple conv_param)";
static PyObject *__pyx_pw_4cupy_5cudnn_69_warn_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW = 0;
  PyObject *__pyx_v_conv_param = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_warn_algorithm_bwd_filter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_dy,&__pyx_n_s_dW,&__pyx_n_s_conv_param,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_filter", 1, 4, 4, 1); __PYX_ERR(0, 640, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dW)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_filter", 1, 4, 4, 2); __PYX_ERR(0, 640, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_filter", 1, 4, 4, 3); __PYX_ERR(0, 640, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_warn_algorithm_bwd_filter") < 0)) __PYX_ERR(0, 640, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_dy = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_dW = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_filter", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 640, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 641, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dy), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dy", 0))) __PYX_ERR(0, 641, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dW), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dW", 0))) __PYX_ERR(0, 641, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 641, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_68_warn_algorithm_bwd_filter(__pyx_self, __pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_68_warn_algorithm_bwd_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_warn_algorithm_bwd_filter", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__warn_algorithm_bwd_filter(__pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":651
 * 
 * 
 * cpdef _Algorithm _find_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t dy_desc, size_t conv_desc,
 */

static PyObject *__pyx_pw_4cupy_5cudnn_71_find_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__find_algorithm_bwd_filter(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_dy_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_algo = 0;
  PyObject *__pyx_v_key = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_v_perf = NULL;
  PyObject *__pyx_v_perf_old = NULL;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_t_11;
  struct __pyx_opt_args_4cupy_5cudnn__get_algorithm __pyx_t_12;
  __Pyx_RefNannySetupContext("_find_algorithm_bwd_filter", 0);

  /* "cupy/cudnn.pyx":656
 *         size_t filter_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, x.shape, dW.shape, dy.shape, conv_param,             # <<<<<<<<<<<<<<
 *            max_workspace_size)
 *     algo = _algorithm_bwd_filter_cache.get(key, None)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x), __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dW), __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dy), __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":657
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, x.shape, dW.shape, dy.shape, conv_param,
 *            max_workspace_size)             # <<<<<<<<<<<<<<
 *     algo = _algorithm_bwd_filter_cache.get(key, None)
 *     if algo is not None:
 */
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 657, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "cupy/cudnn.pyx":656
 *         size_t filter_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, x.shape, dW.shape, dy.shape, conv_param,             # <<<<<<<<<<<<<<
 *            max_workspace_size)
 *     algo = _algorithm_bwd_filter_cache.get(key, None)
 */
  __pyx_t_6 = PyTuple_New(6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_conv_param);
  __Pyx_GIVEREF(__pyx_v_conv_param);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_conv_param);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_key = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "cupy/cudnn.pyx":658
 *     key = (x.data.device.id, x.shape, dW.shape, dy.shape, conv_param,
 *            max_workspace_size)
 *     algo = _algorithm_bwd_filter_cache.get(key, None)             # <<<<<<<<<<<<<<
 *     if algo is not None:
 *         return algo
 */
  if (unlikely(__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 658, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyDict_GetItemDefault(__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache, __pyx_v_key, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_4cupy_5cudnn__Algorithm))))) __PYX_ERR(0, 658, __pyx_L1_error)
  __pyx_v_algo = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "cupy/cudnn.pyx":659
 *            max_workspace_size)
 *     algo = _algorithm_bwd_filter_cache.get(key, None)
 *     if algo is not None:             # <<<<<<<<<<<<<<
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 */
  __pyx_t_7 = (((PyObject *)__pyx_v_algo) != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "cupy/cudnn.pyx":660
 *     algo = _algorithm_bwd_filter_cache.get(key, None)
 *     if algo is not None:
 *         return algo             # <<<<<<<<<<<<<<
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_algo));
    __pyx_r = __pyx_v_algo;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":659
 *            max_workspace_size)
 *     algo = _algorithm_bwd_filter_cache.get(key, None)
 *     if algo is not None:             # <<<<<<<<<<<<<<
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 */
  }

  /* "cupy/cudnn.pyx":661
 *     if algo is not None:
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)             # <<<<<<<<<<<<<<
 *     if _cudnn_version >= 7000:
 *         perf = cudnn.findConvolutionBackwardFilterAlgorithmEx_v7(
 */
  __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_6, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "cupy/cudnn.pyx":662
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         perf = cudnn.findConvolutionBackwardFilterAlgorithmEx_v7(
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 */
  __pyx_t_8 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  if (__pyx_t_8) {

    /* "cupy/cudnn.pyx":663
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:
 *         perf = cudnn.findConvolutionBackwardFilterAlgorithmEx_v7(             # <<<<<<<<<<<<<<
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 */
    __pyx_t_5 = __pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardFilterAlgorithmEx_v7(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_dy_desc, __pyx_v_dy->data->ptr, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_dW->data->ptr, 1, __pyx_v_workspace->ptr, __pyx_v_max_workspace_size, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 663, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "cupy/cudnn.pyx":665
 *         perf = cudnn.findConvolutionBackwardFilterAlgorithmEx_v7(
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]             # <<<<<<<<<<<<<<
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 665, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_perf = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "cupy/cudnn.pyx":666
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)             # <<<<<<<<<<<<<<
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_algo); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_6); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_6); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.mathType = __pyx_t_11;
    __pyx_t_6 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_9, __pyx_t_10, &__pyx_t_12)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF_SET(__pyx_v_algo, ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "cupy/cudnn.pyx":667
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 */
    __pyx_t_7 = (__pyx_v_use_tensor_core != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 667, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 667, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 667, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 667, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_8) {

      /* "cupy/cudnn.pyx":668
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)             # <<<<<<<<<<<<<<
 *     else:
 *         perf_old = cudnn.findConvolutionBackwardFilterAlgorithmEx(
 */
      __pyx_t_4 = __pyx_f_4cupy_5cudnn__warn_algorithm_bwd_filter(__pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 668, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":667
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *         if use_tensor_core and perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 */
    }

    /* "cupy/cudnn.pyx":662
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         perf = cudnn.findConvolutionBackwardFilterAlgorithmEx_v7(
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 */
    goto __pyx_L4;
  }

  /* "cupy/cudnn.pyx":670
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 *         perf_old = cudnn.findConvolutionBackwardFilterAlgorithmEx(             # <<<<<<<<<<<<<<
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":672
 *         perf_old = cudnn.findConvolutionBackwardFilterAlgorithmEx(
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]             # <<<<<<<<<<<<<<
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardFilterAlgorithmEx(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_dy_desc, __pyx_v_dy->data->ptr, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_dW->data->ptr, 1, __pyx_v_workspace->ptr, __pyx_v_max_workspace_size, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 670, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 672, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_perf_old = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "cupy/cudnn.pyx":674
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)             # <<<<<<<<<<<<<<
 *     _algorithm_bwd_filter_cache[key] = algo
 *     return algo
 */
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_perf_old, __pyx_n_s_algo); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_perf_old, __pyx_n_s_memory); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_5); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 674, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "cupy/cudnn.pyx":673
 *             handle, x_desc, x.data.ptr, dy_desc, dy.data.ptr, conv_desc,
 *             filter_desc, dW.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(             # <<<<<<<<<<<<<<
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_bwd_filter_cache[key] = algo
 */
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.mathType = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH;
    __pyx_t_5 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_11, __pyx_t_10, &__pyx_t_12)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 673, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_algo, ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_5));
    __pyx_t_5 = 0;
  }
  __pyx_L4:;

  /* "cupy/cudnn.pyx":675
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_bwd_filter_cache[key] = algo             # <<<<<<<<<<<<<<
 *     return algo
 * 
 */
  if (unlikely(__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 675, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache, __pyx_v_key, ((PyObject *)__pyx_v_algo)) < 0)) __PYX_ERR(0, 675, __pyx_L1_error)

  /* "cupy/cudnn.pyx":676
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_bwd_filter_cache[key] = algo
 *     return algo             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_algo));
  __pyx_r = __pyx_v_algo;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":651
 * 
 * 
 * cpdef _Algorithm _find_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t dy_desc, size_t conv_desc,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_algo);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF(__pyx_v_perf);
  __Pyx_XDECREF(__pyx_v_perf_old);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_71_find_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_70_find_algorithm_bwd_filter[] = "_find_algorithm_bwd_filter(ndarray x, ndarray dy, ndarray dW, tuple conv_param, size_t handle, size_t x_desc, size_t dy_desc, size_t conv_desc, size_t filter_desc, size_t max_workspace_size, bool use_tensor_core) -> _Algorithm";
static PyObject *__pyx_pw_4cupy_5cudnn_71_find_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW = 0;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_dy_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_max_workspace_size;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_find_algorithm_bwd_filter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_dy,&__pyx_n_s_dW,&__pyx_n_s_conv_param,&__pyx_n_s_handle,&__pyx_n_s_x_desc,&__pyx_n_s_dy_desc,&__pyx_n_s_conv_desc,&__pyx_n_s_filter_desc,&__pyx_n_s_max_workspace_size,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 1); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dW)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 2); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 3); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 4); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 5); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dy_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 6); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 7); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filter_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 8); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_workspace_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 9); __PYX_ERR(0, 651, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, 10); __PYX_ERR(0, 651, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_find_algorithm_bwd_filter") < 0)) __PYX_ERR(0, 651, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 11) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_dy = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_dW = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
    __pyx_v_handle = __Pyx_PyInt_As_size_t(values[4]); if (unlikely((__pyx_v_handle == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 653, __pyx_L3_error)
    __pyx_v_x_desc = __Pyx_PyInt_As_size_t(values[5]); if (unlikely((__pyx_v_x_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 653, __pyx_L3_error)
    __pyx_v_dy_desc = __Pyx_PyInt_As_size_t(values[6]); if (unlikely((__pyx_v_dy_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 653, __pyx_L3_error)
    __pyx_v_conv_desc = __Pyx_PyInt_As_size_t(values[7]); if (unlikely((__pyx_v_conv_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 653, __pyx_L3_error)
    __pyx_v_filter_desc = __Pyx_PyInt_As_size_t(values[8]); if (unlikely((__pyx_v_filter_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 654, __pyx_L3_error)
    __pyx_v_max_workspace_size = __Pyx_PyInt_As_size_t(values[9]); if (unlikely((__pyx_v_max_workspace_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 654, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 654, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_filter", 1, 11, 11, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 651, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dy), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dy", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dW), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dW", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 652, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_70_find_algorithm_bwd_filter(__pyx_self, __pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_dy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_70_find_algorithm_bwd_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_dy_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_find_algorithm_bwd_filter", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5cudnn__find_algorithm_bwd_filter(__pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_dy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":679
 * 
 * 
 * cpdef _Algorithm _get_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t gy_desc, size_t conv_desc,
 */

static PyObject *__pyx_pw_4cupy_5cudnn_73_get_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm_bwd_filter(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_gy_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_ret = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_perf = NULL;
  PyObject *__pyx_v_algo = NULL;
  PyObject *__pyx_v_workspace_size = NULL;
  PyObject *__pyx_v_math_type = NULL;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  size_t __pyx_t_13;
  int __pyx_t_14;
  struct __pyx_opt_args_4cupy_5cudnn__get_algorithm __pyx_t_15;
  __Pyx_RefNannySetupContext("_get_algorithm_bwd_filter", 0);

  /* "cupy/cudnn.pyx":684
 *         size_t filter_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         ret = cudnn.getConvolutionBackwardFilterAlgorithm_v7(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 */
  __pyx_t_2 = (__pyx_v_use_tensor_core != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":685
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:
 *         ret = cudnn.getConvolutionBackwardFilterAlgorithm_v7(             # <<<<<<<<<<<<<<
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 *         for i, perf in enumerate(ret):
 */
    __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterAlgorithm_v7(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, 10, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 685, __pyx_L1_error)
    __pyx_v_ret = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":687
 *         ret = cudnn.getConvolutionBackwardFilterAlgorithm_v7(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    __pyx_t_4 = 0;
    __pyx_t_3 = __pyx_v_ret; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    for (;;) {
      if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 687, __pyx_L1_error)
      #else
      __pyx_t_6 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_perf, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_v_i = __pyx_t_4;
      __pyx_t_4 = (__pyx_t_4 + 1);

      /* "cupy/cudnn.pyx":688
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:             # <<<<<<<<<<<<<<
 *                 break
 *         else:
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(__pyx_t_6, __pyx_t_7, Py_LE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 688, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 688, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (__pyx_t_1) {

        /* "cupy/cudnn.pyx":689
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:
 *                 break             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError(
 */
        goto __pyx_L7_break;

        /* "cupy/cudnn.pyx":688
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:             # <<<<<<<<<<<<<<
 *                 break
 *         else:
 */
      }

      /* "cupy/cudnn.pyx":687
 *         ret = cudnn.getConvolutionBackwardFilterAlgorithm_v7(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    }
    /*else*/ {

      /* "cupy/cudnn.pyx":693
 *             raise RuntimeError(
 *                 'No conv bwd filter algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))             # <<<<<<<<<<<<<<
 *         if i != 0:
 *             warnings.warn(
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_No_conv_bwd_filter_algo_availabl, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 693, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 693, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_9) {
        __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 693, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_8);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 693, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 693, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 693, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 693, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "cupy/cudnn.pyx":691
 *                 break
 *         else:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'No conv bwd filter algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_RuntimeError, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 691, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 691, __pyx_L1_error)
    }

    /* "cupy/cudnn.pyx":687
 *         ret = cudnn.getConvolutionBackwardFilterAlgorithm_v7(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    __pyx_L7_break:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":694
 *                 'No conv bwd filter algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))
 *         if i != 0:             # <<<<<<<<<<<<<<
 *             warnings.warn(
 *                 'The best algo of conv bwd filter might not not selected due '
 */
    __pyx_t_1 = ((__pyx_v_i != 0) != 0);
    if (__pyx_t_1) {

      /* "cupy/cudnn.pyx":695
 *                 'equal {}'.format(max_workspace_size))
 *         if i != 0:
 *             warnings.warn(             # <<<<<<<<<<<<<<
 *                 'The best algo of conv bwd filter might not not selected due '
 *                 'to lack of workspace size ({})'.format(max_workspace_size))
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 695, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_warn); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 695, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "cupy/cudnn.pyx":697
 *             warnings.warn(
 *                 'The best algo of conv bwd filter might not not selected due '
 *                 'to lack of workspace size ({})'.format(max_workspace_size))             # <<<<<<<<<<<<<<
 *         algo = perf.algo
 *         workspace_size = perf.memory
 */
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_The_best_algo_of_conv_bwd_filter, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 697, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 697, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      if (!__pyx_t_9) {
        __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 697, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 697, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 697, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 697, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 697, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (!__pyx_t_10) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 695, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_11, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "cupy/cudnn.pyx":694
 *                 'No conv bwd filter algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))
 *         if i != 0:             # <<<<<<<<<<<<<<
 *             warnings.warn(
 *                 'The best algo of conv bwd filter might not not selected due '
 */
    }

    /* "cupy/cudnn.pyx":698
 *                 'The best algo of conv bwd filter might not not selected due '
 *                 'to lack of workspace size ({})'.format(max_workspace_size))
 *         algo = perf.algo             # <<<<<<<<<<<<<<
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_algo); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 698, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_algo = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":699
 *                 'to lack of workspace size ({})'.format(max_workspace_size))
 *         algo = perf.algo
 *         workspace_size = perf.memory             # <<<<<<<<<<<<<<
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_workspace_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":700
 *         algo = perf.algo
 *         workspace_size = perf.memory
 *         math_type = perf.mathType             # <<<<<<<<<<<<<<
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_math_type = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":701
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = PyObject_RichCompare(__pyx_v_math_type, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_1) {

      /* "cupy/cudnn.pyx":702
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)             # <<<<<<<<<<<<<<
 *     else:
 *         algo = cudnn.getConvolutionBackwardFilterAlgorithm_v6(
 */
      __pyx_t_8 = __pyx_f_4cupy_5cudnn__warn_algorithm_bwd_filter(__pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 702, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cudnn.pyx":701
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 */
    }

    /* "cupy/cudnn.pyx":684
 *         size_t filter_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         ret = cudnn.getConvolutionBackwardFilterAlgorithm_v7(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, 10)
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":704
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 *         algo = cudnn.getConvolutionBackwardFilterAlgorithm_v6(             # <<<<<<<<<<<<<<
 *             handle, x_desc, gy_desc, conv_desc, filter_desc,
 *             cudnn.CUDNN_CONVOLUTION_BWD_FILTER_SPECIFY_WORKSPACE_LIMIT,
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":707
 *             handle, x_desc, gy_desc, conv_desc, filter_desc,
 *             cudnn.CUDNN_CONVOLUTION_BWD_FILTER_SPECIFY_WORKSPACE_LIMIT,
 *             max_workspace_size)             # <<<<<<<<<<<<<<
 *         workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 */
    __pyx_t_12 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterAlgorithm_v6(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_SPECIFY_WORKSPACE_LIMIT, __pyx_v_max_workspace_size, 0); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 704, __pyx_L1_error)

    /* "cupy/cudnn.pyx":704
 *             _warn_algorithm_bwd_filter(x, dy, dW, conv_param)
 *     else:
 *         algo = cudnn.getConvolutionBackwardFilterAlgorithm_v6(             # <<<<<<<<<<<<<<
 *             handle, x_desc, gy_desc, conv_desc, filter_desc,
 *             cudnn.CUDNN_CONVOLUTION_BWD_FILTER_SPECIFY_WORKSPACE_LIMIT,
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 704, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_algo = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cupy/cudnn.pyx":709
 *             max_workspace_size)
 *         workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, algo)             # <<<<<<<<<<<<<<
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 *     return _get_algorithm(algo, workspace_size, math_type)
 */
    __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 709, __pyx_L1_error)

    /* "cupy/cudnn.pyx":708
 *             cudnn.CUDNN_CONVOLUTION_BWD_FILTER_SPECIFY_WORKSPACE_LIMIT,
 *             max_workspace_size)
 *         workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(             # <<<<<<<<<<<<<<
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterWorkspaceSize(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_t_12, 0); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 708, __pyx_L1_error)
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 708, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_workspace_size = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cupy/cudnn.pyx":710
 *         workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH             # <<<<<<<<<<<<<<
 *     return _get_algorithm(algo, workspace_size, math_type)
 * 
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 710, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_math_type = __pyx_t_8;
    __pyx_t_8 = 0;
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":711
 *             handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 *     return _get_algorithm(algo, workspace_size, math_type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 711, __pyx_L1_error)
  __pyx_t_13 = __Pyx_PyInt_As_size_t(__pyx_v_workspace_size); if (unlikely((__pyx_t_13 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 711, __pyx_L1_error)
  __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_v_math_type); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 711, __pyx_L1_error)
  __pyx_t_15.__pyx_n = 1;
  __pyx_t_15.mathType = __pyx_t_14;
  __pyx_t_8 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_12, __pyx_t_13, &__pyx_t_15)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_8);
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":679
 * 
 * 
 * cpdef _Algorithm _get_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray dy, core.ndarray dW, tuple conv_param,
 *         size_t handle, size_t x_desc, size_t gy_desc, size_t conv_desc,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_perf);
  __Pyx_XDECREF(__pyx_v_algo);
  __Pyx_XDECREF(__pyx_v_workspace_size);
  __Pyx_XDECREF(__pyx_v_math_type);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_73_get_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_72_get_algorithm_bwd_filter[] = "_get_algorithm_bwd_filter(ndarray x, ndarray dy, ndarray dW, tuple conv_param, size_t handle, size_t x_desc, size_t gy_desc, size_t conv_desc, size_t filter_desc, size_t max_workspace_size, bool use_tensor_core) -> _Algorithm";
static PyObject *__pyx_pw_4cupy_5cudnn_73_get_algorithm_bwd_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW = 0;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_gy_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_max_workspace_size;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_algorithm_bwd_filter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_dy,&__pyx_n_s_dW,&__pyx_n_s_conv_param,&__pyx_n_s_handle,&__pyx_n_s_x_desc,&__pyx_n_s_gy_desc,&__pyx_n_s_conv_desc,&__pyx_n_s_filter_desc,&__pyx_n_s_max_workspace_size,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 1); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dW)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 2); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 3); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 4); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 5); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_gy_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 6); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 7); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filter_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 8); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_workspace_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 9); __PYX_ERR(0, 679, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, 10); __PYX_ERR(0, 679, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_algorithm_bwd_filter") < 0)) __PYX_ERR(0, 679, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 11) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_dy = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_dW = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
    __pyx_v_handle = __Pyx_PyInt_As_size_t(values[4]); if (unlikely((__pyx_v_handle == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 681, __pyx_L3_error)
    __pyx_v_x_desc = __Pyx_PyInt_As_size_t(values[5]); if (unlikely((__pyx_v_x_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 681, __pyx_L3_error)
    __pyx_v_gy_desc = __Pyx_PyInt_As_size_t(values[6]); if (unlikely((__pyx_v_gy_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 681, __pyx_L3_error)
    __pyx_v_conv_desc = __Pyx_PyInt_As_size_t(values[7]); if (unlikely((__pyx_v_conv_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 681, __pyx_L3_error)
    __pyx_v_filter_desc = __Pyx_PyInt_As_size_t(values[8]); if (unlikely((__pyx_v_filter_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L3_error)
    __pyx_v_max_workspace_size = __Pyx_PyInt_As_size_t(values[9]); if (unlikely((__pyx_v_max_workspace_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 682, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_filter", 1, 11, 11, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 679, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dy), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dy", 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dW), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "dW", 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 680, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_72_get_algorithm_bwd_filter(__pyx_self, __pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_72_get_algorithm_bwd_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_dW, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_x_desc, size_t __pyx_v_gy_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_filter_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_get_algorithm_bwd_filter", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm_bwd_filter(__pyx_v_x, __pyx_v_dy, __pyx_v_dW, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_bwd_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":714
 * 
 * 
 * cpdef _warn_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param):
 *     warnings.warn(
 */

static PyObject *__pyx_pw_4cupy_5cudnn_75_warn_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_4cupy_5cudnn__warn_algorithm_bwd_data(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("_warn_algorithm_bwd_data", 0);

  /* "cupy/cudnn.pyx":716
 * cpdef _warn_algorithm_bwd_data(
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param):
 *     warnings.warn(             # <<<<<<<<<<<<<<
 *         'Tensor Core mode is set but the selected convolution backward '
 *         'filter algorithm is not a Tensor Core enabled algorithm. '
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_warn); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":721
 *         'This might be due to lack of workspace memory. '
 *         'W.shape:{}, x.shape:{}, y.shape:{}, pad:{}, stride:{}'
 *         .format(W.shape, x.shape, y.shape, conv_param[0], conv_param[1]),             # <<<<<<<<<<<<<<
 *         RuntimeWarning)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_Tensor_Core_mode_is_set_but_the_3, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_W), __pyx_n_s_shape); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x), __pyx_n_s_shape); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y), __pyx_n_s_shape); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (unlikely(__pyx_v_conv_param == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 721, __pyx_L1_error)
  }
  __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v_conv_param, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (unlikely(__pyx_v_conv_param == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 721, __pyx_L1_error)
  }
  __pyx_t_9 = __Pyx_GetItemInt_Tuple(__pyx_v_conv_param, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[6] = {__pyx_t_10, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 5+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 721, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
    PyObject *__pyx_temp[6] = {__pyx_t_10, __pyx_t_5, __pyx_t_6, __pyx_t_7, __pyx_t_8, __pyx_t_9};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_11, 5+__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 721, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_12 = PyTuple_New(5+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 721, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_12, 2+__pyx_t_11, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_12, 3+__pyx_t_11, __pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_12, 4+__pyx_t_11, __pyx_t_9);
    __pyx_t_5 = 0;
    __pyx_t_6 = 0;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_12, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 721, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":722
 *         'W.shape:{}, x.shape:{}, y.shape:{}, pad:{}, stride:{}'
 *         .format(W.shape, x.shape, y.shape, conv_param[0], conv_param[1]),
 *         RuntimeWarning)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_builtin_RuntimeWarning};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_builtin_RuntimeWarning};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_11, 2+__pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  {
    __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_2);
    __Pyx_INCREF(__pyx_builtin_RuntimeWarning);
    __Pyx_GIVEREF(__pyx_builtin_RuntimeWarning);
    PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_builtin_RuntimeWarning);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":714
 * 
 * 
 * cpdef _warn_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param):
 *     warnings.warn(
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_75_warn_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_74_warn_algorithm_bwd_data[] = "_warn_algorithm_bwd_data(ndarray W, ndarray x, ndarray y, tuple conv_param)";
static PyObject *__pyx_pw_4cupy_5cudnn_75_warn_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_conv_param = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_warn_algorithm_bwd_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_W,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_conv_param,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_data", 1, 4, 4, 1); __PYX_ERR(0, 714, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_data", 1, 4, 4, 2); __PYX_ERR(0, 714, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_data", 1, 4, 4, 3); __PYX_ERR(0, 714, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_warn_algorithm_bwd_data") < 0)) __PYX_ERR(0, 714, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_warn_algorithm_bwd_data", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 714, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 715, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 715, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 715, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 715, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_74_warn_algorithm_bwd_data(__pyx_self, __pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_74_warn_algorithm_bwd_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_warn_algorithm_bwd_data", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__warn_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._warn_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":725
 * 
 * 
 * cpdef _Algorithm _find_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t filter_desc, size_t x_desc, size_t conv_desc,
 */

static PyObject *__pyx_pw_4cupy_5cudnn_77_find_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__find_algorithm_bwd_data(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_filter_desc, size_t __pyx_v_x_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_algo = 0;
  PyObject *__pyx_v_key = NULL;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_v_perf = NULL;
  PyObject *__pyx_v_perf_old = NULL;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_t_11;
  struct __pyx_opt_args_4cupy_5cudnn__get_algorithm __pyx_t_12;
  __Pyx_RefNannySetupContext("_find_algorithm_bwd_data", 0);

  /* "cupy/cudnn.pyx":730
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, W.shape, x.shape, y.shape, conv_param,             # <<<<<<<<<<<<<<
 *            max_workspace_size)
 *     algo = _algorithm_bwd_data_cache.get(key, None)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_W), __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x), __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y), __pyx_n_s_shape); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "cupy/cudnn.pyx":731
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, W.shape, x.shape, y.shape, conv_param,
 *            max_workspace_size)             # <<<<<<<<<<<<<<
 *     algo = _algorithm_bwd_data_cache.get(key, None)
 *     if algo is not None:
 */
  __pyx_t_5 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "cupy/cudnn.pyx":730
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef _Algorithm algo
 *     key = (x.data.device.id, W.shape, x.shape, y.shape, conv_param,             # <<<<<<<<<<<<<<
 *            max_workspace_size)
 *     algo = _algorithm_bwd_data_cache.get(key, None)
 */
  __pyx_t_6 = PyTuple_New(6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 730, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_t_4);
  __Pyx_INCREF(__pyx_v_conv_param);
  __Pyx_GIVEREF(__pyx_v_conv_param);
  PyTuple_SET_ITEM(__pyx_t_6, 4, __pyx_v_conv_param);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 5, __pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_v_key = ((PyObject*)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "cupy/cudnn.pyx":732
 *     key = (x.data.device.id, W.shape, x.shape, y.shape, conv_param,
 *            max_workspace_size)
 *     algo = _algorithm_bwd_data_cache.get(key, None)             # <<<<<<<<<<<<<<
 *     if algo is not None:
 *         return algo
 */
  if (unlikely(__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 732, __pyx_L1_error)
  }
  __pyx_t_6 = __Pyx_PyDict_GetItemDefault(__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache, __pyx_v_key, Py_None); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 732, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_4cupy_5cudnn__Algorithm))))) __PYX_ERR(0, 732, __pyx_L1_error)
  __pyx_v_algo = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_6);
  __pyx_t_6 = 0;

  /* "cupy/cudnn.pyx":733
 *            max_workspace_size)
 *     algo = _algorithm_bwd_data_cache.get(key, None)
 *     if algo is not None:             # <<<<<<<<<<<<<<
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 */
  __pyx_t_7 = (((PyObject *)__pyx_v_algo) != Py_None);
  __pyx_t_8 = (__pyx_t_7 != 0);
  if (__pyx_t_8) {

    /* "cupy/cudnn.pyx":734
 *     algo = _algorithm_bwd_data_cache.get(key, None)
 *     if algo is not None:
 *         return algo             # <<<<<<<<<<<<<<
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_algo));
    __pyx_r = __pyx_v_algo;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":733
 *            max_workspace_size)
 *     algo = _algorithm_bwd_data_cache.get(key, None)
 *     if algo is not None:             # <<<<<<<<<<<<<<
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 */
  }

  /* "cupy/cudnn.pyx":735
 *     if algo is not None:
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)             # <<<<<<<<<<<<<<
 *     if _cudnn_version >= 7000:
 *         perf = cudnn.findConvolutionBackwardDataAlgorithmEx_v7(
 */
  __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_6, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "cupy/cudnn.pyx":736
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         perf = cudnn.findConvolutionBackwardDataAlgorithmEx_v7(
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 */
  __pyx_t_8 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  if (__pyx_t_8) {

    /* "cupy/cudnn.pyx":737
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:
 *         perf = cudnn.findConvolutionBackwardDataAlgorithmEx_v7(             # <<<<<<<<<<<<<<
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 */
    __pyx_t_5 = __pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardDataAlgorithmEx_v7(__pyx_v_handle, __pyx_v_filter_desc, __pyx_v_W->data->ptr, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_y->data->ptr, 1, __pyx_v_workspace->ptr, __pyx_v_max_workspace_size, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 737, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "cupy/cudnn.pyx":739
 *         perf = cudnn.findConvolutionBackwardDataAlgorithmEx_v7(
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]             # <<<<<<<<<<<<<<
 *         if use_tensor_core:
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 */
    __pyx_t_6 = __Pyx_GetItemInt(__pyx_t_5, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 739, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_perf = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "cupy/cudnn.pyx":740
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core:             # <<<<<<<<<<<<<<
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *                 _warn_algorithm_bwd_data(W, x, y, conv_param)
 */
    __pyx_t_8 = (__pyx_v_use_tensor_core != 0);
    if (__pyx_t_8) {

      /* "cupy/cudnn.pyx":741
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core:
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *                 _warn_algorithm_bwd_data(W, x, y, conv_param)
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_6, __pyx_t_5, Py_NE); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 741, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_8) {

        /* "cupy/cudnn.pyx":742
 *         if use_tensor_core:
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *                 _warn_algorithm_bwd_data(W, x, y, conv_param)             # <<<<<<<<<<<<<<
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *     else:
 */
        __pyx_t_4 = __pyx_f_4cupy_5cudnn__warn_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 742, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "cupy/cudnn.pyx":741
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core:
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *                 _warn_algorithm_bwd_data(W, x, y, conv_param)
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 */
      }

      /* "cupy/cudnn.pyx":740
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         if use_tensor_core:             # <<<<<<<<<<<<<<
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *                 _warn_algorithm_bwd_data(W, x, y, conv_param)
 */
    }

    /* "cupy/cudnn.pyx":743
 *             if perf.mathType != cudnn.CUDNN_TENSOR_OP_MATH:
 *                 _warn_algorithm_bwd_data(W, x, y, conv_param)
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)             # <<<<<<<<<<<<<<
 *     else:
 *         perf_old = cudnn.findConvolutionBackwardDataAlgorithmEx(
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_algo); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_9 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_4); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_4); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.mathType = __pyx_t_11;
    __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_9, __pyx_t_10, &__pyx_t_12)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 743, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_algo, ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":736
 *         return algo
 *     workspace = memory.alloc(max_workspace_size)
 *     if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         perf = cudnn.findConvolutionBackwardDataAlgorithmEx_v7(
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 */
    goto __pyx_L4;
  }

  /* "cupy/cudnn.pyx":745
 *         algo = _get_algorithm(perf.algo, perf.memory, perf.mathType)
 *     else:
 *         perf_old = cudnn.findConvolutionBackwardDataAlgorithmEx(             # <<<<<<<<<<<<<<
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":747
 *         perf_old = cudnn.findConvolutionBackwardDataAlgorithmEx(
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]             # <<<<<<<<<<<<<<
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardDataAlgorithmEx(__pyx_v_handle, __pyx_v_filter_desc, __pyx_v_W->data->ptr, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_y->data->ptr, 1, __pyx_v_workspace->ptr, __pyx_v_max_workspace_size, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 745, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_4, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 747, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_perf_old = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "cupy/cudnn.pyx":749
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)             # <<<<<<<<<<<<<<
 *     _algorithm_bwd_data_cache[key] = algo
 *     return algo
 */
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_perf_old, __pyx_n_s_algo); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_11 = __Pyx_PyInt_As_int(__pyx_t_5); if (unlikely((__pyx_t_11 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Dict_GetItem(__pyx_v_perf_old, __pyx_n_s_memory); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_10 = __Pyx_PyInt_As_size_t(__pyx_t_5); if (unlikely((__pyx_t_10 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "cupy/cudnn.pyx":748
 *             handle, filter_desc, W.data.ptr, x_desc, x.data.ptr, conv_desc,
 *             y_desc, y.data.ptr, 1, workspace.ptr, max_workspace_size)[0]
 *         algo = _get_algorithm(             # <<<<<<<<<<<<<<
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_bwd_data_cache[key] = algo
 */
    __pyx_t_12.__pyx_n = 1;
    __pyx_t_12.mathType = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH;
    __pyx_t_5 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_11, __pyx_t_10, &__pyx_t_12)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 748, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF_SET(__pyx_v_algo, ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_5));
    __pyx_t_5 = 0;
  }
  __pyx_L4:;

  /* "cupy/cudnn.pyx":750
 *         algo = _get_algorithm(
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_bwd_data_cache[key] = algo             # <<<<<<<<<<<<<<
 *     return algo
 * 
 */
  if (unlikely(__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 750, __pyx_L1_error)
  }
  if (unlikely(PyDict_SetItem(__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache, __pyx_v_key, ((PyObject *)__pyx_v_algo)) < 0)) __PYX_ERR(0, 750, __pyx_L1_error)

  /* "cupy/cudnn.pyx":751
 *             perf_old['algo'], perf_old['memory'], cudnn.CUDNN_DEFAULT_MATH)
 *     _algorithm_bwd_data_cache[key] = algo
 *     return algo             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_algo));
  __pyx_r = __pyx_v_algo;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":725
 * 
 * 
 * cpdef _Algorithm _find_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t filter_desc, size_t x_desc, size_t conv_desc,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_algo);
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF(__pyx_v_perf);
  __Pyx_XDECREF(__pyx_v_perf_old);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_77_find_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_76_find_algorithm_bwd_data[] = "_find_algorithm_bwd_data(ndarray W, ndarray x, ndarray y, tuple conv_param, size_t handle, size_t filter_desc, size_t x_desc, size_t conv_desc, size_t y_desc, size_t max_workspace_size, bool use_tensor_core) -> _Algorithm";
static PyObject *__pyx_pw_4cupy_5cudnn_77_find_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_y_desc;
  size_t __pyx_v_max_workspace_size;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_find_algorithm_bwd_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_W,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_conv_param,&__pyx_n_s_handle,&__pyx_n_s_filter_desc,&__pyx_n_s_x_desc,&__pyx_n_s_conv_desc,&__pyx_n_s_y_desc,&__pyx_n_s_max_workspace_size,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 1); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 2); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 3); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 4); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filter_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 5); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 6); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 7); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 8); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_workspace_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 9); __PYX_ERR(0, 725, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, 10); __PYX_ERR(0, 725, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_find_algorithm_bwd_data") < 0)) __PYX_ERR(0, 725, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 11) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
    }
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
    __pyx_v_handle = __Pyx_PyInt_As_size_t(values[4]); if (unlikely((__pyx_v_handle == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 727, __pyx_L3_error)
    __pyx_v_filter_desc = __Pyx_PyInt_As_size_t(values[5]); if (unlikely((__pyx_v_filter_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 727, __pyx_L3_error)
    __pyx_v_x_desc = __Pyx_PyInt_As_size_t(values[6]); if (unlikely((__pyx_v_x_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 727, __pyx_L3_error)
    __pyx_v_conv_desc = __Pyx_PyInt_As_size_t(values[7]); if (unlikely((__pyx_v_conv_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 727, __pyx_L3_error)
    __pyx_v_y_desc = __Pyx_PyInt_As_size_t(values[8]); if (unlikely((__pyx_v_y_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 728, __pyx_L3_error)
    __pyx_v_max_workspace_size = __Pyx_PyInt_As_size_t(values[9]); if (unlikely((__pyx_v_max_workspace_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 728, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 728, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_find_algorithm_bwd_data", 1, 11, 11, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 725, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 726, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 726, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 726, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 726, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_76_find_algorithm_bwd_data(__pyx_self, __pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_76_find_algorithm_bwd_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_filter_desc, size_t __pyx_v_x_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_find_algorithm_bwd_data", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5cudnn__find_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 725, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._find_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":754
 * 
 * 
 * cpdef _Algorithm _get_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t filter_desc, size_t x_desc, size_t conv_desc,
 */

static PyObject *__pyx_pw_4cupy_5cudnn_79_get_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_f_4cupy_5cudnn__get_algorithm_bwd_data(struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_filter_desc, size_t __pyx_v_x_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_ret = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_v_perf = NULL;
  PyObject *__pyx_v_algo = NULL;
  PyObject *__pyx_v_workspace_size = NULL;
  PyObject *__pyx_v_math_type = NULL;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  size_t __pyx_t_13;
  int __pyx_t_14;
  struct __pyx_opt_args_4cupy_5cudnn__get_algorithm __pyx_t_15;
  __Pyx_RefNannySetupContext("_get_algorithm_bwd_data", 0);

  /* "cupy/cudnn.pyx":759
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         ret = cudnn.getConvolutionBackwardDataAlgorithm_v7(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 */
  __pyx_t_2 = (__pyx_v_use_tensor_core != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":760
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:
 *         ret = cudnn.getConvolutionBackwardDataAlgorithm_v7(             # <<<<<<<<<<<<<<
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):
 */
    __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataAlgorithm_v7(__pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, 10, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 760, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyList_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 760, __pyx_L1_error)
    __pyx_v_ret = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":762
 *         ret = cudnn.getConvolutionBackwardDataAlgorithm_v7(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    __pyx_t_4 = 0;
    __pyx_t_3 = __pyx_v_ret; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    for (;;) {
      if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_6 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_6); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 762, __pyx_L1_error)
      #else
      __pyx_t_6 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 762, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_perf, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_v_i = __pyx_t_4;
      __pyx_t_4 = (__pyx_t_4 + 1);

      /* "cupy/cudnn.pyx":763
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:             # <<<<<<<<<<<<<<
 *                 break
 *         else:
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(__pyx_t_6, __pyx_t_7, Py_LE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 763, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (__pyx_t_1) {

        /* "cupy/cudnn.pyx":764
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:
 *                 break             # <<<<<<<<<<<<<<
 *         else:
 *             raise RuntimeError(
 */
        goto __pyx_L7_break;

        /* "cupy/cudnn.pyx":763
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):
 *             if perf.memory <= max_workspace_size:             # <<<<<<<<<<<<<<
 *                 break
 *         else:
 */
      }

      /* "cupy/cudnn.pyx":762
 *         ret = cudnn.getConvolutionBackwardDataAlgorithm_v7(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    }
    /*else*/ {

      /* "cupy/cudnn.pyx":768
 *             raise RuntimeError(
 *                 'No conv bwd data algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))             # <<<<<<<<<<<<<<
 *         if i != 0:
 *             warnings.warn(
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_No_conv_bwd_data_algo_available, __pyx_n_s_format); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 768, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 768, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_9) {
        __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 768, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_8);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_8 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 768, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_8 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 768, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 768, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 768, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "cupy/cudnn.pyx":766
 *                 break
 *         else:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 'No conv bwd data algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))
 */
      __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_builtin_RuntimeError, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 766, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_Raise(__pyx_t_7, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __PYX_ERR(0, 766, __pyx_L1_error)
    }

    /* "cupy/cudnn.pyx":762
 *         ret = cudnn.getConvolutionBackwardDataAlgorithm_v7(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 *         for i, perf in enumerate(ret):             # <<<<<<<<<<<<<<
 *             if perf.memory <= max_workspace_size:
 *                 break
 */
    __pyx_L7_break:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":769
 *                 'No conv bwd data algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))
 *         if i != 0:             # <<<<<<<<<<<<<<
 *             warnings.warn(
 *                 'The best algo of conv bwd data might not not selected due '
 */
    __pyx_t_1 = ((__pyx_v_i != 0) != 0);
    if (__pyx_t_1) {

      /* "cupy/cudnn.pyx":770
 *                 'equal {}'.format(max_workspace_size))
 *         if i != 0:
 *             warnings.warn(             # <<<<<<<<<<<<<<
 *                 'The best algo of conv bwd data might not not selected due '
 *                 'to lack of workspace size ({})'.format(max_workspace_size))
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_warnings); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 770, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_warn); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 770, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "cupy/cudnn.pyx":772
 *             warnings.warn(
 *                 'The best algo of conv bwd data might not not selected due '
 *                 'to lack of workspace size ({})'.format(max_workspace_size))             # <<<<<<<<<<<<<<
 *         algo = perf.algo
 *         workspace_size = perf.memory
 */
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_The_best_algo_of_conv_bwd_data_m, __pyx_n_s_format); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 772, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_6 = __Pyx_PyInt_FromSize_t(__pyx_v_max_workspace_size); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 772, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_10, function);
        }
      }
      if (!__pyx_t_9) {
        __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 772, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_7);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 772, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_6};
          __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 772, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 772, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
          __Pyx_GIVEREF(__pyx_t_6);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
          __pyx_t_6 = 0;
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 772, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_10)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (!__pyx_t_10) {
        __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 770, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 770, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
          PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_7};
          __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 770, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        } else
        #endif
        {
          __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 770, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
          __Pyx_GIVEREF(__pyx_t_7);
          PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_7);
          __pyx_t_7 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_11, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 770, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "cupy/cudnn.pyx":769
 *                 'No conv bwd data algo available with workspace size less '
 *                 'equal {}'.format(max_workspace_size))
 *         if i != 0:             # <<<<<<<<<<<<<<
 *             warnings.warn(
 *                 'The best algo of conv bwd data might not not selected due '
 */
    }

    /* "cupy/cudnn.pyx":773
 *                 'The best algo of conv bwd data might not not selected due '
 *                 'to lack of workspace size ({})'.format(max_workspace_size))
 *         algo = perf.algo             # <<<<<<<<<<<<<<
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_algo); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 773, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_algo = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":774
 *                 'to lack of workspace size ({})'.format(max_workspace_size))
 *         algo = perf.algo
 *         workspace_size = perf.memory             # <<<<<<<<<<<<<<
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_memory); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 774, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_workspace_size = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":775
 *         algo = perf.algo
 *         workspace_size = perf.memory
 *         math_type = perf.mathType             # <<<<<<<<<<<<<<
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_bwd_data(W, x, y, conv_param)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_perf, __pyx_n_s_mathType); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 775, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_math_type = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cupy/cudnn.pyx":776
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_bwd_data(W, x, y, conv_param)
 *     else:
 */
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_OP_MATH); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = PyObject_RichCompare(__pyx_v_math_type, __pyx_t_3, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 776, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (__pyx_t_1) {

      /* "cupy/cudnn.pyx":777
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:
 *             _warn_algorithm_bwd_data(W, x, y, conv_param)             # <<<<<<<<<<<<<<
 *     else:
 *         algo = cudnn.getConvolutionBackwardDataAlgorithm_v6(
 */
      __pyx_t_8 = __pyx_f_4cupy_5cudnn__warn_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 777, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "cupy/cudnn.pyx":776
 *         workspace_size = perf.memory
 *         math_type = perf.mathType
 *         if math_type != cudnn.CUDNN_TENSOR_OP_MATH:             # <<<<<<<<<<<<<<
 *             _warn_algorithm_bwd_data(W, x, y, conv_param)
 *     else:
 */
    }

    /* "cupy/cudnn.pyx":759
 *         size_t y_desc, size_t max_workspace_size, bint use_tensor_core):
 *     cdef list ret
 *     if use_tensor_core and _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *         ret = cudnn.getConvolutionBackwardDataAlgorithm_v7(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, 10)
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":779
 *             _warn_algorithm_bwd_data(W, x, y, conv_param)
 *     else:
 *         algo = cudnn.getConvolutionBackwardDataAlgorithm_v6(             # <<<<<<<<<<<<<<
 *             handle, filter_desc, x_desc, conv_desc, y_desc,
 *             cudnn.CUDNN_CONVOLUTION_BWD_DATA_SPECIFY_WORKSPACE_LIMIT,
 */
  /*else*/ {

    /* "cupy/cudnn.pyx":782
 *             handle, filter_desc, x_desc, conv_desc, y_desc,
 *             cudnn.CUDNN_CONVOLUTION_BWD_DATA_SPECIFY_WORKSPACE_LIMIT,
 *             max_workspace_size)             # <<<<<<<<<<<<<<
 *         workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 */
    __pyx_t_12 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataAlgorithm_v6(__pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_SPECIFY_WORKSPACE_LIMIT, __pyx_v_max_workspace_size, 0); if (unlikely(__pyx_t_12 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 779, __pyx_L1_error)

    /* "cupy/cudnn.pyx":779
 *             _warn_algorithm_bwd_data(W, x, y, conv_param)
 *     else:
 *         algo = cudnn.getConvolutionBackwardDataAlgorithm_v6(             # <<<<<<<<<<<<<<
 *             handle, filter_desc, x_desc, conv_desc, y_desc,
 *             cudnn.CUDNN_CONVOLUTION_BWD_DATA_SPECIFY_WORKSPACE_LIMIT,
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_t_12); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 779, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_algo = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cupy/cudnn.pyx":784
 *             max_workspace_size)
 *         workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, algo)             # <<<<<<<<<<<<<<
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 *     return _get_algorithm(algo, workspace_size, math_type)
 */
    __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 784, __pyx_L1_error)

    /* "cupy/cudnn.pyx":783
 *             cudnn.CUDNN_CONVOLUTION_BWD_DATA_SPECIFY_WORKSPACE_LIMIT,
 *             max_workspace_size)
 *         workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(             # <<<<<<<<<<<<<<
 *             handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 */
    __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataWorkspaceSize(__pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_t_12, 0); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 783, __pyx_L1_error)
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 783, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_workspace_size = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "cupy/cudnn.pyx":785
 *         workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 *             handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH             # <<<<<<<<<<<<<<
 *     return _get_algorithm(algo, workspace_size, math_type)
 * 
 */
    __pyx_t_8 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 785, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_v_math_type = __pyx_t_8;
    __pyx_t_8 = 0;
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":786
 *             handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 *         math_type = cudnn.CUDNN_DEFAULT_MATH
 *     return _get_algorithm(algo, workspace_size, math_type)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_12 = __Pyx_PyInt_As_int(__pyx_v_algo); if (unlikely((__pyx_t_12 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 786, __pyx_L1_error)
  __pyx_t_13 = __Pyx_PyInt_As_size_t(__pyx_v_workspace_size); if (unlikely((__pyx_t_13 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 786, __pyx_L1_error)
  __pyx_t_14 = __Pyx_PyInt_As_int(__pyx_v_math_type); if (unlikely((__pyx_t_14 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 786, __pyx_L1_error)
  __pyx_t_15.__pyx_n = 1;
  __pyx_t_15.mathType = __pyx_t_14;
  __pyx_t_8 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm(__pyx_t_12, __pyx_t_13, &__pyx_t_15)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 786, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_8);
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "cupy/cudnn.pyx":754
 * 
 * 
 * cpdef _Algorithm _get_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray y, tuple conv_param,
 *         size_t handle, size_t filter_desc, size_t x_desc, size_t conv_desc,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_perf);
  __Pyx_XDECREF(__pyx_v_algo);
  __Pyx_XDECREF(__pyx_v_workspace_size);
  __Pyx_XDECREF(__pyx_v_math_type);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_79_get_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_78_get_algorithm_bwd_data[] = "_get_algorithm_bwd_data(ndarray W, ndarray x, ndarray y, tuple conv_param, size_t handle, size_t filter_desc, size_t x_desc, size_t conv_desc, size_t y_desc, size_t max_workspace_size, bool use_tensor_core) -> _Algorithm";
static PyObject *__pyx_pw_4cupy_5cudnn_79_get_algorithm_bwd_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_y_desc;
  size_t __pyx_v_max_workspace_size;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_algorithm_bwd_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_W,&__pyx_n_s_x,&__pyx_n_s_y,&__pyx_n_s_conv_param,&__pyx_n_s_handle,&__pyx_n_s_filter_desc,&__pyx_n_s_x_desc,&__pyx_n_s_conv_desc,&__pyx_n_s_y_desc,&__pyx_n_s_max_workspace_size,&__pyx_n_s_use_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case 11: values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
        CYTHON_FALLTHROUGH;
        case 10: values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
        CYTHON_FALLTHROUGH;
        case  9: values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
        CYTHON_FALLTHROUGH;
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 1); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 2); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_param)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 3); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_handle)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 4); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_filter_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 5); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 6); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_conv_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 7); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y_desc)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 8); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  9:
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_workspace_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 9); __PYX_ERR(0, 754, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case 10:
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, 10); __PYX_ERR(0, 754, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_get_algorithm_bwd_data") < 0)) __PYX_ERR(0, 754, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 11) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
      values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
      values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
      values[8] = PyTuple_GET_ITEM(__pyx_args, 8);
      values[9] = PyTuple_GET_ITEM(__pyx_args, 9);
      values[10] = PyTuple_GET_ITEM(__pyx_args, 10);
    }
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_conv_param = ((PyObject*)values[3]);
    __pyx_v_handle = __Pyx_PyInt_As_size_t(values[4]); if (unlikely((__pyx_v_handle == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 756, __pyx_L3_error)
    __pyx_v_filter_desc = __Pyx_PyInt_As_size_t(values[5]); if (unlikely((__pyx_v_filter_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 756, __pyx_L3_error)
    __pyx_v_x_desc = __Pyx_PyInt_As_size_t(values[6]); if (unlikely((__pyx_v_x_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 756, __pyx_L3_error)
    __pyx_v_conv_desc = __Pyx_PyInt_As_size_t(values[7]); if (unlikely((__pyx_v_conv_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 756, __pyx_L3_error)
    __pyx_v_y_desc = __Pyx_PyInt_As_size_t(values[8]); if (unlikely((__pyx_v_y_desc == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 757, __pyx_L3_error)
    __pyx_v_max_workspace_size = __Pyx_PyInt_As_size_t(values[9]); if (unlikely((__pyx_v_max_workspace_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 757, __pyx_L3_error)
    __pyx_v_use_tensor_core = __Pyx_PyObject_IsTrue(values[10]); if (unlikely((__pyx_v_use_tensor_core == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 757, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_get_algorithm_bwd_data", 1, 11, 11, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 754, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 755, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 755, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 755, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_conv_param), (&PyTuple_Type), 1, "conv_param", 1))) __PYX_ERR(0, 755, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_78_get_algorithm_bwd_data(__pyx_self, __pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_78_get_algorithm_bwd_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_conv_param, size_t __pyx_v_handle, size_t __pyx_v_filter_desc, size_t __pyx_v_x_desc, size_t __pyx_v_conv_desc, size_t __pyx_v_y_desc, size_t __pyx_v_max_workspace_size, int __pyx_v_use_tensor_core) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("_get_algorithm_bwd_data", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 754, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("cupy.cudnn._get_algorithm_bwd_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":789
 * 
 * 
 * cpdef bint _should_use_tensor_core(             # <<<<<<<<<<<<<<
 *         str tensor_core_mode, object dtype) except *:
 *     if tensor_core_mode == 'auto':
 */

static PyObject *__pyx_pw_4cupy_5cudnn_81_should_use_tensor_core(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_f_4cupy_5cudnn__should_use_tensor_core(PyObject *__pyx_v_tensor_core_mode, PyObject *__pyx_v_dtype, CYTHON_UNUSED int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("_should_use_tensor_core", 0);

  /* "cupy/cudnn.pyx":791
 * cpdef bint _should_use_tensor_core(
 *         str tensor_core_mode, object dtype) except *:
 *     if tensor_core_mode == 'auto':             # <<<<<<<<<<<<<<
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'always':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_tensor_core_mode, __pyx_n_s_auto, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 791, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "cupy/cudnn.pyx":792
 *         str tensor_core_mode, object dtype) except *:
 *     if tensor_core_mode == 'auto':
 *         return is_tensor_core_available(dtype)             # <<<<<<<<<<<<<<
 *     elif tensor_core_mode == 'always':
 *         # TODO(oktua): more strict condition
 */
    __pyx_t_2 = __pyx_f_4cupy_5cudnn_is_tensor_core_available(__pyx_v_dtype, 0); if (unlikely(__pyx_t_2 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 792, __pyx_L1_error)
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":791
 * cpdef bint _should_use_tensor_core(
 *         str tensor_core_mode, object dtype) except *:
 *     if tensor_core_mode == 'auto':             # <<<<<<<<<<<<<<
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'always':
 */
  }

  /* "cupy/cudnn.pyx":793
 *     if tensor_core_mode == 'auto':
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'always':             # <<<<<<<<<<<<<<
 *         # TODO(oktua): more strict condition
 *         return is_tensor_core_available(dtype)
 */
  __pyx_t_2 = (__Pyx_PyString_Equals(__pyx_v_tensor_core_mode, __pyx_n_s_always, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 793, __pyx_L1_error)
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "cupy/cudnn.pyx":795
 *     elif tensor_core_mode == 'always':
 *         # TODO(oktua): more strict condition
 *         return is_tensor_core_available(dtype)             # <<<<<<<<<<<<<<
 *     elif tensor_core_mode == 'never':
 *         return False
 */
    __pyx_t_1 = __pyx_f_4cupy_5cudnn_is_tensor_core_available(__pyx_v_dtype, 0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 795, __pyx_L1_error)
    __pyx_r = __pyx_t_1;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":793
 *     if tensor_core_mode == 'auto':
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'always':             # <<<<<<<<<<<<<<
 *         # TODO(oktua): more strict condition
 *         return is_tensor_core_available(dtype)
 */
  }

  /* "cupy/cudnn.pyx":796
 *         # TODO(oktua): more strict condition
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'never':             # <<<<<<<<<<<<<<
 *         return False
 *     else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_tensor_core_mode, __pyx_n_s_never, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 796, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (likely(__pyx_t_2)) {

    /* "cupy/cudnn.pyx":797
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'never':
 *         return False             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError(
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "cupy/cudnn.pyx":796
 *         # TODO(oktua): more strict condition
 *         return is_tensor_core_available(dtype)
 *     elif tensor_core_mode == 'never':             # <<<<<<<<<<<<<<
 *         return False
 *     else:
 */
  }

  /* "cupy/cudnn.pyx":799
 *         return False
 *     else:
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             'tensor_code_mode must be either of "always", "auto", or "never".')
 * 
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 799, __pyx_L1_error)
  }

  /* "cupy/cudnn.pyx":789
 * 
 * 
 * cpdef bint _should_use_tensor_core(             # <<<<<<<<<<<<<<
 *         str tensor_core_mode, object dtype) except *:
 *     if tensor_core_mode == 'auto':
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cupy.cudnn._should_use_tensor_core", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_81_should_use_tensor_core(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_80_should_use_tensor_core[] = "_should_use_tensor_core(str tensor_core_mode, dtype) -> bool";
static PyObject *__pyx_pw_4cupy_5cudnn_81_should_use_tensor_core(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_tensor_core_mode = 0;
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_should_use_tensor_core (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_tensor_core_mode,&__pyx_n_s_dtype,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tensor_core_mode)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dtype)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_should_use_tensor_core", 1, 2, 2, 1); __PYX_ERR(0, 789, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_should_use_tensor_core") < 0)) __PYX_ERR(0, 789, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_tensor_core_mode = ((PyObject*)values[0]);
    __pyx_v_dtype = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_should_use_tensor_core", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 789, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn._should_use_tensor_core", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tensor_core_mode), (&PyString_Type), 1, "tensor_core_mode", 1))) __PYX_ERR(0, 790, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_80_should_use_tensor_core(__pyx_self, __pyx_v_tensor_core_mode, __pyx_v_dtype);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_80_should_use_tensor_core(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tensor_core_mode, PyObject *__pyx_v_dtype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("_should_use_tensor_core", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_4cupy_5cudnn__should_use_tensor_core(__pyx_v_tensor_core_mode, __pyx_v_dtype, 0); if (unlikely(__pyx_t_1 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 789, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 789, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cupy.cudnn._should_use_tensor_core", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":803
 * 
 * 
 * def convolution_forward(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_83convolution_forward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_82convolution_forward[] = "convolution_forward(ndarray x, ndarray W, ndarray b, ndarray y, tuple pad, tuple stride, tuple dilation, int groups, *, bool auto_tune, str tensor_core)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_83convolution_forward = {"convolution_forward", (PyCFunction)__pyx_pw_4cupy_5cudnn_83convolution_forward, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_82convolution_forward};
static PyObject *__pyx_pw_4cupy_5cudnn_83convolution_forward(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_b = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_pad = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_dilation = 0;
  int __pyx_v_groups;
  int __pyx_v_auto_tune;
  PyObject *__pyx_v_tensor_core = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convolution_forward (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_W,&__pyx_n_s_b,&__pyx_n_s_y,&__pyx_n_s_pad,&__pyx_n_s_stride,&__pyx_n_s_dilation,&__pyx_n_s_groups,&__pyx_n_s_auto_tune,&__pyx_n_s_tensor_core,0};
    PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
    if (likely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 1); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 2); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 3); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pad)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 4); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 5); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dilation)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 6); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, 7); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_auto_tune)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_forward", __pyx_n_s_auto_tune); __PYX_ERR(0, 803, __pyx_L3_error)
        }
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_forward", __pyx_n_s_tensor_core); __PYX_ERR(0, 803, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "convolution_forward") < 0)) __PYX_ERR(0, 803, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("convolution_forward", __pyx_n_s_auto_tune); __PYX_ERR(0, 803, __pyx_L3_error)
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_b = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[3]);
    __pyx_v_pad = ((PyObject*)values[4]);
    __pyx_v_stride = ((PyObject*)values[5]);
    __pyx_v_dilation = ((PyObject*)values[6]);
    __pyx_v_groups = __Pyx_PyInt_As_int(values[7]); if (unlikely((__pyx_v_groups == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 805, __pyx_L3_error)
    __pyx_v_auto_tune = __Pyx_PyObject_IsTrue(values[8]); if (unlikely((__pyx_v_auto_tune == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 806, __pyx_L3_error)
    __pyx_v_tensor_core = ((PyObject*)values[9]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("convolution_forward", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 803, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.convolution_forward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 804, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 804, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "b", 0))) __PYX_ERR(0, 804, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 804, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_pad), (&PyTuple_Type), 1, "pad", 1))) __PYX_ERR(0, 805, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stride), (&PyTuple_Type), 1, "stride", 1))) __PYX_ERR(0, 805, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dilation), (&PyTuple_Type), 1, "dilation", 1))) __PYX_ERR(0, 805, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tensor_core), (&PyString_Type), 1, "tensor_core", 1))) __PYX_ERR(0, 806, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_82convolution_forward(__pyx_self, __pyx_v_x, __pyx_v_W, __pyx_v_b, __pyx_v_y, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_v_auto_tune, __pyx_v_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_82convolution_forward(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_b, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, int __pyx_v_auto_tune, PyObject *__pyx_v_tensor_core) {
  int __pyx_v_dev_id;
  float __pyx_v_float_zero;
  float __pyx_v_float_one;
  double __pyx_v_double_zero;
  double __pyx_v_double_one;
  size_t __pyx_v_zero;
  size_t __pyx_v_one;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_v_conv_param = 0;
  PyObject *__pyx_v_i = NULL;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_y_desc;
  size_t __pyx_v_b_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_conv_desc;
  size_t __pyx_v_max_workspace_size;
  std::vector<Py_ssize_t>  __pyx_v_b_shape;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_perf = 0;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  size_t __pyx_t_7;
  struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor __pyx_t_8;
  struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  __Pyx_RefNannySetupContext("convolution_forward", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_x);
  __Pyx_INCREF((PyObject *)__pyx_v_W);
  __Pyx_INCREF((PyObject *)__pyx_v_b);

  /* "cupy/cudnn.pyx":807
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 *         bint auto_tune, str tensor_core):
 *     cdef int dev_id = x.data.device.id             # <<<<<<<<<<<<<<
 *     assert dev_id == W.data.device.id
 *     assert dev_id == y.data.device.id
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dev_id = __pyx_t_3;

  /* "cupy/cudnn.pyx":808
 *         bint auto_tune, str tensor_core):
 *     cdef int dev_id = x.data.device.id
 *     assert dev_id == W.data.device.id             # <<<<<<<<<<<<<<
 *     assert dev_id == y.data.device.id
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_W->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 808, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 808, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":809
 *     cdef int dev_id = x.data.device.id
 *     assert dev_id == W.data.device.id
 *     assert dev_id == y.data.device.id             # <<<<<<<<<<<<<<
 * 
 *     cdef float float_zero = 0, float_one = 1
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y->data), __pyx_n_s_device); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 809, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 809, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":811
 *     assert dev_id == y.data.device.id
 * 
 *     cdef float float_zero = 0, float_one = 1             # <<<<<<<<<<<<<<
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 */
  __pyx_v_float_zero = 0.0;
  __pyx_v_float_one = 1.0;

  /* "cupy/cudnn.pyx":812
 * 
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1             # <<<<<<<<<<<<<<
 *     cdef size_t zero, one
 *     if x.dtype == 'd':
 */
  __pyx_v_double_zero = 0.0;
  __pyx_v_double_one = 1.0;

  /* "cupy/cudnn.pyx":814
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
  __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_x->dtype, __pyx_n_s_d, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 814, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupy/cudnn.pyx":815
 *     cdef size_t zero, one
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&double_one
 *     else:
 */
    __pyx_v_zero = ((size_t)(&__pyx_v_double_zero));

    /* "cupy/cudnn.pyx":816
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one             # <<<<<<<<<<<<<<
 *     else:
 *         zero = <size_t>&float_zero
 */
    __pyx_v_one = ((size_t)(&__pyx_v_double_one));

    /* "cupy/cudnn.pyx":814
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":818
 *         one = <size_t>&double_one
 *     else:
 *         zero = <size_t>&float_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&float_one
 * 
 */
  /*else*/ {
    __pyx_v_zero = ((size_t)(&__pyx_v_float_zero));

    /* "cupy/cudnn.pyx":819
 *     else:
 *         zero = <size_t>&float_zero
 *         one = <size_t>&float_one             # <<<<<<<<<<<<<<
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)
 */
    __pyx_v_one = ((size_t)(&__pyx_v_float_one));
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":821
 *         one = <size_t>&float_one
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)             # <<<<<<<<<<<<<<
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)
 * 
 */
  __pyx_t_4 = __pyx_v_x->dtype;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = __pyx_f_4cupy_5cudnn__should_use_tensor_core(__pyx_v_tensor_core, __pyx_t_4, 0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_use_tensor_core = __pyx_t_5;

  /* "cupy/cudnn.pyx":822
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)             # <<<<<<<<<<<<<<
 * 
 *     # cuDNN 7 supports dilation only in *_FWD_ALGO_IMPLICIT_GEMM, but
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_use_tensor_core); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 822, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_pad);
  __Pyx_GIVEREF(__pyx_v_pad);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_pad);
  __Pyx_INCREF(__pyx_v_stride);
  __Pyx_GIVEREF(__pyx_v_stride);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_stride);
  __Pyx_INCREF(__pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_v_x->dtype);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_conv_param = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":826
 *     # cuDNN 7 supports dilation only in *_FWD_ALGO_IMPLICIT_GEMM, but
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:             # <<<<<<<<<<<<<<
 *         for i in dilation:
 *             if i > 1:
 */
  __pyx_t_5 = (__pyx_v_use_tensor_core != 0);
  if (__pyx_t_5) {

    /* "cupy/cudnn.pyx":827
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:
 *         for i in dilation:             # <<<<<<<<<<<<<<
 *             if i > 1:
 *                 use_tensor_core = False
 */
    if (unlikely(__pyx_v_dilation == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 827, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_v_dilation; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 827, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 827, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":828
 *     if use_tensor_core:
 *         for i in dilation:
 *             if i > 1:             # <<<<<<<<<<<<<<
 *                 use_tensor_core = False
 *                 break
 */
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_i, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 828, __pyx_L1_error)
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 828, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_5) {

        /* "cupy/cudnn.pyx":829
 *         for i in dilation:
 *             if i > 1:
 *                 use_tensor_core = False             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
        __pyx_v_use_tensor_core = 0;

        /* "cupy/cudnn.pyx":830
 *             if i > 1:
 *                 use_tensor_core = False
 *                 break             # <<<<<<<<<<<<<<
 * 
 *     handle = get_handle()
 */
        goto __pyx_L6_break;

        /* "cupy/cudnn.pyx":828
 *     if use_tensor_core:
 *         for i in dilation:
 *             if i > 1:             # <<<<<<<<<<<<<<
 *                 use_tensor_core = False
 *                 break
 */
      }

      /* "cupy/cudnn.pyx":827
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:
 *         for i in dilation:             # <<<<<<<<<<<<<<
 *             if i > 1:
 *                 use_tensor_core = False
 */
    }
    __pyx_L6_break:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":826
 *     # cuDNN 7 supports dilation only in *_FWD_ALGO_IMPLICIT_GEMM, but
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:             # <<<<<<<<<<<<<<
 *         for i in dilation:
 *             if i > 1:
 */
  }

  /* "cupy/cudnn.pyx":832
 *                 break
 * 
 *     handle = get_handle()             # <<<<<<<<<<<<<<
 *     x = core.ascontiguousarray(x)
 *     W = core.ascontiguousarray(W)
 */
  __pyx_t_7 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 832, __pyx_L1_error)
  __pyx_v_handle = __pyx_t_7;

  /* "cupy/cudnn.pyx":833
 * 
 *     handle = get_handle()
 *     x = core.ascontiguousarray(x)             # <<<<<<<<<<<<<<
 *     W = core.ascontiguousarray(W)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_x, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_x, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":834
 *     handle = get_handle()
 *     x = core.ascontiguousarray(x)
 *     W = core.ascontiguousarray(W)             # <<<<<<<<<<<<<<
 * 
 *     # TODO(okuta) check performance
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_W, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_W, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":837
 * 
 *     # TODO(okuta) check performance
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 837, __pyx_L1_error)
  __pyx_v_x_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":838
 *     # TODO(okuta) check performance
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 838, __pyx_L1_error)
  __pyx_v_y_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":839
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 839, __pyx_L1_error)
  __pyx_v_b_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":840
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 * 
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 840, __pyx_L1_error)
  __pyx_v_filter_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":841
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t max_workspace_size = get_max_workspace_size()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createConvolutionDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 841, __pyx_L1_error)
  __pyx_v_conv_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":843
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 * 
 *     cdef size_t max_workspace_size = get_max_workspace_size()             # <<<<<<<<<<<<<<
 *     cdef vector.vector[Py_ssize_t] b_shape
 *     cdef _Algorithm perf
 */
  __pyx_v_max_workspace_size = __pyx_f_4cupy_5cudnn_get_max_workspace_size(0);

  /* "cupy/cudnn.pyx":846
 *     cdef vector.vector[Py_ssize_t] b_shape
 *     cdef _Algorithm perf
 *     try:             # <<<<<<<<<<<<<<
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":847
 *     cdef _Algorithm perf
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)             # <<<<<<<<<<<<<<
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 */
    __pyx_t_8.__pyx_n = 1;
    __pyx_t_8.data_type = -1;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_x_desc, __pyx_v_x, 0, &__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 847, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":848
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(y_desc, y, -1)             # <<<<<<<<<<<<<<
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(
 */
    __pyx_t_8.__pyx_n = 1;
    __pyx_t_8.data_type = -1;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_y_desc, __pyx_v_y, 0, &__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 848, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":849
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)             # <<<<<<<<<<<<<<
 *         _create_convolution_descriptor(
 *             conv_desc, pad, stride, dilation, groups, x.dtype,
 */
    __pyx_t_9.__pyx_n = 1;
    __pyx_t_9.format = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_filter_descriptor(__pyx_v_filter_desc, __pyx_v_W, 0, &__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 849, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":851
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(
 *             conv_desc, pad, stride, dilation, groups, x.dtype,             # <<<<<<<<<<<<<<
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 */
    __pyx_t_2 = __pyx_v_x->dtype;
    __Pyx_INCREF(__pyx_t_2);

    /* "cupy/cudnn.pyx":850
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(             # <<<<<<<<<<<<<<
 *             conv_desc, pad, stride, dilation, groups, x.dtype,
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 */
    __pyx_t_4 = __pyx_f_4cupy_5cudnn__create_convolution_descriptor(__pyx_v_conv_desc, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_t_2, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CROSS_CORRELATION, __pyx_v_use_tensor_core, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 850, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":854
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 *         if auto_tune and _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *             perf = _find_algorithm_fwd(
 *                 x, W, y, conv_param, handle, x_desc, filter_desc,
 */
    __pyx_t_10 = (__pyx_v_auto_tune != 0);
    if (__pyx_t_10) {
    } else {
      __pyx_t_5 = __pyx_t_10;
      goto __pyx_L12_bool_binop_done;
    }
    __pyx_t_10 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1388) != 0);
    __pyx_t_5 = __pyx_t_10;
    __pyx_L12_bool_binop_done:;
    if (__pyx_t_5) {

      /* "cupy/cudnn.pyx":855
 * 
 *         if auto_tune and _cudnn_version >= 5000:
 *             perf = _find_algorithm_fwd(             # <<<<<<<<<<<<<<
 *                 x, W, y, conv_param, handle, x_desc, filter_desc,
 *                 conv_desc, y_desc, max_workspace_size, use_tensor_core)
 */
      __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__find_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 855, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_perf = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4);
      __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":854
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 *         if auto_tune and _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *             perf = _find_algorithm_fwd(
 *                 x, W, y, conv_param, handle, x_desc, filter_desc,
 */
      goto __pyx_L11;
    }

    /* "cupy/cudnn.pyx":859
 *                 conv_desc, y_desc, max_workspace_size, use_tensor_core)
 *         else:
 *             perf = _get_algorithm_fwd(             # <<<<<<<<<<<<<<
 *                 x, W, y, conv_param, handle, x_desc, filter_desc,
 *                 conv_desc, y_desc, max_workspace_size, use_tensor_core)
 */
    /*else*/ {

      /* "cupy/cudnn.pyx":861
 *             perf = _get_algorithm_fwd(
 *                 x, W, y, conv_param, handle, x_desc, filter_desc,
 *                 conv_desc, y_desc, max_workspace_size, use_tensor_core)             # <<<<<<<<<<<<<<
 * 
 *         if _cudnn_version >= 7000:
 */
      __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm_fwd(__pyx_v_x, __pyx_v_W, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_filter_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 859, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_v_perf = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4);
      __pyx_t_4 = 0;
    }
    __pyx_L11:;

    /* "cupy/cudnn.pyx":863
 *                 conv_desc, y_desc, max_workspace_size, use_tensor_core)
 * 
 *         if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(conv_desc, perf.mathType)
 * 
 */
    __pyx_t_5 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
    if (__pyx_t_5) {

      /* "cupy/cudnn.pyx":864
 * 
 *         if _cudnn_version >= 7000:
 *             cudnn.setConvolutionMathType(conv_desc, perf.mathType)             # <<<<<<<<<<<<<<
 * 
 *         workspace = memory.alloc(perf.memory)
 */
      __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_setConvolutionMathType(__pyx_v_conv_desc, __pyx_v_perf->mathType, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 864, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":863
 *                 conv_desc, y_desc, max_workspace_size, use_tensor_core)
 * 
 *         if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(conv_desc, perf.mathType)
 * 
 */
    }

    /* "cupy/cudnn.pyx":866
 *             cudnn.setConvolutionMathType(conv_desc, perf.mathType)
 * 
 *         workspace = memory.alloc(perf.memory)             # <<<<<<<<<<<<<<
 * 
 *         cudnn.convolutionForward(
 */
    __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_perf->memory); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 866, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_4, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 866, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":868
 *         workspace = memory.alloc(perf.memory)
 * 
 *         cudnn.convolutionForward(             # <<<<<<<<<<<<<<
 *             handle, one, x_desc, x.data.ptr, filter_desc, W.data.ptr,
 *             conv_desc, perf.algo, workspace.ptr, perf.memory, zero, y_desc,
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_convolutionForward(__pyx_v_handle, __pyx_v_one, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_filter_desc, __pyx_v_W->data->ptr, __pyx_v_conv_desc, __pyx_v_perf->algo, __pyx_v_workspace->ptr, __pyx_v_perf->memory, __pyx_v_zero, __pyx_v_y_desc, __pyx_v_y->data->ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 868, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":872
 *             conv_desc, perf.algo, workspace.ptr, perf.memory, zero, y_desc,
 *             y.data.ptr)
 *         del workspace, x, W             # <<<<<<<<<<<<<<
 * 
 *         if b is not None:
 */
    __Pyx_DECREF(((PyObject *)__pyx_v_workspace));
    __pyx_v_workspace = NULL;
    __Pyx_DECREF(((PyObject *)__pyx_v_x));
    __pyx_v_x = NULL;
    __Pyx_DECREF(((PyObject *)__pyx_v_W));
    __pyx_v_W = NULL;

    /* "cupy/cudnn.pyx":874
 *         del workspace, x, W
 * 
 *         if b is not None:             # <<<<<<<<<<<<<<
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)
 */
    __pyx_t_5 = (((PyObject *)__pyx_v_b) != Py_None);
    __pyx_t_10 = (__pyx_t_5 != 0);
    if (__pyx_t_10) {

      /* "cupy/cudnn.pyx":875
 * 
 *         if b is not None:
 *             assert dev_id == b.data.device.id             # <<<<<<<<<<<<<<
 *             b_shape.assign(y._shape.size(), 1)
 *             b_shape[1] = -1
 */
      #ifndef CYTHON_WITHOUT_ASSERTIONS
      if (unlikely(!Py_OptimizeFlag)) {
        __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 875, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_b->data), __pyx_n_s_device); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 875, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 875, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 875, __pyx_L9_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 875, __pyx_L9_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_10)) {
          PyErr_SetNone(PyExc_AssertionError);
          __PYX_ERR(0, 875, __pyx_L9_error)
        }
      }
      #endif

      /* "cupy/cudnn.pyx":876
 *         if b is not None:
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)             # <<<<<<<<<<<<<<
 *             b_shape[1] = -1
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 */
      __pyx_v_b_shape.assign(__pyx_v_y->_shape.size(), 1);

      /* "cupy/cudnn.pyx":877
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)
 *             b_shape[1] = -1             # <<<<<<<<<<<<<<
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 *             _create_tensor_nd_descriptor(b_desc, b, -1)
 */
      (__pyx_v_b_shape[1]) = -1L;

      /* "cupy/cudnn.pyx":878
 *             b_shape.assign(y._shape.size(), 1)
 *             b_shape[1] = -1
 *             b = core.ascontiguousarray(b)._reshape(b_shape)             # <<<<<<<<<<<<<<
 *             _create_tensor_nd_descriptor(b_desc, b, -1)
 *             cudnn.addTensor_v3(handle, one, b_desc,
 */
      __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_b, 0, NULL)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 878, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_4cupy_4core_4core_ndarray *)((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_4)->__pyx_vtab)->_reshape(((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_4), __pyx_v_b_shape, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 878, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_b, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":879
 *             b_shape[1] = -1
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 *             _create_tensor_nd_descriptor(b_desc, b, -1)             # <<<<<<<<<<<<<<
 *             cudnn.addTensor_v3(handle, one, b_desc,
 *                                b.data.ptr, one, y_desc, y.data.ptr)
 */
      __pyx_t_8.__pyx_n = 1;
      __pyx_t_8.data_type = -1;
      __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_b_desc, __pyx_v_b, 0, &__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 879, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":880
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 *             _create_tensor_nd_descriptor(b_desc, b, -1)
 *             cudnn.addTensor_v3(handle, one, b_desc,             # <<<<<<<<<<<<<<
 *                                b.data.ptr, one, y_desc, y.data.ptr)
 *     finally:
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_addTensor_v3(__pyx_v_handle, __pyx_v_one, __pyx_v_b_desc, __pyx_v_b->data->ptr, __pyx_v_one, __pyx_v_y_desc, __pyx_v_y->data->ptr, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 880, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":874
 *         del workspace, x, W
 * 
 *         if b is not None:             # <<<<<<<<<<<<<<
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)
 */
    }
  }

  /* "cupy/cudnn.pyx":883
 *                                b.data.ptr, one, y_desc, y.data.ptr)
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 883, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":884
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_y_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":885
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_b_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":886
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 * 
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_filter_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 886, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":887
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor(__pyx_v_conv_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 887, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L10;
    }
    __pyx_L9_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __pyx_t_3 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
      {

        /* "cupy/cudnn.pyx":883
 *                                b.data.ptr, one, y_desc, y.data.ptr)
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 883, __pyx_L17_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":884
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_y_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 884, __pyx_L17_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":885
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_b_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 885, __pyx_L17_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":886
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 * 
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_filter_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 886, __pyx_L17_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":887
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor(__pyx_v_conv_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 887, __pyx_L17_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
      goto __pyx_L1_error;
      __pyx_L17_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L10:;
  }

  /* "cupy/cudnn.pyx":803
 * 
 * 
 * def convolution_forward(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cudnn.convolution_forward", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_conv_param);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF((PyObject *)__pyx_v_perf);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XDECREF((PyObject *)__pyx_v_W);
  __Pyx_XDECREF((PyObject *)__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":890
 * 
 * 
 * def convolution_backward_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray gy, core.ndarray gW,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_85convolution_backward_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_84convolution_backward_filter[] = "convolution_backward_filter(ndarray x, ndarray gy, ndarray gW, tuple pad, tuple stride, tuple dilation, int groups, *, bool deterministic, bool auto_tune, str tensor_core)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_85convolution_backward_filter = {"convolution_backward_filter", (PyCFunction)__pyx_pw_4cupy_5cudnn_85convolution_backward_filter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_84convolution_backward_filter};
static PyObject *__pyx_pw_4cupy_5cudnn_85convolution_backward_filter(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gy = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gW = 0;
  PyObject *__pyx_v_pad = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_dilation = 0;
  int __pyx_v_groups;
  int __pyx_v_deterministic;
  int __pyx_v_auto_tune;
  PyObject *__pyx_v_tensor_core = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convolution_backward_filter (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_gy,&__pyx_n_s_gW,&__pyx_n_s_pad,&__pyx_n_s_stride,&__pyx_n_s_dilation,&__pyx_n_s_groups,&__pyx_n_s_deterministic,&__pyx_n_s_auto_tune,&__pyx_n_s_tensor_core,0};
    PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
    if (likely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_gy)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, 1); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_gW)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, 2); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pad)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, 3); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, 4); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dilation)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, 5); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, 6); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deterministic)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_backward_filter", __pyx_n_s_deterministic); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_auto_tune)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_backward_filter", __pyx_n_s_auto_tune); __PYX_ERR(0, 890, __pyx_L3_error)
        }
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_backward_filter", __pyx_n_s_tensor_core); __PYX_ERR(0, 890, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "convolution_backward_filter") < 0)) __PYX_ERR(0, 890, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 7) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("convolution_backward_filter", __pyx_n_s_deterministic); __PYX_ERR(0, 890, __pyx_L3_error)
    }
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_gy = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_gW = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_pad = ((PyObject*)values[3]);
    __pyx_v_stride = ((PyObject*)values[4]);
    __pyx_v_dilation = ((PyObject*)values[5]);
    __pyx_v_groups = __Pyx_PyInt_As_int(values[6]); if (unlikely((__pyx_v_groups == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 892, __pyx_L3_error)
    __pyx_v_deterministic = __Pyx_PyObject_IsTrue(values[7]); if (unlikely((__pyx_v_deterministic == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 893, __pyx_L3_error)
    __pyx_v_auto_tune = __Pyx_PyObject_IsTrue(values[8]); if (unlikely((__pyx_v_auto_tune == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 893, __pyx_L3_error)
    __pyx_v_tensor_core = ((PyObject*)values[9]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("convolution_backward_filter", 1, 7, 7, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 890, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.convolution_backward_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 891, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_gy), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "gy", 0))) __PYX_ERR(0, 891, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_gW), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "gW", 0))) __PYX_ERR(0, 891, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_pad), (&PyTuple_Type), 1, "pad", 1))) __PYX_ERR(0, 892, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stride), (&PyTuple_Type), 1, "stride", 1))) __PYX_ERR(0, 892, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dilation), (&PyTuple_Type), 1, "dilation", 1))) __PYX_ERR(0, 892, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tensor_core), (&PyString_Type), 1, "tensor_core", 1))) __PYX_ERR(0, 893, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_84convolution_backward_filter(__pyx_self, __pyx_v_x, __pyx_v_gy, __pyx_v_gW, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_v_deterministic, __pyx_v_auto_tune, __pyx_v_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_84convolution_backward_filter(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gy, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_gW, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, int __pyx_v_deterministic, int __pyx_v_auto_tune, PyObject *__pyx_v_tensor_core) {
  int __pyx_v_dev_id;
  float __pyx_v_float_zero;
  float __pyx_v_float_one;
  double __pyx_v_double_zero;
  double __pyx_v_double_one;
  size_t __pyx_v_zero;
  size_t __pyx_v_one;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_v_conv_param = 0;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_gy_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_conv_desc;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_perf = 0;
  int __pyx_v_algo;
  size_t __pyx_v_max_workspace_size;
  size_t __pyx_v_workspace_size;
  int __pyx_v_math_type;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor __pyx_t_7;
  struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  __Pyx_RefNannySetupContext("convolution_backward_filter", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_x);
  __Pyx_INCREF((PyObject *)__pyx_v_gy);

  /* "cupy/cudnn.pyx":894
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 *         bint deterministic, bint auto_tune, str tensor_core):
 *     cdef int dev_id = x.data.device.id             # <<<<<<<<<<<<<<
 *     assert dev_id == gy.data.device.id
 *     assert dev_id == gW.data.device.id
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 894, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dev_id = __pyx_t_3;

  /* "cupy/cudnn.pyx":895
 *         bint deterministic, bint auto_tune, str tensor_core):
 *     cdef int dev_id = x.data.device.id
 *     assert dev_id == gy.data.device.id             # <<<<<<<<<<<<<<
 *     assert dev_id == gW.data.device.id
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_gy->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 895, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 895, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":896
 *     cdef int dev_id = x.data.device.id
 *     assert dev_id == gy.data.device.id
 *     assert dev_id == gW.data.device.id             # <<<<<<<<<<<<<<
 * 
 *     cdef float float_zero = 0, float_one = 1
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_gW->data), __pyx_n_s_device); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 896, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 896, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":898
 *     assert dev_id == gW.data.device.id
 * 
 *     cdef float float_zero = 0, float_one = 1             # <<<<<<<<<<<<<<
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 */
  __pyx_v_float_zero = 0.0;
  __pyx_v_float_one = 1.0;

  /* "cupy/cudnn.pyx":899
 * 
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1             # <<<<<<<<<<<<<<
 *     cdef size_t zero, one
 *     if x.dtype == 'd':
 */
  __pyx_v_double_zero = 0.0;
  __pyx_v_double_one = 1.0;

  /* "cupy/cudnn.pyx":901
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
  __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_x->dtype, __pyx_n_s_d, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 901, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupy/cudnn.pyx":902
 *     cdef size_t zero, one
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&double_one
 *     else:
 */
    __pyx_v_zero = ((size_t)(&__pyx_v_double_zero));

    /* "cupy/cudnn.pyx":903
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one             # <<<<<<<<<<<<<<
 *     else:
 *         zero = <size_t>&float_zero
 */
    __pyx_v_one = ((size_t)(&__pyx_v_double_one));

    /* "cupy/cudnn.pyx":901
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":905
 *         one = <size_t>&double_one
 *     else:
 *         zero = <size_t>&float_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&float_one
 * 
 */
  /*else*/ {
    __pyx_v_zero = ((size_t)(&__pyx_v_float_zero));

    /* "cupy/cudnn.pyx":906
 *     else:
 *         zero = <size_t>&float_zero
 *         one = <size_t>&float_one             # <<<<<<<<<<<<<<
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)
 */
    __pyx_v_one = ((size_t)(&__pyx_v_float_one));
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":908
 *         one = <size_t>&float_one
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)             # <<<<<<<<<<<<<<
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)
 * 
 */
  __pyx_t_4 = __pyx_v_x->dtype;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = __pyx_f_4cupy_5cudnn__should_use_tensor_core(__pyx_v_tensor_core, __pyx_t_4, 0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 908, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_use_tensor_core = __pyx_t_5;

  /* "cupy/cudnn.pyx":909
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)             # <<<<<<<<<<<<<<
 * 
 *     handle = get_handle()
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_use_tensor_core); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 909, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_pad);
  __Pyx_GIVEREF(__pyx_v_pad);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_pad);
  __Pyx_INCREF(__pyx_v_stride);
  __Pyx_GIVEREF(__pyx_v_stride);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_stride);
  __Pyx_INCREF(__pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_v_x->dtype);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_conv_param = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":911
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)
 * 
 *     handle = get_handle()             # <<<<<<<<<<<<<<
 *     x = core.ascontiguousarray(x)
 *     gy = core.ascontiguousarray(gy)
 */
  __pyx_t_6 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_6 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 911, __pyx_L1_error)
  __pyx_v_handle = __pyx_t_6;

  /* "cupy/cudnn.pyx":912
 * 
 *     handle = get_handle()
 *     x = core.ascontiguousarray(x)             # <<<<<<<<<<<<<<
 *     gy = core.ascontiguousarray(gy)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_x, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_x, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":913
 *     handle = get_handle()
 *     x = core.ascontiguousarray(x)
 *     gy = core.ascontiguousarray(gy)             # <<<<<<<<<<<<<<
 * 
 *     # TODO(okuta) check performance
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_gy, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 913, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_gy, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":916
 * 
 *     # TODO(okuta) check performance
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t gy_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 */
  __pyx_t_6 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_6 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 916, __pyx_L1_error)
  __pyx_v_x_desc = __pyx_t_6;

  /* "cupy/cudnn.pyx":917
 *     # TODO(okuta) check performance
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()
 *     cdef size_t gy_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 */
  __pyx_t_6 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_6 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 917, __pyx_L1_error)
  __pyx_v_gy_desc = __pyx_t_6;

  /* "cupy/cudnn.pyx":918
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()
 *     cdef size_t gy_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 * 
 */
  __pyx_t_6 = __pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor(0); if (unlikely(__pyx_t_6 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 918, __pyx_L1_error)
  __pyx_v_filter_desc = __pyx_t_6;

  /* "cupy/cudnn.pyx":919
 *     cdef size_t gy_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()             # <<<<<<<<<<<<<<
 * 
 *     cdef _Algorithm perf
 */
  __pyx_t_6 = __pyx_f_4cupy_4cuda_5cudnn_createConvolutionDescriptor(0); if (unlikely(__pyx_t_6 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 919, __pyx_L1_error)
  __pyx_v_conv_desc = __pyx_t_6;

  /* "cupy/cudnn.pyx":923
 *     cdef _Algorithm perf
 *     cdef int algo
 *     cdef size_t max_workspace_size = get_max_workspace_size()             # <<<<<<<<<<<<<<
 *     cdef size_t workspace_size = 0
 *     try:
 */
  __pyx_v_max_workspace_size = __pyx_f_4cupy_5cudnn_get_max_workspace_size(0);

  /* "cupy/cudnn.pyx":924
 *     cdef int algo
 *     cdef size_t max_workspace_size = get_max_workspace_size()
 *     cdef size_t workspace_size = 0             # <<<<<<<<<<<<<<
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 */
  __pyx_v_workspace_size = 0;

  /* "cupy/cudnn.pyx":925
 *     cdef size_t max_workspace_size = get_max_workspace_size()
 *     cdef size_t workspace_size = 0
 *     try:             # <<<<<<<<<<<<<<
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(gy_desc, gy, -1)
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":926
 *     cdef size_t workspace_size = 0
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)             # <<<<<<<<<<<<<<
 *         _create_tensor_nd_descriptor(gy_desc, gy, -1)
 *         _create_filter_descriptor(filter_desc, gW, cudnn.CUDNN_TENSOR_NCHW)
 */
    __pyx_t_7.__pyx_n = 1;
    __pyx_t_7.data_type = -1;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_x_desc, __pyx_v_x, 0, &__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 926, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":927
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(gy_desc, gy, -1)             # <<<<<<<<<<<<<<
 *         _create_filter_descriptor(filter_desc, gW, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(
 */
    __pyx_t_7.__pyx_n = 1;
    __pyx_t_7.data_type = -1;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_gy_desc, __pyx_v_gy, 0, &__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 927, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":928
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(gy_desc, gy, -1)
 *         _create_filter_descriptor(filter_desc, gW, cudnn.CUDNN_TENSOR_NCHW)             # <<<<<<<<<<<<<<
 *         _create_convolution_descriptor(
 *             conv_desc, pad, stride, dilation, groups, x.dtype,
 */
    __pyx_t_8.__pyx_n = 1;
    __pyx_t_8.format = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_filter_descriptor(__pyx_v_filter_desc, __pyx_v_gW, 0, &__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 928, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":930
 *         _create_filter_descriptor(filter_desc, gW, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(
 *             conv_desc, pad, stride, dilation, groups, x.dtype,             # <<<<<<<<<<<<<<
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 */
    __pyx_t_2 = __pyx_v_x->dtype;
    __Pyx_INCREF(__pyx_t_2);

    /* "cupy/cudnn.pyx":929
 *         _create_tensor_nd_descriptor(gy_desc, gy, -1)
 *         _create_filter_descriptor(filter_desc, gW, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(             # <<<<<<<<<<<<<<
 *             conv_desc, pad, stride, dilation, groups, x.dtype,
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 */
    __pyx_t_4 = __pyx_f_4cupy_5cudnn__create_convolution_descriptor(__pyx_v_conv_desc, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_t_2, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CROSS_CORRELATION, __pyx_v_use_tensor_core, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 929, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":933
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 *         if deterministic:             # <<<<<<<<<<<<<<
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_FILTER_ALGO_1
 *             workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 */
    __pyx_t_5 = (__pyx_v_deterministic != 0);
    if (__pyx_t_5) {

      /* "cupy/cudnn.pyx":934
 * 
 *         if deterministic:
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_FILTER_ALGO_1             # <<<<<<<<<<<<<<
 *             workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 *                 handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 */
      __pyx_v_algo = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_FILTER_ALGO_1;

      /* "cupy/cudnn.pyx":935
 *         if deterministic:
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_FILTER_ALGO_1
 *             workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(             # <<<<<<<<<<<<<<
 *                 handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 *             math_type = cudnn.CUDNN_DEFAULT_MATH
 */
      __pyx_t_9 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterWorkspaceSize(__pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_algo, 0); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 935, __pyx_L5_error)
      __pyx_v_workspace_size = __pyx_t_9;

      /* "cupy/cudnn.pyx":937
 *             workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 *                 handle, x_desc, gy_desc, conv_desc, filter_desc, algo)
 *             math_type = cudnn.CUDNN_DEFAULT_MATH             # <<<<<<<<<<<<<<
 *             # TODO(okuta): check workspace size
 *         else:
 */
      __pyx_v_math_type = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH;

      /* "cupy/cudnn.pyx":933
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 *         if deterministic:             # <<<<<<<<<<<<<<
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_FILTER_ALGO_1
 *             workspace_size = cudnn.getConvolutionBackwardFilterWorkspaceSize(
 */
      goto __pyx_L7;
    }

    /* "cupy/cudnn.pyx":940
 *             # TODO(okuta): check workspace size
 *         else:
 *             if auto_tune and _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *                 perf = _find_algorithm_bwd_filter(
 *                     x, gy, gW, conv_param, handle, x_desc, gy_desc, conv_desc,
 */
    /*else*/ {
      __pyx_t_10 = (__pyx_v_auto_tune != 0);
      if (__pyx_t_10) {
      } else {
        __pyx_t_5 = __pyx_t_10;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1388) != 0);
      __pyx_t_5 = __pyx_t_10;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_5) {

        /* "cupy/cudnn.pyx":941
 *         else:
 *             if auto_tune and _cudnn_version >= 5000:
 *                 perf = _find_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *                     x, gy, gW, conv_param, handle, x_desc, gy_desc, conv_desc,
 *                     filter_desc, max_workspace_size, use_tensor_core)
 */
        __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__find_algorithm_bwd_filter(__pyx_v_x, __pyx_v_gy, __pyx_v_gW, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 941, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_v_perf = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4);
        __pyx_t_4 = 0;

        /* "cupy/cudnn.pyx":940
 *             # TODO(okuta): check workspace size
 *         else:
 *             if auto_tune and _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *                 perf = _find_algorithm_bwd_filter(
 *                     x, gy, gW, conv_param, handle, x_desc, gy_desc, conv_desc,
 */
        goto __pyx_L8;
      }

      /* "cupy/cudnn.pyx":945
 *                     filter_desc, max_workspace_size, use_tensor_core)
 *             else:
 *                 perf = _get_algorithm_bwd_filter(             # <<<<<<<<<<<<<<
 *                     x, gy, gW, conv_param, handle, x_desc, gy_desc, conv_desc,
 *                     filter_desc, max_workspace_size, use_tensor_core)
 */
      /*else*/ {

        /* "cupy/cudnn.pyx":947
 *                 perf = _get_algorithm_bwd_filter(
 *                     x, gy, gW, conv_param, handle, x_desc, gy_desc, conv_desc,
 *                     filter_desc, max_workspace_size, use_tensor_core)             # <<<<<<<<<<<<<<
 *             algo = perf.algo
 *             workspace_size = perf.memory
 */
        __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm_bwd_filter(__pyx_v_x, __pyx_v_gy, __pyx_v_gW, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_x_desc, __pyx_v_gy_desc, __pyx_v_conv_desc, __pyx_v_filter_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 945, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_v_perf = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4);
        __pyx_t_4 = 0;
      }
      __pyx_L8:;

      /* "cupy/cudnn.pyx":948
 *                     x, gy, gW, conv_param, handle, x_desc, gy_desc, conv_desc,
 *                     filter_desc, max_workspace_size, use_tensor_core)
 *             algo = perf.algo             # <<<<<<<<<<<<<<
 *             workspace_size = perf.memory
 *             math_type = perf.mathType
 */
      __pyx_t_3 = __pyx_v_perf->algo;
      __pyx_v_algo = __pyx_t_3;

      /* "cupy/cudnn.pyx":949
 *                     filter_desc, max_workspace_size, use_tensor_core)
 *             algo = perf.algo
 *             workspace_size = perf.memory             # <<<<<<<<<<<<<<
 *             math_type = perf.mathType
 * 
 */
      __pyx_t_6 = __pyx_v_perf->memory;
      __pyx_v_workspace_size = __pyx_t_6;

      /* "cupy/cudnn.pyx":950
 *             algo = perf.algo
 *             workspace_size = perf.memory
 *             math_type = perf.mathType             # <<<<<<<<<<<<<<
 * 
 *         if _cudnn_version >= 7000:
 */
      __pyx_t_3 = __pyx_v_perf->mathType;
      __pyx_v_math_type = __pyx_t_3;
    }
    __pyx_L7:;

    /* "cupy/cudnn.pyx":952
 *             math_type = perf.mathType
 * 
 *         if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(conv_desc, math_type)
 * 
 */
    __pyx_t_5 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
    if (__pyx_t_5) {

      /* "cupy/cudnn.pyx":953
 * 
 *         if _cudnn_version >= 7000:
 *             cudnn.setConvolutionMathType(conv_desc, math_type)             # <<<<<<<<<<<<<<
 * 
 *         workspace = memory.alloc(workspace_size)
 */
      __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_setConvolutionMathType(__pyx_v_conv_desc, __pyx_v_math_type, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 953, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":952
 *             math_type = perf.mathType
 * 
 *         if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(conv_desc, math_type)
 * 
 */
    }

    /* "cupy/cudnn.pyx":955
 *             cudnn.setConvolutionMathType(conv_desc, math_type)
 * 
 *         workspace = memory.alloc(workspace_size)             # <<<<<<<<<<<<<<
 * 
 *         cudnn.convolutionBackwardFilter_v3(
 */
    __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_workspace_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 955, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_4, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 955, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":957
 *         workspace = memory.alloc(workspace_size)
 * 
 *         cudnn.convolutionBackwardFilter_v3(             # <<<<<<<<<<<<<<
 *             handle, one, x_desc, x.data.ptr, gy_desc,
 *             gy.data.ptr, conv_desc, algo, workspace.ptr,
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_convolutionBackwardFilter_v3(__pyx_v_handle, __pyx_v_one, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_gy_desc, __pyx_v_gy->data->ptr, __pyx_v_conv_desc, __pyx_v_algo, __pyx_v_workspace->ptr, __pyx_v_workspace_size, __pyx_v_zero, __pyx_v_filter_desc, __pyx_v_gW->data->ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 957, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }

  /* "cupy/cudnn.pyx":962
 *             workspace_size, zero, filter_desc, gW.data.ptr)
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(gy_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cudnn.pyx":963
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(gy_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_gy_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 963, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cudnn.pyx":964
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(gy_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 * 
 */
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_filter_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "cupy/cudnn.pyx":965
 *         cudnn.destroyTensorDescriptor(gy_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor(__pyx_v_conv_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 965, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L6;
    }
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __pyx_t_3 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
      {

        /* "cupy/cudnn.pyx":962
 *             workspace_size, zero, filter_desc, gW.data.ptr)
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(gy_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "cupy/cudnn.pyx":963
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(gy_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_gy_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 963, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "cupy/cudnn.pyx":964
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(gy_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 * 
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_filter_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 964, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "cupy/cudnn.pyx":965
 *         cudnn.destroyTensorDescriptor(gy_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)             # <<<<<<<<<<<<<<
 * 
 * 
 */
        __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor(__pyx_v_conv_desc, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 965, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
      goto __pyx_L1_error;
      __pyx_L13_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L6:;
  }

  /* "cupy/cudnn.pyx":890
 * 
 * 
 * def convolution_backward_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray gy, core.ndarray gW,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cudnn.convolution_backward_filter", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_conv_param);
  __Pyx_XDECREF((PyObject *)__pyx_v_perf);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XDECREF((PyObject *)__pyx_v_gy);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "cupy/cudnn.pyx":968
 * 
 * 
 * def convolution_backward_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_87convolution_backward_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_86convolution_backward_data[] = "convolution_backward_data(ndarray W, ndarray x, ndarray b, ndarray y, tuple pad, tuple stride, tuple dilation, int groups, *, bool deterministic, bool auto_tune, str tensor_core)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_87convolution_backward_data = {"convolution_backward_data", (PyCFunction)__pyx_pw_4cupy_5cudnn_87convolution_backward_data, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_86convolution_backward_data};
static PyObject *__pyx_pw_4cupy_5cudnn_87convolution_backward_data(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_b = 0;
  struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y = 0;
  PyObject *__pyx_v_pad = 0;
  PyObject *__pyx_v_stride = 0;
  PyObject *__pyx_v_dilation = 0;
  int __pyx_v_groups;
  int __pyx_v_deterministic;
  int __pyx_v_auto_tune;
  PyObject *__pyx_v_tensor_core = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("convolution_backward_data (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_W,&__pyx_n_s_x,&__pyx_n_s_b,&__pyx_n_s_y,&__pyx_n_s_pad,&__pyx_n_s_stride,&__pyx_n_s_dilation,&__pyx_n_s_groups,&__pyx_n_s_deterministic,&__pyx_n_s_auto_tune,&__pyx_n_s_tensor_core,0};
    PyObject* values[11] = {0,0,0,0,0,0,0,0,0,0,0};
    if (likely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  8: values[7] = PyTuple_GET_ITEM(__pyx_args, 7);
        CYTHON_FALLTHROUGH;
        case  7: values[6] = PyTuple_GET_ITEM(__pyx_args, 6);
        CYTHON_FALLTHROUGH;
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        CYTHON_FALLTHROUGH;
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        CYTHON_FALLTHROUGH;
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_W)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 1); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 2); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (likely((values[3] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 3); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  4:
        if (likely((values[4] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pad)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 4); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  5:
        if (likely((values[5] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_stride)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 5); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  6:
        if (likely((values[6] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_dilation)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 6); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  7:
        if (likely((values[7] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_groups)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, 7); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  8:
        if (likely((values[8] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_deterministic)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_backward_data", __pyx_n_s_deterministic); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        if (likely((values[9] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_auto_tune)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_backward_data", __pyx_n_s_auto_tune); __PYX_ERR(0, 968, __pyx_L3_error)
        }
        if (likely((values[10] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_tensor_core)) != 0)) kw_args--;
        else {
          __Pyx_RaiseKeywordRequired("convolution_backward_data", __pyx_n_s_tensor_core); __PYX_ERR(0, 968, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "convolution_backward_data") < 0)) __PYX_ERR(0, 968, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 8) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("convolution_backward_data", __pyx_n_s_deterministic); __PYX_ERR(0, 968, __pyx_L3_error)
    }
    __pyx_v_W = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[0]);
    __pyx_v_x = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[1]);
    __pyx_v_b = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[2]);
    __pyx_v_y = ((struct __pyx_obj_4cupy_4core_4core_ndarray *)values[3]);
    __pyx_v_pad = ((PyObject*)values[4]);
    __pyx_v_stride = ((PyObject*)values[5]);
    __pyx_v_dilation = ((PyObject*)values[6]);
    __pyx_v_groups = __Pyx_PyInt_As_int(values[7]); if (unlikely((__pyx_v_groups == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 970, __pyx_L3_error)
    __pyx_v_deterministic = __Pyx_PyObject_IsTrue(values[8]); if (unlikely((__pyx_v_deterministic == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 971, __pyx_L3_error)
    __pyx_v_auto_tune = __Pyx_PyObject_IsTrue(values[9]); if (unlikely((__pyx_v_auto_tune == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 971, __pyx_L3_error)
    __pyx_v_tensor_core = ((PyObject*)values[10]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("convolution_backward_data", 1, 8, 8, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 968, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.convolution_backward_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_W), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "W", 0))) __PYX_ERR(0, 969, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_x), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "x", 0))) __PYX_ERR(0, 969, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_b), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "b", 0))) __PYX_ERR(0, 969, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_y), __pyx_ptype_4cupy_4core_4core_ndarray, 1, "y", 0))) __PYX_ERR(0, 969, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_pad), (&PyTuple_Type), 1, "pad", 1))) __PYX_ERR(0, 970, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stride), (&PyTuple_Type), 1, "stride", 1))) __PYX_ERR(0, 970, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dilation), (&PyTuple_Type), 1, "dilation", 1))) __PYX_ERR(0, 970, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tensor_core), (&PyString_Type), 1, "tensor_core", 1))) __PYX_ERR(0, 971, __pyx_L1_error)
  __pyx_r = __pyx_pf_4cupy_5cudnn_86convolution_backward_data(__pyx_self, __pyx_v_W, __pyx_v_x, __pyx_v_b, __pyx_v_y, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_v_deterministic, __pyx_v_auto_tune, __pyx_v_tensor_core);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_86convolution_backward_data(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_W, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_x, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_b, struct __pyx_obj_4cupy_4core_4core_ndarray *__pyx_v_y, PyObject *__pyx_v_pad, PyObject *__pyx_v_stride, PyObject *__pyx_v_dilation, int __pyx_v_groups, int __pyx_v_deterministic, int __pyx_v_auto_tune, PyObject *__pyx_v_tensor_core) {
  int __pyx_v_dev_id;
  float __pyx_v_float_zero;
  float __pyx_v_float_one;
  double __pyx_v_double_zero;
  double __pyx_v_double_one;
  size_t __pyx_v_zero;
  size_t __pyx_v_one;
  int __pyx_v_use_tensor_core;
  PyObject *__pyx_v_conv_param = 0;
  PyObject *__pyx_v_i = NULL;
  size_t __pyx_v_handle;
  size_t __pyx_v_x_desc;
  size_t __pyx_v_y_desc;
  size_t __pyx_v_b_desc;
  size_t __pyx_v_filter_desc;
  size_t __pyx_v_conv_desc;
  struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v_perf = 0;
  int __pyx_v_algo;
  size_t __pyx_v_max_workspace_size;
  size_t __pyx_v_workspace_size;
  std::vector<Py_ssize_t>  __pyx_v_b_shape;
  int __pyx_v_math_type;
  struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *__pyx_v_workspace = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  size_t __pyx_t_7;
  struct __pyx_opt_args_4cupy_5cudnn__create_tensor_nd_descriptor __pyx_t_8;
  struct __pyx_opt_args_4cupy_5cudnn__create_filter_descriptor __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  char const *__pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  __Pyx_RefNannySetupContext("convolution_backward_data", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_W);
  __Pyx_INCREF((PyObject *)__pyx_v_x);
  __Pyx_INCREF((PyObject *)__pyx_v_b);

  /* "cupy/cudnn.pyx":972
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 *         bint deterministic, bint auto_tune, str tensor_core):
 *     cdef int dev_id = W.data.device.id             # <<<<<<<<<<<<<<
 *     assert dev_id == x.data.device.id
 *     assert dev_id == y.data.device.id
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_W->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_dev_id = __pyx_t_3;

  /* "cupy/cudnn.pyx":973
 *         bint deterministic, bint auto_tune, str tensor_core):
 *     cdef int dev_id = W.data.device.id
 *     assert dev_id == x.data.device.id             # <<<<<<<<<<<<<<
 *     assert dev_id == y.data.device.id
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_x->data), __pyx_n_s_device); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_id); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 973, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 973, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":974
 *     cdef int dev_id = W.data.device.id
 *     assert dev_id == x.data.device.id
 *     assert dev_id == y.data.device.id             # <<<<<<<<<<<<<<
 * 
 *     cdef float float_zero = 0, float_one = 1
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 974, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_y->data), __pyx_n_s_device); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 974, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 974, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 974, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 974, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 974, __pyx_L1_error)
    }
  }
  #endif

  /* "cupy/cudnn.pyx":976
 *     assert dev_id == y.data.device.id
 * 
 *     cdef float float_zero = 0, float_one = 1             # <<<<<<<<<<<<<<
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 */
  __pyx_v_float_zero = 0.0;
  __pyx_v_float_one = 1.0;

  /* "cupy/cudnn.pyx":977
 * 
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1             # <<<<<<<<<<<<<<
 *     cdef size_t zero, one
 *     if x.dtype == 'd':
 */
  __pyx_v_double_zero = 0.0;
  __pyx_v_double_one = 1.0;

  /* "cupy/cudnn.pyx":979
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
  __pyx_t_5 = (__Pyx_PyString_Equals(__pyx_v_x->dtype, __pyx_n_s_d, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 979, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "cupy/cudnn.pyx":980
 *     cdef size_t zero, one
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&double_one
 *     else:
 */
    __pyx_v_zero = ((size_t)(&__pyx_v_double_zero));

    /* "cupy/cudnn.pyx":981
 *     if x.dtype == 'd':
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one             # <<<<<<<<<<<<<<
 *     else:
 *         zero = <size_t>&float_zero
 */
    __pyx_v_one = ((size_t)(&__pyx_v_double_one));

    /* "cupy/cudnn.pyx":979
 *     cdef double double_zero = 0, double_one = 1
 *     cdef size_t zero, one
 *     if x.dtype == 'd':             # <<<<<<<<<<<<<<
 *         zero = <size_t>&double_zero
 *         one = <size_t>&double_one
 */
    goto __pyx_L3;
  }

  /* "cupy/cudnn.pyx":983
 *         one = <size_t>&double_one
 *     else:
 *         zero = <size_t>&float_zero             # <<<<<<<<<<<<<<
 *         one = <size_t>&float_one
 * 
 */
  /*else*/ {
    __pyx_v_zero = ((size_t)(&__pyx_v_float_zero));

    /* "cupy/cudnn.pyx":984
 *     else:
 *         zero = <size_t>&float_zero
 *         one = <size_t>&float_one             # <<<<<<<<<<<<<<
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)
 */
    __pyx_v_one = ((size_t)(&__pyx_v_float_one));
  }
  __pyx_L3:;

  /* "cupy/cudnn.pyx":986
 *         one = <size_t>&float_one
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)             # <<<<<<<<<<<<<<
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)
 * 
 */
  __pyx_t_4 = __pyx_v_x->dtype;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = __pyx_f_4cupy_5cudnn__should_use_tensor_core(__pyx_v_tensor_core, __pyx_t_4, 0); if (unlikely(__pyx_t_5 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 986, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_use_tensor_core = __pyx_t_5;

  /* "cupy/cudnn.pyx":987
 * 
 *     cdef bint use_tensor_core = _should_use_tensor_core(tensor_core, x.dtype)
 *     cdef tuple conv_param = (pad, stride, x.dtype, use_tensor_core)             # <<<<<<<<<<<<<<
 * 
 *     # cuDNN 7 supports dilation only in *_FWD_ALGO_IMPLICIT_GEMM, but
 */
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_use_tensor_core); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_pad);
  __Pyx_GIVEREF(__pyx_v_pad);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_pad);
  __Pyx_INCREF(__pyx_v_stride);
  __Pyx_GIVEREF(__pyx_v_stride);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_stride);
  __Pyx_INCREF(__pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_v_x->dtype);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_x->dtype);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_v_conv_param = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":991
 *     # cuDNN 7 supports dilation only in *_FWD_ALGO_IMPLICIT_GEMM, but
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:             # <<<<<<<<<<<<<<
 *         for i in dilation:
 *             if i > 1:
 */
  __pyx_t_5 = (__pyx_v_use_tensor_core != 0);
  if (__pyx_t_5) {

    /* "cupy/cudnn.pyx":992
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:
 *         for i in dilation:             # <<<<<<<<<<<<<<
 *             if i > 1:
 *                 use_tensor_core = False
 */
    if (unlikely(__pyx_v_dilation == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 992, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_v_dilation; __Pyx_INCREF(__pyx_t_2); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 992, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_2, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 992, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_i, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":993
 *     if use_tensor_core:
 *         for i in dilation:
 *             if i > 1:             # <<<<<<<<<<<<<<
 *                 use_tensor_core = False
 *                 break
 */
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_i, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 993, __pyx_L1_error)
      __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 993, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_5) {

        /* "cupy/cudnn.pyx":994
 *         for i in dilation:
 *             if i > 1:
 *                 use_tensor_core = False             # <<<<<<<<<<<<<<
 *                 break
 * 
 */
        __pyx_v_use_tensor_core = 0;

        /* "cupy/cudnn.pyx":995
 *             if i > 1:
 *                 use_tensor_core = False
 *                 break             # <<<<<<<<<<<<<<
 * 
 *     handle = get_handle()
 */
        goto __pyx_L6_break;

        /* "cupy/cudnn.pyx":993
 *     if use_tensor_core:
 *         for i in dilation:
 *             if i > 1:             # <<<<<<<<<<<<<<
 *                 use_tensor_core = False
 *                 break
 */
      }

      /* "cupy/cudnn.pyx":992
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:
 *         for i in dilation:             # <<<<<<<<<<<<<<
 *             if i > 1:
 *                 use_tensor_core = False
 */
    }
    __pyx_L6_break:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":991
 *     # cuDNN 7 supports dilation only in *_FWD_ALGO_IMPLICIT_GEMM, but
 *     # it supports Tensor Cores only in *_FWD_ALGO_IMPLICIT_PRECOMP_GEMM.
 *     if use_tensor_core:             # <<<<<<<<<<<<<<
 *         for i in dilation:
 *             if i > 1:
 */
  }

  /* "cupy/cudnn.pyx":997
 *                 break
 * 
 *     handle = get_handle()             # <<<<<<<<<<<<<<
 *     x = core.ascontiguousarray(x)
 *     W = core.ascontiguousarray(W)
 */
  __pyx_t_7 = __pyx_f_4cupy_5cudnn_get_handle(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 997, __pyx_L1_error)
  __pyx_v_handle = __pyx_t_7;

  /* "cupy/cudnn.pyx":998
 * 
 *     handle = get_handle()
 *     x = core.ascontiguousarray(x)             # <<<<<<<<<<<<<<
 *     W = core.ascontiguousarray(W)
 * 
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_x, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 998, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_x, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":999
 *     handle = get_handle()
 *     x = core.ascontiguousarray(x)
 *     W = core.ascontiguousarray(W)             # <<<<<<<<<<<<<<
 * 
 *     # TODO(okuta) check performance
 */
  __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_W, 0, NULL)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_W, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":1002
 * 
 *     # TODO(okuta) check performance
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1002, __pyx_L1_error)
  __pyx_v_x_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":1003
 *     # TODO(okuta) check performance
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1003, __pyx_L1_error)
  __pyx_v_y_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":1004
 *     cdef size_t x_desc = cudnn.createTensorDescriptor()
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1004, __pyx_L1_error)
  __pyx_v_b_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":1005
 *     cdef size_t y_desc = cudnn.createTensorDescriptor()
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()             # <<<<<<<<<<<<<<
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()
 * 
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1005, __pyx_L1_error)
  __pyx_v_filter_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":1006
 *     cdef size_t b_desc = cudnn.createTensorDescriptor()
 *     cdef size_t filter_desc = cudnn.createFilterDescriptor()
 *     cdef size_t conv_desc = cudnn.createConvolutionDescriptor()             # <<<<<<<<<<<<<<
 * 
 *     cdef _Algorithm perf
 */
  __pyx_t_7 = __pyx_f_4cupy_4cuda_5cudnn_createConvolutionDescriptor(0); if (unlikely(__pyx_t_7 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 1006, __pyx_L1_error)
  __pyx_v_conv_desc = __pyx_t_7;

  /* "cupy/cudnn.pyx":1010
 *     cdef _Algorithm perf
 *     cdef int algo
 *     cdef size_t max_workspace_size = get_max_workspace_size()             # <<<<<<<<<<<<<<
 *     cdef size_t workspace_size = 0
 *     cdef vector.vector[Py_ssize_t] b_shape
 */
  __pyx_v_max_workspace_size = __pyx_f_4cupy_5cudnn_get_max_workspace_size(0);

  /* "cupy/cudnn.pyx":1011
 *     cdef int algo
 *     cdef size_t max_workspace_size = get_max_workspace_size()
 *     cdef size_t workspace_size = 0             # <<<<<<<<<<<<<<
 *     cdef vector.vector[Py_ssize_t] b_shape
 *     try:
 */
  __pyx_v_workspace_size = 0;

  /* "cupy/cudnn.pyx":1013
 *     cdef size_t workspace_size = 0
 *     cdef vector.vector[Py_ssize_t] b_shape
 *     try:             # <<<<<<<<<<<<<<
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 */
  /*try:*/ {

    /* "cupy/cudnn.pyx":1014
 *     cdef vector.vector[Py_ssize_t] b_shape
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)             # <<<<<<<<<<<<<<
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 */
    __pyx_t_8.__pyx_n = 1;
    __pyx_t_8.data_type = -1;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_x_desc, __pyx_v_x, 0, &__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1014, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":1015
 *     try:
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(y_desc, y, -1)             # <<<<<<<<<<<<<<
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(
 */
    __pyx_t_8.__pyx_n = 1;
    __pyx_t_8.data_type = -1;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_y_desc, __pyx_v_y, 0, &__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1015, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":1016
 *         _create_tensor_nd_descriptor(x_desc, x, -1)
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)             # <<<<<<<<<<<<<<
 *         _create_convolution_descriptor(
 *             conv_desc, pad, stride, dilation, groups, x.dtype,
 */
    __pyx_t_9.__pyx_n = 1;
    __pyx_t_9.format = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW;
    __pyx_t_2 = __pyx_f_4cupy_5cudnn__create_filter_descriptor(__pyx_v_filter_desc, __pyx_v_W, 0, &__pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1016, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":1018
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(
 *             conv_desc, pad, stride, dilation, groups, x.dtype,             # <<<<<<<<<<<<<<
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 */
    __pyx_t_2 = __pyx_v_x->dtype;
    __Pyx_INCREF(__pyx_t_2);

    /* "cupy/cudnn.pyx":1017
 *         _create_tensor_nd_descriptor(y_desc, y, -1)
 *         _create_filter_descriptor(filter_desc, W, cudnn.CUDNN_TENSOR_NCHW)
 *         _create_convolution_descriptor(             # <<<<<<<<<<<<<<
 *             conv_desc, pad, stride, dilation, groups, x.dtype,
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 */
    __pyx_t_4 = __pyx_f_4cupy_5cudnn__create_convolution_descriptor(__pyx_v_conv_desc, __pyx_v_pad, __pyx_v_stride, __pyx_v_dilation, __pyx_v_groups, __pyx_t_2, __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CROSS_CORRELATION, __pyx_v_use_tensor_core, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1017, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "cupy/cudnn.pyx":1021
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 *         if deterministic:             # <<<<<<<<<<<<<<
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_DATA_ALGO_1
 *             workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 */
    __pyx_t_5 = (__pyx_v_deterministic != 0);
    if (__pyx_t_5) {

      /* "cupy/cudnn.pyx":1022
 * 
 *         if deterministic:
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_DATA_ALGO_1             # <<<<<<<<<<<<<<
 *             workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 *                 handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 */
      __pyx_v_algo = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_CONVOLUTION_BWD_DATA_ALGO_1;

      /* "cupy/cudnn.pyx":1023
 *         if deterministic:
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_DATA_ALGO_1
 *             workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(             # <<<<<<<<<<<<<<
 *                 handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 *             math_type = cudnn.CUDNN_DEFAULT_MATH
 */
      __pyx_t_6 = __pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataWorkspaceSize(__pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_algo, 0); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1L) && PyErr_Occurred())) __PYX_ERR(0, 1023, __pyx_L9_error)
      __pyx_v_workspace_size = __pyx_t_6;

      /* "cupy/cudnn.pyx":1025
 *             workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 *                 handle, filter_desc, x_desc, conv_desc, y_desc, algo)
 *             math_type = cudnn.CUDNN_DEFAULT_MATH             # <<<<<<<<<<<<<<
 *             # TODO(okuta): check workspace size
 *         else:
 */
      __pyx_v_math_type = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_DEFAULT_MATH;

      /* "cupy/cudnn.pyx":1021
 *             cudnn.CUDNN_CROSS_CORRELATION, use_tensor_core)
 * 
 *         if deterministic:             # <<<<<<<<<<<<<<
 *             algo = cudnn.CUDNN_CONVOLUTION_BWD_DATA_ALGO_1
 *             workspace_size = cudnn.getConvolutionBackwardDataWorkspaceSize(
 */
      goto __pyx_L11;
    }

    /* "cupy/cudnn.pyx":1028
 *             # TODO(okuta): check workspace size
 *         else:
 *             if auto_tune and _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *                 perf = _find_algorithm_bwd_data(
 *                     W, x, y, conv_param, handle, filter_desc, x_desc,
 */
    /*else*/ {
      __pyx_t_10 = (__pyx_v_auto_tune != 0);
      if (__pyx_t_10) {
      } else {
        __pyx_t_5 = __pyx_t_10;
        goto __pyx_L13_bool_binop_done;
      }
      __pyx_t_10 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1388) != 0);
      __pyx_t_5 = __pyx_t_10;
      __pyx_L13_bool_binop_done:;
      if (__pyx_t_5) {

        /* "cupy/cudnn.pyx":1029
 *         else:
 *             if auto_tune and _cudnn_version >= 5000:
 *                 perf = _find_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *                     W, x, y, conv_param, handle, filter_desc, x_desc,
 *                     conv_desc, y_desc, max_workspace_size, use_tensor_core)
 */
        __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__find_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1029, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_v_perf = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4);
        __pyx_t_4 = 0;

        /* "cupy/cudnn.pyx":1028
 *             # TODO(okuta): check workspace size
 *         else:
 *             if auto_tune and _cudnn_version >= 5000:             # <<<<<<<<<<<<<<
 *                 perf = _find_algorithm_bwd_data(
 *                     W, x, y, conv_param, handle, filter_desc, x_desc,
 */
        goto __pyx_L12;
      }

      /* "cupy/cudnn.pyx":1033
 *                     conv_desc, y_desc, max_workspace_size, use_tensor_core)
 *             else:
 *                 perf = _get_algorithm_bwd_data(             # <<<<<<<<<<<<<<
 *                     W, x, y, conv_param, handle, filter_desc, x_desc,
 *                     conv_desc, y_desc, max_workspace_size, use_tensor_core)
 */
      /*else*/ {

        /* "cupy/cudnn.pyx":1035
 *                 perf = _get_algorithm_bwd_data(
 *                     W, x, y, conv_param, handle, filter_desc, x_desc,
 *                     conv_desc, y_desc, max_workspace_size, use_tensor_core)             # <<<<<<<<<<<<<<
 *             algo = perf.algo
 *             workspace_size = perf.memory
 */
        __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_5cudnn__get_algorithm_bwd_data(__pyx_v_W, __pyx_v_x, __pyx_v_y, __pyx_v_conv_param, __pyx_v_handle, __pyx_v_filter_desc, __pyx_v_x_desc, __pyx_v_conv_desc, __pyx_v_y_desc, __pyx_v_max_workspace_size, __pyx_v_use_tensor_core, 0)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1033, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_v_perf = ((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_t_4);
        __pyx_t_4 = 0;
      }
      __pyx_L12:;

      /* "cupy/cudnn.pyx":1036
 *                     W, x, y, conv_param, handle, filter_desc, x_desc,
 *                     conv_desc, y_desc, max_workspace_size, use_tensor_core)
 *             algo = perf.algo             # <<<<<<<<<<<<<<
 *             workspace_size = perf.memory
 *             math_type = perf.mathType
 */
      __pyx_t_3 = __pyx_v_perf->algo;
      __pyx_v_algo = __pyx_t_3;

      /* "cupy/cudnn.pyx":1037
 *                     conv_desc, y_desc, max_workspace_size, use_tensor_core)
 *             algo = perf.algo
 *             workspace_size = perf.memory             # <<<<<<<<<<<<<<
 *             math_type = perf.mathType
 * 
 */
      __pyx_t_7 = __pyx_v_perf->memory;
      __pyx_v_workspace_size = __pyx_t_7;

      /* "cupy/cudnn.pyx":1038
 *             algo = perf.algo
 *             workspace_size = perf.memory
 *             math_type = perf.mathType             # <<<<<<<<<<<<<<
 * 
 *         if _cudnn_version >= 7000:
 */
      __pyx_t_3 = __pyx_v_perf->mathType;
      __pyx_v_math_type = __pyx_t_3;
    }
    __pyx_L11:;

    /* "cupy/cudnn.pyx":1040
 *             math_type = perf.mathType
 * 
 *         if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(conv_desc, math_type)
 * 
 */
    __pyx_t_5 = ((__pyx_v_4cupy_5cudnn__cudnn_version >= 0x1B58) != 0);
    if (__pyx_t_5) {

      /* "cupy/cudnn.pyx":1041
 * 
 *         if _cudnn_version >= 7000:
 *             cudnn.setConvolutionMathType(conv_desc, math_type)             # <<<<<<<<<<<<<<
 * 
 *         workspace = memory.alloc(workspace_size)
 */
      __pyx_t_4 = __pyx_f_4cupy_4cuda_5cudnn_setConvolutionMathType(__pyx_v_conv_desc, __pyx_v_math_type, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1041, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "cupy/cudnn.pyx":1040
 *             math_type = perf.mathType
 * 
 *         if _cudnn_version >= 7000:             # <<<<<<<<<<<<<<
 *             cudnn.setConvolutionMathType(conv_desc, math_type)
 * 
 */
    }

    /* "cupy/cudnn.pyx":1043
 *             cudnn.setConvolutionMathType(conv_desc, math_type)
 * 
 *         workspace = memory.alloc(workspace_size)             # <<<<<<<<<<<<<<
 * 
 *         cudnn.convolutionBackwardData_v3(
 */
    __pyx_t_4 = __Pyx_PyInt_FromSize_t(__pyx_v_workspace_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1043, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_2 = ((PyObject *)__pyx_f_4cupy_4cuda_6memory_alloc(__pyx_t_4, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1043, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_workspace = ((struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":1045
 *         workspace = memory.alloc(workspace_size)
 * 
 *         cudnn.convolutionBackwardData_v3(             # <<<<<<<<<<<<<<
 *             handle, one, filter_desc, W.data.ptr, x_desc, x.data.ptr,
 *             conv_desc, algo, workspace.ptr, workspace_size, zero, y_desc,
 */
    __pyx_t_2 = __pyx_f_4cupy_4cuda_5cudnn_convolutionBackwardData_v3(__pyx_v_handle, __pyx_v_one, __pyx_v_filter_desc, __pyx_v_W->data->ptr, __pyx_v_x_desc, __pyx_v_x->data->ptr, __pyx_v_conv_desc, __pyx_v_algo, __pyx_v_workspace->ptr, __pyx_v_workspace_size, __pyx_v_zero, __pyx_v_y_desc, __pyx_v_y->data->ptr, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1045, __pyx_L9_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "cupy/cudnn.pyx":1050
 *             y.data.ptr)
 * 
 *         del workspace, x, W             # <<<<<<<<<<<<<<
 * 
 *         if b is not None:
 */
    __Pyx_DECREF(((PyObject *)__pyx_v_workspace));
    __pyx_v_workspace = NULL;
    __Pyx_DECREF(((PyObject *)__pyx_v_x));
    __pyx_v_x = NULL;
    __Pyx_DECREF(((PyObject *)__pyx_v_W));
    __pyx_v_W = NULL;

    /* "cupy/cudnn.pyx":1052
 *         del workspace, x, W
 * 
 *         if b is not None:             # <<<<<<<<<<<<<<
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)
 */
    __pyx_t_5 = (((PyObject *)__pyx_v_b) != Py_None);
    __pyx_t_10 = (__pyx_t_5 != 0);
    if (__pyx_t_10) {

      /* "cupy/cudnn.pyx":1053
 * 
 *         if b is not None:
 *             assert dev_id == b.data.device.id             # <<<<<<<<<<<<<<
 *             b_shape.assign(y._shape.size(), 1)
 *             b_shape[1] = -1
 */
      #ifndef CYTHON_WITHOUT_ASSERTIONS
      if (unlikely(!Py_OptimizeFlag)) {
        __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_dev_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1053, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_b->data), __pyx_n_s_device); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1053, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1053, __pyx_L9_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1053, __pyx_L9_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 1053, __pyx_L9_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_10)) {
          PyErr_SetNone(PyExc_AssertionError);
          __PYX_ERR(0, 1053, __pyx_L9_error)
        }
      }
      #endif

      /* "cupy/cudnn.pyx":1054
 *         if b is not None:
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)             # <<<<<<<<<<<<<<
 *             b_shape[1] = -1
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 */
      __pyx_v_b_shape.assign(__pyx_v_y->_shape.size(), 1);

      /* "cupy/cudnn.pyx":1055
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)
 *             b_shape[1] = -1             # <<<<<<<<<<<<<<
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 *             _create_tensor_nd_descriptor(b_desc, b, -1)
 */
      (__pyx_v_b_shape[1]) = -1L;

      /* "cupy/cudnn.pyx":1056
 *             b_shape.assign(y._shape.size(), 1)
 *             b_shape[1] = -1
 *             b = core.ascontiguousarray(b)._reshape(b_shape)             # <<<<<<<<<<<<<<
 *             _create_tensor_nd_descriptor(b_desc, b, -1)
 *             cudnn.addTensor_v3(handle, one, b_desc, b.data.ptr, one, y_desc,
 */
      __pyx_t_4 = ((PyObject *)__pyx_f_4cupy_4core_4core_ascontiguousarray(__pyx_v_b, 0, NULL)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1056, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_4cupy_4core_4core_ndarray *)((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_4)->__pyx_vtab)->_reshape(((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_4), __pyx_v_b_shape, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1056, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF_SET(__pyx_v_b, ((struct __pyx_obj_4cupy_4core_4core_ndarray *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1057
 *             b_shape[1] = -1
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 *             _create_tensor_nd_descriptor(b_desc, b, -1)             # <<<<<<<<<<<<<<
 *             cudnn.addTensor_v3(handle, one, b_desc, b.data.ptr, one, y_desc,
 *                                y.data.ptr)
 */
      __pyx_t_8.__pyx_n = 1;
      __pyx_t_8.data_type = -1;
      __pyx_t_1 = __pyx_f_4cupy_5cudnn__create_tensor_nd_descriptor(__pyx_v_b_desc, __pyx_v_b, 0, &__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1057, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1058
 *             b = core.ascontiguousarray(b)._reshape(b_shape)
 *             _create_tensor_nd_descriptor(b_desc, b, -1)
 *             cudnn.addTensor_v3(handle, one, b_desc, b.data.ptr, one, y_desc,             # <<<<<<<<<<<<<<
 *                                y.data.ptr)
 *     finally:
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_addTensor_v3(__pyx_v_handle, __pyx_v_one, __pyx_v_b_desc, __pyx_v_b->data->ptr, __pyx_v_one, __pyx_v_y_desc, __pyx_v_y->data->ptr, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1058, __pyx_L9_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1052
 *         del workspace, x, W
 * 
 *         if b is not None:             # <<<<<<<<<<<<<<
 *             assert dev_id == b.data.device.id
 *             b_shape.assign(y._shape.size(), 1)
 */
    }
  }

  /* "cupy/cudnn.pyx":1061
 *                                y.data.ptr)
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 */
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1061, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1062
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_y_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1063
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_b_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1063, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1064
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_filter_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1064, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "cupy/cudnn.pyx":1065
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)             # <<<<<<<<<<<<<<
 */
      __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor(__pyx_v_conv_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L10;
    }
    __pyx_L9_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15) < 0)) __Pyx_ErrFetch(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_17);
      __Pyx_XGOTREF(__pyx_t_18);
      __pyx_t_3 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_12 = __pyx_filename;
      {

        /* "cupy/cudnn.pyx":1061
 *                                y.data.ptr)
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_x_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1061, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":1062
 *     finally:
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_y_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1062, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":1063
 *         cudnn.destroyTensorDescriptor(x_desc)
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor(__pyx_v_b_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1063, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":1064
 *         cudnn.destroyTensorDescriptor(y_desc)
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)             # <<<<<<<<<<<<<<
 *         cudnn.destroyConvolutionDescriptor(conv_desc)
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor(__pyx_v_filter_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1064, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "cupy/cudnn.pyx":1065
 *         cudnn.destroyTensorDescriptor(b_desc)
 *         cudnn.destroyFilterDescriptor(filter_desc)
 *         cudnn.destroyConvolutionDescriptor(conv_desc)             # <<<<<<<<<<<<<<
 */
        __pyx_t_1 = __pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor(__pyx_v_conv_desc, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1065, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ErrRestore(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_12;
      goto __pyx_L1_error;
      __pyx_L18_error:;
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_XGIVEREF(__pyx_t_17);
        __Pyx_XGIVEREF(__pyx_t_18);
        __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_17, __pyx_t_18);
      }
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L10:;
  }

  /* "cupy/cudnn.pyx":968
 * 
 * 
 * def convolution_backward_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("cupy.cudnn.convolution_backward_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_conv_param);
  __Pyx_XDECREF(__pyx_v_i);
  __Pyx_XDECREF((PyObject *)__pyx_v_perf);
  __Pyx_XDECREF((PyObject *)__pyx_v_workspace);
  __Pyx_XDECREF((PyObject *)__pyx_v_W);
  __Pyx_XDECREF((PyObject *)__pyx_v_x);
  __Pyx_XDECREF((PyObject *)__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4715b48:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_4cupy_5cudnn_89__pyx_unpickle__Algorithm(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4cupy_5cudnn_88__pyx_unpickle__Algorithm[] = "__pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_4cupy_5cudnn_89__pyx_unpickle__Algorithm = {"__pyx_unpickle__Algorithm", (PyCFunction)__pyx_pw_4cupy_5cudnn_89__pyx_unpickle__Algorithm, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_88__pyx_unpickle__Algorithm};
static PyObject *__pyx_pw_4cupy_5cudnn_89__pyx_unpickle__Algorithm(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__Algorithm (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Algorithm", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Algorithm", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle__Algorithm") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__Algorithm", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("cupy.cudnn.__pyx_unpickle__Algorithm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4cupy_5cudnn_88__pyx_unpickle__Algorithm(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4cupy_5cudnn_88__pyx_unpickle__Algorithm(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle__Algorithm", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x4715b48:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x4715b48) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x4715b48:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)
 *     __pyx_result = _Algorithm.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x4715b48:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = _Algorithm.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x47, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x4715b48:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)
 *     __pyx_result = _Algorithm.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_4cupy_5cudnn__Algorithm), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)
 *     __pyx_result = _Algorithm.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = _Algorithm.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_4cupy_5cudnn___pyx_unpickle__Algorithm__set_state(((struct __pyx_obj_4cupy_5cudnn__Algorithm *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x4715b48 = (algo, mathType, memory))" % __pyx_checksum)
 *     __pyx_result = _Algorithm.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4715b48:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("cupy.cudnn.__pyx_unpickle__Algorithm", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_4cupy_5cudnn___pyx_unpickle__Algorithm__set_state(struct __pyx_obj_4cupy_5cudnn__Algorithm *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle__Algorithm__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->algo = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->mathType = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_size_t(__pyx_t_1); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->memory = __pyx_t_3;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 3) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":12
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle__Algorithm__set_state(<_Algorithm> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__Algorithm__set_state(_Algorithm __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.algo = __pyx_state[0]; __pyx_result.mathType = __pyx_state[1]; __pyx_result.memory = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("cupy.cudnn.__pyx_unpickle__Algorithm__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "vector.from_py":45
 * 
 * @cname("__pyx_convert_vector_from_py_int")
 * cdef vector[X] __pyx_convert_vector_from_py_int(object o) except *:             # <<<<<<<<<<<<<<
 *     cdef vector[X] v
 *     for item in o:
 */

static std::vector<int>  __pyx_convert_vector_from_py_int(PyObject *__pyx_v_o) {
  std::vector<int>  __pyx_v_v;
  PyObject *__pyx_v_item = NULL;
  std::vector<int>  __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *(*__pyx_t_3)(PyObject *);
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__pyx_convert_vector_from_py_int", 0);

  /* "vector.from_py":47
 * cdef vector[X] __pyx_convert_vector_from_py_int(object o) except *:
 *     cdef vector[X] v
 *     for item in o:             # <<<<<<<<<<<<<<
 *         v.push_back(<X>item)
 *     return v
 */
  if (likely(PyList_CheckExact(__pyx_v_o)) || PyTuple_CheckExact(__pyx_v_o)) {
    __pyx_t_1 = __pyx_v_o; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
    __pyx_t_3 = NULL;
  } else {
    __pyx_t_2 = -1; __pyx_t_1 = PyObject_GetIter(__pyx_v_o); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 47, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = Py_TYPE(__pyx_t_1)->tp_iternext; if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 47, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_3)) {
      if (likely(PyList_CheckExact(__pyx_t_1))) {
        if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(1, 47, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 47, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      } else {
        if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_4 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_4); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(1, 47, __pyx_L1_error)
        #else
        __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 47, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        #endif
      }
    } else {
      __pyx_t_4 = __pyx_t_3(__pyx_t_1);
      if (unlikely(!__pyx_t_4)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(1, 47, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "vector.from_py":48
 *     cdef vector[X] v
 *     for item in o:
 *         v.push_back(<X>item)             # <<<<<<<<<<<<<<
 *     return v
 * 
 */
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_v_item); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 48, __pyx_L1_error)
    __pyx_v_v.push_back(((int)__pyx_t_5));

    /* "vector.from_py":47
 * cdef vector[X] __pyx_convert_vector_from_py_int(object o) except *:
 *     cdef vector[X] v
 *     for item in o:             # <<<<<<<<<<<<<<
 *         v.push_back(<X>item)
 *     return v
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "vector.from_py":49
 *     for item in o:
 *         v.push_back(<X>item)
 *     return v             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_v;
  goto __pyx_L0;

  /* "vector.from_py":45
 * 
 * @cname("__pyx_convert_vector_from_py_int")
 * cdef vector[X] __pyx_convert_vector_from_py_int(object o) except *:             # <<<<<<<<<<<<<<
 *     cdef vector[X] v
 *     for item in o:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("vector.from_py.__pyx_convert_vector_from_py_int", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "vector.to_py":60
 * 
 * @cname("__pyx_convert_vector_to_py_Py_ssize_t")
 * cdef object __pyx_convert_vector_to_py_Py_ssize_t(vector[X]& v):             # <<<<<<<<<<<<<<
 *     return [v[i] for i in range(v.size())]
 * 
 */

static PyObject *__pyx_convert_vector_to_py_Py_ssize_t(const std::vector<Py_ssize_t>  &__pyx_v_v) {
  size_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__pyx_convert_vector_to_py_Py_ssize_t", 0);

  /* "vector.to_py":61
 * @cname("__pyx_convert_vector_to_py_Py_ssize_t")
 * cdef object __pyx_convert_vector_to_py_Py_ssize_t(vector[X]& v):
 *     return [v[i] for i in range(v.size())]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_v_v.size();
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;
    __pyx_t_5 = PyInt_FromSsize_t((__pyx_v_v[__pyx_v_i])); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 61, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) __PYX_ERR(1, 61, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "vector.to_py":60
 * 
 * @cname("__pyx_convert_vector_to_py_Py_ssize_t")
 * cdef object __pyx_convert_vector_to_py_Py_ssize_t(vector[X]& v):             # <<<<<<<<<<<<<<
 *     return [v[i] for i in range(v.size())]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("vector.to_py.__pyx_convert_vector_to_py_Py_ssize_t", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_4cupy_5cudnn__Algorithm(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_4cupy_5cudnn__Algorithm(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyMethodDef __pyx_methods_4cupy_5cudnn__Algorithm[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_4cupy_5cudnn_10_Algorithm_1__reduce_cython__, METH_NOARGS, __pyx_doc_4cupy_5cudnn_10_Algorithm___reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_4cupy_5cudnn_10_Algorithm_3__setstate_cython__, METH_O, __pyx_doc_4cupy_5cudnn_10_Algorithm_2__setstate_cython__},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_4cupy_5cudnn__Algorithm = {
  PyVarObject_HEAD_INIT(0, 0)
  "cupy.cudnn._Algorithm", /*tp_name*/
  sizeof(struct __pyx_obj_4cupy_5cudnn__Algorithm), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_4cupy_5cudnn__Algorithm, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_4cupy_5cudnn__Algorithm, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_4cupy_5cudnn__Algorithm, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"get_handle", (PyCFunction)__pyx_pw_4cupy_5cudnn_1get_handle, METH_NOARGS, __pyx_doc_4cupy_5cudnn_get_handle},
  {"_get_nd_tensor_cache", (PyCFunction)__pyx_pw_4cupy_5cudnn_5_get_nd_tensor_cache, METH_NOARGS, __pyx_doc_4cupy_5cudnn_4_get_nd_tensor_cache},
  {"get_max_workspace_size", (PyCFunction)__pyx_pw_4cupy_5cudnn_7get_max_workspace_size, METH_NOARGS, __pyx_doc_4cupy_5cudnn_6get_max_workspace_size},
  {"set_max_workspace_size", (PyCFunction)__pyx_pw_4cupy_5cudnn_9set_max_workspace_size, METH_O, __pyx_doc_4cupy_5cudnn_8set_max_workspace_size},
  {"get_data_type", (PyCFunction)__pyx_pw_4cupy_5cudnn_11get_data_type, METH_O, __pyx_doc_4cupy_5cudnn_10get_data_type},
  {"_get_byte_size", (PyCFunction)__pyx_pw_4cupy_5cudnn_13_get_byte_size, METH_O, __pyx_doc_4cupy_5cudnn_12_get_byte_size},
  {"_create_tensor_nd_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_15_create_tensor_nd_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_14_create_tensor_nd_descriptor},
  {"_create_tensor_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_17_create_tensor_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_16_create_tensor_descriptor},
  {"_create_tensor_descriptor_as4darray", (PyCFunction)__pyx_pw_4cupy_5cudnn_19_create_tensor_descriptor_as4darray, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_18_create_tensor_descriptor_as4darray},
  {"_create_filter_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_21_create_filter_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_20_create_filter_descriptor},
  {"_create_convolution_descriptor", (PyCFunction)__pyx_pw_4cupy_5cudnn_23_create_convolution_descriptor, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_22_create_convolution_descriptor},
  {"is_tensor_core_available", (PyCFunction)__pyx_pw_4cupy_5cudnn_61is_tensor_core_available, METH_O, __pyx_doc_4cupy_5cudnn_60is_tensor_core_available},
  {"_warn_algorithm_fwd", (PyCFunction)__pyx_pw_4cupy_5cudnn_63_warn_algorithm_fwd, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_62_warn_algorithm_fwd},
  {"_find_algorithm_fwd", (PyCFunction)__pyx_pw_4cupy_5cudnn_65_find_algorithm_fwd, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_64_find_algorithm_fwd},
  {"_get_algorithm_fwd", (PyCFunction)__pyx_pw_4cupy_5cudnn_67_get_algorithm_fwd, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_66_get_algorithm_fwd},
  {"_warn_algorithm_bwd_filter", (PyCFunction)__pyx_pw_4cupy_5cudnn_69_warn_algorithm_bwd_filter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_68_warn_algorithm_bwd_filter},
  {"_find_algorithm_bwd_filter", (PyCFunction)__pyx_pw_4cupy_5cudnn_71_find_algorithm_bwd_filter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_70_find_algorithm_bwd_filter},
  {"_get_algorithm_bwd_filter", (PyCFunction)__pyx_pw_4cupy_5cudnn_73_get_algorithm_bwd_filter, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_72_get_algorithm_bwd_filter},
  {"_warn_algorithm_bwd_data", (PyCFunction)__pyx_pw_4cupy_5cudnn_75_warn_algorithm_bwd_data, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_74_warn_algorithm_bwd_data},
  {"_find_algorithm_bwd_data", (PyCFunction)__pyx_pw_4cupy_5cudnn_77_find_algorithm_bwd_data, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_76_find_algorithm_bwd_data},
  {"_get_algorithm_bwd_data", (PyCFunction)__pyx_pw_4cupy_5cudnn_79_get_algorithm_bwd_data, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_78_get_algorithm_bwd_data},
  {"_should_use_tensor_core", (PyCFunction)__pyx_pw_4cupy_5cudnn_81_should_use_tensor_core, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4cupy_5cudnn_80_should_use_tensor_core},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_cudnn(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_cudnn},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "cudnn",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_DeprecationWarning, __pyx_k_DeprecationWarning, sizeof(__pyx_k_DeprecationWarning), 0, 0, 1, 1},
  {&__pyx_n_s_Descriptor, __pyx_k_Descriptor, sizeof(__pyx_k_Descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_Descriptor___del, __pyx_k_Descriptor___del, sizeof(__pyx_k_Descriptor___del), 0, 0, 1, 1},
  {&__pyx_n_s_Descriptor___init, __pyx_k_Descriptor___init, sizeof(__pyx_k_Descriptor___init), 0, 0, 1, 1},
  {&__pyx_n_s_DropoutStates, __pyx_k_DropoutStates, sizeof(__pyx_k_DropoutStates), 0, 0, 1, 1},
  {&__pyx_n_s_DropoutStates___init, __pyx_k_DropoutStates___init, sizeof(__pyx_k_DropoutStates___init), 0, 0, 1, 1},
  {&__pyx_n_s_DropoutStates_backward, __pyx_k_DropoutStates_backward, sizeof(__pyx_k_DropoutStates_backward), 0, 0, 1, 1},
  {&__pyx_n_s_DropoutStates_forward, __pyx_k_DropoutStates_forward, sizeof(__pyx_k_DropoutStates_forward), 0, 0, 1, 1},
  {&__pyx_kp_s_Dtype_is_not_supported_in_cuDNN, __pyx_k_Dtype_is_not_supported_in_cuDNN, sizeof(__pyx_k_Dtype_is_not_supported_in_cuDNN), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x47, __pyx_k_Incompatible_checksums_s_vs_0x47, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x47), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_cuDNN_data_type, __pyx_k_Invalid_cuDNN_data_type, sizeof(__pyx_k_Invalid_cuDNN_data_type), 0, 0, 1, 0},
  {&__pyx_kp_s_No_conv_bwd_data_algo_available, __pyx_k_No_conv_bwd_data_algo_available, sizeof(__pyx_k_No_conv_bwd_data_algo_available), 0, 0, 1, 0},
  {&__pyx_kp_s_No_conv_bwd_filter_algo_availabl, __pyx_k_No_conv_bwd_filter_algo_availabl, sizeof(__pyx_k_No_conv_bwd_filter_algo_availabl), 0, 0, 1, 0},
  {&__pyx_kp_s_No_conv_fwd_algo_available_with, __pyx_k_No_conv_fwd_algo_available_with, sizeof(__pyx_k_No_conv_fwd_algo_available_with), 0, 0, 1, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeWarning, __pyx_k_RuntimeWarning, sizeof(__pyx_k_RuntimeWarning), 0, 0, 1, 1},
  {&__pyx_kp_s_Tensor_Core_mode_is_set_but_the, __pyx_k_Tensor_Core_mode_is_set_but_the, sizeof(__pyx_k_Tensor_Core_mode_is_set_but_the), 0, 0, 1, 0},
  {&__pyx_kp_s_Tensor_Core_mode_is_set_but_the_2, __pyx_k_Tensor_Core_mode_is_set_but_the_2, sizeof(__pyx_k_Tensor_Core_mode_is_set_but_the_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Tensor_Core_mode_is_set_but_the_3, __pyx_k_Tensor_Core_mode_is_set_but_the_3, sizeof(__pyx_k_Tensor_Core_mode_is_set_but_the_3), 0, 0, 1, 0},
  {&__pyx_kp_s_The_best_algo_of_conv_bwd_data_m, __pyx_k_The_best_algo_of_conv_bwd_data_m, sizeof(__pyx_k_The_best_algo_of_conv_bwd_data_m), 0, 0, 1, 0},
  {&__pyx_kp_s_The_best_algo_of_conv_bwd_filter, __pyx_k_The_best_algo_of_conv_bwd_filter, sizeof(__pyx_k_The_best_algo_of_conv_bwd_filter), 0, 0, 1, 0},
  {&__pyx_kp_s_The_best_algo_of_conv_fwd_might, __pyx_k_The_best_algo_of_conv_fwd_might, sizeof(__pyx_k_The_best_algo_of_conv_fwd_might), 0, 0, 1, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_W, __pyx_k_W, sizeof(__pyx_k_W), 0, 0, 1, 1},
  {&__pyx_n_s__42, __pyx_k__42, sizeof(__pyx_k__42), 0, 0, 1, 1},
  {&__pyx_n_s_act_desc, __pyx_k_act_desc, sizeof(__pyx_k_act_desc), 0, 0, 1, 1},
  {&__pyx_n_s_activation_backward, __pyx_k_activation_backward, sizeof(__pyx_k_activation_backward), 0, 0, 1, 1},
  {&__pyx_n_s_activation_forward, __pyx_k_activation_forward, sizeof(__pyx_k_activation_forward), 0, 0, 1, 1},
  {&__pyx_n_s_add_tensor, __pyx_k_add_tensor, sizeof(__pyx_k_add_tensor), 0, 0, 1, 1},
  {&__pyx_n_s_algo, __pyx_k_algo, sizeof(__pyx_k_algo), 0, 0, 1, 1},
  {&__pyx_n_s_alpha, __pyx_k_alpha, sizeof(__pyx_k_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_always, __pyx_k_always, sizeof(__pyx_k_always), 0, 0, 1, 1},
  {&__pyx_n_s_arr, __pyx_k_arr, sizeof(__pyx_k_arr), 0, 0, 1, 1},
  {&__pyx_n_s_atexit, __pyx_k_atexit, sizeof(__pyx_k_atexit), 0, 0, 1, 1},
  {&__pyx_n_s_auto, __pyx_k_auto, sizeof(__pyx_k_auto), 0, 0, 1, 1},
  {&__pyx_n_s_auto_tune, __pyx_k_auto_tune, sizeof(__pyx_k_auto_tune), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_b_desc, __pyx_k_b_desc, sizeof(__pyx_k_b_desc), 0, 0, 1, 1},
  {&__pyx_n_s_b_shape, __pyx_k_b_shape, sizeof(__pyx_k_b_shape), 0, 0, 1, 1},
  {&__pyx_n_s_backward, __pyx_k_backward, sizeof(__pyx_k_backward), 0, 0, 1, 1},
  {&__pyx_n_s_beta, __pyx_k_beta, sizeof(__pyx_k_beta), 0, 0, 1, 1},
  {&__pyx_n_s_bias, __pyx_k_bias, sizeof(__pyx_k_bias), 0, 0, 1, 1},
  {&__pyx_n_s_biasData, __pyx_k_biasData, sizeof(__pyx_k_biasData), 0, 0, 1, 1},
  {&__pyx_n_s_biasDesc, __pyx_k_biasDesc, sizeof(__pyx_k_biasDesc), 0, 0, 1, 1},
  {&__pyx_n_s_bias_desc, __pyx_k_bias_desc, sizeof(__pyx_k_bias_desc), 0, 0, 1, 1},
  {&__pyx_n_s_byte_size, __pyx_k_byte_size, sizeof(__pyx_k_byte_size), 0, 0, 1, 1},
  {&__pyx_n_s_c_contiguous, __pyx_k_c_contiguous, sizeof(__pyx_k_c_contiguous), 0, 0, 1, 1},
  {&__pyx_n_s_c_ksize, __pyx_k_c_ksize, sizeof(__pyx_k_c_ksize), 0, 0, 1, 1},
  {&__pyx_n_s_c_pad, __pyx_k_c_pad, sizeof(__pyx_k_c_pad), 0, 0, 1, 1},
  {&__pyx_n_s_c_stride, __pyx_k_c_stride, sizeof(__pyx_k_c_stride), 0, 0, 1, 1},
  {&__pyx_n_s_cache, __pyx_k_cache, sizeof(__pyx_k_cache), 0, 0, 1, 1},
  {&__pyx_n_s_char, __pyx_k_char, sizeof(__pyx_k_char), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_coef, __pyx_k_coef, sizeof(__pyx_k_coef), 0, 0, 1, 1},
  {&__pyx_n_s_conv_desc, __pyx_k_conv_desc, sizeof(__pyx_k_conv_desc), 0, 0, 1, 1},
  {&__pyx_n_s_conv_param, __pyx_k_conv_param, sizeof(__pyx_k_conv_param), 0, 0, 1, 1},
  {&__pyx_n_s_convolution_backward_data, __pyx_k_convolution_backward_data, sizeof(__pyx_k_convolution_backward_data), 0, 0, 1, 1},
  {&__pyx_n_s_convolution_backward_filter, __pyx_k_convolution_backward_filter, sizeof(__pyx_k_convolution_backward_filter), 0, 0, 1, 1},
  {&__pyx_n_s_convolution_forward, __pyx_k_convolution_forward, sizeof(__pyx_k_convolution_forward), 0, 0, 1, 1},
  {&__pyx_n_s_create_convolution_descriptor, __pyx_k_create_convolution_descriptor, sizeof(__pyx_k_create_convolution_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_dropout_descriptor, __pyx_k_create_dropout_descriptor, sizeof(__pyx_k_create_dropout_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_dropout_states, __pyx_k_create_dropout_states, sizeof(__pyx_k_create_dropout_states), 0, 0, 1, 1},
  {&__pyx_kp_s_create_dropout_states_is_depreca, __pyx_k_create_dropout_states_is_depreca, sizeof(__pyx_k_create_dropout_states_is_depreca), 0, 0, 1, 0},
  {&__pyx_n_s_create_filter_descriptor, __pyx_k_create_filter_descriptor, sizeof(__pyx_k_create_filter_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_op_tensor_descriptor, __pyx_k_create_op_tensor_descriptor, sizeof(__pyx_k_create_op_tensor_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_pooling_descriptor, __pyx_k_create_pooling_descriptor, sizeof(__pyx_k_create_pooling_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_reduce_tensor_descriptor, __pyx_k_create_reduce_tensor_descriptor, sizeof(__pyx_k_create_reduce_tensor_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_rnn_descriptor, __pyx_k_create_rnn_descriptor, sizeof(__pyx_k_create_rnn_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_spatial_transformer_descr, __pyx_k_create_spatial_transformer_descr, sizeof(__pyx_k_create_spatial_transformer_descr), 0, 0, 1, 1},
  {&__pyx_n_s_create_tensor_descriptor, __pyx_k_create_tensor_descriptor, sizeof(__pyx_k_create_tensor_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_tensor_nd_descriptor, __pyx_k_create_tensor_nd_descriptor, sizeof(__pyx_k_create_tensor_nd_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_create_uninitialized_tensor_desc, __pyx_k_create_uninitialized_tensor_desc, sizeof(__pyx_k_create_uninitialized_tensor_desc), 0, 0, 1, 1},
  {&__pyx_n_s_cudnn, __pyx_k_cudnn, sizeof(__pyx_k_cudnn), 0, 0, 1, 1},
  {&__pyx_n_s_cudnn_nd_tensor_cache, __pyx_k_cudnn_nd_tensor_cache, sizeof(__pyx_k_cudnn_nd_tensor_cache), 0, 0, 1, 1},
  {&__pyx_n_s_cupy_cuda, __pyx_k_cupy_cuda, sizeof(__pyx_k_cupy_cuda), 0, 0, 1, 1},
  {&__pyx_n_s_cupy_cudnn, __pyx_k_cupy_cudnn, sizeof(__pyx_k_cupy_cudnn), 0, 0, 1, 1},
  {&__pyx_kp_s_cupy_cudnn_pyx, __pyx_k_cupy_cudnn_pyx, sizeof(__pyx_k_cupy_cudnn_pyx), 0, 0, 1, 0},
  {&__pyx_kp_s_cupy_cudnn_supports_c_contiguous, __pyx_k_cupy_cudnn_supports_c_contiguous, sizeof(__pyx_k_cupy_cudnn_supports_c_contiguous), 0, 0, 1, 0},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_n_s_dW, __pyx_k_dW, sizeof(__pyx_k_dW), 0, 0, 1, 1},
  {&__pyx_n_s_data_type, __pyx_k_data_type, sizeof(__pyx_k_data_type), 0, 0, 1, 1},
  {&__pyx_n_s_del, __pyx_k_del, sizeof(__pyx_k_del), 0, 0, 1, 1},
  {&__pyx_n_s_desc, __pyx_k_desc, sizeof(__pyx_k_desc), 0, 0, 1, 1},
  {&__pyx_n_s_desc_2, __pyx_k_desc_2, sizeof(__pyx_k_desc_2), 0, 0, 1, 1},
  {&__pyx_n_s_descriptor, __pyx_k_descriptor, sizeof(__pyx_k_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroy, __pyx_k_destroy, sizeof(__pyx_k_destroy), 0, 0, 1, 1},
  {&__pyx_n_s_destroyConvolutionDescriptor, __pyx_k_destroyConvolutionDescriptor, sizeof(__pyx_k_destroyConvolutionDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyDropoutDescriptor, __pyx_k_destroyDropoutDescriptor, sizeof(__pyx_k_destroyDropoutDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyFilterDescriptor, __pyx_k_destroyFilterDescriptor, sizeof(__pyx_k_destroyFilterDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyOpTensorDescriptor, __pyx_k_destroyOpTensorDescriptor, sizeof(__pyx_k_destroyOpTensorDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyPoolingDescriptor, __pyx_k_destroyPoolingDescriptor, sizeof(__pyx_k_destroyPoolingDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyRNNDescriptor, __pyx_k_destroyRNNDescriptor, sizeof(__pyx_k_destroyRNNDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyReduceTensorDescriptor, __pyx_k_destroyReduceTensorDescriptor, sizeof(__pyx_k_destroyReduceTensorDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroySpatialTransformerDescrip, __pyx_k_destroySpatialTransformerDescrip, sizeof(__pyx_k_destroySpatialTransformerDescrip), 0, 0, 1, 1},
  {&__pyx_n_s_destroyTensorDescriptor, __pyx_k_destroyTensorDescriptor, sizeof(__pyx_k_destroyTensorDescriptor), 0, 0, 1, 1},
  {&__pyx_n_s_destroyer, __pyx_k_destroyer, sizeof(__pyx_k_destroyer), 0, 0, 1, 1},
  {&__pyx_n_s_deterministic, __pyx_k_deterministic, sizeof(__pyx_k_deterministic), 0, 0, 1, 1},
  {&__pyx_n_s_dev_id, __pyx_k_dev_id, sizeof(__pyx_k_dev_id), 0, 0, 1, 1},
  {&__pyx_n_s_device, __pyx_k_device, sizeof(__pyx_k_device), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_dilation, __pyx_k_dilation, sizeof(__pyx_k_dilation), 0, 0, 1, 1},
  {&__pyx_kp_s_dilation_must_be_one_when_cuDNN, __pyx_k_dilation_must_be_one_when_cuDNN, sizeof(__pyx_k_dilation_must_be_one_when_cuDNN), 0, 0, 1, 0},
  {&__pyx_n_s_dim, __pyx_k_dim, sizeof(__pyx_k_dim), 0, 0, 1, 1},
  {&__pyx_n_s_dim_A, __pyx_k_dim_A, sizeof(__pyx_k_dim_A), 0, 0, 1, 1},
  {&__pyx_n_s_direction, __pyx_k_direction, sizeof(__pyx_k_direction), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_double_one, __pyx_k_double_one, sizeof(__pyx_k_double_one), 0, 0, 1, 1},
  {&__pyx_n_s_double_zero, __pyx_k_double_zero, sizeof(__pyx_k_double_zero), 0, 0, 1, 1},
  {&__pyx_n_s_dropout, __pyx_k_dropout, sizeof(__pyx_k_dropout), 0, 0, 1, 1},
  {&__pyx_n_s_dropout_desc, __pyx_k_dropout_desc, sizeof(__pyx_k_dropout_desc), 0, 0, 1, 1},
  {&__pyx_n_s_dropout_ratio, __pyx_k_dropout_ratio, sizeof(__pyx_k_dropout_ratio), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_dx, __pyx_k_dx, sizeof(__pyx_k_dx), 0, 0, 1, 1},
  {&__pyx_n_s_dy, __pyx_k_dy, sizeof(__pyx_k_dy), 0, 0, 1, 1},
  {&__pyx_n_s_dy_desc, __pyx_k_dy_desc, sizeof(__pyx_k_dy_desc), 0, 0, 1, 1},
  {&__pyx_n_s_e, __pyx_k_e, sizeof(__pyx_k_e), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_filter_desc, __pyx_k_filter_desc, sizeof(__pyx_k_filter_desc), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_float_one, __pyx_k_float_one, sizeof(__pyx_k_float_one), 0, 0, 1, 1},
  {&__pyx_n_s_float_zero, __pyx_k_float_zero, sizeof(__pyx_k_float_zero), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_forward, __pyx_k_forward, sizeof(__pyx_k_forward), 0, 0, 1, 1},
  {&__pyx_n_s_gW, __pyx_k_gW, sizeof(__pyx_k_gW), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_get_rnn_lin_layer_bias_params, __pyx_k_get_rnn_lin_layer_bias_params, sizeof(__pyx_k_get_rnn_lin_layer_bias_params), 0, 0, 1, 1},
  {&__pyx_n_s_get_rnn_lin_layer_matrix_params, __pyx_k_get_rnn_lin_layer_matrix_params, sizeof(__pyx_k_get_rnn_lin_layer_matrix_params), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_groups, __pyx_k_groups, sizeof(__pyx_k_groups), 0, 0, 1, 1},
  {&__pyx_kp_s_groups_must_be_one_when_cuDNN_7, __pyx_k_groups_must_be_one_when_cuDNN_7, sizeof(__pyx_k_groups_must_be_one_when_cuDNN_7), 0, 0, 1, 0},
  {&__pyx_n_s_gx, __pyx_k_gx, sizeof(__pyx_k_gx), 0, 0, 1, 1},
  {&__pyx_n_s_gy, __pyx_k_gy, sizeof(__pyx_k_gy), 0, 0, 1, 1},
  {&__pyx_n_s_gy_desc, __pyx_k_gy_desc, sizeof(__pyx_k_gy_desc), 0, 0, 1, 1},
  {&__pyx_n_s_handle, __pyx_k_handle, sizeof(__pyx_k_handle), 0, 0, 1, 1},
  {&__pyx_n_s_handle_2, __pyx_k_handle_2, sizeof(__pyx_k_handle_2), 0, 0, 1, 1},
  {&__pyx_n_s_hidden_size, __pyx_k_hidden_size, sizeof(__pyx_k_hidden_size), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_indices, __pyx_k_indices, sizeof(__pyx_k_indices), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_input_mode, __pyx_k_input_mode, sizeof(__pyx_k_input_mode), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_ksize, __pyx_k_ksize, sizeof(__pyx_k_ksize), 0, 0, 1, 1},
  {&__pyx_kp_s_ksize_stride_and_pad_must_be_of, __pyx_k_ksize_stride_and_pad_must_be_of, sizeof(__pyx_k_ksize_stride_and_pad_must_be_of), 0, 0, 1, 0},
  {&__pyx_n_s_layer, __pyx_k_layer, sizeof(__pyx_k_layer), 0, 0, 1, 1},
  {&__pyx_n_s_lin_layer_id, __pyx_k_lin_layer_id, sizeof(__pyx_k_lin_layer_id), 0, 0, 1, 1},
  {&__pyx_n_s_local, __pyx_k_local, sizeof(__pyx_k_local), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mat, __pyx_k_mat, sizeof(__pyx_k_mat), 0, 0, 1, 1},
  {&__pyx_n_s_mat_desc, __pyx_k_mat_desc, sizeof(__pyx_k_mat_desc), 0, 0, 1, 1},
  {&__pyx_n_s_mathType, __pyx_k_mathType, sizeof(__pyx_k_mathType), 0, 0, 1, 1},
  {&__pyx_n_s_math_type, __pyx_k_math_type, sizeof(__pyx_k_math_type), 0, 0, 1, 1},
  {&__pyx_n_s_max_workspace_size, __pyx_k_max_workspace_size, sizeof(__pyx_k_max_workspace_size), 0, 0, 1, 1},
  {&__pyx_n_s_memory, __pyx_k_memory, sizeof(__pyx_k_memory), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_nb_dims, __pyx_k_nb_dims, sizeof(__pyx_k_nb_dims), 0, 0, 1, 1},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_never, __pyx_k_never, sizeof(__pyx_k_never), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_num_layers, __pyx_k_num_layers, sizeof(__pyx_k_num_layers), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_n_s_offset, __pyx_k_offset, sizeof(__pyx_k_offset), 0, 0, 1, 1},
  {&__pyx_n_s_one, __pyx_k_one, sizeof(__pyx_k_one), 0, 0, 1, 1},
  {&__pyx_n_s_op_type, __pyx_k_op_type, sizeof(__pyx_k_op_type), 0, 0, 1, 1},
  {&__pyx_n_s_pad, __pyx_k_pad, sizeof(__pyx_k_pad), 0, 0, 1, 1},
  {&__pyx_kp_s_pad_and_stride_must_be_of_same_l, __pyx_k_pad_and_stride_must_be_of_same_l, sizeof(__pyx_k_pad_and_stride_must_be_of_same_l), 0, 0, 1, 0},
  {&__pyx_n_s_perf, __pyx_k_perf, sizeof(__pyx_k_perf), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_ptr, __pyx_k_ptr, sizeof(__pyx_k_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_py_cudnn, __pyx_k_py_cudnn, sizeof(__pyx_k_py_cudnn), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle__Algorithm, __pyx_k_pyx_unpickle__Algorithm, sizeof(__pyx_k_pyx_unpickle__Algorithm), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_type, __pyx_k_reduce_type, sizeof(__pyx_k_reduce_type), 0, 0, 1, 1},
  {&__pyx_n_s_register, __pyx_k_register, sizeof(__pyx_k_register), 0, 0, 1, 1},
  {&__pyx_n_s_reserve_size, __pyx_k_reserve_size, sizeof(__pyx_k_reserve_size), 0, 0, 1, 1},
  {&__pyx_n_s_reserve_space, __pyx_k_reserve_space, sizeof(__pyx_k_reserve_space), 0, 0, 1, 1},
  {&__pyx_n_s_reset_handles, __pyx_k_reset_handles, sizeof(__pyx_k_reset_handles), 0, 0, 1, 1},
  {&__pyx_n_s_rnn_desc, __pyx_k_rnn_desc, sizeof(__pyx_k_rnn_desc), 0, 0, 1, 1},
  {&__pyx_n_s_sampler_type, __pyx_k_sampler_type, sizeof(__pyx_k_sampler_type), 0, 0, 1, 1},
  {&__pyx_n_s_seed, __pyx_k_seed, sizeof(__pyx_k_seed), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_set_dropout_descriptor, __pyx_k_set_dropout_descriptor, sizeof(__pyx_k_set_dropout_descriptor), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_srcDestData, __pyx_k_srcDestData, sizeof(__pyx_k_srcDestData), 0, 0, 1, 1},
  {&__pyx_n_s_srcDestDesc, __pyx_k_srcDestDesc, sizeof(__pyx_k_srcDestDesc), 0, 0, 1, 1},
  {&__pyx_n_s_state_size, __pyx_k_state_size, sizeof(__pyx_k_state_size), 0, 0, 1, 1},
  {&__pyx_n_s_state_size_in_bytes, __pyx_k_state_size_in_bytes, sizeof(__pyx_k_state_size_in_bytes), 0, 0, 1, 1},
  {&__pyx_n_s_states, __pyx_k_states, sizeof(__pyx_k_states), 0, 0, 1, 1},
  {&__pyx_n_s_states_2, __pyx_k_states_2, sizeof(__pyx_k_states_2), 0, 0, 1, 1},
  {&__pyx_n_s_stride, __pyx_k_stride, sizeof(__pyx_k_stride), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_kp_s_tensor_code_mode_must_be_either, __pyx_k_tensor_code_mode_must_be_either, sizeof(__pyx_k_tensor_code_mode_must_be_either), 0, 0, 1, 0},
  {&__pyx_n_s_tensor_core, __pyx_k_tensor_core, sizeof(__pyx_k_tensor_core), 0, 0, 1, 1},
  {&__pyx_n_s_tensor_core_mode, __pyx_k_tensor_core_mode, sizeof(__pyx_k_tensor_core_mode), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_threading, __pyx_k_threading, sizeof(__pyx_k_threading), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_tensor_core, __pyx_k_use_tensor_core, sizeof(__pyx_k_use_tensor_core), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_w, __pyx_k_w, sizeof(__pyx_k_w), 0, 0, 1, 1},
  {&__pyx_n_s_w_data_ptr, __pyx_k_w_data_ptr, sizeof(__pyx_k_w_data_ptr), 0, 0, 1, 1},
  {&__pyx_n_s_w_desc, __pyx_k_w_desc, sizeof(__pyx_k_w_desc), 0, 0, 1, 1},
  {&__pyx_n_s_warn, __pyx_k_warn, sizeof(__pyx_k_warn), 0, 0, 1, 1},
  {&__pyx_n_s_warnings, __pyx_k_warnings, sizeof(__pyx_k_warnings), 0, 0, 1, 1},
  {&__pyx_n_s_workspace, __pyx_k_workspace, sizeof(__pyx_k_workspace), 0, 0, 1, 1},
  {&__pyx_n_s_workspace_size, __pyx_k_workspace_size, sizeof(__pyx_k_workspace_size), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_x_desc, __pyx_k_x_desc, sizeof(__pyx_k_x_desc), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_y_desc, __pyx_k_y_desc, sizeof(__pyx_k_y_desc), 0, 0, 1, 1},
  {&__pyx_n_s_zero, __pyx_k_zero, sizeof(__pyx_k_zero), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 79, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 100, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 133, __pyx_L1_error)
  __pyx_builtin_DeprecationWarning = __Pyx_GetBuiltinName(__pyx_n_s_DeprecationWarning); if (!__pyx_builtin_DeprecationWarning) __PYX_ERR(0, 443, __pyx_L1_error)
  __pyx_builtin_RuntimeWarning = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeWarning); if (!__pyx_builtin_RuntimeWarning) __PYX_ERR(0, 575, __pyx_L1_error)
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) __PYX_ERR(0, 614, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(0, 618, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(1, 61, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "cupy/cudnn.pyx":133
 *                                 int format):
 *     if not arr._c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')             # <<<<<<<<<<<<<<
 *     if arr._shape.size() == 4:
 *         data_type = get_data_type(arr.dtype)
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_cupy_cudnn_supports_c_contiguous); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "cupy/cudnn.pyx":179
 *     ndim = len(pad)
 *     if ndim != len(stride):
 *         raise ValueError('pad and stride must be of same length')             # <<<<<<<<<<<<<<
 * 
 *     compute_type = get_data_type(dtype)
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_pad_and_stride_must_be_of_same_l); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "cupy/cudnn.pyx":197
 *                 for i in c_dilation:
 *                     if i != 1:
 *                         raise ValueError(             # <<<<<<<<<<<<<<
 *                             'dilation must be one when cuDNN < 6.0')
 *         cudnn.setConvolutionNdDescriptor_v3(
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_dilation_must_be_one_when_cuDNN); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 197, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "cupy/cudnn.pyx":208
 *             d0, d1 = dilation
 *             if _cudnn_version < 6000 and (d0 != 1 or d1 != 1):
 *                 raise ValueError('dilation must be one when cuDNN < 6.0')             # <<<<<<<<<<<<<<
 *         p0, p1 = pad
 *         s0, s1 = stride
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_dilation_must_be_one_when_cuDNN); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "cupy/cudnn.pyx":224
 *             cudnn.setConvolutionGroupCount(desc, groups)
 *     elif groups > 1:
 *         raise ValueError('groups must be one when cuDNN < 7.0')             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_groups_must_be_one_when_cuDNN_7); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "cupy/cudnn.pyx":247
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:
 *         raise ValueError('cupy.cudnn supports c-contiguous arrays only')             # <<<<<<<<<<<<<<
 *     data_type = get_data_type(arr.dtype)
 *     key = (data_type, tuple(arr._shape))
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_cupy_cudnn_supports_c_contiguous); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "cupy/cudnn.pyx":287
 *     cdef int ndim = len(ksize)
 *     if ndim != len(stride) or ndim != len(pad):
 *         raise ValueError('ksize, stride, and pad must be of same length')             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createPoolingDescriptor(),
 *                       py_cudnn.destroyPoolingDescriptor)
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_ksize_stride_and_pad_must_be_of); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "cupy/cudnn.pyx":441
 * 
 * def create_dropout_states(handle):
 *     warnings.warn('create_dropout_states is deprecated.'             # <<<<<<<<<<<<<<
 *                   'Please use DropoutStates class instead.',
 *                   DeprecationWarning)
 */
  __pyx_tuple__12 = PyTuple_Pack(2, __pyx_kp_s_create_dropout_states_is_depreca, __pyx_builtin_DeprecationWarning); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 441, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "cupy/cudnn.pyx":799
 *         return False
 *     else:
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             'tensor_code_mode must be either of "always", "auto", or "never".')
 * 
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_tensor_code_mode_must_be_either); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "cupy/cudnn.pyx":38
 * 
 * @atexit.register
 * def reset_handles():             # <<<<<<<<<<<<<<
 *     for handle in _handles:
 *         if handle:
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_n_s_handle); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);
  __pyx_codeobj__15 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__14, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_reset_handles, 38, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__15)) __PYX_ERR(0, 38, __pyx_L1_error)

  /* "cupy/cudnn.pyx":79
 * 
 * 
 * class Descriptor(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, descriptor, destroyer):
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "cupy/cudnn.pyx":81
 * class Descriptor(object):
 * 
 *     def __init__(self, descriptor, destroyer):             # <<<<<<<<<<<<<<
 *         self.value = descriptor
 *         self.destroy = destroyer
 */
  __pyx_tuple__17 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_descriptor, __pyx_n_s_destroyer); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_init, 81, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(0, 81, __pyx_L1_error)

  /* "cupy/cudnn.pyx":85
 *         self.destroy = destroyer
 * 
 *     def __del__(self):             # <<<<<<<<<<<<<<
 *         if self.value:
 *             self.destroy(self.value)
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_del, 85, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(0, 85, __pyx_L1_error)

  /* "cupy/cudnn.pyx":227
 * 
 * 
 * def create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)
 */
  __pyx_tuple__21 = PyTuple_Pack(3, __pyx_n_s_arr, __pyx_n_s_format, __pyx_n_s_desc); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_codeobj__22 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__21, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_tensor_descriptor, 227, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__22)) __PYX_ERR(0, 227, __pyx_L1_error)

  /* "cupy/cudnn.pyx":234
 * 
 * 
 * def create_uninitialized_tensor_descriptor():             # <<<<<<<<<<<<<<
 *     """Create uninitialized tensor descriptor.
 * 
 */
  __pyx_codeobj__23 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_uninitialized_tensor_desc, 234, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__23)) __PYX_ERR(0, 234, __pyx_L1_error)

  /* "cupy/cudnn.pyx":244
 * 
 * 
 * def create_tensor_nd_descriptor(core.ndarray arr):             # <<<<<<<<<<<<<<
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:
 */
  __pyx_tuple__24 = PyTuple_Pack(5, __pyx_n_s_arr, __pyx_n_s_cache, __pyx_n_s_data_type, __pyx_n_s_key, __pyx_n_s_desc); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_codeobj__25 = (PyObject*)__Pyx_PyCode_New(1, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__24, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_tensor_nd_descriptor, 244, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__25)) __PYX_ERR(0, 244, __pyx_L1_error)

  /* "cupy/cudnn.pyx":263
 * 
 * 
 * def create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createFilterDescriptor(),
 *                       py_cudnn.destroyFilterDescriptor)
 */
  __pyx_tuple__26 = PyTuple_Pack(3, __pyx_n_s_arr, __pyx_n_s_format, __pyx_n_s_desc); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);
  __pyx_codeobj__27 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__26, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_filter_descriptor, 263, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__27)) __PYX_ERR(0, 263, __pyx_L1_error)

  /* "cupy/cudnn.pyx":270
 * 
 * 
 * def create_convolution_descriptor(pad, stride, dtype,             # <<<<<<<<<<<<<<
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,
 */
  __pyx_tuple__28 = PyTuple_Pack(8, __pyx_n_s_pad, __pyx_n_s_stride, __pyx_n_s_dtype, __pyx_n_s_mode, __pyx_n_s_dilation, __pyx_n_s_use_tensor_core, __pyx_n_s_groups, __pyx_n_s_desc); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_codeobj__29 = (PyObject*)__Pyx_PyCode_New(7, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__28, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_convolution_descriptor, 270, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__29)) __PYX_ERR(0, 270, __pyx_L1_error)

  /* "cupy/cudnn.pyx":283
 * 
 * 
 * def create_pooling_descriptor(ksize, stride, pad, int mode):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)
 */
  __pyx_tuple__30 = PyTuple_Pack(9, __pyx_n_s_ksize, __pyx_n_s_stride, __pyx_n_s_pad, __pyx_n_s_mode, __pyx_n_s_c_ksize, __pyx_n_s_c_pad, __pyx_n_s_c_stride, __pyx_n_s_ndim, __pyx_n_s_desc); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_codeobj__31 = (PyObject*)__Pyx_PyCode_New(4, 0, 9, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__30, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_pooling_descriptor, 283, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__31)) __PYX_ERR(0, 283, __pyx_L1_error)

  /* "cupy/cudnn.pyx":305
 * 
 * 
 * def activation_forward(core.ndarray x, int mode, double coef=0.0):             # <<<<<<<<<<<<<<
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1
 */
  __pyx_tuple__32 = PyTuple_Pack(13, __pyx_n_s_x, __pyx_n_s_mode, __pyx_n_s_coef, __pyx_n_s_float_zero, __pyx_n_s_float_one, __pyx_n_s_double_zero, __pyx_n_s_double_one, __pyx_n_s_zero, __pyx_n_s_one, __pyx_n_s_y, __pyx_n_s_handle, __pyx_n_s_desc, __pyx_n_s_act_desc); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(3, 0, 13, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_activation_forward, 305, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 305, __pyx_L1_error)

  /* "cupy/cudnn.pyx":336
 * 
 * 
 * def activation_backward(core.ndarray x, core.ndarray y, core.ndarray gy,             # <<<<<<<<<<<<<<
 *                         int mode, float coef=0.0):
 *     cdef float float_zero = 0, float_one = 1
 */
  __pyx_tuple__34 = PyTuple_Pack(15, __pyx_n_s_x, __pyx_n_s_y, __pyx_n_s_gy, __pyx_n_s_mode, __pyx_n_s_coef, __pyx_n_s_float_zero, __pyx_n_s_float_one, __pyx_n_s_double_zero, __pyx_n_s_double_one, __pyx_n_s_zero, __pyx_n_s_one, __pyx_n_s_gx, __pyx_n_s_handle, __pyx_n_s_desc, __pyx_n_s_act_desc); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(5, 0, 15, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_activation_backward, 336, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(0, 336, __pyx_L1_error)

  /* "cupy/cudnn.pyx":371
 * 
 * 
 * def create_dropout_descriptor(             # <<<<<<<<<<<<<<
 *         handle, dropout, states, state_size_in_bytes, seed):
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 */
  __pyx_tuple__36 = PyTuple_Pack(6, __pyx_n_s_handle, __pyx_n_s_dropout, __pyx_n_s_states, __pyx_n_s_state_size_in_bytes, __pyx_n_s_seed, __pyx_n_s_desc); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(0, 371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(5, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_dropout_descriptor, 371, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(0, 371, __pyx_L1_error)

  /* "cupy/cudnn.pyx":380
 * 
 * 
 * def set_dropout_descriptor(desc, handle, dropout):             # <<<<<<<<<<<<<<
 *     # When the fourth argument is NULL, random state is not updated.
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout, 0, 0, 0)
 */
  __pyx_tuple__38 = PyTuple_Pack(3, __pyx_n_s_desc, __pyx_n_s_handle, __pyx_n_s_dropout); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_set_dropout_descriptor, 380, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(0, 380, __pyx_L1_error)

  /* "cupy/cudnn.pyx":385
 * 
 * 
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,             # <<<<<<<<<<<<<<
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 */
  __pyx_tuple__40 = PyTuple_Pack(10, __pyx_n_s_hidden_size, __pyx_n_s_num_layers, __pyx_n_s_dropout_desc, __pyx_n_s_input_mode, __pyx_n_s_direction, __pyx_n_s_mode, __pyx_n_s_data_type, __pyx_n_s_algo, __pyx_n_s_desc, __pyx_n_s_handle_2); if (unlikely(!__pyx_tuple__40)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);
  __pyx_codeobj__41 = (PyObject*)__Pyx_PyCode_New(8, 0, 10, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__40, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_rnn_descriptor, 385, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__41)) __PYX_ERR(0, 385, __pyx_L1_error)

  /* "cupy/cudnn.pyx":403
 * 
 * 
 * def get_rnn_lin_layer_matrix_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */
  __pyx_tuple__43 = PyTuple_Pack(17, __pyx_n_s_handle, __pyx_n_s_rnn_desc, __pyx_n_s_layer, __pyx_n_s_x_desc, __pyx_n_s_w_desc, __pyx_n_s_w, __pyx_n_s_lin_layer_id, __pyx_n_s_ptr, __pyx_n_s_w_data_ptr, __pyx_n_s_mat_desc, __pyx_n_s_data_type, __pyx_n_s__42, __pyx_n_s_dim, __pyx_n_s_byte_size, __pyx_n_s_offset, __pyx_n_s_size, __pyx_n_s_mat); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(7, 0, 17, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_get_rnn_lin_layer_matrix_params, 403, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(0, 403, __pyx_L1_error)

  /* "cupy/cudnn.pyx":422
 * 
 * 
 * def get_rnn_lin_layer_bias_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */
  __pyx_tuple__45 = PyTuple_Pack(16, __pyx_n_s_handle, __pyx_n_s_rnn_desc, __pyx_n_s_layer, __pyx_n_s_x_desc, __pyx_n_s_w_desc, __pyx_n_s_w, __pyx_n_s_lin_layer_id, __pyx_n_s_ptr, __pyx_n_s_bias_desc, __pyx_n_s_data_type, __pyx_n_s__42, __pyx_n_s_dim, __pyx_n_s_byte_size, __pyx_n_s_offset, __pyx_n_s_size, __pyx_n_s_bias); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(7, 0, 16, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_get_rnn_lin_layer_bias_params, 422, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(0, 422, __pyx_L1_error)

  /* "cupy/cudnn.pyx":440
 * 
 * 
 * def create_dropout_states(handle):             # <<<<<<<<<<<<<<
 *     warnings.warn('create_dropout_states is deprecated.'
 *                   'Please use DropoutStates class instead.',
 */
  __pyx_tuple__47 = PyTuple_Pack(2, __pyx_n_s_handle, __pyx_n_s_state_size); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __pyx_codeobj__48 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__47, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_dropout_states, 440, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__48)) __PYX_ERR(0, 440, __pyx_L1_error)

  /* "cupy/cudnn.pyx":448
 * 
 * 
 * def create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),
 *                       py_cudnn.destroySpatialTransformerDescriptor)
 */
  __pyx_tuple__49 = PyTuple_Pack(6, __pyx_n_s_sampler_type, __pyx_n_s_dtype, __pyx_n_s_nb_dims, __pyx_n_s_dim_A, __pyx_n_s_desc, __pyx_n_s_data_type); if (unlikely(!__pyx_tuple__49)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);
  __pyx_codeobj__50 = (PyObject*)__Pyx_PyCode_New(4, 0, 6, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__49, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_spatial_transformer_descr, 448, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__50)) __PYX_ERR(0, 448, __pyx_L1_error)

  /* "cupy/cudnn.pyx":458
 * 
 * 
 * def add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc,             # <<<<<<<<<<<<<<
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,
 */
  __pyx_tuple__51 = PyTuple_Pack(7, __pyx_n_s_handle, __pyx_n_s_alpha, __pyx_n_s_biasDesc, __pyx_n_s_biasData, __pyx_n_s_beta, __pyx_n_s_srcDestDesc, __pyx_n_s_srcDestData); if (unlikely(!__pyx_tuple__51)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);
  __pyx_codeobj__52 = (PyObject*)__Pyx_PyCode_New(7, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__51, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_add_tensor, 458, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__52)) __PYX_ERR(0, 458, __pyx_L1_error)

  /* "cupy/cudnn.pyx":464
 * 
 * 
 * def create_op_tensor_descriptor(op_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),
 *                       py_cudnn.destroyOpTensorDescriptor)
 */
  __pyx_tuple__53 = PyTuple_Pack(4, __pyx_n_s_op_type, __pyx_n_s_dtype, __pyx_n_s_desc, __pyx_n_s_data_type); if (unlikely(!__pyx_tuple__53)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);
  __pyx_codeobj__54 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__53, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_op_tensor_descriptor, 464, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__54)) __PYX_ERR(0, 464, __pyx_L1_error)

  /* "cupy/cudnn.pyx":474
 * 
 * 
 * def create_reduce_tensor_descriptor(reduce_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),
 *                       py_cudnn.destroyReduceTensorDescriptor)
 */
  __pyx_tuple__55 = PyTuple_Pack(5, __pyx_n_s_reduce_type, __pyx_n_s_dtype, __pyx_n_s_desc, __pyx_n_s_data_type, __pyx_n_s_indices); if (unlikely(!__pyx_tuple__55)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);
  __pyx_codeobj__56 = (PyObject*)__Pyx_PyCode_New(2, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__55, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_create_reduce_tensor_descriptor, 474, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__56)) __PYX_ERR(0, 474, __pyx_L1_error)

  /* "cupy/cudnn.pyx":497
 * 
 * 
 * class DropoutStates(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, handle, seed):
 */
  __pyx_tuple__57 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__57)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);

  /* "cupy/cudnn.pyx":499
 * class DropoutStates(object):
 * 
 *     def __init__(self, handle, seed):             # <<<<<<<<<<<<<<
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)
 */
  __pyx_tuple__58 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_handle, __pyx_n_s_seed, __pyx_n_s_state_size); if (unlikely(!__pyx_tuple__58)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  __pyx_codeobj__59 = (PyObject*)__Pyx_PyCode_New(3, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__58, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_init, 499, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__59)) __PYX_ERR(0, 499, __pyx_L1_error)

  /* "cupy/cudnn.pyx":506
 *             state_size, seed)
 * 
 *     def forward(self, handle, core.ndarray x, dropout_ratio):             # <<<<<<<<<<<<<<
 *         cdef core.ndarray y, reserve_space
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)
 */
  __pyx_tuple__60 = PyTuple_Pack(8, __pyx_n_s_self, __pyx_n_s_handle, __pyx_n_s_x, __pyx_n_s_dropout_ratio, __pyx_n_s_y, __pyx_n_s_reserve_space, __pyx_n_s_x_desc, __pyx_n_s_reserve_size); if (unlikely(!__pyx_tuple__60)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);
  __pyx_codeobj__61 = (PyObject*)__Pyx_PyCode_New(4, 0, 8, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__60, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_forward, 506, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__61)) __PYX_ERR(0, 506, __pyx_L1_error)

  /* "cupy/cudnn.pyx":526
 *         return reserve_space, y
 * 
 *     def backward(self, handle, core.ndarray dy, dropout_ratio,             # <<<<<<<<<<<<<<
 *                  core.ndarray reserve_space):
 *         cdef core.ndarray dx
 */
  __pyx_tuple__62 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_handle, __pyx_n_s_dy, __pyx_n_s_dropout_ratio, __pyx_n_s_reserve_space, __pyx_n_s_dx, __pyx_n_s_dy_desc); if (unlikely(!__pyx_tuple__62)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);
  __pyx_codeobj__63 = (PyObject*)__Pyx_PyCode_New(5, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__62, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_backward, 526, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__63)) __PYX_ERR(0, 526, __pyx_L1_error)

  /* "cupy/cudnn.pyx":803
 * 
 * 
 * def convolution_forward(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */
  __pyx_tuple__64 = PyTuple_Pack(30, __pyx_n_s_x, __pyx_n_s_W, __pyx_n_s_b, __pyx_n_s_y, __pyx_n_s_pad, __pyx_n_s_stride, __pyx_n_s_dilation, __pyx_n_s_groups, __pyx_n_s_auto_tune, __pyx_n_s_tensor_core, __pyx_n_s_dev_id, __pyx_n_s_float_zero, __pyx_n_s_float_one, __pyx_n_s_double_zero, __pyx_n_s_double_one, __pyx_n_s_zero, __pyx_n_s_one, __pyx_n_s_use_tensor_core, __pyx_n_s_conv_param, __pyx_n_s_i, __pyx_n_s_handle, __pyx_n_s_x_desc, __pyx_n_s_y_desc, __pyx_n_s_b_desc, __pyx_n_s_filter_desc, __pyx_n_s_conv_desc, __pyx_n_s_max_workspace_size, __pyx_n_s_b_shape, __pyx_n_s_perf, __pyx_n_s_workspace); if (unlikely(!__pyx_tuple__64)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);
  __pyx_codeobj__65 = (PyObject*)__Pyx_PyCode_New(8, 2, 30, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__64, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_convolution_forward, 803, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__65)) __PYX_ERR(0, 803, __pyx_L1_error)

  /* "cupy/cudnn.pyx":890
 * 
 * 
 * def convolution_backward_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray gy, core.ndarray gW,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */
  __pyx_tuple__66 = PyTuple_Pack(30, __pyx_n_s_x, __pyx_n_s_gy, __pyx_n_s_gW, __pyx_n_s_pad, __pyx_n_s_stride, __pyx_n_s_dilation, __pyx_n_s_groups, __pyx_n_s_deterministic, __pyx_n_s_auto_tune, __pyx_n_s_tensor_core, __pyx_n_s_dev_id, __pyx_n_s_float_zero, __pyx_n_s_float_one, __pyx_n_s_double_zero, __pyx_n_s_double_one, __pyx_n_s_zero, __pyx_n_s_one, __pyx_n_s_use_tensor_core, __pyx_n_s_conv_param, __pyx_n_s_handle, __pyx_n_s_x_desc, __pyx_n_s_gy_desc, __pyx_n_s_filter_desc, __pyx_n_s_conv_desc, __pyx_n_s_perf, __pyx_n_s_algo, __pyx_n_s_max_workspace_size, __pyx_n_s_workspace_size, __pyx_n_s_math_type, __pyx_n_s_workspace); if (unlikely(!__pyx_tuple__66)) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);
  __pyx_codeobj__67 = (PyObject*)__Pyx_PyCode_New(7, 3, 30, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__66, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_convolution_backward_filter, 890, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__67)) __PYX_ERR(0, 890, __pyx_L1_error)

  /* "cupy/cudnn.pyx":968
 * 
 * 
 * def convolution_backward_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */
  __pyx_tuple__68 = PyTuple_Pack(34, __pyx_n_s_W, __pyx_n_s_x, __pyx_n_s_b, __pyx_n_s_y, __pyx_n_s_pad, __pyx_n_s_stride, __pyx_n_s_dilation, __pyx_n_s_groups, __pyx_n_s_deterministic, __pyx_n_s_auto_tune, __pyx_n_s_tensor_core, __pyx_n_s_dev_id, __pyx_n_s_float_zero, __pyx_n_s_float_one, __pyx_n_s_double_zero, __pyx_n_s_double_one, __pyx_n_s_zero, __pyx_n_s_one, __pyx_n_s_use_tensor_core, __pyx_n_s_conv_param, __pyx_n_s_i, __pyx_n_s_handle, __pyx_n_s_x_desc, __pyx_n_s_y_desc, __pyx_n_s_b_desc, __pyx_n_s_filter_desc, __pyx_n_s_conv_desc, __pyx_n_s_perf, __pyx_n_s_algo, __pyx_n_s_max_workspace_size, __pyx_n_s_workspace_size, __pyx_n_s_b_shape, __pyx_n_s_math_type, __pyx_n_s_workspace); if (unlikely(!__pyx_tuple__68)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_codeobj__69 = (PyObject*)__Pyx_PyCode_New(8, 3, 34, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__68, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_cupy_cudnn_pyx, __pyx_n_s_convolution_backward_data, 968, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__69)) __PYX_ERR(0, 968, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4715b48:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__70 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__70)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);
  __pyx_codeobj__71 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__70, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle__Algorithm, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__71)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_ = PyFloat_FromDouble(0.); if (unlikely(!__pyx_float_0_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_70 = PyInt_FromLong(70); if (unlikely(!__pyx_int_70)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_74537800 = PyInt_FromLong(74537800L); if (unlikely(!__pyx_int_74537800)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_modinit_global_init_code(void); /*proto*/
static int __Pyx_modinit_variable_export_code(void); /*proto*/
static int __Pyx_modinit_function_export_code(void); /*proto*/
static int __Pyx_modinit_type_init_code(void); /*proto*/
static int __Pyx_modinit_type_import_code(void); /*proto*/
static int __Pyx_modinit_variable_import_code(void); /*proto*/
static int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_4cupy_5cudnn__thread_local = Py_None; Py_INCREF(Py_None);
  __pyx_v_4cupy_5cudnn__algorithm_fwd_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_4cupy_5cudnn__Algorithm) < 0) __PYX_ERR(0, 547, __pyx_L1_error)
  __pyx_type_4cupy_5cudnn__Algorithm.tp_print = 0;
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_4cupy_5cudnn__Algorithm.tp_dictoffset && __pyx_type_4cupy_5cudnn__Algorithm.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_4cupy_5cudnn__Algorithm.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttrString(__pyx_m, "_Algorithm", (PyObject *)&__pyx_type_4cupy_5cudnn__Algorithm) < 0) __PYX_ERR(0, 547, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_4cupy_5cudnn__Algorithm) < 0) __PYX_ERR(0, 547, __pyx_L1_error)
  __pyx_ptype_4cupy_5cudnn__Algorithm = &__pyx_type_4cupy_5cudnn__Algorithm;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_ptype_4cupy_4cuda_6device_Device = __Pyx_ImportType("cupy.cuda.device", "Device", sizeof(struct __pyx_obj_4cupy_4cuda_6device_Device), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_6device_Device)) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6device_Device = (struct __pyx_vtabstruct_4cupy_4cuda_6device_Device*)__Pyx_GetVtable(__pyx_ptype_4cupy_4cuda_6device_Device->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6device_Device)) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_6memory_BaseMemory = __Pyx_ImportType("cupy.cuda.memory", "BaseMemory", sizeof(struct __pyx_obj_4cupy_4cuda_6memory_BaseMemory), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_6memory_BaseMemory)) __PYX_ERR(3, 10, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_6memory_MemoryPointer = __Pyx_ImportType("cupy.cuda.memory", "MemoryPointer", sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer)) __PYX_ERR(3, 19, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPointer*)__Pyx_GetVtable(__pyx_ptype_4cupy_4cuda_6memory_MemoryPointer->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPointer)) __PYX_ERR(3, 19, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_6memory_MemoryPool = __Pyx_ImportType("cupy.cuda.memory", "MemoryPool", sizeof(struct __pyx_obj_4cupy_4cuda_6memory_MemoryPool), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_6memory_MemoryPool)) __PYX_ERR(3, 47, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool = (struct __pyx_vtabstruct_4cupy_4cuda_6memory_MemoryPool*)__Pyx_GetVtable(__pyx_ptype_4cupy_4cuda_6memory_MemoryPool->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_6memory_MemoryPool)) __PYX_ERR(3, 47, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_8function_CPointer = __Pyx_ImportType("cupy.cuda.function", "CPointer", sizeof(struct __pyx_obj_4cupy_4cuda_8function_CPointer), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_8function_CPointer)) __PYX_ERR(4, 1, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_8function_Function = __Pyx_ImportType("cupy.cuda.function", "Function", sizeof(struct __pyx_obj_4cupy_4cuda_8function_Function), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_8function_Function)) __PYX_ERR(4, 5, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_8function_Function = (struct __pyx_vtabstruct_4cupy_4cuda_8function_Function*)__Pyx_GetVtable(__pyx_ptype_4cupy_4cuda_8function_Function->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_8function_Function)) __PYX_ERR(4, 5, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_8function_Module = __Pyx_ImportType("cupy.cuda.function", "Module", sizeof(struct __pyx_obj_4cupy_4cuda_8function_Module), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_8function_Module)) __PYX_ERR(4, 15, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_8function_Module = (struct __pyx_vtabstruct_4cupy_4cuda_8function_Module*)__Pyx_GetVtable(__pyx_ptype_4cupy_4cuda_8function_Module->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_8function_Module)) __PYX_ERR(4, 15, __pyx_L1_error)
  __pyx_ptype_4cupy_4cuda_8function_LinkState = __Pyx_ImportType("cupy.cuda.function", "LinkState", sizeof(struct __pyx_obj_4cupy_4cuda_8function_LinkState), 1); if (unlikely(!__pyx_ptype_4cupy_4cuda_8function_LinkState)) __PYX_ERR(4, 26, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4cuda_8function_LinkState = (struct __pyx_vtabstruct_4cupy_4cuda_8function_LinkState*)__Pyx_GetVtable(__pyx_ptype_4cupy_4cuda_8function_LinkState->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4cuda_8function_LinkState)) __PYX_ERR(4, 26, __pyx_L1_error)
  __pyx_ptype_4cupy_4core_4core_ndarray = __Pyx_ImportType("cupy.core.core", "ndarray", sizeof(struct __pyx_obj_4cupy_4core_4core_ndarray), 1); if (unlikely(!__pyx_ptype_4cupy_4core_4core_ndarray)) __PYX_ERR(5, 7, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4core_4core_ndarray = (struct __pyx_vtabstruct_4cupy_4core_4core_ndarray*)__Pyx_GetVtable(__pyx_ptype_4cupy_4core_4core_ndarray->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4core_4core_ndarray)) __PYX_ERR(5, 7, __pyx_L1_error)
  __pyx_ptype_4cupy_4core_4core_broadcast = __Pyx_ImportType("cupy.core.core", "broadcast", sizeof(struct __pyx_obj_4cupy_4core_4core_broadcast), 1); if (unlikely(!__pyx_ptype_4cupy_4core_4core_broadcast)) __PYX_ERR(5, 86, __pyx_L1_error)
  __pyx_ptype_4cupy_4core_4core_Indexer = __Pyx_ImportType("cupy.core.core", "Indexer", sizeof(struct __pyx_obj_4cupy_4core_4core_Indexer), 1); if (unlikely(!__pyx_ptype_4cupy_4core_4core_Indexer)) __PYX_ERR(5, 94, __pyx_L1_error)
  __pyx_vtabptr_4cupy_4core_4core_Indexer = (struct __pyx_vtabstruct_4cupy_4core_4core_Indexer*)__Pyx_GetVtable(__pyx_ptype_4cupy_4core_4core_Indexer->tp_dict); if (unlikely(!__pyx_vtabptr_4cupy_4core_4core_Indexer)) __PYX_ERR(5, 94, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("cupy.cuda.device"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "get_device_id", (void (**)(void))&__pyx_f_4cupy_4cuda_6device_get_device_id, "int (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "get_compute_capability", (void (**)(void))&__pyx_f_4cupy_4cuda_6device_get_compute_capability, "PyObject *(int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = __Pyx_ImportModule("cupy.cuda.memory"); if (!__pyx_t_2) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "alloc", (void (**)(void))&__pyx_f_4cupy_4cuda_6memory_alloc, "struct __pyx_obj_4cupy_4cuda_6memory_MemoryPointer *(PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __Pyx_ImportModule("cupy.core.core"); if (!__pyx_t_3) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_3, "ascontiguousarray", (void (**)(void))&__pyx_f_4cupy_4core_4core_ascontiguousarray, "struct __pyx_obj_4cupy_4core_4core_ndarray *(struct __pyx_obj_4cupy_4core_4core_ndarray *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_4core_ascontiguousarray *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_ImportModule("cupy.cuda.cudnn"); if (!__pyx_t_4) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getVersion", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getVersion, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "create", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_create, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "destroy", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_destroy, "PyObject *(size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createTensorDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createTensorDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setTensor4dDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setTensor4dDescriptor, "PyObject *(size_t, int, int, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setTensorNdDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setTensorNdDescriptor, "PyObject *(size_t, int, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "destroyTensorDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_destroyTensorDescriptor, "PyObject *(size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "addTensor_v3", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_addTensor_v3, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createOpTensorDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createOpTensorDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setOpTensorDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setOpTensorDescriptor, "PyObject *(size_t, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createReduceTensorDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createReduceTensorDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setReduceTensorDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setReduceTensorDescriptor, "PyObject *(size_t, int, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createFilterDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createFilterDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setFilter4dDescriptor_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setFilter4dDescriptor_v4, "PyObject *(size_t, int, int, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setFilterNdDescriptor_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setFilterNdDescriptor_v4, "PyObject *(size_t, int, int, int, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getFilterNdDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getFilterNdDescriptor, "PyObject *(size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "destroyFilterDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_destroyFilterDescriptor, "PyObject *(size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createConvolutionDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createConvolutionDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setConvolutionMathType", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setConvolutionMathType, "PyObject *(size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setConvolutionGroupCount", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setConvolutionGroupCount, "PyObject *(size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setConvolution2dDescriptor_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setConvolution2dDescriptor_v4, "PyObject *(size_t, int, int, int, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setConvolution2dDescriptor_v5", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setConvolution2dDescriptor_v5, "PyObject *(size_t, int, int, int, int, int, int, int, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setConvolutionNdDescriptor_v3", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setConvolutionNdDescriptor_v3, "PyObject *(size_t, int, size_t, size_t, size_t, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "destroyConvolutionDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_destroyConvolutionDescriptor, "PyObject *(size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "findConvolutionForwardAlgorithmEx", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_findConvolutionForwardAlgorithmEx, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "findConvolutionForwardAlgorithmEx_v7", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_findConvolutionForwardAlgorithmEx_v7, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionForwardAlgorithm_v6", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardAlgorithm_v6, "int (size_t, size_t, size_t, size_t, size_t, int, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionForwardAlgorithm_v7", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardAlgorithm_v7, "PyObject *(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionForwardWorkspaceSize", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionForwardWorkspaceSize, "Py_ssize_t (size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "convolutionForward", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_convolutionForward, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "findConvolutionBackwardFilterAlgorithmEx", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardFilterAlgorithmEx, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "findConvolutionBackwardFilterAlgorithmEx_v7", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardFilterAlgorithmEx_v7, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionBackwardFilterAlgorithm_v6", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterAlgorithm_v6, "int (size_t, size_t, size_t, size_t, size_t, int, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionBackwardFilterAlgorithm_v7", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterAlgorithm_v7, "PyObject *(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionBackwardFilterWorkspaceSize", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardFilterWorkspaceSize, "Py_ssize_t (size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "convolutionBackwardFilter_v3", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_convolutionBackwardFilter_v3, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "findConvolutionBackwardDataAlgorithmEx", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardDataAlgorithmEx, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "findConvolutionBackwardDataAlgorithmEx_v7", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_findConvolutionBackwardDataAlgorithmEx_v7, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionBackwardDataAlgorithm_v6", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataAlgorithm_v6, "int (size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionBackwardDataAlgorithm_v7", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataAlgorithm_v7, "PyObject *(size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getConvolutionBackwardDataWorkspaceSize", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getConvolutionBackwardDataWorkspaceSize, "Py_ssize_t (size_t, size_t, size_t, size_t, size_t, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "convolutionBackwardData_v3", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_convolutionBackwardData_v3, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, int, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createPoolingDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createPoolingDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setPooling2dDescriptor_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setPooling2dDescriptor_v4, "PyObject *(size_t, int, int, int, int, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setPoolingNdDescriptor_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setPoolingNdDescriptor_v4, "PyObject *(size_t, int, int, int, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createActivationDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createActivationDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setActivationDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setActivationDescriptor, "PyObject *(size_t, int, int, double, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "destroyActivationDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_destroyActivationDescriptor, "PyObject *(size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "activationForward_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_activationForward_v4, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "activationBackward_v4", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_activationBackward_v4, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createDropoutDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createDropoutDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "dropoutGetStatesSize", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_dropoutGetStatesSize, "Py_ssize_t (size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setDropoutDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setDropoutDescriptor, "PyObject *(size_t, size_t, float, size_t, size_t, unsigned PY_LONG_LONG, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getDropoutReserveSpaceSize", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getDropoutReserveSpaceSize, "size_t (size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "dropoutForward", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_dropoutForward, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "dropoutBackward", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_dropoutBackward, "PyObject *(size_t, size_t, size_t, size_t, size_t, size_t, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createRNNDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createRNNDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setRNNDescriptor_v5", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setRNNDescriptor_v5, "PyObject *(size_t, int, int, size_t, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setRNNDescriptor_v6", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setRNNDescriptor_v6, "PyObject *(size_t, size_t, int, int, size_t, int, int, int, int, int, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getRNNLinLayerMatrixParams", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getRNNLinLayerMatrixParams, "PyObject *(size_t, size_t, int, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "getRNNLinLayerBiasParams", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_getRNNLinLayerBiasParams, "PyObject *(size_t, size_t, int, size_t, size_t, size_t, int, size_t, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "createSpatialTransformerDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_createSpatialTransformerDescriptor, "size_t (int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_4, "setSpatialTransformerDescriptor", (void (**)(void))&__pyx_f_4cupy_4cuda_5cudnn_setSpatialTransformerDescriptor, "PyObject *(size_t, size_t, int, int, size_t, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_ImportModule("cupy.core.internal"); if (!__pyx_t_5) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_5, "prod", (void (**)(void))&__pyx_f_4cupy_4core_8internal_prod, "Py_ssize_t (PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_4cupy_4core_8internal_prod *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#if PY_MAJOR_VERSION < 3
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC void
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#else
#ifdef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#endif
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (!(defined(__cplusplus)) || (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ > 4)))
    #define CYTHON_SMALL_CODE __attribute__((optimize("Os")))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initcudnn(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initcudnn(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_cudnn(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_cudnn(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_cudnn(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_cudnn(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("cudnn", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_cupy__cudnn) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "cupy.cudnn")) {
      if (unlikely(PyDict_SetItemString(modules, "cupy.cudnn", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() != 0)) goto __pyx_L1_error;
  if (unlikely(__Pyx_modinit_type_import_code() != 0)) goto __pyx_L1_error;
  (void)__Pyx_modinit_variable_import_code();
  if (unlikely(__Pyx_modinit_function_import_code() != 0)) goto __pyx_L1_error;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "cupy/cudnn.pyx":3
 * from libcpp cimport vector
 * 
 * import atexit             # <<<<<<<<<<<<<<
 * import threading
 * import warnings
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_atexit, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_atexit, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":4
 * 
 * import atexit
 * import threading             # <<<<<<<<<<<<<<
 * import warnings
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_1) < 0) __PYX_ERR(0, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":5
 * import atexit
 * import threading
 * import warnings             # <<<<<<<<<<<<<<
 * 
 * import numpy
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_warnings, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warnings, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":7
 * import warnings
 * 
 * import numpy             # <<<<<<<<<<<<<<
 * 
 * from cupy.core cimport core
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_numpy, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":15
 * from cupy.cuda cimport memory
 * 
 * from cupy.cuda import cudnn as py_cudnn             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_cudnn);
  __Pyx_GIVEREF(__pyx_n_s_cudnn);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_cudnn);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_cupy_cuda, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_cudnn); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_py_cudnn, __pyx_t_1) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":18
 * 
 * 
 * cdef int _cudnn_version = cudnn.getVersion()             # <<<<<<<<<<<<<<
 * cdef _thread_local = threading.local()
 * 
 */
  __pyx_t_3 = __pyx_f_4cupy_4cuda_5cudnn_getVersion(0); if (unlikely(__pyx_t_3 == ((size_t)0) && PyErr_Occurred())) __PYX_ERR(0, 18, __pyx_L1_error)
  __pyx_v_4cupy_5cudnn__cudnn_version = __pyx_t_3;

  /* "cupy/cudnn.pyx":19
 * 
 * cdef int _cudnn_version = cudnn.getVersion()
 * cdef _thread_local = threading.local()             # <<<<<<<<<<<<<<
 * 
 * cdef vector.vector[size_t] _handles
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_local); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_4cupy_5cudnn__thread_local);
  __Pyx_DECREF_SET(__pyx_v_4cupy_5cudnn__thread_local, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":37
 * 
 * 
 * @atexit.register             # <<<<<<<<<<<<<<
 * def reset_handles():
 *     for handle in _handles:
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_atexit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_register); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "cupy/cudnn.pyx":38
 * 
 * @atexit.register
 * def reset_handles():             # <<<<<<<<<<<<<<
 *     for handle in _handles:
 *         if handle:
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_3reset_handles, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "cupy/cudnn.pyx":37
 * 
 * 
 * @atexit.register             # <<<<<<<<<<<<<<
 * def reset_handles():
 *     for handle in _handles:
 */
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_reset_handles, __pyx_t_4) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":51
 * 
 * 
 * cdef size_t _max_workspace_size = 8 * 1024 * 1024             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_4cupy_5cudnn__max_workspace_size = 0x800000;

  /* "cupy/cudnn.pyx":79
 * 
 * 
 * class Descriptor(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, descriptor, destroyer):
 */
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__16); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_tuple__16, __pyx_n_s_Descriptor, __pyx_n_s_Descriptor, (PyObject *) NULL, __pyx_n_s_cupy_cudnn, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "cupy/cudnn.pyx":81
 * class Descriptor(object):
 * 
 *     def __init__(self, descriptor, destroyer):             # <<<<<<<<<<<<<<
 *         self.value = descriptor
 *         self.destroy = destroyer
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_10Descriptor_1__init__, 0, __pyx_n_s_Descriptor___init, NULL, __pyx_n_s_cupy_cudnn, __pyx_d, ((PyObject *)__pyx_codeobj__18)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_1) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":85
 *         self.destroy = destroyer
 * 
 *     def __del__(self):             # <<<<<<<<<<<<<<
 *         if self.value:
 *             self.destroy(self.value)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_10Descriptor_3__del__, 0, __pyx_n_s_Descriptor___del, NULL, __pyx_n_s_cupy_cudnn, __pyx_d, ((PyObject *)__pyx_codeobj__20)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_del, __pyx_t_1) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":79
 * 
 * 
 * class Descriptor(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, descriptor, destroyer):
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_n_s_Descriptor, __pyx_tuple__16, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Descriptor, __pyx_t_1) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":157
 * 
 * cpdef _create_filter_descriptor(
 *         size_t desc, core.ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_shape
 *     cdef Py_ssize_t s, ndim = arr._shape.size()
 */
  __pyx_k__2 = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW;

  /* "cupy/cudnn.pyx":156
 * 
 * 
 * cpdef _create_filter_descriptor(             # <<<<<<<<<<<<<<
 *         size_t desc, core.ndarray arr, int format=cudnn.CUDNN_TENSOR_NCHW):
 *     cdef vector.vector[int] c_shape
 */
  __pyx_k__2 = __pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW;

  /* "cupy/cudnn.pyx":227
 * 
 * 
 * def create_tensor_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createTensorDescriptor(),
 *                       py_cudnn.destroyTensorDescriptor)
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_k__7 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_25create_tensor_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_tensor_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":234
 * 
 * 
 * def create_uninitialized_tensor_descriptor():             # <<<<<<<<<<<<<<
 *     """Create uninitialized tensor descriptor.
 * 
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_27create_uninitialized_tensor_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_uninitialized_tensor_desc, __pyx_t_4) < 0) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":244
 * 
 * 
 * def create_tensor_nd_descriptor(core.ndarray arr):             # <<<<<<<<<<<<<<
 *     cdef dict cache
 *     if not arr.flags.c_contiguous:
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_29create_tensor_nd_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_tensor_nd_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":263
 * 
 * 
 * def create_filter_descriptor(arr, format=cudnn.CUDNN_TENSOR_NCHW):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createFilterDescriptor(),
 *                       py_cudnn.destroyFilterDescriptor)
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_TENSOR_NCHW); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_k__9 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_31create_filter_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_filter_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":271
 * 
 * def create_convolution_descriptor(pad, stride, dtype,
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,             # <<<<<<<<<<<<<<
 *                                   dilation=None,
 *                                   use_tensor_core=False,
 */
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_e_4cupy_4cuda_5cudnn_CUDNN_CROSS_CORRELATION); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_k__10 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":270
 * 
 * 
 * def create_convolution_descriptor(pad, stride, dtype,             # <<<<<<<<<<<<<<
 *                                   mode=cudnn.CUDNN_CROSS_CORRELATION,
 *                                   dilation=None,
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_33create_convolution_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_convolution_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":283
 * 
 * 
 * def create_pooling_descriptor(ksize, stride, pad, int mode):             # <<<<<<<<<<<<<<
 *     cdef vector.vector[int] c_ksize, c_pad, c_stride
 *     cdef int ndim = len(ksize)
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_35create_pooling_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_pooling_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":305
 * 
 * 
 * def activation_forward(core.ndarray x, int mode, double coef=0.0):             # <<<<<<<<<<<<<<
 *     cdef float float_zero = 0, float_one = 1
 *     cdef double double_zero = 0, double_one = 1
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_37activation_forward, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_activation_forward, __pyx_t_4) < 0) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":336
 * 
 * 
 * def activation_backward(core.ndarray x, core.ndarray y, core.ndarray gy,             # <<<<<<<<<<<<<<
 *                         int mode, float coef=0.0):
 *     cdef float float_zero = 0, float_one = 1
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_39activation_backward, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_activation_backward, __pyx_t_4) < 0) __PYX_ERR(0, 336, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":371
 * 
 * 
 * def create_dropout_descriptor(             # <<<<<<<<<<<<<<
 *         handle, dropout, states, state_size_in_bytes, seed):
 *     desc = Descriptor(cudnn.createDropoutDescriptor(),
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_41create_dropout_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 371, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_dropout_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 371, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":380
 * 
 * 
 * def set_dropout_descriptor(desc, handle, dropout):             # <<<<<<<<<<<<<<
 *     # When the fourth argument is NULL, random state is not updated.
 *     cudnn.setDropoutDescriptor(desc.value, handle, dropout, 0, 0, 0)
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_43set_dropout_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_dropout_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 380, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":385
 * 
 * 
 * def create_rnn_descriptor(hidden_size, num_layers, dropout_desc,             # <<<<<<<<<<<<<<
 *                           input_mode, direction, mode, data_type, algo=None):
 *     desc = Descriptor(cudnn.createRNNDescriptor(),
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_45create_rnn_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_rnn_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":403
 * 
 * 
 * def get_rnn_lin_layer_matrix_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_47get_rnn_lin_layer_matrix_params, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_rnn_lin_layer_matrix_params, __pyx_t_4) < 0) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":422
 * 
 * 
 * def get_rnn_lin_layer_bias_params(             # <<<<<<<<<<<<<<
 *         handle, rnn_desc, layer, x_desc, w_desc, core.ndarray w, lin_layer_id):
 *     cdef size_t ptr = 0
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_49get_rnn_lin_layer_bias_params, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_rnn_lin_layer_bias_params, __pyx_t_4) < 0) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":440
 * 
 * 
 * def create_dropout_states(handle):             # <<<<<<<<<<<<<<
 *     warnings.warn('create_dropout_states is deprecated.'
 *                   'Please use DropoutStates class instead.',
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_51create_dropout_states, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_dropout_states, __pyx_t_4) < 0) __PYX_ERR(0, 440, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":448
 * 
 * 
 * def create_spatial_transformer_descriptor(sampler_type, dtype, nb_dims, dim_A):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createSpatialTransformerDescriptor(),
 *                       py_cudnn.destroySpatialTransformerDescriptor)
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_53create_spatial_transformer_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_spatial_transformer_descr, __pyx_t_4) < 0) __PYX_ERR(0, 448, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":458
 * 
 * 
 * def add_tensor(handle, alpha, biasDesc, biasData, beta, srcDestDesc,             # <<<<<<<<<<<<<<
 *                srcDestData):
 *     cudnn.addTensor_v3(handle, alpha, biasDesc,
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_55add_tensor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_add_tensor, __pyx_t_4) < 0) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":464
 * 
 * 
 * def create_op_tensor_descriptor(op_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createOpTensorDescriptor(),
 *                       py_cudnn.destroyOpTensorDescriptor)
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_57create_op_tensor_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_op_tensor_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 464, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":474
 * 
 * 
 * def create_reduce_tensor_descriptor(reduce_type, dtype):             # <<<<<<<<<<<<<<
 *     desc = Descriptor(cudnn.createReduceTensorDescriptor(),
 *                       py_cudnn.destroyReduceTensorDescriptor)
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_59create_reduce_tensor_descriptor, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_reduce_tensor_descriptor, __pyx_t_4) < 0) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":497
 * 
 * 
 * class DropoutStates(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, handle, seed):
 */
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__57); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_tuple__57, __pyx_n_s_DropoutStates, __pyx_n_s_DropoutStates, (PyObject *) NULL, __pyx_n_s_cupy_cudnn, (PyObject *) NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "cupy/cudnn.pyx":499
 * class DropoutStates(object):
 * 
 *     def __init__(self, handle, seed):             # <<<<<<<<<<<<<<
 *         state_size = cudnn.dropoutGetStatesSize(handle)
 *         self._states = memory.alloc(state_size)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_13DropoutStates_1__init__, 0, __pyx_n_s_DropoutStates___init, NULL, __pyx_n_s_cupy_cudnn, __pyx_d, ((PyObject *)__pyx_codeobj__59)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_init, __pyx_t_1) < 0) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":506
 *             state_size, seed)
 * 
 *     def forward(self, handle, core.ndarray x, dropout_ratio):             # <<<<<<<<<<<<<<
 *         cdef core.ndarray y, reserve_space
 *         set_dropout_descriptor(self._desc, handle, dropout_ratio)
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_13DropoutStates_3forward, 0, __pyx_n_s_DropoutStates_forward, NULL, __pyx_n_s_cupy_cudnn, __pyx_d, ((PyObject *)__pyx_codeobj__61)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_forward, __pyx_t_1) < 0) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":526
 *         return reserve_space, y
 * 
 *     def backward(self, handle, core.ndarray dy, dropout_ratio,             # <<<<<<<<<<<<<<
 *                  core.ndarray reserve_space):
 *         cdef core.ndarray dx
 */
  __pyx_t_1 = __Pyx_CyFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_13DropoutStates_5backward, 0, __pyx_n_s_DropoutStates_backward, NULL, __pyx_n_s_cupy_cudnn, __pyx_d, ((PyObject *)__pyx_codeobj__63)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_SetNameInClass(__pyx_t_2, __pyx_n_s_backward, __pyx_t_1) < 0) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "cupy/cudnn.pyx":497
 * 
 * 
 * class DropoutStates(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, handle, seed):
 */
  __pyx_t_1 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_n_s_DropoutStates, __pyx_tuple__57, __pyx_t_2, NULL, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_DropoutStates, __pyx_t_1) < 0) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":562
 * 
 * 
 * cdef dict _algorithm_fwd_cache = {}             # <<<<<<<<<<<<<<
 * cdef dict _algorithm_bwd_filter_cache = {}
 * cdef dict _algorithm_bwd_data_cache = {}
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_XGOTREF(__pyx_v_4cupy_5cudnn__algorithm_fwd_cache);
  __Pyx_DECREF_SET(__pyx_v_4cupy_5cudnn__algorithm_fwd_cache, ((PyObject*)__pyx_t_4));
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":563
 * 
 * cdef dict _algorithm_fwd_cache = {}
 * cdef dict _algorithm_bwd_filter_cache = {}             # <<<<<<<<<<<<<<
 * cdef dict _algorithm_bwd_data_cache = {}
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 563, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_XGOTREF(__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache);
  __Pyx_DECREF_SET(__pyx_v_4cupy_5cudnn__algorithm_bwd_filter_cache, ((PyObject*)__pyx_t_4));
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":564
 * cdef dict _algorithm_fwd_cache = {}
 * cdef dict _algorithm_bwd_filter_cache = {}
 * cdef dict _algorithm_bwd_data_cache = {}             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 564, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_XGOTREF(__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache);
  __Pyx_DECREF_SET(__pyx_v_4cupy_5cudnn__algorithm_bwd_data_cache, ((PyObject*)__pyx_t_4));
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":803
 * 
 * 
 * def convolution_forward(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray W, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_83convolution_forward, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convolution_forward, __pyx_t_4) < 0) __PYX_ERR(0, 803, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":890
 * 
 * 
 * def convolution_backward_filter(             # <<<<<<<<<<<<<<
 *         core.ndarray x, core.ndarray gy, core.ndarray gW,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_85convolution_backward_filter, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convolution_backward_filter, __pyx_t_4) < 0) __PYX_ERR(0, 890, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":968
 * 
 * 
 * def convolution_backward_data(             # <<<<<<<<<<<<<<
 *         core.ndarray W, core.ndarray x, core.ndarray b, core.ndarray y,
 *         tuple pad, tuple stride, tuple dilation, int groups, *,
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_87convolution_backward_data, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_convolution_backward_data, __pyx_t_4) < 0) __PYX_ERR(0, 968, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__Algorithm(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x4715b48:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_4 = PyCFunction_NewEx(&__pyx_mdef_4cupy_5cudnn_89__pyx_unpickle__Algorithm, NULL, __pyx_n_s_cupy_cudnn); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle__Algorithm, __pyx_t_4) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "cupy/cudnn.pyx":1
 * from libcpp cimport vector             # <<<<<<<<<<<<<<
 * 
 * import atexit
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_4) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "vector.to_py":60
 * 
 * @cname("__pyx_convert_vector_to_py_Py_ssize_t")
 * cdef object __pyx_convert_vector_to_py_Py_ssize_t(vector[X]& v):             # <<<<<<<<<<<<<<
 *     return [v[i] for i in range(v.size())]
 * 
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init cupy.cudnn", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init cupy.cudnn");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* UnicodeAsUCS4 */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", c->ob_type->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* None */
static CYTHON_INLINE Py_ssize_t __Pyx_div_Py_ssize_t(Py_ssize_t a, Py_ssize_t b) {
    Py_ssize_t q = a / b;
    Py_ssize_t r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    if (likely(result)) {
        Py_INCREF(result);
    } else if (unlikely(PyErr_Occurred())) {
        result = NULL;
    } else {
#else
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* ExtTypeTest */
    static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* DictGetItem */
    #if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* GetItemInt */
    static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* BytesEquals */
    static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
    static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* GetException */
    #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* SwapException */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = *type;
    tstate->exc_state.exc_value = *value;
    tstate->exc_state.exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* SaveResetException */
      #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* SliceObject */
      static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* PyIntBinop */
      #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                    CYTHON_FALLTHROUGH;
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* PyErrExceptionMatches */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr3 */
      static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* UnpackUnboundCMethod */
      static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST);
    }
#endif
    return 0;
}

/* CallUnboundCMethod1 */
      #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    if (likely(cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return (*(cfunc->func))(self, arg);
        } else if (PY_VERSION_HEX >= 0x030600B1 && flag == METH_FASTCALL) {
            if (PY_VERSION_HEX >= 0x030700A0) {
                return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, &arg, 1);
            } else {
                return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
            }
        } else if (PY_VERSION_HEX >= 0x030700A0 && flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, &arg, 1, NULL);
        }
    }
    return __Pyx__CallUnboundCMethod1(cfunc, self, arg);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(1);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 1, arg);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(2, self, arg);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* CallUnboundCMethod2 */
      #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030600B1
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    if (likely(cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            #if PY_VERSION_HEX >= 0x030700A0
            return (*(__Pyx_PyCFunctionFast)cfunc->func)(self, args, 2);
            #else
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, args, 2, NULL);
            #endif
        }
        #if PY_VERSION_HEX >= 0x030700A0
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return (*(__Pyx_PyCFunctionFastWithKeywords)cfunc->func)(self, args, 2, NULL);
        #endif
    }
    return __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        args = PyTuple_New(2);
        if (unlikely(!args)) goto bad;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = (*(PyCFunctionWithKeywords)cfunc->func)(self, args, NULL);
        else
            result = (*cfunc->func)(self, args);
    } else {
        args = PyTuple_New(3);
        if (unlikely(!args)) goto bad;
        Py_INCREF(self);
        PyTuple_SET_ITEM(args, 0, self);
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 1, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 2, arg2);
        result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    }
#else
    args = PyTuple_Pack(3, self, arg1, arg2);
    if (unlikely(!args)) goto bad;
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
#endif
bad:
    Py_XDECREF(args);
    return result;
}

/* dict_getitem_default */
      static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
    if ((1));
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    }
#endif
    else {
        if (default_value == Py_None)
            value = __Pyx_CallUnboundCMethod1(&__pyx_umethod_PyDict_Type_get, d, key);
        else
            value = __Pyx_CallUnboundCMethod2(&__pyx_umethod_PyDict_Type_get, d, key, default_value);
    }
    return value;
}

/* RaiseKeywordRequired */
      static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name) {
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() needs keyword-only argument %U", func_name, kw_name);
        #else
        "%s() needs keyword-only argument %s", func_name,
        PyString_AS_STRING(kw_name));
        #endif
}

/* Import */
      static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
      static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* PyObject_GenericGetAttrNoDict */
      #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
      #if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetupReduce */
      static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* GetVTable */
      static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObjectCallNoArg */
      #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* CalculateMetaclass */
        static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
        static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
        #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Py3ClassCreate */
            static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
            #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
            static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
            #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
            static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
            #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
            static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
            static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
            static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
            static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
            static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_As_unsigned_PY_LONG_LONG(PyObject *x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG) -1, const_zero = (unsigned PY_LONG_LONG) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned PY_LONG_LONG) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned PY_LONG_LONG) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned PY_LONG_LONG) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) >= 2 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) >= 3 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) >= 4 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned PY_LONG_LONG) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned PY_LONG_LONG) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) ((((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) ((((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) ((((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned PY_LONG_LONG) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned PY_LONG_LONG val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned PY_LONG_LONG) -1;
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned PY_LONG_LONG) -1;
        val = __Pyx_PyInt_As_unsigned_PY_LONG_LONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned PY_LONG_LONG");
    return (unsigned PY_LONG_LONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned PY_LONG_LONG");
    return (unsigned PY_LONG_LONG) -1;
}

/* FastTypeChecks */
            #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
            static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
            #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
            #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* FunctionImport */
            #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
            static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
